<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: ff78b0c577ea $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-07-25 11:24 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
define('WE_DOCUMENT_ROOT', (string) realpath(__DIR__ . '/../../'));
if(file_exists('includes/proxysettings.inc.php')){
	include_once('includes/proxysettings.inc.php'); //@phan-suppress-current-line PhanMissingRequireFile
} elseif(file_exists(WE_DOCUMENT_ROOT . '/webEdition/custom/conf/proxysettings.inc.php')){
	include_once(WE_DOCUMENT_ROOT . '/webEdition/custom/conf/proxysettings.inc.php');
}
require_once(WE_DOCUMENT_ROOT . '/webEdition/we/include/we_autoload.inc.php');

if(!defined('IS_LIVE_SERVER') && is_dir('updateClient')){
	we_base_file::deleteLocalFolder('updateClient', true);
}

/*
 * Include all needed files
 */
require_once('includes/includes.inc.php');
we_html_tools::protect();

function dealUpdateCmd(string $cmd): void{

	/*
	 * Gather all needed Variables for the update-Request
	 */
	$parameters = [];
	foreach(welu_update_const::LU_ParameterNames as $parameterName){
		if(isset($_REQUEST[$parameterName])){
			$parameters[$parameterName] = $_REQUEST[$parameterName];
		}
	}

	/*
	 * For command checkConnection, it is not needed to create a session on the
	 * server. Therefore treat this command in a special way.
	 */
	if($cmd === 'checkConnection'){

		$response = liveUpdateHttp::getHttpResponse(welu_update_const::SERVER, welu_update_const::SCRIPT, $parameters);
		$liveUpdateResponse = new liveUpdateResponse();

		echo ($response && $liveUpdateResponse->initByHttpResponse($response) ?
			welu_update_frames::htmlConnectionSuccess($liveUpdateResponse->isError() ? $liveUpdateResponse->getField('Message') : '') :
			welu_update_frames::htmlConnectionError()
		);

		exit();
	}
	/*
	 * Before an update_cmd is submitted to the server, there must be an
	 * existing session on the server. $_REQUEST[liveUpdateSession] contains
	 * the session_id of the server. If this id is missing, create a new
	 * session on the server.
	 */
	if(empty($_REQUEST['liveUpdateSession'])){

		/*
		 * exit after submitting the form
		 */
		echo liveUpdateHttp::getServerSessionForm();
		exit;
	}
	/*
	 * $_REQUEST['liveUpdateSession'] exists => Session on server is up
	 * prepare all needed variables to submit to the updateServer
	 * These are stored in $LU_ParameterNames
	 */

	// add all other request parameters to the request
	$reqVars = [];
	$ignored = welu_update_const::getIgnoredParams();
	foreach($_REQUEST as $key => $value){
		if(!isset($parameters[$key]) && !in_array($key, $ignored, true) && !array_key_exists($key, $_COOKIE)){
			$reqVars[$key] = $value;
		}
	}
	$parameters['reqArray'] = base64_encode(we_serialize($reqVars));

	$response = liveUpdateHttp::getHttpResponse(welu_update_const::SERVER, welu_update_const::SCRIPT, $parameters);


	/*
	 * There is a response from the Update-Server.
	 */
	if($response){

		$liveUpdateResponse = new liveUpdateResponse();

		echo ($liveUpdateResponse->initByHttpResponse($response) ?
			$liveUpdateResponse->getOutput() :
			welu_update_frames::htmlConnectionError());
		return;
	}
	/*
	 * No response from the update-server. Error message
	 */
	echo welu_update_frames::htmlConnectionError();
}

/*
 * Deal with update_cmd
 */
if(isset($_REQUEST['update_cmd'])){
	dealUpdateCmd((string) $_REQUEST['update_cmd']);
} else {
	/*
	 * No update_cmd exists, show normal frameset
	 */
	$updateFrames = new welu_update_frames();
	echo $updateFrames->getFrame();
}
