<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 4836b978d48d $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2021-06-22 16:35 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_backup_exportBase{
	const EXPORT_DIR = 'database';
	const SITE_DIR = 'site';
	const FILES_DIR = 'files';
	const EXTERNALS_DIR = 'extern';
	const VERSIONS_DIR = 'versions';
	const TEMPLATES_DIR = 'templates';
	const CONF_DIR = 'conf';
	const HOOK_DIR = 'hook';
	const TAG_DIR = 'tag';
	const TW_DIR = 'tw';
	const MAX_BUFFER = we_base_util::MBYTE;

	/** @var string */
	protected $backup_file = '';

	/** @var string */
	protected $buffer = '';

	/** @var int */
	protected $binFiles = 0;
	protected $tableMap = [];

	abstract public function open(bool $init = false): bool;

	abstract protected function writeBackupDataToFile(string $data);

	abstract public function writeBinFile(string $file, string $dir = self::FILES_DIR, string $as = ''): bool;

	abstract public function writeBinWEFile(int $id, string $type = we_base_ContentTypes::BACKUP_BINARY): bool;

	abstract public function flush(bool $force = false);

	abstract protected function getOffset(): int;

	protected function __construct(string $filename){
		$settings = we_backup_settings::i();
		$settings->filename = $filename;
		$settings->backup_file = $this->backup_file = TEMP_PATH . $filename;
		we_base_file::insertIntoCleanUp($this->backup_file, 24 * 3600);
	}

	public function writeBackupData(string $data, string $newTable = ''): bool{
		if($newTable){
			$this->tableMap[$newTable] = [$this->getOffset() => $this->binFiles];
		}
		$this->buffer .= $data . "\n";
		if(strlen($this->buffer) > self::MAX_BUFFER){
			$this->writeBuffer();
		}
		return true;
	}

	private function writeBuffer(): bool{
		$ret = $this->writeBackupDataToFile($this->buffer);
		$this->buffer = '';
		return $ret;
	}

	public function close(): bool{
		if($this->buffer){
			$this->writeBuffer();
		}
		return true;
	}

	public function finish(): void{
//copy the file to right location
		$settings = we_backup_settings::i();
		$download = $settings->option(we_backup_settings::OPTION_BACKUP_EXPORT2SEND);
		if($settings->option(we_backup_settings::OPTION_BACKUP_EXPORT2SERVER) || !$download){//if no download, put it into export at least
			$backup_filename = WE_BACKUP_PATH . we_backup_util::SAVE_DIR . $settings->filename;

			we_backup_util::addLog('Move file to ' . $backup_filename);

			if($download){
				we_base_file::makeHardLink($settings->backup_file, $backup_filename);
			} else {
				we_base_file::moveFile($settings->backup_file, $backup_filename);
				$settings->backup_file = $backup_filename;
			}
		}

		if($settings->option(we_backup_settings::OPTION_BACKUP_EXPORT2SEND)){
			we_base_file::insertIntoCleanUp($settings->backup_file, 8 * 3600); //8h
		}
		we_backup_util::addLog('Backup export finished', true);
		//we_backup_settings::clearSessionData();
		//FIXME unsetting this causes problems in next step (download)
	}

}
