<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 9de6ddac66e5 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2021-11-18 21:01 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
/* a class for handling templates */
class we_contents_object extends we_contents_objectBase implements we_exim_xmlIF{

	public const BACKUP_XML_TAG = 'we:class';
	const QUERY_PREFIX = 'object_';
	protected const VARIANT_TYPES = [
		self::TYPE_INPUT,
		self::TYPE_LINK,
		self::TYPE_TEXT,
		self::TYPE_IMG,
		self::TYPE_INT,
		self::TYPE_FLOAT,
		self::TYPE_SELECT,
		self::TYPE_DATETIME,
		self::TYPE_DATE,
		self::TYPE_TIME,
		self::TYPE_HREF
	];
	protected const DEFAULT_FIELDS = [
		'OF_ID' => 'INT unsigned NOT NULL',
		'OF_DocumentTable' => "enum('tblObjectFiles','" . TEMPORARY_DOCTARGET . "') NOT NULL DEFAULT 'tblObjectFiles'"
	];
	protected const DEFAULT_INDEXES = ['PRIMARY KEY (OF_DocumentTable,OF_ID)'];
	private const URL_UNIQUE = ['urlunique' => 16,];

	/** @var int[] */
	var $DefaultWorkspaces = [];

	/** @var bool */
	var $WorkspaceFlag = true;

	/** @var int[] */
	var $Templates = [];

	/** @var array<string,mixed> */
	var $DefaultValues = [];

	/** @var string */
	protected $DefaultText = '';

	/** @var int[] */
	protected $DefaultCategory = [];

	/** @var string */
	protected $DefaultTitle = '';

	/** @var string */
	protected $DefaultKeywords = '';

	/** @var string */
	protected $DefaultUrl = '';

	/** @var string */
	protected $DefaultUrlfield0 = '';

	/** @var string */
	protected $DefaultUrlfield1 = '';

	/** @var string */
	protected $DefaultUrlfield2 = '';

	/** @var string */
	protected $DefaultUrlfield3 = '';

	/** @var int */
	protected $DefaultTriggerID = 0;

	/** @var int id of the sidebar document */
	protected $SidebarDocID = 0;

	/** @var string */
	protected $DefaultDesc = '';

	/** @var bool */
	private $delWorkspaceError = false;

	/** @var string database table in which the object is stored */
	public $Table = OBJECT_TABLE;

	public function __construct(int $id = 0){
		parent::__construct();
		$this->Publish_Date = new DateTime();
		$this->initByID($id);
	}

	public function initByID(int $id, string $table = OBJECT_TABLE, int $from = self::LOAD_MAIN_DB): void{
		parent::initByID($id, $table, $from);
		if(isWE()){
			$this->setSort();
		}
	}

	/** initializes all data not static in class and not saved by persistent slots/elements.
	 * This is called from constructor and __wakeup on serialized documents
	 */
	protected function initStatic(): void{
		parent::initStatic();
		$this->persistent_slots = array_merge($this->persistent_slots, [
			'WorkspaceFlag' => we_base_request::BOOL,
			'Templates' => we_base_request::INTLISTA,
			'Workspaces' => we_base_request::INTLISTA,
			'DefaultWorkspaces' => we_base_request::INTLISTA,
			'Category' => we_base_request::INTLISTA,
			'DefaultCategory' => we_base_request::INTLISTA,
			'DefaultText' => we_base_request::STRING,
			'DefaultValues' => we_base_request::SERIALIZED_JSON,
			'DefaultTitle' => we_base_request::STRING,
			'DefaultKeywords' => we_base_request::STRING,
			'DefaultUrl' => we_base_request::STRING,
			'DefaultUrlfield0' => we_base_request::STRING,
			'DefaultUrlfield1' => we_base_request::STRING,
			'DefaultUrlfield2' => we_base_request::STRING,
			'DefaultUrlfield3' => we_base_request::STRING,
			'DefaultTriggerID' => we_base_request::INT,
			'SidebarDocID' => we_base_request::INT,
			'DefaultDesc' => we_base_request::STRING,
			'CSS' => we_base_request::INTLISTA
		]);
		if(isWE()){
			$this->EditPageNrs = array_merge($this->EditPageNrs, [we_base_constants::WE_EDITPAGE_PROPERTIES, we_base_constants::WE_EDITPAGE_WORKSPACE, we_base_constants::WE_EDITPAGE_INFO,
				we_base_constants::WE_EDITPAGE_CONTENT]); // ,we_base_constants::WE_EDITPAGE_PREVIEW
			$this->setSort();
			$this->loadDefaultsValues();
		}
	}

	function save(bool $resave = false, bool $skipHook = false): bool{
		if(!$this->checkIfPathOk()){
			return false;
		}

		$this->Mod_Date = new DateTime();
		$this->ModifierID = !we_base_WE::$schedulerActive ? we_users_user::getID() : 0;

		if(!$this->saveToDB()){
			return false;
		}

		we_editor_save::$responseText = g_l('weClass', '[response_save_ok]');
		we_editor_save::$responseTextType = we_base_util::WE_MESSAGE_NOTICE;

		if(!$this->OldPath || ($this->OldPath != $this->Path)){
			$fID = fI('SELECT ID FROM ' . OBJECT_FILES_TABLE . ' WHERE Path="' . $this->DB_WE->escape($this->OldPath) . '"');
			$pID = fI('SELECT ID FROM ' . OBJECT_FILES_TABLE . ' WHERE Path="' . str_replace('\\', '/', dirname($this->Path)) . '"');
			$cf = new we_folder_objectFile();
			$cf->updateEntry($this->ID, $this->Filename, $pID, $fID);
		}

		$this->OldPath = $this->Path; // reset oldPath
		we_editor_save::$javaScript[] = ['reload_editpage'];
		we_editor_save::$javaScript[] = ['setEditorDocumentId', $this->ID];
		we_editor_save::$javaScript[] = we_gui_headermenu::getMenuReloadCode();

		return true;
	}

	function saveToDB(): bool{
		$defaultsArray = [
			'WE_CSS_FOR_CLASS' => $this->CSS,
			'WorkspaceFlag' => $this->WorkspaceFlag,
		];

		$this->wasUpdate = $this->ID > 0;

		$this->checkSetDefaultTextFromElements();
		$this->checkSetDefaultUrlFromElements();

		if(!$this->wasUpdate){//create table, add element to tblObject
			$q = self::DEFAULT_FIELDS;

			//FIXME this is wrong, see e.g. triggerID
			$this->DefaultTitle = ($tmp = $this->getElementS('title')) ? $this->getElementS($tmp . self::NAME_DIVIDER . self::ELEMENT_TYPE) . '_' . $this->getElementS($tmp . self::NAME_DIVIDER . 'name') : '_';
			$this->DefaultDesc = ($tmp = $this->getElementS('desc')) ? $this->getElementS($tmp . self::NAME_DIVIDER . self::ELEMENT_TYPE) . '_' . $this->getElementS($tmp) : '_';
			$this->DefaultKeywords = ($tmp = $this->getElementS('keywords')) ? $this->getElementS($tmp . self::NAME_DIVIDER . self::ELEMENT_TYPE) . '_' . $this->getElementS($tmp . self::NAME_DIVIDER . 'name') : '_';
			$this->DefaultUrlfield0 = ($tmp = $this->getElementS('urlfield0')) ? $this->getElementS($tmp . self::NAME_DIVIDER . self::ELEMENT_TYPE) . '_' . $this->getElementS($tmp . self::NAME_DIVIDER . 'name') : '_';
			$this->DefaultUrlfield1 = ($tmp = $this->getElementS('urlfield1')) ? $this->getElementS($tmp . self::NAME_DIVIDER . self::ELEMENT_TYPE) . '_' . $this->getElementS($this->getElementS('urlfield1')) : '_';
			$this->DefaultUrlfield2 = ($tmp = $this->getElementS('urlfield2')) ? $this->getElementS($tmp . self::NAME_DIVIDER . self::ELEMENT_TYPE) . '_' . $this->getElementS($tmp . self::NAME_DIVIDER . 'name') : '_';
			$this->DefaultUrlfield3 = ($tmp = $this->getElementS('urlfield3')) ? $this->getElementS($tmp . self::NAME_DIVIDER . self::ELEMENT_TYPE) . '_' . $this->getElementS($tmp . self::NAME_DIVIDER . 'name') : '_';
			//$this->DefaultTriggerID = ($tmp = $this->getElementS('triggerid')) ? $this->getElementS($tmp . self::ELEMENT_TYPE) . '_' . $this->getElementS($tmp) : 0;


			$this->DefaultCategory = $this->Category;
			$this->savePersistentSlotsToDB();

			$ctable = OBJECT_X_TABLE . intval($this->ID);

			$this->DB_WE->delTable($ctable);
			$this->DB_WE->addTable($ctable, $q, self::DEFAULT_INDEXES);

			$this->wasUpdate = true;
		}

		$we_sort = $this->getElementA('we_sort');
		asort($we_sort, SORT_NUMERIC);
		$this->setElementA('we_sort', $we_sort);

		$ctable = OBJECT_X_TABLE . intval($this->ID);
		$tableInfo = $this->DB_WE->metaFieldNames($ctable);
		$q = [];
		$fieldsToDelete = array_intersect($this->getElementA('felderloeschen'), $tableInfo);
		foreach($fieldsToDelete as $cur){
			$q[] = ' DROP `' . $cur . '` ';
		}

		$neu = $this->getElementA('neuefelder');
		$lastElement = 'OF_DocumentTable';

		foreach(array_keys($we_sort) as $id){
			$cur = $this->getElementS('wholename' . $id);
			$newType = $this->getElementS($cur . self::NAME_DIVIDER . self::ELEMENT_TYPE);
			$name = $newType . '_' . $this->getElementS($cur . self::NAME_DIVIDER . 'name');

			if(in_array($cur, $neu)){
				$q[] = ' ADD `' . $name . '` ' . $this->getSQLType($cur) . ' AFTER `' . $lastElement . '`';
			} else {
				//change from object is indexed to unindexed
				if((str_starts_with($cur, self::QUERY_PREFIX)) && ($newType . '_' != self::QUERY_PREFIX)){
					$q[] = ' DROP KEY `' . $cur . '` ';
				}

				$q[] = ' CHANGE `' . $cur . '` `' . $name . '` ' . $this->getSQLType($cur) . ' AFTER `' . $lastElement . '`';
			}
			//add index for complex queries
			if((!str_starts_with($cur, self::QUERY_PREFIX)) && ($newType . '_' == self::QUERY_PREFIX)){
				$q[] = 'ADD INDEX (`' . $name . '`) ';
			}
			$lastElement = $name;

			$arr = [];
			foreach(array_merge(self::COMMON_DEFAULTS_VALUES, self::DEFAULTS_VALUES[$newType]['defaults'] ?? []) as $key => $default){
				switch($default[1]){
					case we_base_request::BOOL:
						$def = (bool) $default[0];
						$arr[$key] = ($val = $this->getElementB($cur . self::NAME_DIVIDER . $key, $def)) !== $def ? $val : we_base_request::NOT_VALID_RAW;
						break;
					case we_base_request::INT:
						$def = (int) $default[0];
						$arr[$key] = ($val = $this->getElementI($cur . self::NAME_DIVIDER . $key, $def)) !== $def ? $val : we_base_request::NOT_VALID_RAW;
						break;
					case we_base_request::FLOAT:
						$def = (float) $default[0];
						$arr[$key] = ($val = $this->getElementF($cur . self::NAME_DIVIDER . $key, $def)) !== $def ? $val : we_base_request::NOT_VALID_RAW;
						break;
					case we_base_request::HTML:
					case we_base_request::STRING:
						$def = (string) $default[0];
						$arr[$key] = ($val = $this->getElementS($cur . self::NAME_DIVIDER . $key, $def)) !== $def ? $val : we_base_request::NOT_VALID_RAW;
						break;
					case we_base_request::INTLISTA:
					case we_base_request::STRING_LIST:
						$def = (array) $default[0];
						$arr[$key] = ($val = array_filter($this->getElementA($cur . self::NAME_DIVIDER . $key, $def))) !== $def ? $val : we_base_request::NOT_VALID_RAW;
						break;
					case 'arrayHref':
					case 'arrayLink':
						$def = (array) $default[0];
						$arr[$key] = ($val = array_filter($this->getElementA($cur . self::NAME_DIVIDER . $key, $def))) !== $def ? $val : we_base_request::NOT_VALID_RAW;
						break;
				}
			}
			$defaultsArray[$name] = array_filter($arr, function($value){
				return $value !== we_base_request::NOT_VALID_RAW;
			});

			// FIXME: $this->DefaultValues[$cur]: $cur or $name?
			$defaultsArray[$name]['uniqueID'] = $this->DefaultValues[$name]['uniqueID'] ?? md5(uniqid(__FILE__, true)); // FIXME: write to elements and get it from there
			if(!$this->isVariantField($cur)){
				unset($defaultsArray[$name]['variant']);
				if($this->issetElement($cur . self::NAME_DIVIDER . 'variant')){
					$this->delElement($cur . self::NAME_DIVIDER . 'variant');
				}
			}
		}

		$this->DefaultCategory = $this->Category;
		$this->DefaultTitle = $this->getElementS(($tmp = $this->getElementS('title')) . self::NAME_DIVIDER . self::ELEMENT_TYPE) . '_' . $this->getElementS($tmp . self::NAME_DIVIDER . 'name');
		$this->DefaultDesc = $this->getElementS(($tmp = $this->getElementS('desc')) . self::NAME_DIVIDER . self::ELEMENT_TYPE) . '_' . $this->getElementS($tmp . self::NAME_DIVIDER . 'name');
		$this->DefaultKeywords = $this->getElementS(($tmp = $this->getElementS('keywords')) . self::NAME_DIVIDER . self::ELEMENT_TYPE) . '_' . $this->getElementS($tmp . self::NAME_DIVIDER . 'name');

		$this->DefaultUrlfield0 = ($tmp = $this->getElementS('urlfield0')) ? $this->getElementS($tmp . self::NAME_DIVIDER . self::ELEMENT_TYPE) . '_' . $this->getElementS($tmp . self::NAME_DIVIDER . 'name') : '_';
		$this->DefaultUrlfield1 = ($tmp = $this->getElementS('urlfield1')) ? $this->getElementS($tmp . self::NAME_DIVIDER . self::ELEMENT_TYPE) . '_' . $this->getElementS($tmp . self::NAME_DIVIDER . 'name') : '_';
		$this->DefaultUrlfield2 = ($tmp = $this->getElementS('urlfield2')) ? $this->getElementS($tmp . self::NAME_DIVIDER . self::ELEMENT_TYPE) . '_' . $this->getElementS($tmp . self::NAME_DIVIDER . 'name') : '_';
		$this->DefaultUrlfield3 = ($tmp = $this->getElementS('urlfield3')) ? $this->getElementS($tmp . self::NAME_DIVIDER . self::ELEMENT_TYPE) . '_' . $this->getElementS($tmp . self::NAME_DIVIDER . 'name') : '_';
//$this->DefaultTriggerID = ($tmp = $this->getElement('triggerid')) ? $this->getElement($tmp.self::ELEMENT_TYPE).'_'.$this->getElement($tmp) : '0';

		$this->DefaultValues = $defaultsArray;

		$variant_field = 'variant_' . we_base_variants::ELEMENT_NAME;

		$exists = $this->DB_WE->isColExist($ctable, $variant_field);

		if($this->hasVariantFields()){
			if(!$exists){
				$this->DB_WE->addCol($ctable, $variant_field, 'TEXT NOT NULL');
			}
		} elseif($exists){
			$this->DB_WE->delCol($ctable, $variant_field);
		}

		//temporary fix, since NO_ZERO_DATE makes some date queries fail
		$this->DB_WE->query('SET SESSION sql_mode="STRICT_TRANS_TABLES,ERROR_FOR_DIVISION_BY_ZERO,NO_ENGINE_SUBSTITUTION,NO_ZERO_IN_DATE"');
		if(($q = implode(',', array_filter($q))) && !$this->DB_WE->query('ALTER TABLE ' . $ctable . ' ' . $q)){
			$this->errMsg = 'problems on ALTER TABLE';

			return false;
		}
		//reset sql-mode
		$this->DB_WE->query('SET SESSION sql_mode="' . DB_WE::SQLMODE . '"');
		$this->savePersistentSlotsToDB();

		$this->elements = [];
		$this->getContentData($this->Table);

		return true;
	}

	private function getSQLType(string $name): string{
		$len = $this->getElementI($name . self::NAME_DIVIDER . 'length');
		switch($this->getElementS($name . self::NAME_DIVIDER . self::ELEMENT_TYPE)){
			case self::TYPE_SELECT://FIXME set to enum!!
				return 'VARCHAR(' . (($len > 0 && ($len < 255)) ? $len : 255) . ') NOT NULL';
			case self::TYPE_INPUT:
				return 'VARCHAR(' . (($len > 0 && ($len < 4096)) ? $len : 255) . ') NOT NULL';
			case self::TYPE_DATETIME:
				return 'INT unsigned NOT NULL';
			case self::TYPE_DATE:
				return 'DATE  NOT NULL';
			case self::TYPE_TIME:
				return 'TIME NOT NULL';
			case self::TYPE_COUNTRY:
			case self::TYPE_LANGUAGE:
				return 'CHAR(2) NOT NULL';
			case self::TYPE_LINK:
			case self::TYPE_HREF:
			case self::TYPE_MULTIOBJECT:
			case self::TYPE_SHOPCATEGORY:
				return 'TEXT NOT NULL';
			case self::TYPE_TEXT:
				return 'LONGTEXT NOT NULL';
			case self::TYPE_IMG:
			case self::TYPE_AUDIO:
			case self::TYPE_VIDEO:
			case self::TYPE_BINARY:
			case self::TYPE_COLLECTION:
			case self::TYPE_OBJECT:
				return 'INT unsigned NOT NULL DEFAULT "0"';
			case self::TYPE_CHECKBOX:
				return 'BOOLEAN NOT NULL DEFAULT \'' . ($this->getElementS($name . self::NAME_DIVIDER . 'defaultB') ? '1' : '0') . '\'';
			case self::TYPE_INT:
				$value = explode('_', $this->getElementS($name . self::NAME_DIVIDER . 'typeLen'));
				return ' ' . ($value[0] ?: 'INT') . (!empty($value[1]) && $value[1] === 'U' ? ' UNSIGNED' : '') . ' DEFAULT NULL ';
			case self::TYPE_FLOAT:
				return 'DOUBLE NOT NULL DEFAULT "0.0"';
			case self::TYPE_SHOPVAT:
				return 'decimal(4,2) default "0" NOT NULL';
			default:
				return '';
		}
	}

	function ModifyPathInformation(int $parentID): void{
		$this->setParentID($parentID);
		$this->wasUpdate = true;
		$this->savePersistentSlotsToDB(['Filename', 'Path', 'ParentID']);
	}

	function setSort(): void{
		if(!$this->ID || $this->issetElement('we_sort')){
			return;
		}
		$ctable = OBJECT_X_TABLE . intval($this->ID);
		$tableInfo = $this->ID ? $this->DB_WE->metaFieldNames($ctable) : ['OF_ID', 'OF_DocumentTable'];
		$sort = [];
		$i = 0;
		'@phan-var string[] $tableInfo';
		foreach($tableInfo as $name){
			[$type, $name] = explode('_', $name, 2);
			switch($type){
				case 'OF':
				case 'variant':
					break;
				default:
					$sort[str_replace('.', '', uniqid($name))] = $i++;
			}
		}
		$this->setElementA('we_sort', $sort);
	}

	/* must be called from the editor-script. Returns a filename which has to be included from the global-Script */

	function editor(we_base_jsCmd $jsCmd = null): we_editor_base{
		if(we_base_request::string('we_cmd', '', 0) === 'save_document'){
			$this->save();
			we_editor_save::$responseText = sprintf(g_l('weClass', '[response_save_ok]'), $this->Path);
			we_editor_save::$responseTextType = we_base_util::WE_MESSAGE_NOTICE;
			return new we_editor_save($this, $jsCmd);
		}
		switch($this->EditPageNr){
			default:
				$_SESSION['weS']['EditPageNr'] = $this->EditPageNr = we_base_constants::WE_EDITPAGE_PROPERTIES;
			case we_base_constants::WE_EDITPAGE_PROPERTIES:
			case we_base_constants::WE_EDITPAGE_WORKSPACE:
				return new we_editor_properties($this, $jsCmd);
			case we_base_constants::WE_EDITPAGE_INFO:
				return new we_editor_info_object($this, $jsCmd);
			case we_base_constants::WE_EDITPAGE_CONTENT:
				return new we_editor_contentobject($this, $jsCmd);
		}
	}

	private function moveEntryAtClass(string $identifier, int $move): void{
		$sort = $this->getElementA('we_sort');
		$pos = $sort[$identifier];

		$newPos = array_search($pos + $move, $sort);
		if($newPos !== false){
			$sort[$identifier] = $sort[$newPos];
			$sort[$newPos] = $pos;
			asort($sort, SORT_NUMERIC);
			$this->setElementA('we_sort', $sort);
		}
	}

	function downEntryAtClass(string $identifier): void{
		$this->moveEntryAtClass($identifier, 1);
	}

	function upEntryAtClass(string $identifier): void{
		$this->moveEntryAtClass($identifier, -1);
	}

	function addEntryToClass(string $identifier, string $after = ''): void{
		$sort = $this->getElementA('we_sort');
		$uid = uniqid();

		$this->setElementS($uid, '');
		$this->setElementI($uid . self::NAME_DIVIDER . self::ELEMENT_LENGHT, 0);
		$this->setElementS($uid . self::NAME_DIVIDER . self::ELEMENT_TYPE, '');
		$this->setElementI($uid . self::NAME_DIVIDER . self::ELEMENT_HEIGHT, 180);
		$this->setElementS($uid . self::NAME_DIVIDER . self::ELEMENT_CLASS, '');
		$this->setElementI($uid . self::NAME_DIVIDER . self::ELEMENT_MAX, 0);
		$this->setElementS('wholename' . $identifier, $uid);
		$nf = $this->getElementA('neuefelder');
		$nf[] = $uid;
		$this->setElementA('neuefelder', $nf);

		if($after && isset($sort[$after])){
			$pos = $sort[$after];

			foreach($sort as &$identpos){
				if($identpos > $pos){
					$identpos += 1;
				}
			}
			$sort[$identifier] = $pos + 1;
		} else {
			$sort[$identifier] = count($sort);
		}
		asort($sort, SORT_NUMERIC);
		$this->setElementA('we_sort', $sort);
	}

	function removeEntryFromClass(string $identifier): void{
		$sort = $this->getElementA('we_sort');

		$uid = $this->getElementS('wholename' . $identifier);

		if(($pos = array_search($uid, ($nf = $this->getElementA('neuefelder')))) !== false){
			unset($nf[$pos]);
			$this->setElementA('neuefelder', $nf);
		} else {
			$er = $this->getElementA('felderloeschen');
			$er[] = $uid;
			$this->setElementA('felderloeschen', $er);
		}

		$this->delElement('wholename' . $identifier);
		$this->delElement($uid . self::NAME_DIVIDER . 'name');
		$this->delElement($uid . self::NAME_DIVIDER . self::ELEMENT_LENGHT);
		$this->delElement($uid . self::NAME_DIVIDER . self::ELEMENT_TYPE);
		$this->delElement($uid . self::NAME_DIVIDER . self::ELEMENT_HEIGHT);
		$this->delElement($uid . self::NAME_DIVIDER . self::ELEMENT_DEFAULT); // FIXME: need correct defaultField!
		$this->delElement($uid . self::NAME_DIVIDER . self::ELEMENT_CLASS);
		$this->delElement($uid . self::NAME_DIVIDER . self::ELEMENT_MAX);

//move elements
		$pos = $sort[$identifier];
		unset($sort[$identifier]);
		foreach($sort as &$identpos){
			if($identpos > $pos){
				$identpos -= 1;
			}
		}

		asort($sort, SORT_NUMERIC);
		$this->setElementA('we_sort', ($sort ?: []));
	}

	function upDownMultiobjectAtClass(string $name, int $i, bool $up): void{
		$fullName = $name . self::NAME_DIVIDER . self::DEFAULTS_VALUES[self::TYPE_MULTIOBJECT]['defField'];
		$defaultValue = $this->getElementA($fullName);
		$dir = ($up ? -1 : 1);
		$tempM = $defaultValue[($i + $dir)];
		$defaultValue[($i + $dir)] = $defaultValue[$i];
		$defaultValue[$i] = $tempM;
		$this->setElementA($fullName, array_merge($defaultValue));
	}

	function addMultiobjectToClass(string $name, int $pos): void{
		$fullName = $name . self::NAME_DIVIDER . self::DEFAULTS_VALUES[self::TYPE_MULTIOBJECT]['defField'];
		$defaultValue = $this->getElementA($fullName);
		array_splice($defaultValue, ($pos + 1), 0, [0]);
		$defaultValue = $defaultValue ?: [0];
		$this->setElementA($fullName, array_merge($defaultValue));
		$this->setElementI($name . self::NAME_DIVIDER . 'count', count($defaultValue));
	}

	function removeMultiobjectFromClass(string $name, int $pos): void{
		$fullName = $name . self::NAME_DIVIDER . self::DEFAULTS_VALUES[self::TYPE_MULTIOBJECT]['defField'];
		$defaultValue = $this->getElementA($fullName);
		array_splice($defaultValue, $pos, 1);
		$defaultValue = $defaultValue ?: [0];
		$this->setElementA($fullName, array_values($defaultValue));
		$this->setElementI($name . self::NAME_DIVIDER . 'count', count($defaultValue));
	}

	function removeAllMultiobjectFromClass(string $name): void{
		$fullName = $name . self::NAME_DIVIDER . self::DEFAULTS_VALUES[self::TYPE_MULTIOBJECT]['defField'];
		$defaultValue = $this->getElementA($fullName) ?: [0];
		$this->setElementA($fullName, array_values($defaultValue));
		$this->setElementI($name . self::NAME_DIVIDER . 'count', count($defaultValue));
	}

	function upDownSelectAtClass(string $name, int $i, bool $up): void{
		$selectKeys = $this->getElementA($name . self::NAME_DIVIDER . 'selectKeys', []);
		$dir = ($up ? -1 : 1);
		$tempK = $selectKeys[($i + $dir)];
		$selectKeys[($i + $dir)] = $selectKeys[$i];
		$selectKeys[$i] = $tempK;
		$this->setElementA($name . self::NAME_DIVIDER . 'selectKeys', array_merge($selectKeys));

		$selectValues = $this->getElementA($name . self::NAME_DIVIDER . 'selectValues', []);
		$tempV = $selectValues[($i + $dir)];
		$selectValues[($i + $dir)] = $selectValues[$i];
		$selectValues[$i] = $tempV;
		$this->setElementA($name . self::NAME_DIVIDER . 'selectValues', array_merge($selectValues));
	}

	function addSelectToClass(string $name, int $pos): void{
		$amount = we_base_request::int('amount_insert_select_at_class_' . $name . $pos, 1);
		$cnt = $this->getElementI($name . self::NAME_DIVIDER . 'count') + $amount;
		$this->setElementI($name . self::NAME_DIVIDER . 'count', $cnt);

		$selectKeys = $this->getElementA($name . self::NAME_DIVIDER . 'selectKeys', []);
		array_splice($selectKeys, ($pos + 1), 0, array_fill(0, $amount, ''));
		$this->setElementA($name . self::NAME_DIVIDER . 'selectKeys', array_merge($selectKeys));

		$selectValues = $this->getElementA($name . self::NAME_DIVIDER . 'selectValues', []);
		array_splice($selectValues, ($pos + 1), 0, array_fill(0, $amount, ''));
		$this->setElementA($name . self::NAME_DIVIDER . 'selectValues', array_merge($selectValues));
	}

	function removeSelectFromClass(string $name, int $nr): void{
		$selectKeys = $this->getElementA($name . self::NAME_DIVIDER . 'selectKeys', []);
		array_splice($selectKeys, $nr, 1);
		$this->setElementA($name . self::NAME_DIVIDER . 'selectKeys', array_merge($selectKeys));

		$selectValues = $this->getElementA($name . self::NAME_DIVIDER . 'selectValues', []);
		array_splice($selectValues, $nr, 1);
		$this->setElementA($name . self::NAME_DIVIDER . 'selectValues', array_merge($selectValues));

		$this->setElementI($name . self::NAME_DIVIDER . 'count', count($selectKeys));
	}

	function getEmptyDefaultFields(): string{
		$ret = '';
		foreach([
	'title', 'desc', 'keywords',
	'urlfield0', 'urlfield1', 'urlfield2', 'urlfield3'
		] as $field){
			$ret .= '<input type="radio" value="0" name="we_' . $this->Name . '_' . self::TYPE_GENERAL . '[' . $field . ']" id="empty_' . $this->Name . '_' . self::TYPE_GENERAL . '[' . $field . ']"/>';
		}
		return we_html_element::div(['style' => "display:none"], $ret);
	}

	/**
	 *
	 * @param string $type
	 * @param string $name
	 * @param string $identifier
	 * @return array{new:bool,name:string,html:string}
	 */
	private function getFieldHTMLName(string $type, string $name, string $identifier): array{

		switch($type){
			case self::TYPE_OBJECT:
				$val = $this->getElementS($name . self::NAME_DIVIDER . 'name');
				$vals = $this->DB_WE->getAllFirstq('SELECT ID,Path FROM ' . OBJECT_TABLE . ' ORDER BY Path', false);
				return [
					'new' => empty($val),
					'name' => empty($vals[$val]) ? '' : $vals[$val],
					'html' => we_html_tools::select('we_' . $this->Name . '_' . self::TYPE_GENERAL . '[' . $name . self::NAME_DIVIDER . 'name]', $vals, 1, $val, false, ['onchange' => "if(this.form.elements['we_" . $this->Name . "_" . self::TYPE_OBJECT . "[" . $name . self::NAME_DIVIDER . "defaultI]']){this.form.elements['we_" . $this->Name . "_" . self::TYPE_OBJECT . "[" . $name . self::NAME_DIVIDER . "defaultI]'].value='' };_EditorFrame.setEditorIsHot(true);we_cmd('we_object_class_changeEntry','" . we_base_WE::$transaction . "','" . $identifier . "')"], true)
				];
			case self::TYPE_SHOPVAT:
			case self::TYPE_SHOPCATEGORY:
				$fieldName = $type === self::TYPE_SHOPCATEGORY ? we_shop_shop::CATEGORY_FIELD_NAME : we_shop_shop::VAT_FIELD_NAME;
				return [
					'new' => false,
					'name' => $fieldName,
					'html' => we_html_element::hidden('we_' . $this->Name . '_' . self::TYPE_GENERAL . '[' . $name . self::NAME_DIVIDER . 'name]', $fieldName) .
					we_html_tools::textInput('tmp' . $fieldName, $fieldName, 52, ['readonly' => 'readonly', 'disabled' => 'disabled'], 'text')
				];
			default:
				$val = $this->getElementS($name . self::NAME_DIVIDER . 'name');
				$fieldName = $val ?: g_l('modules_object', '[field][new]');
				return [
					'new' => empty($val),
					'name' => $fieldName,
					'html' => we_html_tools::textInput('we_' . $this->Name . '_' . self::TYPE_GENERAL . '[' . $name . self::NAME_DIVIDER . 'name]', $fieldName, 52, ['data-oldValue' => $fieldName,
						'onblur' => "we_checkObjFieldname(this);",
						'onchange' => "_EditorFrame.setEditorIsHot(true);"], 'text')
				];
		}
	}

	private function getFieldHTMLLength(string $type, string $name): string{
		switch($type){
			case self::TYPE_AUDIO:
			case self::TYPE_BINARY:
			case self::TYPE_CHECKBOX:
			case self::TYPE_COLLECTION:
			case self::TYPE_COUNTRY:
			case self::TYPE_DATETIME:
			case self::TYPE_DATE:
			case self::TYPE_TIME:
			case self::TYPE_FLOAT:
			case self::TYPE_HREF:
			case self::TYPE_IMG:
			case self::TYPE_LANGUAGE:
			case self::TYPE_LINK:
			case self::TYPE_SELECT:
			case self::TYPE_MULTIOBJECT:
			case self::TYPE_OBJECT:
			case self::TYPE_SHOPCATEGORY:
			case self::TYPE_SHOPVAT:
			case self::TYPE_TEXT:
			case self::TYPE_VIDEO:
				return '';
			case self::TYPE_INT:
				$values = [
					g_l('modules_object', '[int][signed]') => we_html_tools::OPTGROUP,
					'TINYINT' => '[-128 .. 127] (tiny)',
					'SMALLINT' => '[-32.768 .. 32.767] (small)',
					'MEDIUMINT' => '[-8.388.608 .. 8.388.607] (medium)',
					'INT' => '[-2.147.483.648 .. 2.147.483.647] (int)',
					'BIGINT' => '[-9.223.372.036.854.775.808 .. 9.223.372.036.854.775.807] (big)',
					g_l('modules_object', '[int][unsigned]') => we_html_tools::OPTGROUP,
					'TINYINT_U' => '[0 .. 255] (tiny)',
					'SMALLINT_U' => '[0 .. 65.535] (small)',
					'MEDIUMINT_U' => '[0 .. 16.777.215] (medium)',
					'INT_U' => '[0 .. 4.294.967.295] (int)',
					'BIGINT_U' => '[0 .. 18.446.744.073.709.551.615] (big)',
				];
				$sel = $this->getElementS($name . self::NAME_DIVIDER . 'typeLen');

				return '<tr><td class="weMultiIconBoxHeadlineThin">' . g_l('modules_object', '[range][int]') . '</br>' .
						we_html_tools::select('we_' . $this->Name . '_' . self::TYPE_GENERAL . '[' . $name . self::NAME_DIVIDER . 'typeLen]', $values, 1, $sel ?: 'INT', false, ['onchange' => "_EditorFrame.setEditorIsHot(true);",
						]) .
						'</td></tr>';
			default:
				$maxLengthVal = 1023;
				$len = $this->getElementI($name . self::NAME_DIVIDER . self::ELEMENT_LENGHT);
				return '<tr><td class="weMultiIconBoxHeadlineThin">' . g_l('modules_object', '[length]') . '<br/>' .
						we_html_tools::textInput('we_' . $this->Name . '_' . self::TYPE_GENERAL . '[' . $name . self::NAME_DIVIDER . self::ELEMENT_LENGHT . ']', (string) ($len ?: $maxLengthVal), 4, [
							'onchange' => "_EditorFrame.setEditorIsHot(true);",
							'weType' => 'weObject_' . $type . '_length', 'max' => '1023', 'min' => '1', 'step' => '1'], 'number') .
						'</td></tr>';
		}
	}

	private function getDateFieldHTML(string $name, string $type): string{
		$dd = $this->getElementB($name . self::NAME_DIVIDER . 'defaultB');
		return we_html_forms::checkboxWithHidden($dd, 'we_' . $this->Name . '_' . $type . '[' . $name . self::NAME_DIVIDER . 'defaultB]', g_l('modules_object', '[creationDate]'), false, '', "_EditorFrame.setEditorIsHot(true);");
	}

	private function getCommonBinaryFieldHTML(we_base_jsCmd $jsCmd, string $name, string $identifier, string $type): string{
		return $this->formDirChooser(0, FILE_TABLE, 'ParentPath', $type . '[' . $name . self::NAME_DIVIDER . 'rootdir]', '', true, false, $this->getElementI($name . self::NAME_DIVIDER . 'rootdir'), $identifier) .
				'</td></tr>' .
				'<tr><td class="weMultiIconBoxHeadlineThin">' . g_l('modules_object', '[defaultdir]') . '<br/>' .
				$this->formDirChooser(0, FILE_TABLE, 'StartPath', $type . '[' . $name . self::NAME_DIVIDER . 'defaultdir]', '', true, false, $this->getElementI($name . self::NAME_DIVIDER . 'defaultdir'), $identifier, $jsCmd) .
				'</td></tr>' .
				'<tr><td class="weMultiIconBoxHeadlineThin">' . g_l('modules_object', '[default]') . '<br/>';
	}

	private function getSelectFieldHTML(string $name, string $identifier): string{
		$content = '<table><tr><td>Default</td><td>Key</td><td>Value</td><td></td></tr>';
		if(!$this->issetElement($name . 'count')){
			$this->setElementI($name . 'count', 0);
		}

		$addArray = [1 => 1, 2 => 2, 3 => 3, 4 => 4, 5 => 5, 6 => 6, 7 => 7, 8 => 8, 9 => 9, 10 => 10];

		$defaultField = self::DEFAULTS_VALUES[self::TYPE_SELECT]['defField'];
		$default = $this->getElementI($name . self::NAME_DIVIDER . $defaultField, 0);
		$selectKeys = $this->getElementA($name . self::NAME_DIVIDER . 'selectKeys', ['']);
		$selectValues = $this->getElementA($name . self::NAME_DIVIDER . 'selectValues', ['']);
		$count = count($selectKeys);

		for($f = 0; $f < count($selectKeys); $f++){
			$content .= '<tr><td>' . we_html_forms::radiobuttonV((string) $f, (string) $default, 'we_' . $this->Name . '_' . self::TYPE_SELECT . '[' . $name . self::NAME_DIVIDER . 'defaultI]', '') .
					'</td><td>' . we_html_tools::textInput('we_' . $this->Name . '_' . self::TYPE_SELECT . '[' . $name . self::NAME_DIVIDER . 'selectKeys][' . $f . ']', (string) $selectKeys[$f], 255, [
						'onchange' => "_EditorFrame.setEditorIsHot(true);", 'class' => 'wetextinput medium', 'required' => 'required'], 'text', '105px') .
					'</td><td>' . we_html_tools::textInput('we_' . $this->Name . '_' . self::TYPE_SELECT . '[' . $name . self::NAME_DIVIDER . 'selectValues][' . $f . ']', (string) $selectValues[$f], 255, [
						'onchange' => "_EditorFrame.setEditorIsHot(true);", 'class' => 'wetextinput medium', 'required' => 'required'], 'text', '105px');

			$content .= '</td><td>' .
					we_html_tools::select('amount_insert_select_at_class_' . $name . $f, $addArray, 1, [], false, ['class' => 'weSelect tiny']) .
					we_html_button::create_button(we_html_button::LIST_ADD, "javascript:we_cmd('we_object_class_selectInsert','" . we_base_WE::$transaction . "','" . ($identifier) . "','" . $name . "','" . ($f) . "')") .
					(($f > 0) ?
					we_html_button::create_button(we_html_button::DIRUP, "javascript:we_cmd('we_object_class_selectUp','" . we_base_WE::$transaction . "','" . ($identifier) . "','" . $name . "','" . ($f) . "')") :
					we_html_button::create_button(we_html_button::DIRUP, '#', '', '', true)
					) .
					($f < ($count - 1) ?
					we_html_button::create_button(we_html_button::DIRDOWN, "javascript:we_cmd('we_object_class_selectDown','" . we_base_WE::$transaction . "','" . ($identifier) . "','" . $name . "','" . ($f) . "')") :
					we_html_button::create_button(we_html_button::DIRDOWN, '#', '', '', true)
					) .
					we_html_button::create_button(we_html_button::TRASH, "javascript:we_cmd('we_object_class_selectDelete','" . we_base_WE::$transaction . "','" . ($identifier) . "','" . $name . "','" . ($f) . "')") .
					'</td></tr>';
		}
		return $content . '</table>';
	}

	private function getHrefFieldHTML(string $name, string $identifier): string{
		$typeVal = $this->getElementS($name . self::NAME_DIVIDER . 'hreftype'); //href_href__hreftype
		// FIXME: use we_html_select
		$cmd = "'we_object_class_reloadEntry','" . we_base_WE::$transaction . "','" . $identifier . "'";
		$typeSelect = '<select style="margin-right:4em;" id="we_' . $this->Name . '_' . self::TYPE_HREF . '[' . $name . self::NAME_DIVIDER . 'hreftype]" name="we_' . $this->Name . '_' . self::TYPE_HREF . '[' . $name . self::NAME_DIVIDER . 'hreftype]" onchange="we_cmd(' . $cmd . ')">
			<option' . (($typeVal === we_base_link::TYPE_ALL || !$typeVal) ? ' selected' : '') . ' value="' . we_base_link::TYPE_ALL . '">all</option>
			<option' . (($typeVal === we_base_link::TYPE_INT) ? ' selected' : '') . ' value="' . we_base_link::TYPE_INT . '">int</option>
			<option' . (($typeVal === we_base_link::TYPE_EXT) ? ' selected' : '') . ' value="' . we_base_link::TYPE_EXT . '">ext</option>
			</select>';
		$fileVal = $this->getElementB($name . self::NAME_DIVIDER . 'hreffile', false);
		$fileSelect = '<select style="margin-right:4em;" id="we_' . $this->Name . '_' . self::TYPE_HREF . '[' . $name . self::NAME_DIVIDER . 'hreffile]" name="we_' . $this->Name . '_' . self::TYPE_HREF . '[' . $name . self::NAME_DIVIDER . 'hreffile]">
			<option' . (($fileVal) ? ' selected="selected"' : '') . ' value="1">true</option>
			<option' . ((!$fileVal) ? ' selected="selected"' : '') . ' value="0">false</option>
			</select>';
		$dirVal = $this->getElementB($name . self::NAME_DIVIDER . 'hrefdirectory', false); // options anzeige umgedreht wegen 4363
		$dirSelect = '<select id="we_' . $this->Name . '_' . self::TYPE_HREF . '[' . $name . self::NAME_DIVIDER . 'hrefdirectory]" name="we_' . $this->Name . '_' . self::TYPE_HREF . '[' . $name . self::NAME_DIVIDER . 'hrefdirectory]">
			<option' . (($dirVal) ? ' selected="selected"' : '') . ' value="1">true</option>
			<option' . ((!$dirVal) ? ' selected="selected"' : '') . ' value="0">false</option>
			</select>';
		return $typeSelect . 'file ' .
				$fileSelect . 'directory ' .
				$dirSelect .
				'</td></tr>
					<tr style="vertical-align:top"><td class="weMultiIconBoxHeadlineThin">' . g_l('modules_object', '[default]') . '<br/>' .
				$this->htmlHref($name);
	}

	private function getFullMultiobjectFieldHTML(string $name, string $identifier): string{
		$vals = $this->DB_WE->getAllFirstq('SELECT ID,Path FROM ' . OBJECT_TABLE . ' ORDER BY Path', false);
//FIXME: JS!!
		$content = we_html_tools::select('we_' . $this->Name . '_' . self::TYPE_MULTIOBJECT . '[' . $name . self::NAME_DIVIDER . 'tableid]', $vals, 1, [(string) ($this->getElementI($name . self::NAME_DIVIDER . 'tableid'))], false, [
					'onchange' => "if(this.form.elements['we_" . $this->Name . '_' . self::TYPE_MULTIOBJECT . '[' . $name . self::NAME_DIVIDER . 'defaultA]' . "']){this.form.elements['we_" . $this->Name . '_input[' . $name . 'default]' . "'].value='';};_EditorFrame.setEditorIsHot(true);we_cmd('we_object_class_changeMultiobject','" . we_base_WE::$transaction . "','" . $identifier . "','" . $name . "')"]) .
				'</td></tr>
<tr style="vertical-align:top"><td class="weMultiIconBoxHeadlineThin">' . g_l('modules_object', '[max_objects]') . '<br/>' . we_html_tools::textInput('we_' . $this->Name . '_' . self::TYPE_MULTIOBJECT . '[' . $name . self::NAME_DIVIDER . 'max]', (string) $this->getElementI($name . self::NAME_DIVIDER . 'max'), 3, [
					'onchange' => "we_cmd('we_object_class_reloadEntry','" . we_base_WE::$transaction . "','" . $identifier . "');"], 'text', '50px') . ' (' . g_l('modules_object', '[no_maximum]') . ')</td>
</tr>
<tr style="vertical-align:top"><td class="weMultiIconBoxHeadlineThin">' . g_l('modules_object', '[default]') . '<br/>';

		if(!$this->issetElement($name . self::NAME_DIVIDER . 'count')){
			$this->setElementI($name . self::NAME_DIVIDER . 'count', 0);
		}

		$defaultField = self::DEFAULTS_VALUES[self::TYPE_MULTIOBJECT]['defField'];
		$defaultValue = self::DEFAULTS_VALUES[self::TYPE_MULTIOBJECT]['defaults'][$defaultField][0];
		$default = $this->getElementA($name . self::NAME_DIVIDER . $defaultField, $defaultValue) ?: [0];
		$weSuggest = we_gui_suggest::getInstance();
		$weSuggest->setContentType([we_base_ContentTypes::FOLDER_OBJECTFILE, we_base_ContentTypes::OBJECT_FILE])
				->setMaxResults(10)
				->setSelector(we_gui_suggest::DocSelector)
				->setTable(OBJECT_FILES_TABLE);
		$innerContent = '';
		for($f = 0; $f < count($default); $f++){
			$innerContent .= $this->getMultiObjectFieldHTML($name, $identifier, $f, $weSuggest, !($f === 0 && count($default) === 1));
		}

		return $content . we_html_element::div(['class' => 'objectSelect'], $innerContent);
	}

	function getFieldHTML(we_base_jsCmd $jsCmd, string $name, string $identifier, string $we_transaction): string{
		$type = $this->getElementS($name . self::NAME_DIVIDER . self::ELEMENT_TYPE, self::TYPE_INPUT) ?: 'input';
		static $pos = -1;
		$pos++;
		$defaultValueB = false;
		$defaultValueI = 0;
		$defaultValueF = 0.0;
		$defaultValueS = '';
		//$defaultValueA = [];
		switch(($defaultField = self::DEFAULTS_VALUES[$type]['defField'] ?? self::DEFAULT_S)){
			case self::DEFAULT_B:
				$defaultValueB = $this->getElementB($name . self::NAME_DIVIDER . $defaultField, (bool) self::DEFAULTS_VALUES[$type]['defaults'][$defaultField][0]);
				break;
			case self::DEFAULT_I:
				$defaultValueI = $this->getElementI($name . self::NAME_DIVIDER . $defaultField, (int) self::DEFAULTS_VALUES[$type]['defaults'][$defaultField][0]);
				break;
			case self::DEFAULT_F:
				$defaultValueF = $this->getElementF($name . self::NAME_DIVIDER . $defaultField, (float) self::DEFAULTS_VALUES[$type]['defaults'][$defaultField][0]);
				break;
			default://needed since $defaultValue must be declared
			case self::DEFAULT_HTML:
			case self::DEFAULT_S:
				$defaultValueS = $this->getElementS($name . self::NAME_DIVIDER . $defaultField, (string) self::DEFAULTS_VALUES[$type]['defaults'][$defaultField][0]);
				break;
			case self::DEFAULT_AI:
			case self::DEFAULT_AS:
			case self::DEFAULT_HREF:
			case self::DEFAULT_LINK:
				//$defaultValueA = $this->getElementA($name . self::NAME_DIVIDER . $defaultField, (array) self::DEFAULTS_VALUES[$type]['defaults'][$defaultField][0]);
				break;
		}

		$val = [
			g_l('modules_object', '[fieldgroups][input]') => we_html_tools::OPTGROUP,
			self::TYPE_INPUT => g_l('modules_object', '[field][input]'),
			self::TYPE_TEXT => g_l('modules_object', '[field][textarea]'),
			self::TYPE_INT => g_l('modules_object', '[field][int]'),
			self::TYPE_FLOAT => g_l('modules_object', '[field][float]'),
			g_l('modules_object', '[fieldgroups][datetime]') => we_html_tools::OPTGROUP,
			self::TYPE_DATE => g_l('modules_object', '[field][date]'),
			self::TYPE_DATETIME => g_l('modules_object', '[field][datetime]'),
			self::TYPE_TIME => g_l('modules_object', '[field][time]'),
			g_l('modules_object', '[fieldgroups][select]') => we_html_tools::OPTGROUP,
			self::TYPE_CHECKBOX => g_l('modules_object', '[field][checkbox]'),
			self::TYPE_SELECT => g_l('modules_object', '[field][meta]'),
			g_l('modules_object', '[fieldgroups][media]') => we_html_tools::OPTGROUP,
			self::TYPE_IMG => g_l('modules_object', '[field][img]'),
			self::TYPE_AUDIO => g_l('modules_object', '[field][audio]'),
			self::TYPE_VIDEO => g_l('modules_object', '[field][video]'),
			self::TYPE_COLLECTION => g_l('modules_object', '[field][collection]'),
			g_l('modules_object', '[fieldgroups][language]') => we_html_tools::OPTGROUP,
			self::TYPE_COUNTRY => g_l('modules_object', '[field][country]'),
			self::TYPE_LANGUAGE => g_l('modules_object', '[field][language]'),
			g_l('modules_object', '[fieldgroups][link]') => we_html_tools::OPTGROUP,
			self::TYPE_LINK => g_l('modules_object', '[field][link]'),
			self::TYPE_HREF => g_l('modules_object', '[field][href]'),
			self::TYPE_BINARY => g_l('modules_object', '[field][binary]'),
			self::TYPE_OBJECT => g_l('modules_object', '[field][objectFile]'),
			self::TYPE_MULTIOBJECT => g_l('modules_object', '[field][multiObjectFile]'),
		];
		if(we_modules_info::isActive(we_modules_info::SHOP)){
			$val[g_l('modules_object', '[fieldgroups][shop]')] = we_html_tools::OPTGROUP;
			$val[self::TYPE_SHOPVAT] = g_l('modules_object', '[field][shopVat]');
			$val[self::TYPE_SHOPCATEGORY] = g_l('modules_object', '[field][shopCat]');
		}
		['new' => $new, 'name' => $fieldName, 'html' => $nameField] = $this->getFieldHTMLName($type, $name, $identifier);

		$parts = new we_html_multiIconBoxContent();
		$preContent = '<tr>
			<td class="weMultiIconBoxHeadline leftSpace-med" rowspan="100">' . we_html_element::div(['class' => 'weObjectHeadline'], ($fieldName ?: g_l('weClass', '[name]')) .
						we_html_element::div(['class' => 'addTrashUpDown'], we_html_element::div([], we_html_button::create_button(we_html_button::ADD_FIELD, "javascript:we_cmd('we_object_class_insertEntry','" . $we_transaction . "','" . $identifier . "');") .
										we_html_button::create_button(we_html_button::TRASH, "javascript:we_cmd('we_object_class_deleteEntry','" . $we_transaction . "','" . $identifier . "');")) .
								we_html_element::div(['class' => 'UpDown'], we_html_button::create_button(we_html_button::DIRUP, "javascript:we_cmd('we_object_class_entryUp','" . $we_transaction . "','" . $identifier . "');", '', '', false, false, '_' . $identifier) .
										we_html_button::create_button(we_html_button::DIRDOWN, "javascript:we_cmd('we_object_class_entryDown','" . $we_transaction . "','" . $identifier . "');", '', '', false, false, "_" . $identifier))
						)
				) .
				'</td>
			<td class="weMultiIconBoxHeadlineThin">' . g_l('weClass', '[name]') . '<br/>' . $nameField . '</td></tr>' .
				'<tr><td class="weMultiIconBoxHeadlineThin">' . g_l('modules_object', '[type]') . '<br/>' .
				we_html_tools::select('we_' . $this->Name . '_' . self::TYPE_GENERAL . '[' . $name . self::NAME_DIVIDER . self::ELEMENT_TYPE . ']', $val, 1, $type, false, ['onchange' => "if(this.form.elements['we_" . $this->Name . '_' . $type . '[' . $name . self::NAME_DIVIDER . $defaultField . ']' . "']){this.form.elements['we_" . $this->Name . '_' . $type . '[' . $name . self::NAME_DIVIDER . $defaultField . ']' . "'].value='' };_EditorFrame.setEditorIsHot(true);we_cmd('we_object_class_changeEntry','" . we_base_WE::$transaction . "','" . $identifier . "');"]) .
				'</td></tr>';

		$content = $this->getFieldHTMLLength($type, $name) .
				'<tr><td class="weMultiIconBoxHeadlineThin">' .
				$this->formTextArea($type, $name . self::NAME_DIVIDER . 'editdescription', g_l('global', '[description]'), 1, 40, ['onchange' => "_EditorFrame.setEditorIsHot(true);",
					'class' => 'wetextarea']) .
				'</td></tr>';
		switch($type){
			case self::TYPE_MULTIOBJECT:
				$content .= '<tr><td class="weMultiIconBoxHeadlineThin multiobject">' . g_l('contentTypes', '[object]') . '<br/>' .
						$this->getFullMultiobjectFieldHTML($name, $identifier) .
						'</td></tr>';
				break;
			case self::TYPE_HREF:
				$content .= '<tr><td>type ' . $this->getHrefFieldHTML($name, $identifier) . '</td></tr>';
				break;
			case self::TYPE_CHECKBOX:
				$content .= '<tr><td class="weMultiIconBoxHeadlineThin">' . g_l('modules_object', '[default]') . '<br/>' .
						we_html_forms::checkbox('1', $defaultValueB, 'we_' . $this->Name . '_' . self::TYPE_CHECKBOX . '[' . $name . "default1]", g_l('modules_object', '[checked]'), true, '', "if(this.checked){document.we_form.elements['we_" . $this->Name . "_" . self::TYPE_CHECKBOX . "[" . $name . self::NAME_DIVIDER . $defaultField . "]" . "'].value=1;}else{ document.we_form.elements['we_" . $this->Name . "_" . self::TYPE_CHECKBOX . "[" . $name . self::NAME_DIVIDER . $defaultField . "]" . "'].value=0;}") .
						'<input type=hidden name="we_' . $this->Name . "_" . self::TYPE_CHECKBOX . "[" . $name . self::NAME_DIVIDER . $defaultField . "]" . '" value="' . ((int) $defaultValueB) . '" />' .
						'</td></tr>';
				break;
			case self::TYPE_IMG:
				$content .= '<tr><td class="weMultiIconBoxHeadlineThin">' . g_l('modules_object', '[rootdir]') . '<br/>' .
						$this->getCommonBinaryFieldHTML($jsCmd, $name, $identifier, self::TYPE_IMG) .
						$this->getImageHTML($name, $defaultValueI, $identifier) .
						'</td></tr>';
				break;
			case self::TYPE_AUDIO:
				$content .= '<tr><td class="weMultiIconBoxHeadlineThin">' . g_l('modules_object', '[rootdir]') . '<br/>' .
						$this->getCommonBinaryFieldHTML($jsCmd, $name, $identifier, self::TYPE_AUDIO) .
						$this->getAudioHTML($name, $defaultValueI, $identifier) .
						'</td></tr>';
				break;
			case self::TYPE_VIDEO:
				$content .= '<tr><td class="weMultiIconBoxHeadlineThin">' . g_l('modules_object', '[rootdir]') . '<br/>' .
						$this->getCommonBinaryFieldHTML($jsCmd, $name, $identifier, self::TYPE_VIDEO) .
						$this->getVideoHTML($name, $defaultValueI, $identifier) .
						'</td></tr>';
				break;
			case self::TYPE_BINARY:
				$content .= '<tr><td class="weMultiIconBoxHeadlineThin">' . g_l('modules_object', '[rootdir]') . '<br/>' .
						$this->getCommonBinaryFieldHTML($jsCmd, $name, $identifier, self::TYPE_BINARY) .
						$this->getBinaryHTML($name, $defaultValueI, $identifier) .
						'</td></tr>';
				break;
			case self::TYPE_DATETIME:
			case self::TYPE_DATE:
			case self::TYPE_TIME:
				// as long as convertDateInRequest() is needed and it kills all elems in TYPE_DATE-request but the date itself, we need TYPE_GENERAL
				$content .= '<tr><td>Default<br/>' .
						$this->getDateFieldHTML($name, /* $type */ self::TYPE_GENERAL) .
						'</td></tr>';
				break;
			case self::TYPE_TEXT:
				$content .= '<tr><td class="weMultiIconBoxHeadlineThin">' . g_l('modules_object', '[default]') . '<br/>' .
						$this->wysiwyg($name, $identifier) .
						'</td></tr>';
				break;
			case self::TYPE_OBJECT:
				$content .= '<tr><td class="weMultiIconBoxHeadlineThin">' . g_l('modules_object', '[default]') . '<br/>' .
						$this->getObjectFieldHTML($name) .
						'</td></tr>';
				break;
			case self::TYPE_COLLECTION:
				$content .= '<tr><td class="weMultiIconBoxHeadlineThin">' . g_l('modules_object', '[default]') . '<br/>' .
						$this->getCollectionFieldHTML($name, $defaultValueI) .
						'</td></tr>';
				break;
			case self::TYPE_SELECT:
				$content .= '<tr><td class="weMultiIconBoxHeadlineThin">' . g_l('modules_object', '[default]') . '<br/>' .
						$this->getSelectFieldHTML($name, $identifier) .
						'</td></tr>';
				break;
			case self::TYPE_COUNTRY:
				$content .= '<tr><td class="weMultiIconBoxHeadlineThin">' . g_l('modules_object', '[default]') . '<br/>' .
						we_html_tools::textInput('we_' . $this->Name . '_' . self::TYPE_COUNTRY . '[' . $name . self::NAME_DIVIDER . $defaultField . ']', $defaultValueS, 10, ['onchange' => "_EditorFrame.setEditorIsHot(true);",
							'weType' => $type], 'text') .
						'</td></tr>';
				break;
			case self::TYPE_LANGUAGE:
				$content .= '<tr><td class="weMultiIconBoxHeadlineThin">' . g_l('modules_object', '[default]') . '<br/>' .
						we_html_tools::textInput('we_' . $this->Name . '_' . self::TYPE_COUNTRY . '[' . $name . self::NAME_DIVIDER . $defaultField . ']', $defaultValueS, 15, ['onchange' => "_EditorFrame.setEditorIsHot(true);",
							'weType' => $type], 'text') .
						'</td></tr>';
				break;
			case self::TYPE_LINK:
				$content .= '<tr><td class="weMultiIconBoxHeadlineThin">' . g_l('modules_object', '[default]') . '<br/>' .
						$this->getLinkFieldHTML($name, $identifier) .
						'</td></tr>';
				break;
			case self::TYPE_SHOPVAT:
				$values = [];
				if(we_modules_info::isActive(we_modules_info::SHOP)){
					$allVats = we_shop_vats::getAllShopVATs();
					foreach($allVats as $id => $shopVat){
						$values[$id] = $shopVat->vat . ' - ' . $shopVat->getNaturalizedText() . ' (' . $shopVat->territory . ')';
					}

					$content .= '<tr><td class="weMultiIconBoxHeadlineThin">' . g_l('modules_object', '[default]') . '<br/>' .
							we_html_tools::select('we_' . $this->Name . '_' . self::TYPE_SHOPVAT . '[' . $name . self::NAME_DIVIDER . $defaultField . ']', $values, 1, [$defaultValueI]) .
							'</td></tr>';
				}
				break;
			case self::TYPE_SHOPCATEGORY:
				if(we_modules_info::isActive(we_modules_info::SHOP)){
					$selectField = we_html_tools::select('we_' . $this->Name . '_' . self::TYPE_SHOPCATEGORY . '[' . $name . self::NAME_DIVIDER . 'shopcatField]', ['0' => '', 'ID' => 'ID',
								'Category' => 'Category', 'Path' => 'Path',
								'Title' => 'Title', 'Description' => 'Description', 'destPrinciple' => 'destPrinciple'], 1, [$defaultValueI]);
					$selectShopPath = we_html_tools::select('we_' . $this->Name . '_' . self::TYPE_SHOPCATEGORY . '[' . $name . self::NAME_DIVIDER . 'shopcatShowPath]', ['true' => 'true',
								'false' => 'false'], 1, $this->getElementS($name . self::NAME_DIVIDER . 'shopcatShowPath'));
					$textRootdir = we_html_tools::textInput('we_' . $this->Name . '_' . self::TYPE_SHOPCATEGORY . '[' . $name . self::NAME_DIVIDER . 'shopcatRootdir]', $this->getElementS($name . self::NAME_DIVIDER . 'shopcatRootdir'));
					$values = ['0' => ' '];
					foreach(we_shop_category::getShopCatsFromDir() as $cat){
						$values[$cat->ID] = $cat->Path;
					}
					$selectCategories = we_html_tools::select('we_' . $this->Name . '_' . self::TYPE_SHOPCATEGORY . '[' . $name . self::NAME_DIVIDER . $defaultField . ']', $values, 1, [
								$defaultValueI], false, [
									], true);
					$selectLimitChoice = we_html_forms::checkboxWithHidden($this->getElementB($name . self::NAME_DIVIDER . 'shopcatLimitChoice'), 'we_' . $this->Name . '_' . self::TYPE_SHOPCATEGORY . '[' . $name . self::NAME_DIVIDER . 'shopcatLimitChoice]', 'use default only', false, '', '_EditorFrame.setEditorIsHot(true);');

					$content .= '<tr><td>' .
							'field ' . $selectField . ' showpath ' . $selectShopPath . '</td>
						</tr>
						<tr><td>' .
							'rootdir ' . $textRootdir . '</td>
						</tr>
						<tr><td class="weMultiIconBoxHeadlineThin">' . g_l('modules_object', '[default]') . '<br/>' .
							$selectCategories . '<br/>' . $selectLimitChoice . '</td>
						</tr>';
				}
				break;
			case self::TYPE_INPUT: // default for input, int and float
				$content .= '<tr><td class="weMultiIconBoxHeadlineThin">' . g_l('modules_object', '[default]') . '<br/>' .
						we_html_tools::textInput('we_' . $this->Name . '_' . self::TYPE_INPUT . '[' . $name . self::NAME_DIVIDER . $defaultField . ']', $defaultValueS, 255, [
							'onchange' => "_EditorFrame.setEditorIsHot(true);", 'weType' => $type], 'text') .
						'</td></tr>';
				break;
			case self::TYPE_INT: // default for input, int and float
				$content .= '<tr><td class="weMultiIconBoxHeadlineThin">' . g_l('modules_object', '[default]') . '<br/>' .
						we_html_tools::textInput('we_' . $this->Name . '_' . self::TYPE_INT . '[' . $name . self::NAME_DIVIDER . $defaultField . ']', (string) $defaultValueI, 9, [
							'onchange' => "_EditorFrame.setEditorIsHot(true);", 'weType' => $type], 'text') .
						'</td></tr>';
				break;
			case self::TYPE_FLOAT: // default for input, int and float
				$content .= '<tr><td class="weMultiIconBoxHeadlineThin">' . g_l('modules_object', '[default]') . '<br/>' .
						we_html_tools::textInput('we_' . $this->Name . '_' . self::TYPE_FLOAT . '[' . $name . self::NAME_DIVIDER . $defaultField . ']', (string) $defaultValueF, 19, [
							'onchange' => "_EditorFrame.setEditorIsHot(true);", 'weType' => $type], 'text') .
						'</td></tr>';
				break;
		}

		switch($type){
			case self::TYPE_DATE:
			case self::TYPE_DATETIME:
			case self::TYPE_TIME:
				$content .= '<tr><td class="weMultiIconBoxHeadlineThin"><br/>';
				break;
			case self::TYPE_TEXT:
			case self::TYPE_INPUT:
			case self::TYPE_SELECT:
			case self::TYPE_LINK:
			case self::TYPE_HREF:
				$content .= '<tr><td class="weMultiIconBoxHeadlineThin">' . we_html_element::br() . '<div style="float: left;width:50%;">';
				foreach(['title' => g_l('global', '[title]'), 'desc' => g_l('global', '[description]'), 'keywords' => g_l('weClass', '[Keywords]')] as $fname => $gl){
					$content .= we_html_forms::radiobuttonV($name, $this->getElementS($fname), 'we_' . $this->Name . '_' . self::TYPE_GENERAL . '[' . $fname . ']', $gl, true, '', "if(this.waschecked){document.getElementById('empty_" . $this->Name . '_' . self::TYPE_GENERAL . '[' . $fname . "]').checked=true;this.waschecked=false;}_EditorFrame.setEditorIsHot(true);", false, '', 0, 0, "if(this.checked){this.waschecked=true}");
				}
				$content .= '</div>';
				break;
			default:
		}

		switch($type){
			case self::TYPE_TEXT:
			case self::TYPE_INPUT:
			case self::TYPE_DATE:
			case self::TYPE_DATETIME:
			case self::TYPE_TIME:
				$content .= '<div>';
				$fields = (in_array($type, [self::TYPE_DATE, self::TYPE_DATETIME, self::TYPE_TIME]) ? ['urlfield0'] : array_keys(self::URL_FIELDS));
				foreach($fields as $key){
					$content .= we_html_forms::radiobuttonV($name, $this->getElementS($key), 'we_' . $this->Name . '_' . self::TYPE_GENERAL . '[' . $key . ']', g_l('weClass', '[' . $key . ']'), true, '', "if(this.waschecked){document.getElementById('empty_" . $this->Name . '_' . self::TYPE_GENERAL . '[' . $key . "]').checked=true;this.waschecked=false;}_EditorFrame.setEditorIsHot(true);", false, '', 0, 0, "if(this.checked){this.waschecked=true}");
				}
				$content .= '</div>';
			//nobreak;
			case self::TYPE_SELECT:
			case self::TYPE_LINK:
			case self::TYPE_HREF:
				$content .= '</td></tr>';
				break;
			default:
		}

		if($type != self::TYPE_CHECKBOX){
//Pflichtfeld
			$content .= '<tr style="vertical-align:top"><td><div style="float: left;width:50%;">' .
					we_html_forms::checkbox('1', $this->getElementB($name . self::NAME_DIVIDER . 'required'), 'we_' . $this->Name . '_' . $type . '[' . $name . self::NAME_DIVIDER . 'required1]', g_l('global', '[required_field]'), true, '', "if(this.checked){document.we_form.elements['we_" . $this->Name . '_' . $type . '[' . $name . self::NAME_DIVIDER . "required]'].value=1;}else{ document.we_form.elements['we_" . $this->Name . '_' . $type . '[' . $name . self::NAME_DIVIDER . "required]'].value=0;}") .
					($this->isVariantField($name) ?
					'</div><div style="float: left;width:50%;">' .
					we_html_forms::checkboxWithHidden($this->getElementB($name . self::NAME_DIVIDER . 'variant'), 'we_' . $this->Name . '_' . $type . '[' . $name . self::NAME_DIVIDER . 'variant]', g_l('global', '[variant_field]'), false, '', '_EditorFrame.setEditorIsHot(true);') :
					'') .
					'<input type=hidden name="we_' . $this->Name . '_' . $type . '[' . $name . self::NAME_DIVIDER . 'required]" value="' . $this->getElementI($name . self::NAME_DIVIDER . 'required') . '" />' .
					'</div></td></tr>';
// description for editmode.
		} else {
			$content .= (we_modules_info::isActive(we_modules_info::SHOP) ?
//Pflichtfeld
					'<tr style="vertical-align:top"><td>' .
					($this->isVariantField($name) ?
					we_html_forms::checkboxWithHidden($this->getElementB($name . self::NAME_DIVIDER . 'variant'), 'we_' . $this->Name . '_' . $type . '[' . $name . self::NAME_DIVIDER . 'variant]', g_l('global', '[variant_field]'), false, '', '_EditorFrame.setEditorIsHot(true);') :
					'') .
					'<input type=hidden name="we_' . $this->Name . '_' . $type . '[' . $name . self::NAME_DIVIDER . 'required]" value="0" />' .
					'</td></tr>' :
					'<input type=hidden name="we_' . $this->Name . '_' . $type . '[' . $name . self::NAME_DIVIDER . 'required]" value="0" />' );
		}


		$content .= '<tr><td class="weMultiIconBoxHeadlineThin">' . g_l('weClass', '[fieldusers]') . '<br/>' .
				$this->formUsers1($jsCmd, $name, $type, $identifier) .
				'</td></tr>';

		if($new){
			$content .= we_html_element::hidden('fold_named[' . $name . '_0]', '1');
		}

		$parts->add((new we_html_multiIconBoxContentLine(g_l('weClass', '[moreProps]'), '<table>' . $content . '</table>'))->fold(!$new, true));


		return $preContent . '<tr><td>' . (new we_html_multiIconBox($parts, $name))->get() . '</td></tr>';
	}

	function htmlHref(string $name): string{
		$defaultField = self::DEFAULTS_VALUES[self::TYPE_HREF]['defField'];
		$hreftype = $this->getElementS($name . self::NAME_DIVIDER . 'hreftype');
		$attr = ['size' => '20'];

		$hrefArr = $this->getElementA($name . self::NAME_DIVIDER . $defaultField, self::DEFAULTS_VALUES[self::TYPE_HREF]['defaults'][$defaultField][0]);
		$int = (bool) ($hrefArr['int'] ?? false);
		$intID = (int) ($hrefArr['intID'] ?? 0);
		$intPath = $intID ? id2path($intID) : '';

		$int_elem_Name = 'we_' . $this->Name . '_' . self::TYPE_HREF . '[' . $name . self::NAME_DIVIDER . $defaultField . '][int]';
		$intPath_elem_Name = 'we_' . $this->Name . '_' . self::TYPE_HREF . '[' . $name . self::NAME_DIVIDER . $defaultField . '][intPath]';
		$intID_elem_Name = 'we_' . $this->Name . '_' . self::TYPE_HREF . '[' . $name . self::NAME_DIVIDER . $defaultField . '][intID]'; //TODO: should we use #bdid?
		$ext_elem_Name = 'we_' . $this->Name . '_' . self::TYPE_HREF . '[' . $name . self::NAME_DIVIDER . $defaultField . '][extPath]';


		switch($hreftype){
			case we_base_link::TYPE_INT:
				$out = self::hrefRow($intID_elem_Name, $intID, $intPath_elem_Name, $intPath, $attr, $int_elem_Name);
				break;
			case we_base_link::TYPE_EXT:
				$extPath = $hrefArr['extPath'] ?? '';
				$out = self::hrefRow('', 0, $ext_elem_Name, $extPath, $attr, $int_elem_Name);
				break;
			default:
				$extPath = $hrefArr['extPath'] ?? '';
				$out = self::hrefRow($intID_elem_Name, $intID, $intPath_elem_Name, $intPath, $attr, $int_elem_Name, true, $int) .
						self::hrefRow('', 0, $ext_elem_Name, $extPath, $attr, $int_elem_Name, true, $int);
		}
		return '<table class="default">' . $out . '</table>';
	}

	private function getLinkFieldHTML(string $name, string $i): string{
		$defaultField = self::DEFAULTS_VALUES[self::TYPE_LINK]['defField'];
		$attribs = ['name' => $name . self::NAME_DIVIDER . $defaultField];
		// FIXME: write complete default to defaults_array
		$link = $this->getElementA($name . self::NAME_DIVIDER . $defaultField) ?: ['ctype' => 'text', 'type' => we_base_link::TYPE_EXT, 'href' => '#', 'text' => g_l('global', '[new_link]')];
		'@phan-var array{name:string,text:string,img_src:string,ctype:?string,img_id:int|string,width:string,height:string,alt:?string,img_title:?string} $link';
		$img = new we_document_image();

		unset($link['cssClasses']);
		$content = self::getLinkContent($link, $img) ?: g_l('global', '[new_link]');
		$startTag = $this->getLinkStartTag($link, $attribs, $this->ParentID, $this->Path, $img);
		$editbut = we_html_button::create_button(we_html_button::LINK_EDIT, "javascript:we_cmd('edit_link_at_class',{transaction:'" . we_base_WE::$transaction . "',fieldname:'" . $name . self::NAME_DIVIDER . $defaultField . "','identifier':'" . $i . "'})") .
				$delbut = we_html_button::create_button(we_html_button::TRASH, "javascript:setScrollTo(" . we_base_constants::WE_EDITPAGE_CONTENT . ");we_cmd('we_object_class_deleteLink','" . we_base_WE::$transaction . "','" . $i . "','" . $name . self::NAME_DIVIDER . $defaultField . "')");

		return '<table class="default defaultfont">
	<tr><td>' . ($startTag ?: '') . $content . '</a></td>
		<td width="5"></td><td>' . $editbut . $delbut . '</td>
	</tr>
</table>';
	}

	private function getObjectFieldHTML(string $ObjectID): string{
		$pid = (int) $this->getElementS($ObjectID . self::NAME_DIVIDER . 'name');
		$defaultField = self::DEFAULTS_VALUES[self::TYPE_OBJECT]['defField'];
		$textname = 'we_' . $this->Name . '_' . self::TYPE_OBJECT . '[' . $pid . '_path]';
		$idname = 'we_' . $this->Name . '_' . self::TYPE_OBJECT . '[' . $ObjectID . self::NAME_DIVIDER . $defaultField . ']';
		$myid = $this->getElementI($ObjectID . self::NAME_DIVIDER . $defaultField);
		$DoubleNames = $this->includedObjectHasDoubbleFieldNames($pid);

		$path = $this->getElementS(we_listview_base::OBJECT_FIELD_PREFIX . $pid . '_path') ?: ($myid ? fS('SELECT Path FROM ' . OBJECT_FILES_TABLE . ' WHERE ID=' . $myid) : '');

		$rootDirID = fI('SELECT `of`.ID FROM ' . OBJECT_FILES_TABLE . ' `of` JOIN ' . OBJECT_TABLE . ' o ON (o.ID=`of`.TableID AND `of`.Path=o.Path) WHERE o.ID=' . $pid);

		$table = OBJECT_FILES_TABLE;
		$button = we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',{id:document.we_form.elements['" . $idname . "'].value,table:'" . $table . "',targetID:'" . $idname . "',targetName:'" . $textname . "',jsCmd:'setHot',rootDirID:'" . $rootDirID . "',filter:WE().consts.contentTypes.OBJECT_FILE,open_doc:" . (we_base_permission::hasPerm([
							we_base_perm::OBJECTS_CAN_SELECT_OTHER_USERS]) ? 0 : 1) . "})");
		$delbutton = we_html_button::create_button(we_html_button::TRASH, "javascript:document.we_form.elements['" . $idname . "'].value='';document.we_form.elements['" . $textname . "'].value=''");

		$weSuggest = we_gui_suggest::getInstance();
		$weSuggest->setAcId("TypeObject")
				->setContentType([we_base_ContentTypes::FOLDER_OBJECTFILE, we_base_ContentTypes::OBJECT_FILE])
				->setInput($textname, $path)
				->setMaxResults(20)
				->setRequired(false)
				->setResult($idname, (string) $myid)
				->setSelector(we_gui_suggest::DocSelector)
				->setTable($table)
				->setWidth(246)
				->setSelectButton($button)
				->setTrashButton($delbutton);

		return $weSuggest->getHTML() . ($DoubleNames ? '<span style="color:red" >' . sprintf(g_l('modules_object', '[incObject_sameFieldname]'), implode(', ', $DoubleNames)) . '</span>' : '');
	}

	private function getCollectionFieldHTML(string $name, int $defaultID, bool $editable = true): string{
		if(!$editable){
			return '';
		}
		$defaultField = self::DEFAULTS_VALUES[self::TYPE_COLLECTION]['defField'];
		$textname = 'we_' . $this->Name . '_' . self::TYPE_COLLECTION . '[' . $name . self::NAME_DIVIDER . 'defaultPath]';
		$idname = 'we_' . $this->Name . '_' . self::TYPE_COLLECTION . '[' . $name . self::NAME_DIVIDER . $defaultField . ']';
		$path = id2path($defaultID, VFILE_TABLE);
		$rootDirID = 0;

		$btnSelect = we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',{id:document.we_form.elements['" . $idname . "'].value,table:WE().consts.tables.VFILE_TABLE,targetID:'" . $idname . "',targetName:'" . $textname . "',jsCmd:'setHot',rootDirID:'" . $rootDirID . "',filter:WE().consts.contentTypes.COLLECTION,open_doc:" . (we_base_permission::hasPerm([
							we_base_perm::WORK_COLLECTIONS_CAN_SEE]) ? 0 : 1) . "})");

		$btnDelete = we_html_button::create_button(we_html_button::TRASH, "javascript:document.we_form.elements['" . $idname . "'].value='';document.we_form.elements['" . $textname . "'].value=''");

		$btnNewCollection = we_html_button::create_button(we_html_button::ADD_COLLECTION, "javascript:we_cmd('edit_new_collection','write_back_to_opener," . $idname . ',' . $textname . "','',-1,'" . stripTblPrefix(VFILE_TABLE) . "');", '', '', false, false);

		$weSuggest = we_gui_suggest::getInstance();
		return
						$weSuggest
						->setAcId($idname . we_base_file::getUniqueId())
						->setContentType([we_base_ContentTypes::FOLDER_VFILE, we_base_ContentTypes::COLLECTION])
						->setInput($textname, $path)
						->setMaxResults(20)
						->setRequired(true)
						->setResult($idname, (string) $defaultID)
						->setSelector(we_gui_suggest::DocSelector)
						->setTable(VFILE_TABLE)
						->setWidth(246)
						->setSelectButton($btnSelect)
						->setCreateButton((we_base_permission::hasPerm([we_base_perm::WORK_COLLECTION_NEW]) ? $btnNewCollection : ''))
						->setTrashButton($btnDelete)
						->getHTML();
	}

	private function getMultiObjectFieldHTML(string $name, string $i, int $f, we_gui_suggest $weSuggest, bool $required = true): string{
		$pid = $this->getElementI($name . self::NAME_DIVIDER . 'tableid');
		$defaultField = self::DEFAULTS_VALUES[self::TYPE_MULTIOBJECT]['defField'];
		$defaultValue = self::DEFAULTS_VALUES[self::TYPE_MULTIOBJECT]['defaults'][$defaultField][0];

		$textname = $name . '_' . 'defaultPath][' . $f . ']';
		$idname = 'we_' . $this->Name . '_' . self::TYPE_MULTIOBJECT . '[' . $name . self::NAME_DIVIDER . $defaultField . '][' . $f . ']';
		$ids = $this->getElementA($name . self::NAME_DIVIDER . $defaultField, $defaultValue);

		$myid = !empty($ids[$f]) ? $ids[$f] : 0;
		$count = count($ids);
		$max = $this->getElementI($name . self::NAME_DIVIDER . 'max');

		$path = (
				$this->getElementS(we_listview_base::OBJECT_FIELD_PREFIX . $name . '_path') ?:
				($myid ? fS('SELECT Path FROM ' . OBJECT_FILES_TABLE . ' WHERE ID=' . $myid) : '')
				);

		['ID' => $rootDirID, 'Path' => $rootDirPath] = getHash('SELECT `of`.ID,`of`.Path FROM ' . OBJECT_FILES_TABLE . ' `of` WHERE `of`.IsClassFolder=1 AND `of`.TableID=' . $pid) ?: [
			'ID' => 0, 'Path' => '/'];

		$selectObject = we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',{id:document.we_form.elements['" . $idname . "'].value,table:WE().consts.tables.OBJECT_FILES_TABLE,targetID:'" . $idname . "',targetName:'" . $textname . "',jsCmd:'setHot',rootDirID:" . $rootDirID . ",filter:WE().consts.contentTypes.OBJECT_FILE,open_doc:" . (we_base_permission::hasPerm([
							we_base_perm::OBJECTS_CAN_SELECT_OTHER_USERS]) ? 0 : 1) . "})");
		$delbutton = we_html_button::create_button(we_html_button::TRASH, "javascript:document.we_form.elements['" . $idname . "'].value='';document.we_form.elements['" . $textname . "'].value=''");

		return
						$weSuggest
						->setAcId($textname . we_base_file::getUniqueId(), $rootDirPath)
						->setInput($textname, $path)
						->setRequired($required)
						->setResult($idname, (string) $myid)
						->setSelectButton($selectObject)
						->setTrashButton(($count >= 1 ?
								we_html_button::create_button(we_html_button::TRASH, "javascript:we_cmd('we_object_class_multiobjectDelete','" . we_base_WE::$transaction . "','" . ($i) . "','" . $name . "','" . ($f) . "')") :
								$delbutton
						))
						->setCreateButton((($count < $max || $max === 0) ?
								we_html_button::create_button(we_html_button::LIST_ADD, "javascript:we_cmd('we_object_class_multiobjectInsert','" . we_base_WE::$transaction . "','" . $i . "','" . $name . "','" . $f . "')") :
								(new we_html_button(we_html_button::LIST_ADD))->disable(true)->get()
						))
						->setOpenButton(($myid ?
								we_html_button::create_button(we_html_button::OPEN_TO_EDIT, "javascript:WE().layout.weEditorFrameController.openDocument(WE().consts.tables.OBJECT_FILES_TABLE," . $myid . ",WE().consts.contentTypes.OBJECT_FILE);") :
								we_html_button::create_button(we_html_button::OPEN_TO_EDIT, '', '', '', true)
						))
						->setAdditionalButton((($f > 0) ?
								we_html_button::create_button(we_html_button::DIRUP, "javascript:we_cmd('we_object_class_multiobjectUp','" . we_base_WE::$transaction . "','" . ($i) . "','" . $name . "','" . ($f) . "')") :
								(new we_html_button(we_html_button::DIRUP))->disable(true)->get()
								) .
								(($f < ($count - 1)) ?
								we_html_button::create_button(we_html_button::DIRDOWN, "javascript:we_cmd('we_object_class_multiobjectDown','" . we_base_WE::$transaction . "','" . ($i) . "','" . $name . "','" . ($f) . "')") :
								(new we_html_button(we_html_button::DIRDOWN))->disable(true)->get()
						))
						->getHTML();
	}

	function wysiwyg(string $name, string $i = '0'): string{
		return we_html_element::div([], we_html_button::create_button('attributes', "javascript:setScrollTo(" . we_base_constants::WE_EDITPAGE_CONTENT . ");we_cmd('we_object_class_textAreaEdit','" . $i . "','" . $name . "','" . we_base_WE::$transaction . "');")) .
				$this->getWysiwygArea($name);
	}

	function getWysiwygArea(string $name): string{
		$stylesheets = [];
		foreach($this->CSS as $cssId){
			if(($src = fS('SELECT CONCAT(Path,' . (DISABLE_MEDIA_MODTIME ? '""' : '"?m=",UNIX_TIMESTAMP(Publish_Date)') . ') FROM ' . FILE_TABLE . ' WHERE IsPublished=1 AND ID=' . $cssId))){
				$stylesheets[] = $src;
			}
		}

		$attribs = [
			'bgcolor' => $this->getElementS($name . self::NAME_DIVIDER . 'bgcolor'),
			'class' => $this->getElementS($name . self::NAME_DIVIDER . 'class'),
			'classes' => $this->getElementS($name . self::NAME_DIVIDER . 'cssClasses'),
			'cols' => 30,
			'commands' => preg_replace('/ *, */', ',', $this->getElementS($name . self::NAME_DIVIDER . 'commands') ?: COMMANDS_DEFAULT),
			'contextmenu' => $this->getElementS($name . self::NAME_DIVIDER . 'contextmenu'),
			'fontsizes' => $this->getElementS($name . self::NAME_DIVIDER . 'fontsizes'),
			'formats' => $this->getElementS($name . self::NAME_DIVIDER . 'formats'),
			'gallerytemplates' => $this->getElementS($name . self::NAME_DIVIDER . 'gallerytemplates'),
			'inlineedit' => false /* $this->getElementB($name . self::NAME_DIVIDER . 'inlineedit') */,
			'removefirstparagraph' => $this->getElementB($name . self::NAME_DIVIDER . 'removefirstparagraph', REMOVEFIRSTPARAGRAPH_DEFAULT),
			'rows' => 3,
			'stylesheets' => $stylesheets,
			'templates' => $this->getElementS($name . self::NAME_DIVIDER . 'templates'),
			'tinyparams' => $this->getElementS($name . self::NAME_DIVIDER . 'tinyparams'),
			'wysiwyg' => $this->getElementB($name . self::NAME_DIVIDER . 'wysiwyg'),
				//'xml' => $this->getElementB($name . self::NAME_DIVIDER . 'xml'),
		];

		$defaultField = self::DEFAULTS_VALUES[self::TYPE_TEXT]['defField'];

		return we_html_forms::weTextarea('we_' . $this->Name . '_' . self::TYPE_TEXT . '[' . $name . self::NAME_DIVIDER . $defaultField . ']', $this->getElementS($name . self::NAME_DIVIDER . $defaultField), $attribs, !($this->CSS || $attribs['classes']), false, $attribs['removefirstparagraph']);
	}

	function add_user_to_field(array $ids, string $name): void{
		$users = array_filter($this->getElementA($name . self::NAME_DIVIDER . 'users'));
		foreach($ids as $id){
			if($id && (!in_array($id, $users))){
				$users[] = $id;
			}
		}
		$this->setElementA($name . self::NAME_DIVIDER . 'users', $users);
	}

	function del_user_from_field(int $id, string $name): void{
		$users = array_filter($this->getElementA($name . self::NAME_DIVIDER . 'users'));
		$pos = array_search($id, $users);
		if($pos !== false){
			unset($users[$pos]);
		}
		$this->setElementA($name . self::NAME_DIVIDER . 'users', $users);
	}

	private function formUsers1(we_base_jsCmd $jsCmd, string $name, string $type, string $nr = '0'): string{
		$users = $this->getElementA($name . self::NAME_DIVIDER . 'users') ? array_filter($this->getElementA($name . self::NAME_DIVIDER . 'users')) : [];
		$content = '';
		if($users){
			$allUsers = array_flip($users);
			foreach($this->DB_WE->getAllq('SELECT ID,Path,(IF(IsFolder,"' . we_base_ContentTypes::USERGROUP . '",(IF(Alias>0,"' . we_base_ContentTypes::USERALIAS . '","' . we_base_ContentTypes::USER . '")))) AS ContentType FROM ' . USER_TABLE . ' WHERE ID IN (' . implode(',', $users) . ')') as $item){
				$content .= '<tr><td style="width:20px;" class="userIcon" data-contenttype="' . $item['ContentType'] . '"></td><td style="width:324px;">' . $item['Path'] . '</td><td>' . we_html_button::create_button(we_html_button::TRASH, "javascript:we_cmd('we_object_class_fieldDelUser','" . we_base_WE::$transaction . "','" . $nr . "'," . $item['ID'] . ",'" . $name . "');") . '</td></tr>';
				unset($allUsers[$item['ID']]);
			}
//all non-existing users
			foreach(array_keys($allUsers) as $user){
				$content .= '<tr><td style="width:20px;"></td><td>Unknown</td><td>' . we_html_button::create_button(we_html_button::TRASH, "javascript:we_cmd('we_object_class_fieldDelUser','" . we_base_WE::$transaction . "','" . $nr . "'," . $user . ",'" . $name . "');") . '</td></tr>';
			}
		} else {
			$content .= '<tr><td style="width:20px;" class="userIcon" data-contenttype="' . we_base_ContentTypes::USERGROUP . '"></td><td style="width:324px;">' . g_l('weClass', '[everybody]') . '</td><td></td></tr>';
		}

		$textname = 'we_' . $this->Name . '_' . $type . '[' . $name . self::NAME_DIVIDER . 'usertext]';
		$idname = 'we_' . $this->Name . '_' . $type . '[' . $name . self::NAME_DIVIDER . 'userid]';

		$cmd = "we_object_class_fieldAddUser," . we_base_WE::$transaction . "," . $nr . ",," . $name;

		$jsCmd->addCmd('setIconOfDocClass', 'userIcon');
		return we_html_element::div(['class' => 'multichooserDiv'],
						we_html_element::div(['class' => 'multichooser'],
								'<table>' .
								$content . '</table>') .
						we_html_element::div(['class' => 'button'],
								we_html_button::create_button(we_html_button::DELETE_ALL, "javascript:we_cmd('we_object_class_ownersDeleteAll','" . we_base_WE::$transaction . "','" . $nr . "','" . $name . "')", '', '', empty($users)) .
								we_html_button::create_button(we_html_button::ADD, "javascript:we_cmd('we_users_selector',{id:document.we_form.elements['" . $idname . "'].value,targetName:'" . $textname . "',targetID:'" . $idname . "',jsCmd:'" . $cmd . "',multiple:true})") .
								we_html_element::hiddens([$idname => '0', $textname => ''])
		));
	}

	function del_all_owners(string $name = ''): void{
		if($name === ''){
			parent::del_all_owners();
			return;
		}
		$this->setElementA($name . self::NAME_DIVIDER . 'users', []);
	}

	function add_css(array $ids): void{
		$this->CSS = array_unique(array_filter(array_merge($this->CSS, $ids)), SORT_NUMERIC);
	}

	function del_css(int $id): void{
		if(($pos = array_search($id, $this->CSS, false)) === false){
			return;
		}
		unset($this->CSS[$pos]);
	}

	private function getImageHTML(string $name, int $id, string $i = '0'): string{
		$doc = new we_document_image($id);
		$defaultField = self::DEFAULTS_VALUES[self::TYPE_IMG]['defField'];
		$fname = 'we_' . $this->Name . '_' . self::TYPE_IMG . '[' . $name . self::NAME_DIVIDER . $defaultField . ']';
		$content = '<input type=hidden name="' . $fname . '" value="' . $id . '" />' .
				we_html_button::create_button(we_html_button::IMAGE_SELECT, "javascript:we_cmd('we_selector_image',{id:'" . $id . "',table:WE().consts.tables.FILE_TABLE,targetID:'" . $fname . "',jsCmd:'we_object_class_reloadEntry," . we_base_WE::$transaction . ',' . $i . "',rootDirID:0})") .
				we_html_button::create_button(we_html_button::TRASH, "javascript:we_cmd('we_object_class_removeImage','" . we_base_WE::$transaction . "','" . $i . "','" . $name . self::NAME_DIVIDER . $defaultField . "')") .
				'<br/>' . $doc->getHtml();

// gets thumbnails and shows a select field, if there are any:
		$thumbList = DB_WE::$inst->getAllFirstq('SELECT ID,Text FROM ' . THUMBNAILS_TABLE . ' ORDER BY Text', false);
		if($thumbList){
			$thumbList = ['-'] + $thumbList;
			$content .= '<br/>' . g_l('modules_object', '[use_thumbnail_preview]') . ":<br/>";
			$tmp = $this->getElementI($name . self::NAME_DIVIDER . 'defaultThumb');
			$currentSelection = ($tmp && isset($thumbList[$tmp]) ? $tmp : '');

			$content .= we_html_tools::select('we_' . $this->Name . '_' . self::TYPE_IMG . '[' . $name . self::NAME_DIVIDER . 'defaultThumb]', $thumbList, 1, $currentSelection, false, [
						'onchange' => "_EditorFrame.setEditorIsHot(true);"]);
		}
		return $content;
	}

	private function getVideoHTML(string $name, int $id, string $i = '0'): string{
		$doc = new we_document_video($id);
		$defaultField = self::DEFAULTS_VALUES[self::TYPE_VIDEO]['defField'];
		$fname = 'we_' . $this->Name . '_' . self::TYPE_VIDEO . '[' . $name . self::NAME_DIVIDER . $defaultField . ']';

		return '<input type=hidden name="' . $fname . '" value="' . $id . '" />' .
				we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',{id:'" . $id . "',table:WE().consts.tables.FILE_TABLE,targetID:'" . $fname . "',jsCmd:'we_object_class_reloadEntry," . we_base_WE::$transaction . ',' . $i . "',rootDirID:0,filter:WE().consts.contentTypes.VIDEO})") .
				we_html_button::create_button(we_html_button::TRASH, "javascript:we_cmd('we_object_class_removeImage','" . we_base_WE::$transaction . "','" . $i . "','" . $name . self::NAME_DIVIDER . $defaultField . "')") .
				'<br/>' . $doc->getHtml();
	}

	private function getAudioHTML(string $name, int $id, string $i = '0'): string{
		$doc = new we_document_audio($id);
		$defaultField = self::DEFAULTS_VALUES[self::TYPE_AUDIO]['defField'];
		$fname = 'we_' . $this->Name . '_' . self::TYPE_AUDIO . '[' . $name . self::NAME_DIVIDER . $defaultField . ']';

		return '<input type=hidden name="' . $fname . '" value="' . $id . '" />' .
				we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',{id:'" . $id . "',table:WE().consts.tables.FILE_TABLE,targetID:'" . $fname . "',jsCmd:'we_object_class_reloadEntry," . we_base_WE::$transaction . ',' . $i . "',rootDirID:0,filter:WE().consts.contentTypes.AUDIO})") .
				we_html_button::create_button(we_html_button::TRASH, "javascript:we_cmd('we_object_class_removeImage','" . we_base_WE::$transaction . "','" . $i . "','" . $name . self::NAME_DIVIDER . $defaultField . "')") .
				'<br/>' . $doc->getHtml();
	}

	private function getBinaryHTML(string $name, int $id, string $i = '0'): string{
		$doc = new we_document_other($id);
		$defaultField = self::DEFAULTS_VALUES[self::TYPE_BINARY]['defField'];
		$fname = 'we_' . $this->Name . '_' . self::TYPE_BINARY . '[' . $name . self::NAME_DIVIDER . $defaultField . ']';

		return '<input type=hidden name="' . $fname . '" value="' . $id . '" />' .
				we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',{id:'" . $id . "',table:WE().consts.tables.FILE_TABLE,targetID:'" . $fname . "',jsCmd:'we_object_class_reloadEntry," . we_base_WE::$transaction . ',' . $i . "',rootDirID:0,filter:WE().consts.contentTypes.APPLICATION+','+WE().consts.contentTypes.PDF})") .
				we_html_button::create_button(we_html_button::TRASH, "javascript:we_cmd('we_object_class_removeImage','" . we_base_WE::$transaction . "','" . $i . "','" . $name . self::NAME_DIVIDER . $defaultField . "');") .
				'<br/>' . $doc->getHtml();
	}

	private function checkSetDefaultTextFromElements(): void{
		if(($elem = $this->getElementA('defaultTexts', we_base_request::NOT_VALID_AS)) !== we_base_request::NOT_VALID_AS){
			$this->DefaultText = implode('', array_filter($elem));
		}
	}

	private function checkSetDefaultUrlFromElements(): void{
		if(($defaultUrls = $this->getElementA('defaultUrls', we_base_request::NOT_VALID_AS)) !== we_base_request::NOT_VALID_AS){
			foreach($defaultUrls as &$value){
				if(in_array(($v = trim($value, '%')), array_keys(self::URL_FIELDS))){
					$value = '%' . $v . (($l = $this->getElementI($v . '_len')) > 0 ? $l : self::URL_FIELDS[$v]) . '%';
				}
			}
			$this->DefaultUrl = implode('', array_filter($defaultUrls));
		}
	}

	private function formDefaultText(): string{
		$this->checkSetDefaultTextFromElements();

		$opts = we_base_country::g_l_a('modules_object', '[value]');
		$rows = [];
		$c = 0;
		foreach(array_filter(explode('%', trim($this->DefaultText, '%'))) as $val){
			if(in_array('%' . $val . '%', array_keys($opts))){
				$rows[] = we_html_tools::select('we_' . $this->Name . '_' . self::TYPE_GENERAL . '[defaultTexts][' . $c . ']', $opts, 1, '%' . $val . '%', false, ['onchange' => "_EditorFrame.setEditorIsHot(true);we_cmd('reload_editpage');"]);
			} else {
				$rows[] = we_html_tools::select('textwert_' . $c, $opts, 1, 'Text', false, ['onchange' => "_EditorFrame.setEditorIsHot(true); document.we_form.elements['we_" . $this->Name . "_" . self::TYPE_GENERAL . "[defaultTexts][" . $c . "]'].value=this.options[this.selectedIndex].value; we_cmd('reload_editpage');"], true, 140) .
						we_html_tools::textInput('we_' . $this->Name . '_' . self::TYPE_GENERAL . '[defaultTexts][' . $c . ']', $val, 255, ['onchange' => "_EditorFrame.setEditorIsHot(true);"], 'text', '217px');
			}
			$c++;
		}

		return implode(we_html_element::br(), $rows) . we_html_tools::select('we_' . $this->Name . '_' . self::TYPE_GENERAL . '[defaultTexts][' . $c . ']', $opts, 1, '', false, [
					'onchange' => "_EditorFrame.setEditorIsHot(true);we_cmd('reload_editpage');"]);
	}

	private function formDefaultUrl(): string{
		$this->checkSetDefaultUrlFromElements();

		$opts = we_base_country::g_l_a('modules_object', '[url]');
		asort($opts);
		$rows = [];
		$c = 0;

		foreach(array_filter(explode('%', trim($this->DefaultUrl, '%'))) as $val){
			if(in_array('%' . $val . '%', array_keys($opts))){
				$rows[] = we_html_tools::select('we_' . $this->Name . '_' . self::TYPE_GENERAL . '[defaultUrls][' . $c . ']', $opts, 1, '%' . $val . '%', false, ['onchange' => "_EditorFrame.setEditorIsHot(true);we_cmd('reload_editpage');"]);
			} else {
				if(str_starts_with($val, 'urlfield')){
					$len = (int) str_replace(array_keys(self::URL_FIELDS), '', $val);
					$rows[] = we_html_tools::select('we_' . $this->Name . '_' . self::TYPE_GENERAL . '[defaultUrls][' . $c . ']', $opts, 1, '%' . substr($val, 0, 9) . '%', false, [
								'onchange' => "_EditorFrame.setEditorIsHot(true);we_cmd('reload_editpage');"], true, 140) .
							we_html_element::span(['style' => 'padding: 0 4px 0 6px;'], g_l('modules_object', '[length]') . ':') . we_html_tools::textInput('we_' . $this->Name . '_' . self::TYPE_GENERAL . '[' . substr($val, 0, 9) . '_len]', (string) $len, 255, [
								'onchange' => "_EditorFrame.setEditorIsHot(true);"], "text", '60px');
				} else {
					$rows[] = we_html_tools::select('textwert_' . $c, $opts, 1, 'Text', false, ['onchange' => "_EditorFrame.setEditorIsHot(true); document.we_form.elements['we_" . $this->Name . "_" . self::TYPE_GENERAL . "[defaultUrls][" . $c . "]'].value = this.options[this.selectedIndex].value; we_cmd('reload_editpage');"], true, 140) .
							we_html_tools::textInput('we_' . $this->Name . '_' . self::TYPE_GENERAL . '[defaultUrls][' . $c . ']', $val, 255, ['onchange' => "_EditorFrame.setEditorIsHot(true);"], "text", '217px');
				}
			}
			$c++;
		}

		return implode(we_html_element::br(), $rows) . we_html_tools::select('we_' . $this->Name . '_' . self::TYPE_GENERAL . '[defaultUrls][' . $c . ']', $opts, 1, '', false, [
					'onchange' => "_EditorFrame.setEditorIsHot(true);we_cmd('reload_editpage');"]) .
				we_html_element::hidden('we_' . $this->Name . '_input[DefaultanzahlUrl]', $c);
	}

	private function formDefault(): string{
		return $this->formTriggerDocument($this->DefaultTriggerID, true);
	}

	function formRestrictOwners(bool $canChange = true): string{
		return ($canChange ?
				we_html_element::hidden('we_' . $this->Name . '_RestrictOwners', intval($this->RestrictOwners)) .
				we_html_forms::checkbox('1', $this->RestrictOwners, 'tmpwe_' . $this->Name . '_RestrictOwners', g_l('weClass', '[limitedAccess]'), true, "defaultfont", "_EditorFrame.setEditorIsHot(true);this.form.elements['we_" . $this->Name . "_RestrictOwners'].value=(this.checked ? '1' : '0');we_cmd('reload_editpage');") :
				'<table class="default"><tr><td><i class="fa fa-' . ($this->RestrictOwners ? 'check-' : '') . 'square-o wecheckIcon disabled"></i></td><td class="defaultfont">&nbsp;' . g_l('weClass', '[limitedAccess]') . '</td></tr></table>'
				);
	}

	public function formPath(bool $disablePath = false, bool $notSetHot = false, string $extra = ''): string{
		return $this->formInput('', 'Filename', g_l('modules_object', '[classname]'), 0, 255, ['onchange' => "pathOfDocumentChanged(true);"]);
	}

	private function formWorkspaces(we_base_jsCmd $jsCmd): string{
//remove not existing workspaces - deal with templates as well

		$newTmplArr = $newDefaultArr = $newArr = [];

//    check if workspace exists - correct templates if neccessary !!
		foreach($this->Workspaces as $i => $wsp){
			if(we_base_file::isWeFile($wsp, FILE_TABLE)){
				$newArr[] = $wsp;
				if(in_array($wsp, $this->DefaultWorkspaces)){
					$newDefaultArr[] = $wsp;
				}
				$newTmplArr[] = (int) ($this->Templates[$i] ?? 0);
			}
		}
		$this->DefaultWorkspaces = $newDefaultArr;
		$this->Workspaces = $newArr;
		$this->Templates = $newTmplArr;


		$addbut = we_html_button::create_button(we_html_button::ADD, "javascript:we_cmd('we_selector_directory',{table:WE().consts.tables.FILE_TABLE,jsCmd:'object_add_workspace',multiple:true})");

		$obj = new we_chooser_multiDirTemplateAndDefault($this->Workspaces, 'object_del_workspace', $addbut, we_users_util::get_ws(FILE_TABLE), $this->Templates, 'we_' . $this->Name . '_Templates', [
				], we_users_util::get_ws(TEMPLATES_TABLE), 'we_' . $this->Name . '_DefaultWorkspaces', $this->DefaultWorkspaces);
		$obj->setDeletable(true);
		$obj->create = true;
		$content = $obj->get($jsCmd);

		if($this->delWorkspaceError){
			$jsCmd->addMsg(g_l('weClass', '[we_del_workspace_error]'), we_base_util::WE_MESSAGE_ERROR);
		}
		return $content;
	}

	private function formWorkspacesFlag(): string{
		return '<div style="margin-bottom:8px;">' . we_html_forms::radiobutton('1', $this->WorkspaceFlag, 'we_' . $this->Name . '_WorkspaceFlag', g_l('modules_object', '[behaviour_all]')) . '</div><div>' .
				we_html_forms::radiobutton('0', !$this->WorkspaceFlag, 'we_' . $this->Name . '_WorkspaceFlag', g_l('modules_object', '[behaviour_no]')) . '</div>';
	}

	private function formCSS(we_base_jsCmd $jsCmd): string{
		$addbut = we_html_button::create_button(we_html_button::ADD, "javascript:we_cmd('we_selector_document',{id:0,table: WE().consts.tables.FILE_TABLE,jsCmd:'we_object_class_cssAdd',filter:WE().consts.contentTypes.CSS,open_doc:true,multiple:true})");
		$css = new we_chooser_multiDir($this->CSS, 'we_object_class_cssDel', $addbut, [], 'ContentType', FILE_TABLE);
		return $css->get($jsCmd);
	}

	protected function formCopyDocument(): string{
		return we_html_element::hidden('we_' . $this->Name . '_CopyID', $this->CopyID) .
				we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',{id:document.we_form.elements['we_" . $this->Name . "_CopyID'].value,table:'" . $this->Table . "',targetID:'we_" . $this->Name . "_CopyID',jsCmd:'copyDocumentSelect',rootDirID:0,filter:'" . $this->ContentType . "'});");
	}

	function copyDoc(int $id): bool{
		if(!$id){
			return false;
		}
		$doc = new we_contents_object();
		$doc->InitByID($id, '', self::LOAD_TEMP_DB);
		if($this->ID == 0){
			foreach(array_keys($this->persistent_slots) as $name){
				$this->{$name} = $doc->{$name} ?? '';
			}
			$this->Creation_Date = new DateTime();
			$this->CreatorID = we_users_user::getID();
			$this->ID = 0;
			$this->OldPath = '';
			$this->Publish_Date = new DateTime();
			$this->Filename .= '_copy';
			$this->Path = $this->getPath();
			$this->OldPath = $this->Path;
		}
		$this->elements = $doc->elements;
		$newEls = [];
		foreach($this->elements as $n => $e){
			if(str_starts_with(strtolower($n), 'wholename')){
				$newEls[] = $e['dat'];
			}
		}
		if($newEls){
			$this->setElementA('neuefelder', array_merge($this->getElementA('neuefelder'), $newEls));
		}
		$this->EditPageNr = we_base_constants::WE_EDITPAGE_PROPERTIES;
		$this->Category = $doc->Category;
		return true;
	}

	function changeTempl_ob(int $nr, int $id): void{
		$this->Templates[$nr] = $id;
	}

	function del_workspace(int $id): void{
		if(fB('SELECT 1 FROM ' . OBJECT_FILES_TABLE . ' WHERE IsFolder=0 AND TableID=' . ((int) $this->ID) . ' AND FIND_IN_SET(' . $id . ',Workspaces) LIMIT 1')){
			$this->delWorkspaceError = true;
			return;
		}

		if(($pos = array_search($id, $this->Workspaces)) !== false){
			unset($this->Workspaces[$pos]);
			if(in_array($id, $this->DefaultWorkspaces)){
				unset($this->DefaultWorkspaces[array_search($id, $this->DefaultWorkspaces)]);
			}
			unset($this->Templates[$pos]);
		}
	}

	protected function getContentData(string $table): bool{
		if(!$this->ID){
			return false;
		}
		$f = 0;
		$rec = getHash('SELECT DefaultCategory,DefaultText,DefaultDesc,DefaultTitle,DefaultUrl,DefaultUrlfield0,DefaultUrlfield1,DefaultUrlfield2,DefaultUrlfield3,DefaultTriggerID,SidebarDocID,DefaultKeywords,DefaultValues FROM ' . OBJECT_TABLE . ' WHERE ID=' . $this->ID);

		$this->setSort();

		$this->DefaultValues = we_unserialize($rec['DefaultValues']);

		foreach($this->DefaultValues as $name => $vals){
			$name = (string) $name;
			if($name === 'WE_CSS_FOR_CLASS'){
				$this->CSS = we_base_request::filterIntList($vals);
			}
			if(!is_array($vals)){
				continue;
			}
			/*
			  if(!empty($vals['meta']) && is_array($vals['meta'])){
			  $this->setElementI($name . 'count', (count($vals['meta']) - 1));
			  switch(in_array(($fieldType = explode('_', $name)[0]), array_keys(self::DEFAULTS_VALUES)) ? $fieldType : ''){
			  case self::TYPE_META:
			  $keynames = array_keys($vals['meta']);
			  for($ll = 0; $ll <= count($vals['meta']); $ll++){
			  $this->setElementS($name . self::NAME_DIVIDER . 'defaultkey' . $ll, (string) ($keynames[$ll] ?? ''));
			  $this->setElementS($name . self::NAME_DIVIDER . 'defaultvalue' . $ll, (isset($keynames[$ll]) ? (string) $vals['meta'][$keynames[$ll]] : ''));
			  }
			  break;
			  case self::TYPE_MULTIOBJECT:
			  $keynames = array_keys($vals['meta']);
			  for($ll = 0; $ll <= count($vals['meta']); $ll++){
			  $this->setElementI($name . self::NAME_DIVIDER . 'defaultvalue' . $ll, (isset($keynames[$ll]) ? (int) $vals['meta'][$keynames[$ll]] : 0));
			  }
			  break;
			  }
			  } else {
			  //				$this->setElementI($name . 'count', 0);
			  }
			 *
			 */
		}

		$this->DefaultCategory = we_base_request::filterIntList($rec['DefaultCategory']);
		$this->Category = $this->DefaultCategory;

		$this->WorkspaceFlag = !empty($this->DefaultValues['WorkspaceFlag']);
		$this->setElementS('title', $rec['DefaultTitle']);
		$this->setElementS('desc', $rec['DefaultDesc']);
		$this->setElementS('keywords', $rec['DefaultKeywords']);

		$this->DefaultText = $rec['DefaultText'];
		$this->DefaultUrl = $rec['DefaultUrl'];

		$this->setElementS('urlfield0', $rec['DefaultUrlfield0']);
		$this->setElementS('urlfield1', $rec['DefaultUrlfield1']);
		$this->setElementS('urlfield2', $rec['DefaultUrlfield2']);
		$this->setElementS('urlfield3', $rec['DefaultUrlfield3']);
		$this->setElementI('triggerid', (int) $rec['DefaultTriggerID']);
		$this->DefaultTriggerID = $rec['DefaultTriggerID'];

		$ctable = OBJECT_X_TABLE . intval($this->ID);
		$tableInfo = $this->DB_WE->metadata($ctable);
		'@phan-var array<string,array{table:string,name:string,type:string,len:int,flags:int}> $tableInfo';

		$sort = $this->getElementA('we_sort');
		foreach($tableInfo as $info){
			[$type, $name] = explode('_', $info['name'], 2);
			if(!$name || $type === 'OF' || $type === 'variant'){
				continue;
			}

			$this->setElementS($info['name'] . self::NAME_DIVIDER . 'name', $name);
			$this->setElementS('wholename' . array_search($f, $sort), $info['name']);
			$this->setElementI($info['name'] . self::NAME_DIVIDER . self::ELEMENT_LENGHT, $info['len']);
			$this->setElementS($info['name'] . self::NAME_DIVIDER . self::ELEMENT_TYPE, $type);
			$typeLen = strtoupper($info['type']) . (defined('MYSQLI_UNSIGNED_FLAG') && (($info['flags'] & MYSQLI_UNSIGNED_FLAG) > 0) ? '_U' : '');
			$this->setElementS($info['name'] . self::NAME_DIVIDER . 'typeLen', $typeLen);
			$this->setFieldValues($info['name'], array_merge(self::COMMON_DEFAULTS_VALUES, self::DEFAULTS_VALUES[$type]['defaults']));
			$f++;
		}

		return !empty($rec);
	}

	private function setFieldValues(string $fieldName, array $defaultValue): void{
		foreach($defaultValue as $element => $default){
			$val = $this->DefaultValues[$fieldName][$element] ?? $default[0];

			switch($default[1]){
				case we_base_request::BOOL:
					$this->setElementB($fieldName . self::NAME_DIVIDER . $element, (bool) $val);
					break;
				case we_base_request::INT:
					$this->setElementI($fieldName . self::NAME_DIVIDER . $element, (int) $val);
					break;
				case we_base_request::FLOAT:
					$this->setElementF($fieldName . self::NAME_DIVIDER . $element, (float) $val);
					break;
				case we_base_request::STRING:
				case we_base_request::HTML:
					$this->setElementS($fieldName . self::NAME_DIVIDER . $element, (string) $val);
					break;
				case we_base_request::STRING_LIST:
					$this->setElementA($fieldName . self::NAME_DIVIDER . $element, array_values(array_map('strval', (array) $val)));
					break;
				case we_base_request::INTLISTA:
					$this->setElementA($fieldName . self::NAME_DIVIDER . $element, array_values(array_map('intval', (array) $val)));
					break;
				case 'arrayHref':
					$this->setElementA($fieldName . self::NAME_DIVIDER . $element, [
						'int' => isset($val['int']) ? (bool) $val['int'] : true,
						'intID' => isset($val['intID']) ? (int) $val['intID'] : 0,
						'extPath' => isset($val['extPath']) ? (string) $val['extPath'] : '',
					]);
					break;
				case 'arrayLink':
					// FIXME: cast types like in arrayHref
					$this->setElementA($fieldName . self::NAME_DIVIDER . $element, (array) $val);
					break;
			}
		}
	}

	protected function setPersistentSlot(string $name, /* mixed */ $value): void{
		switch($name){
			default:
				parent::setPersistentSlot($name, $value);
				return;
			case 'Templates_0':
				$tmp = $dfWs = [];
				foreach($this->Workspaces as $i => $cur){
					$tmp[] = we_base_request::int('we_' . $this->Name . '_Templates_' . $i);
					if(we_base_request::int('we_' . $this->Name . '_DefaultWorkspaces_' . $i, -1) !== -1){
						$dfWs[] = $cur;
					}
				}
				$this->Templates = $tmp;
				$this->DefaultWorkspaces = $dfWs;
				return;
		}
	}

	protected function isFilenameEmpty(): bool{
		return ($this->Filename === '');
	}

	protected function isFilenameNotValid(): bool{
		static $allowedReplace = [
			'%ID%',
			'%d%',
			'%j%',
			'%m%',
			'%y%',
			'%Y%',
			'%n%',
			'%h%',
			'%H%',
			'%g%',
			'%G%',
			'%unique%'
		];

		foreach($this->getElementA('defaultTexts') as $namePart){
			if($namePart && !in_array($namePart, $allowedReplace) && preg_match('/[^\w\-.]/', $namePart)){
				return true;
			}
		}

		return (preg_match('/[^\w\-.]/', $this->Filename)) ? true : false;
	}

	protected function isFilenameNotAllowed(): bool{
		return false;
	}

	protected function isFilenameDouble(): bool{
		return fB('SELECT 1 FROM ' . $this->Table . ' WHERE ParentID=' . intval($this->ParentID) . ' AND Filename="' . $this->DB_WE->escape($this->Filename) . '" AND ID!=' . intval($this->ID));
	}

	function checkPathDiffAndCreate(): bool{
		return true;
	}

	protected function hasDoubbleFieldNames(): string{
		$sort = $this->getElementA('we_sort');
		if(empty($sort)){
			return '';
		}
		$usedNames = [];
		foreach(array_keys($sort) as $elem){
			$name = $this->getElementS('wholename' . $elem);
			$foo = $this->getElementS($name . self::NAME_DIVIDER . 'name');
			if(empty($foo)){ //we don't allow field names evaluating to false, report them as double ;-)
				return '';
			}
			if(!in_array($foo, $usedNames)){
				$usedNames[] = $foo;
				continue;
			}
			switch($this->getElementS($name . self::NAME_DIVIDER . self::ELEMENT_TYPE)){
				case self::TYPE_OBJECT:
					return fS('SELECT Path FROM ' . OBJECT_TABLE . ' WHERE ID=' . $foo);
				default:
					return $foo;
			}
		}
		return '';
	}

	protected function fieldNameNotValid(): string{
		$sort = $this->getElementA('we_sort');
		if(empty($sort)){
			return '';
		}
		foreach(array_keys($sort) as $elem){
			$name = $this->getElementS('wholename' . $elem);
			if(we_database_base::isReservedField($name)){
				return $name;
			}
		}
		return '';
	}

	public function checkFieldsOnSave(): string{
		if(($n = $this->hasDoubbleFieldNames())){
			return sprintf(g_l('weEditor', '[doubble_field_alert]'), $n);
		}

		if(($n = $this->fieldNameNotValid())){
			return sprintf(g_l('weEditor', '[reserved_field_alert]'), $n);
		}

		if($this->triggerIdNotValdid()){
			return g_l('weEditor', '[triggerIdNotValid]');
		}
		if($this->triggerDocNotDynamic()){
			return g_l('weEditor', '[triggerDocNotDynamic]');
		}

		return parent::checkFieldsOnSave();
	}

	private function triggerIdNotValdid(): bool{
		return $this->DefaultUrl && (!$this->DefaultTriggerID || !fB('SELECT 1 FROM ' . FILE_TABLE . ' WHERE ID=' . intval($this->DefaultTriggerID) . ' AND IsFolder=0'));
	}

	private function triggerDocNotDynamic(): bool{
		return $this->DefaultUrl && (!fB('SELECT 1 FROM ' . FILE_TABLE . ' WHERE ID=' . intval($this->DefaultTriggerID) . ' AND IsFolder=0 AND IsDynamic=1'));
	}

	private function includedObjectHasDoubbleFieldNames(int $incClass): array{
		$sort = $this->getElementA('we_sort');
		$usedNames = $doubleNames = [];
		foreach(array_keys($sort) as $elem){
			$usedNames[] = $this->getElementS($this->getElementS('wholename' . $elem) . self::NAME_DIVIDER . 'name');
		}

		$incclassobj = new we_contents_object($incClass);
		$isort = $incclassobj->getElementA('we_sort');
		foreach(array_keys($isort) as $elem){
			$foo = $incclassobj->getElementS($incclassobj->getElementS('wholename' . $elem) . self::NAME_DIVIDER . 'name');
			if(in_array($foo, $usedNames)){
				$doubleNames[] = $foo;
			}
		}
		return $doubleNames;
	}

	protected function writeDocument(): bool{
		return true; // we don't have to write!
	}

	protected function setElementsFromHTTP(): void{
		if(!$_REQUEST){
			return;
		}

		$dates = we_base_util::convertDateInRequest('');
		foreach($dates as $n => $v){
			'@phan-var DateTime $v';
			assert($v instanceof DateTime);
			$this->setElementI((string) $n, (int) $v->format('U'));
		}

		// move to contents_object?
		foreach($_REQUEST as $n => $values){
			//($type = count(($nParts = explode('_', $n))) > 1 ? array_pop($nParts) : '');

			if(preg_match('/^we_' . preg_quote($this->Name) . '_([^\[]+)$/', $n) &&
					is_array($values) && ($type = count(($nParts = explode('_', $n))) > 1 ? array_pop($nParts) : '') &&
					in_array($type, array_merge([self::TYPE_GENERAL], array_keys(self::DEFAULTS_VALUES)))){
				$elements = array_merge(self::GENERAL_ELEMENTS, self::COMMON_DEFAULTS_VALUES, (self::DEFAULTS_VALUES[$type]['defaults'] ?? []));
				foreach($values as $name => $value){
					'@phan-var string $name';
					$regs = [];
					$elements = array_merge(self::GENERAL_ELEMENTS, // FIXME: make function to get these...
							self::COMMON_DEFAULTS_VALUES,
							(self::DEFAULTS_VALUES[$type]['defaults'] ?? []),
							(self::DEFAULTS_VALUES[$type]['additionals'] ?? [])
					);

					$elemGeneral = $elemTyped = [];
					if(($type === self::TYPE_GENERAL && !empty(($elemGeneral = self::GENERAL_ELEMENTS[$name] ?? []))) ||
							preg_match('/(.+)' . self::NAME_DIVIDER . '(.+)/', $name, $regs) && !empty(($elemTyped = $elements[$regs[2]] ?? []))){

						$element = $elemGeneral ?: ($elemTyped ?: ['', '']); // due to phan...

						switch($element[1]){
							case we_base_request::BOOL:
								$val = we_base_request::bool($n, false, $name);
								$this->setElementB($name, $val);
								break;
							case we_base_request::INT:
								$val = we_base_request::int($n, 0, $name);
								$this->setElementI($name, $val);
								break;
							case we_base_request::FLOAT:
								$val = we_base_request::float($n, 0, $name);
								$this->setElementF($name, $val);
								break;
							case we_base_request::STRING_LIST:
								if($element[2] ?? false){ // do not apply array_filter
									$v = [];
									assert(is_array($value));
									foreach(array_keys($value) as $i){
										$v[] = we_base_request::string($n, '', $name, $i);
									}
									$this->setElementA($name, $v);
									break;
								}
								$this->setElementA($name, we_base_request::stringList($n, [], $name));
								break;
							case we_base_request::INTLISTA:
								if($element[2] ?? false){ // do not apply array_filter
									$v = [];
									assert(is_array($value));
									foreach(array_keys($value) as $i){
										$v[] = we_base_request::int($n, 0, $name, $i);
									}
									$this->setElementA($name, $v);
									break;
								}
								$this->setElementA($name, we_base_request::intList($n, [], $name));
								break;
							case 'arrayHref':
								$defaultField = self::DEFAULTS_VALUES[self::TYPE_HREF]['defField'];
								$default = self::DEFAULTS_VALUES[self::TYPE_HREF]['defaults'][$defaultField][0];
								$arr = [
									'int' => we_base_request::bool($n, $default['int'], $name, 'int'),
									'intID' => we_base_request::int($n, $default['intID'], $name, 'intID'),
									'extPath' => we_base_request::_(we_base_request::URL, $n, $default['extPath'], $name, 'extPath'),
								];
								$this->setElementA($name, $arr);
								break;
							case 'arrayLink':
								// links are not processed here!
								t_e('we_contents_objectBase::setElementsFromHTTP: unexpected arrayLink request?', $name);
								break;
							case we_base_request::HTML:
								$this->setElementS($name, we_base_request::_(we_base_request::WYSIWYG, $n, '', $name));
							case we_base_request::STRING:
							default:
								$this->setElementS($name, we_base_request::string($n, '', $name));
								break;
						}
						unset($_REQUEST[$n][$name]);
					}
				}
			}
		}

		parent::setElementsFromHTTP();
	}

	public function we_save(bool $resave = false, bool $skipHook = false): bool{
		if(!$this->save()){
			return false;
		}

		if(!$resave){
			we_base_history::insertIntoHistory($this);
		}
		/* hook */
		$this->unregisterMediaLinks();
		$this->registerMediaLinks(true, false);
		if($skipHook){
			return true;
		}
		$hook = new we_hook_handler('save', ['obj' => $this, 'resave' => $resave]);
		//check if doc should be saved
		if(!$hook->executeHook()){
			$this->errMsg = $hook->getErrorString();
			return false;
		}

		//check for warnings and notices
		if(($tmp = $hook->getWarningString())){
			$this->warningMsg[] = $tmp;
		}
		if(($tmp = $hook->getNoticeString())){
			$this->noticeMsg[] = $tmp;
		}

		return true;
	}

	function registerMediaLinks(bool $temp = false, bool $linksReady = false): bool{// FIXME: publish is obsolete for classes
		foreach($this->DefaultValues as $k => $v){
			if(count(($typeName = explode('_', $k, 2))) > 1){
				switch($typeName[0]){
					case 'binary':
					case 'img':
						if(!empty($v['default'])){
							$this->MediaLinks[$typeName[0] . '[name=' . $typeName[1] . ']'] = $v['default'];
						}
						break;
					case 'link':
						$defaultField = self::DEFAULTS_VALUES[self::TYPE_LINK]['defField'];
						if(($default = $v[$defaultField] ?? [])){
							if($default['type'] === 'int' && $default['id']){
								$this->MediaLinks[$typeName[0] . '[name=' . $typeName[1] . ']'] = $default['id'];
							}
							if($default['ctype'] === 'int' && $default['img_id']){
								$this->MediaLinks[$typeName[0] . '[name=' . $typeName[1] . ']'] = $default['img_id'];
							}
						}
						break;
					case 'text':
						if(!empty($v['wysiwyg'])){
							$v['default'] = (string) ($v['default'] ?? '');
							$this->MediaLinks = array_merge($this->MediaLinks, we_wysiwyg_editor::getAllMediaLinks($v['default'], $typeName[1]));
						}
						break;
					default:
					//
				}
			}
		}

		return parent::registerMediaLinks(false, true);
	}

	/**
	 * @return	boolean if the field can have variants the function returns true otherwise false
	 * @param	$field - the name of the field
	 */
	function isVariantField(string $field): bool{
		$type = ($this->getElementS($field . self::NAME_DIVIDER . self::ELEMENT_TYPE) ) ?: '';
		return $type && in_array($type, self::VARIANT_TYPES);
	}

	/**
	 * @return	boolean the function returns the number of variant fields
	 */
	function hasVariantFields(): bool{
		return !empty($this->getVariantFields());
	}

	/**
	 * if document can have variants the function returns true, otherwise false
	 *
	 * if paramter checkField is true, this function checks also, if there are
	 * already fields selected for the variants.
	 *
	 * @param boolean $checkFields
	 * @return boolean
	 */
	function canHaveVariants(bool $checkFields = false): bool{
		return true;
	}

	private function loadDefaultsValues(): void{
		$this->DefaultValues = $this->DefaultValues ?: we_unserialize(fS('SELECT DefaultValues FROM ' . OBJECT_TABLE . ' WHERE ID=' . $this->ID));
	}

	/**
	 * @desc 	the function returns the array with all object field names
	 * @return	array<string,array{type:string,content:string,attributes:array}> with the field name and attributes
	 */
	function getAllVariantFields(): array{
		$return = [];
		$this->loadDefaultsValues();
		$fields = $this->DefaultValues;
		'@phan-var array<string,array{type:string,content:string,attributes:array}> $fields';
		foreach($fields as $name => $field){
			if($this->isVariantField($name)){
				$return[$name] = $field;
			}
		}
		return $return;
	}

	/**
	 * @return array<string,array{type:string,content:string,attributes:array}> with the field names and attributes
	 */
	function getVariantFields(): array{
		$return = [];
		$fields = $this->getAllVariantFields();
		foreach($fields as $name => $field){
			if(!empty($field['variant'])){
				$return[$name] = $field;
			}
		}
		return $return;
	}

	/* creates the DirectoryChoooser field with the 'browse'-Button. Clicking on the Button opens the fileselector */

	protected function formDirChooser(int $rootDirID = 0, string $table = '', string $Pathname = 'ParentPath', string $IDName = 'ParentID', string $cmd = '', /* mixed */ $label = true, bool $disabled = false, int $pathID = 0, string $identifier = '', we_base_jsCmd $jsCmd = null): string{
		$path = id2path($pathID);

		if(!$table){
			$table = $this->Table;
		}
		$textname = 'we_' . $this->Name . '_' . $Pathname . ($identifier ? '_' . $identifier : '');
		$idname = 'we_' . $this->Name . '_' . $IDName;

		// Important: custom callback commands must call we_cmd('pathOfDocumentChanged') at the beginning
		$cmd = $cmd ?: 'pathOfDocumentChanged';

		// Still the old problem: after dynamically loading single fields suggestor is not automatically initialised
		// Activate when replace fields is based on rpc
		$weSuggest = we_gui_suggest::getInstance();
		return $weSuggest->setAcId($idname . we_base_file::getUniqueId())
						->setContentType(array_merge(we_base_ContentTypes::LINK_ABLE, [we_base_ContentTypes::FOLDER_DOCUMENT]))
						->setInput($textname, $path, ['onchange' => $cmd])
						->setMaxResults(10)
						->setResult($idname, (string) $pathID)
						->setSelector(we_gui_suggest::DocSelector)
						->setTable(FILE_TABLE)
						->setRequired(false)
						->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_directory',{id:document.we_form.elements['" . $idname . "'].value,table:'" . $table . "',targetID:'" . $idname . "',targetName:'" . $textname . "',jsCmd:'" . $cmd . "',rootDirID:'" . $rootDirID . "'})"))
						->getHTML();
	}

	function userCanSave(bool $ctConditionOk = false): bool{
		if(we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR])){
			return true;
		}
		return we_base_permission::hasPerm([we_base_perm::OBJECTS_CAN_SEE, we_base_perm::OBJECT_NEW]) && parent::userCanSave() && !we_users_util::isUserInUsers(we_users_user::getID(), array_filter($this->OwnersReadOnly));
	}

	static function isUsedByObjectFile(int $id): bool{
		return $id && fB('SELECT 1 FROM ' . OBJECT_FILES_TABLE . ' WHERE IsClassFolder=0 AND TableID=' . $id . ' LIMIT 1');
	}

	/**
	 * @return bool
	 * @desc	checks if the user has the right to see an objectfile
	 */
	function userHasPerms(): bool{
		return (we_base_permission::hasPerm([we_base_perm::OBJECTS_CAN_SEE]) && (!$this->RestrictOwners) || we_users_util::isOwner($this->Owners) || we_users_util::isOwner([
					$this->CreatorID]));
	}

	public function getPropertyPage(we_base_jsCmd $jsCmd): string{
		$parts = new we_html_multiIconBoxContent();
		switch($this->EditPageNr){
			default:
				$parts
						->add(new we_html_multiIconBoxContentLine(g_l('weClass', '[path]'), $this->formPath(), we_html_multiIconBox::PROP_PATH))
						->add(new we_html_multiIconBoxContentLine(g_l('modules_object', '[default]'), $this->formDefault(), we_html_multiIconBox::PROP_ATTRIB))
						->add((new we_html_multiIconBoxContentLine(g_l('modules_object', '[name]'), $this->formDefaultText(), we_html_multiIconBox::PROP_AUTONAME))->fold())
						->add((new we_html_multiIconBoxContentLine(g_l('modules_object', '[seourl]'), $this->formDefaultUrl(), we_html_multiIconBox::PROP_SEO))->fold())
						->add(we_modules_info::isActive(we_modules_info::CATEGORY) ?
										(new we_html_multiIconBoxContentLine(g_l('global', '[categorys]'), $this->formCategory($jsCmd), we_html_multiIconBox::PROP_CATEGORIES))->fold() : null)
						->add((new we_html_multiIconBoxContentLine(g_l('weClass', '[owners]'), $this->formRestrictOwners() . '<br/>' .
										($this->RestrictOwners ? $this->formOwners($jsCmd) : ''), we_html_multiIconBox::PROP_USER))->fold())
						->add(SIDEBAR_DISABLED ? null : (new we_html_multiIconBoxContentLine(g_l('prefs', '[sidebar]'), $this->formSelectDoc($this->SidebarDocID, 'we_' . $this->Name . '_SidebarDocID', ''), we_html_multiIconBox::PROP_SIDEBAR)))
						->add((new we_html_multiIconBoxContentLine(g_l('weClass', '[CSS]'), $this->formCSS($jsCmd), we_html_multiIconBox::PROP_OTHER))->fold())
						->add((new we_html_multiIconBoxContentLine(g_l('modules_object', '[copyClass]'), $this->formCopyDocument(), we_html_multiIconBox::PROP_COPY)))
				;
				break;
			case we_base_constants::WE_EDITPAGE_WORKSPACE:
				$parts
						->add((new we_html_multiIconBoxContentLine(g_l('weClass', '[workspaces]'), $this->formWorkspaces($jsCmd), we_html_multiIconBox::PROP_WORKSPACE)))
						->add((new we_html_multiIconBoxContentLine(g_l('modules_object', '[behaviour]'), $this->formWorkspacesFlag(), we_html_multiIconBox::PROP_DISPLAY)));
		}
		return (new we_html_multiIconBox($parts, 'PropertyPage'))->marginLeft()->get();
	}

	public static function isObjectXTable(string $table): int{
		$match = [];
		return (preg_match('|^' . OBJECT_X_TABLE . '([0-9]*)$|i', $table, $match) ? ((int) $match[1]) : 0);
	}

	public function toXML(array $attribs = [], array $extraSlots = [], string $content = ''): string{
		$tableInfo = $this->DB_WE->metadata(OBJECT_X_TABLE . $this->ID);
		$defvalues = $this->DefaultValues;
		foreach($tableInfo as $cur){
			$fieldname = $cur['name'];
			if(isset($defvalues[$fieldname])){
				$defvalues[$fieldname]['length'] = ($cur['len'] > 255) ? 255 : $cur['len'];
			}
		}
		$this->DefaultValues = $defvalues;

		return parent::toXML($attribs, $extraSlots);
	}

	function publInScheduleTable(): ?DateTime{
		return null;
	}

	public function getPersistent(): array{
		return $this->persistent_slots;
	}

}
