<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 468def12ca8d $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2021-12-02 16:12 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
/* * this is the abstract super class for DB connections */

if(!defined('DB_DATABASE')){
	file_exists(WE_DOCUMENT_ROOT . '/webEdition/custom/conf/we_conf.inc.php') ? //FIXME: remove in cardada
			require_once(WE_DOCUMENT_ROOT . '/webEdition/custom/conf/we_conf.inc.php') :
			require_once(WE_DOCUMENT_ROOT . '/webEdition/we/include/conf/we_conf.inc.php');
}

abstract class we_database_base implements Serializable{
	private const ENGINES = [
		'MYISAM' => 'myisam',
		'ARIA' => 'aria'
	];
	public const SERVER_MARIA = 'MariaDB';
	public const SERVER_MYSQL = 'MySQL';
	public const DB_COLLATION = 'utf8_unicode_ci';
	public const DB_CHARSET = 'utf8';
	public const DB_CHARSET_LEN = 3;
	const META_SHORT = 'short';
	const META_FULL = 'all';
	const META_NAME = 'name';
	/* database field types */
	const TYPE_TINYINT = 'tinyint';
	const TYPE_SMALLINT = 'smallint';
	const TYPE_INT = 'int';
	const TYPE_MEDIUMINT = 'mediumint';
	const TYPE_BIGINT = 'bigint';
	const TYPE_FLOAT = 'float';
	const TYPE_DOUBLE = 'double';
	const TYPE_NULL = 'null';
	const TYPE_TIMESTAMP = 'timestamp';
	const TYPE_DATE = 'date';
	const TYPE_TIME = 'time';
	const TYPE_DATETIME = 'datetime';
	const TYPE_YEAR = 'year';
	const TYPE_BIT = 'bit';
	const TYPE_DECIMAL = 'decimal';
	const TYPE_ENUM = 'enum';
	const TYPE_SET = 'set';
	const TYPE_TINYBLOB = 'tinyblob';
	const TYPE_TINYTEXT = 'tinytext';
	const TYPE_MEDIUMBLOB = 'mediumblob';
	const TYPE_MEDIUMTEXT = 'mediumtext';
	const TYPE_BLOB = 'blob';
	const TYPE_TEXT = 'text';
	const TYPE_LONGBLOB = 'longblob';
	const TYPE_LONGTEXT = 'longtext';
	const TYPE_VARBIN = 'varbinary';
	const TYPE_VARCHAR = 'varchar';
	const TYPE_BIN = 'binary';
	const TYPE_CHAR = 'char';
	const TYPE_GEOMETRY = 'geometry';
	const TYPE_SHORT_NUMBER = 'number';
	const TYPE_SHORT_TIME = 'time';
	const TYPE_SHORT_DATE = 'date';
	const TYPE_SHORT_DATETIME = 'datetime';
	const TYPE_SHORT_STRING = 'string';
	const TYPE_SHORT_BINARY = 'binary';
	public const DATE_TIME_FORMAT = 'Y-m-d H:i:s';
	public const DATE_FORMAT = 'Y-m-d';
	const RESERVED_WORDS = [
		'add', 'all', 'alter', 'asc', 'by', 'change', 'character', 'collate', 'column', 'constraint', 'default', 'delayed',
		'desc', 'disable', 'distinct', 'distinctrow', 'drop', 'dumpfile', 'duplicate', 'enable', 'enum', 'first', 'for',
		'foreign', 'from', 'fulltext', 'grant', 'group', 'group', 'having', 'high_priority', 'id', 'ignore', 'in', 'index',
		'insert', 'into', 'key', 'limit', 'lock', 'low_priority', 'mode', 'modify', 'none', 'on', 'option', 'order', 'order',
		'outfile', 'password', 'primary', 'privileges', 'procedure', 'quick', 'rename', 'replace', 'require', 'revoke', 'rollup',
		'select', 'set', 'share', 'sql_buffer_result', 'sql_cache', 'sql_cals_found_rows', 'sql_no_cache', 'sql_small_result',
		'straight_join', 'table', 'to', 'truncate', 'union', 'unique', 'update', 'using', 'values', 'where', 'where', 'with',
	];

	/** @var DB_WE */
	public static $inst;

	/** @var mysqli[] */
	private static $pool = [];

	/** @var int */
	protected static $conCount = 0;

	/** @var int */
	protected static $linkCount = 0;
	//states if we have lost connection and try again

	/** @var mysqli|null link handles */
	protected $Link_ID = null;

	/** @var int query handles */
	protected $Query_ID = 0;

	/** @var int */
	private $Insert_ID = 0;

	/** @var int */
	private $Affected_Rows = 0;

	/** @var array<string|int,mixed> */
	protected $Record = [];

	/** @var int current row number */
	public $Row = 0;

	/** @var int current error number */
	public $Errno = 0;
	/*	 * current Error text */

	/** @var string current Error text */
	public $Error = '';

	/** @var string[] */
	protected $Warnings = [];

	/** @var bool true, if a temporary table was created */
	private $hasTempTable = false;

	/** public: connection parameters */
	protected $Database = DB_DATABASE;

	/** @var int */
	private static $Trigger_cnt = 0;

	/** @var string */
	private static $defaultEngine = '';

	/** Connects to the database, which this is done by the constructor */
	abstract protected function connect(string $Database = DB_DATABASE, string $Host = DB_HOST, string $User = DB_USER, string $Password = DB_PASSWORD): bool;

	/** internal query
	 * @param $Query_String string the sql statement
	 * @param $unbuffered bool if this query is executed buffered or unbuffered
	 * @return int a query_id
	 */
	abstract protected function _query(string $Query_String, bool $unbuffered = false);

	abstract protected function _setCharset(string $charset): void;
	/** internal free function */

	/** internal free function */
	abstract protected function _free(): void;

	/** set error / Warnings
	 * @return string error
	 */
	abstract protected function setError(): void;

	/** get info of latest executed query
	 * @return String
	 */
	abstract protected function info();

	/** internal seek to a specific position in result-set
	 * @return bool if seek was successfull
	 */
	abstract protected function _seek(int $pos = 0): bool;

	/** internal fetch the data from DB
	 * @param $resultType int (MYSQLI_BOTH,MYSQLI_ASSOC,MYSQLI_NUM)
	 * @return array depending on resulttype
	 */
	abstract protected function fetch_array(int $resultType): ?array;

	/** internal check if Database is still responding
	 * @return true if DB is still present
	 */
	abstract protected function ping(): bool;

	/** get the last inserted ID
	 * @return int last generated id of the insert-statement
	 */
	abstract protected function _getInsertId(): int;

	/** get the no of rows in the resultset
	 * @return int row count
	 */
	abstract public function num_rows(): int;

	/** get the count of fields in the result-set
	 * @return int no of fields
	 */
	abstract public function num_fields(): int;

	/** get the name of the field no
	 * @param $no int field no
	 * @return string orignal name of the field
	 */
	abstract public function field_name(int $no): string;

	/** get the type of the field no
	 * @param $no int field no
	 * @return string type of the field
	 */
	abstract public function field_type(int $no): string;

	/** get the type of the field no
	 * @param $no int field no
	 * @return string type of the field
	 */
	abstract public function field_type_short(int $no): string;

	/** get the name of the table field no is located
	 * @param $no int field no
	 * @return string orignal table location of the field
	 */
	abstract public function field_table(int $no): string;

	/** get the lenght of the field no
	 * @param $no int field no
	 * @return int length inside the tabledef. of the field
	 */
	abstract public function field_len(int $no): int;

	/** get the flags of the field no
	 * @param $no int field no
	 * @return int field flags as int
	 */
	abstract public function field_flags(int $no): int;

	abstract public function field_default(int $no): string;

	/** close the DB connection
	 *
	 */
	abstract public function close(): void;

	/** get the no of rows that were affected by update/delete/replace
	 * @return int count of rows
	 */
	abstract protected function _affected_rows();

	/** get Information about the used driver etc.
	 * @return array<string,string> containing all information
	 */
	abstract public function _getInfo();

	/** returns the charset of the current connection */
	abstract public function _getCurrentCharset(): string;

	/** translates the flags-bit-field to human readable string */
	abstract public function flags2txt(int $flags_num): string;

	public function __construct(string $Database = DB_DATABASE){
		$this->Database = $Database;
		//make lazy connections, only the first one is executed instantly
		if(!self::$linkCount){
			we_cache_db::setup();
			if(!$this->_connect()){
				if(file_exists(WE_CUSTOM_PATH . '/maintenance.inc.php')){
					echo include(WE_CUSTOM_PATH . '/maintenance.inc.php');
				} else {
					echo (empty($GLOBALS['we']['ERROR']) ? '' : 'Problem Details:<br/>' . $GLOBALS['we']['ERROR']);
				}
				exit('Database error');
			}
			//make the first connection reuseable
			$this->repool();
		}
		self::$conCount++;
	}

	/**
	 * Fill connection pool
	 */
	public function __destruct(){
		$this->free();
	}

	public static function getStatic(): DB_WE{
		if(!self::$inst){
			self::$inst = new DB_WE();
		}
		return self::$inst;
	}

	private function repool(): void{
		if($this->Link_ID && $this->Database === DB_DATABASE && !$this->hasTempTable){
			self::$pool[] = $this->Link_ID;
			$this->Link_ID = null;
		}
	}

	/**
	 * internal connect which uses Connection Pools
	 * @return bool
	 */
	protected function _connect(): bool{
		$this->Link_ID = ($this->Database === DB_DATABASE ? (array_pop(self::$pool) ?: null) : null);
		if(!$this->isConnected(false)){
			if(!$this->connect($this->Database)){
				return false;
			}
		}
		return $this->Link_ID ? true : false;
	}

	/**
	 * Only for debug
	 */
	public static function showStat(): void{
		echo 'tried connections: ' . self::$conCount . '<br/>' . 'real connections: ' . self::$linkCount;
	}

	/**
	 * This function is a replacement for mysql_real_escape_string, which sends the string to mysql to escape it
	 * @param string|float|int|bool $inp value to escape for sql-query
	 * @return string
	 */
	public static function escape_sql(/* mixed */ $inp): string{
		if(is_array($inp)){
			throw new we_exception_base('Escapping arrays is unsupported!');
		}

		return ($inp && is_string($inp) ?
			preg_replace('/[\x00-\x08\x10\x0B\x0C\x0E-\x19\x7F]' . //fix for utf-8 only chars
				'|[\x00-\x7F][\x80-\xBF]+' .
				'|([\xC0\xC1]|[\xF0-\xFF])[\x80-\xBF]*' .
				'|[\xC2-\xDF]((?![\x80-\xBF])|[\x80-\xBF]{2,})' .
				'|[\xE0-\xEF](([\x80-\xBF](?![\x80-\xBF]))|(?![\x80-\xBF]{2})|[\x80-\xBF]{3,})/S',
								'?', strtr($inp, [
				'\\' => '\\\\',
				"\0" => '\\0',
				"\n" => '\\n',
				"\r" => '\\r',
				"'" => "\\'",
				'"' => '\\"',
				"\x1a" => '\\Z'
			])) :
			(string) $inp);
	}

	/**
	 * This function is a replacement for mysql_real_escape_string, which sends the string to mysql to escape it
	 * @param string|float|int|bool $inp value to escape for sql-query
	 * @return string
	 */
	public function escape(/* mixed */ $inp): string{
		return self::escape_sql($inp);
	}

	/** check the connection
	 *
	 * @return bool true, if the DB is connected
	 */
	protected function isConnected(bool $withPing = true): bool{
		return ($this->Link_ID !== null) && ($withPing ? $this->ping() : true);
	}

	private static function checkForUnionStatement(string $Query_String, bool $allowUnion): bool{
		if(!$allowUnion && stripos($Query_String, 'union') !== false || str_contains($Query_String, '/*!')){
			$queryToCheck = str_replace(['\\\\'/* escape for mysql connection */, '\\"', "\\'", '\\\`'], '', $Query_String);
			$quotes = ['\'' => false, '"' => false, '`' => false, '/*' => false, '#' => false];
			$queryWithoutStrings = '';

			for($i = 0; $i < strlen($queryToCheck); $i++){
				$char = $queryToCheck[$i];
				$active = array_filter($quotes);
				$active = !empty($active);
				switch($char){
					case '/':
						if(!$active && $queryToCheck[$i + 1] === '*'){
							if($queryToCheck[$i + 2] === '!'){/* mysql specific code */
								if(defined('ERROR_LOG_TABLE')){
									t_e('error', 'No MySQL specific syntax allowed!', $Query_String);
								}
								//be quiet, no need to give more information
								return false;
							}
							$quotes['/*'] = true;
							$i++;
							continue 2;
						}
						break;
					case '*':
						if($quotes['/*'] && $queryToCheck[$i + 1] === '/'){//no nested comments
							$quotes['/*'] = false;
							$i++;
							continue 2;
						}
						break;
					case "\n":
						if($active && $quotes['#']){
							$quotes['#'] = false;
						}
					case '-':
						if(!$active && $queryToCheck[$i + 1] === '-'){
							$quotes['#'] = true;
							$active = true;
							$i++;
							continue 2;
						}
						break;
					case '#':
						if(!$active){
							$quotes['#'] = true;
							$active = true;
							continue 2;
						}
						break;
					case '"':
					case '`':
					case '\'':
						if(($active && $quotes[$char]) || !$active){//if active close only corresponding pair
							$quotes[$char] = !$quotes[$char];
							$active = true;
						}
						break;
				}
				if(!$active){
					$queryWithoutStrings .= $char;
				}
			}
			if(!$allowUnion && stripos($queryWithoutStrings, 'union') !== false || str_contains($queryWithoutStrings, '/*') || str_contains($queryWithoutStrings, '*/')){
				if(defined('ERROR_LOG_TABLE')){
					t_e('error', 'Attempt to execute union statement/injection', $Query_String);
				}
				//be quiet, no need to give more information
				return false;
			}
		}
		return true;
	}

	/**
	 * make an sql-Query to the DB
	 * @param string $Query_String the sql-query
	 * @param bool $allowUnion this parameter is deprecated; it determines if the query is allowed to have unions
	 * @return bool true, if the query was successfull
	 */
	public function query(string $Query_String, bool $allowUnion = false, bool $unbuffered = false, bool $isRetry = false): bool{
		if(!$Query_String){
			return true;
		}
		$slowQuery = defined('DEBUG_SLOW_QUERY');

		$time = (self::$Trigger_cnt || $slowQuery ? microtime(true) : 0);

		if(!$isRetry){
			$this->Errno = 0;
			$this->Error = '';
			$this->Row = 0;
		}
# New query, discard previous result.
		if($this->Query_ID){
			$this->free();
		}

		if(!$this->isConnected(false) && !$this->_connect()){
			return false;
		}

		$repool = false;
// check for union This is the fastest check
// if union is found in query, then take a closer look
		if(!self::checkForUnionStatement($Query_String, $allowUnion)){
			return false;
		}
		static $date = 0;
		$date = $date ?: date('Y-m-d');

		$isSelect = stripos($Query_String, 'select') === 0;
		//FIX for current MySQL Versions which do not cache queries with dates
		/* if($isSelect){
		  $Query_String = str_replace(['CURDATE()', 'CURRENT_DATE()'], '"' . $date . '"', $Query_String);
		  } */
		$this->Row = 0;
		$this->Query_ID = $this->_query($Query_String, $unbuffered);
		$this->Insert_ID = $this->_getInsertId();
		$this->Affected_Rows = $this->_affected_rows();
		$this->setError();

		if(!$this->Errno){
			if(preg_match('/alter\stable|drop\stable|truncate\stable/i', $Query_String)){
				$this->_query('FLUSH TABLES');
				$this->hasTempTable = false;
				we_cache_db::clean($Query_String);
				$repool = true;
			} elseif(preg_match('/insert\s|delete\s|update\s|replace\s/i', $Query_String)){
				//don't kill query cache if we add errors
				if(!str_contains($Query_String, defined('ERROR_LOG_TABLE') ? ERROR_LOG_TABLE : 'ERROR_LOG_TABLE') || (stripos($Query_String, 'DELETE') === 0)){
// delete getHash DB Cache
					we_cache_db::clean($Query_String);
				}
				$repool = true;
			} elseif(preg_match('/CREATE TEMPORARY/i', $Query_String)){
				//we have to keep this DB connection with that temp table, since tables are connection dependend
				$this->hasTempTable = true;
				$repool = false;
			}

			$matches = [];
			if(preg_match('/^[[:space:]]*alter[[:space:]]*table[[:space:]]*(`?([[:alpha:]]|[[:punct:]])+`?)[[:space:]]*(add|change|modify|drop)/i', $Query_String, $matches)){
				$this->_query('ANALYZE TABLE `' . $matches[1] . '`');
				$repool = true;
			}
		}
		if($slowQuery || self::$Trigger_cnt){
			$time = microtime(true) - $time;
			$slowQuery = $slowQuery ? ($time > 1) : false;
		}
		if((self::$Trigger_cnt || $slowQuery) && (!defined('ERROR_LOG_TABLE') || !str_contains($Query_String, ERROR_LOG_TABLE))){
			$reason = $slowQuery ? 'SLOW QUERY detected' : 'Trigger ' . self::$Trigger_cnt;
			if(self::$Trigger_cnt){
				--self::$Trigger_cnt;
			}
			$tmp = [
				'time' => $time,
				'trigger' => self::$Trigger_cnt,
				'errno' => $this->Errno,
				'error' => $this->Error,
				'affected' => $this->_affected_rows(),
				'rows' => $this->num_rows(),
				'explain' => []
			];
			if($isSelect){
				$this->Query_ID = $this->_query('EXPLAIN ' . $Query_String);

				while($this->next_record()){
					if(empty($tmp['explain'])){
						$tmp['explain'][] = implode(' | ', array_keys($this->Record));
					}
					$tmp['explain'][] = implode(' | ', $this->Record);
				}
				$this->Row = 0;
				!$this->isConnected(false) && !$this->_connect();
				$this->Query_ID = $this->_query($Query_String, $unbuffered);
			}
			t_e($reason, $Query_String, $tmp);
		}

		if($this->Warnings){
			trigger_error('MYSQL-WARNING' . "\n" . implode("\n", $this->Warnings) . "\n" . $Query_String);
		}

		switch($this->Errno){
			case 1062://ignore as error - duplicate entry
				if($repool){
					$this->repool();
				}
				return false;
			case 0:// ignore this
				if($repool){
					$this->repool();
				}
				return true;
			case 2006://SERVER_GONE_ERROR
			case 2013://SERVER_LOST
				if(!$isRetry){
					$this->Link_ID = null;
					$tmp = $this->query($Query_String, $allowUnion, $unbuffered, true);
					return $tmp;
				}
			default:
				$this->free();
				trigger_error('MYSQL-ERROR' . "\nFehler: " . $this->Errno . "\nDetail: " . $this->Error . "\nInfo:" . $this->info() . "\nQuery: " . $Query_String, E_USER_WARNING);
				//error_log('MYSQL-ERROR - Fehler: ' . $this->Errno . ' Detail: ' . $this->Error . ' Query: ' . $Query_String);
		}


		if($repool){
			$this->repool();
		}

		return false;
	}

	/* discard the query result */

	public function free(): void{
		$this->_free();
		$this->hasTempTable = false;
		$this->repool();
		$this->Query_ID = 0;
		$this->Record = [];
	}

	/**
	 * @deprecated since version barrhorn
	 * shorthand notation for num_rows */
	public function nf(): int{
		return $this->num_fields();
	}

	/**
	 * get a value from a field, queried by a prequel query+next_record
	 * @param string $Name name/number of the field, depending on query-type
	 * @return mixed returns the value or '' if not present
	 */
	public function f(string $Name){
		return $this->Record[$Name] ?? '';
	}

	/**
	 * get an array with all field names from a previous query
	 * @return array in order of the query, all fields are added with their name, having one name per row
	 */
	public function fieldNames(): array{
		if(!($this->Query_ID)){
			return [];
		}
		$res = [];
		$count = $this->num_fields();
		for($i = 0; $i < $count; $i++){
			$res[$i] = $this->field_name($i);
		}
		return $res;
	}

	/**
	 * get all tables named (like)
	 * @param string $like if given, make a like query, without any %
	 * @return array<int,array{table_name:string,tablespace_name:string,database:string}> all tables (named like $like)
	 */
	public function table_names(string $like = ''): array{
		$this->query('SHOW TABLES' . (($like != '') ? ' LIKE "' . $like . '"' : ''));
		$return = [];
		while($this->next_record(MYSQLI_NUM)){
			$return[] = [
				'table_name' => $this->f('0'),
				'tablespace_name' => $this->Database,
				'database' => $this->Database];
		}
		return $return;
	}

	/** walk result set
	 * @param $resultType int
	 * @return bool true, if rows was successfully fetched
	 */
	public function next_record(int $resultType = MYSQLI_ASSOC): bool{
		if(!($this->Query_ID)){
			//t_e('next_record called with no query pending.');
			return false;
		}
		$res = $this->fetch_array($resultType);
		$this->Record = $res ?: [];
		$this->Row++;
		$this->setError();

		$stat = is_array($res);
		if(!$stat){
			$this->repool();
		}
		return $stat;
	}

	/** get result at positionset
	 * @param $pos int position in the result set
	 * @param $resultType int
	 * @return bool true, if rows was successfully fetched
	 */
	public function record(int $pos = 0, int $resultType = MYSQLI_BOTH): bool{
		if(!$this->seek($pos)){
			return false;
		}

		return $this->next_record($resultType);
	}

	/** set the position in result set
	 * @param $pos int seek to pos in result set
	 * @return bool true, if seek was successfull
	 */
	public function seek(int $pos = 0): bool{
		if(!$this->Query_ID){
			t_e('seek called with no query pending.');
			return false;
		}
		if($this->_seek($pos)){
			$this->Row = $pos;
			return true;
		}
		t_e("seek($pos) failed: result has " . $this->num_rows() . " rows");

		/* half assed attempt to save the day,
		 * but do not consider this documented or even
		 * desireable behaviour.
		 */
		$this->_seek($this->num_rows());
		$this->Row = $this->num_rows();
		return false;
	}

	/**
	 * Get complete result as array
	 * @return array<int,mixed,mixed[]>
	 * @deprecated since version 8.0.5, this function will be private soon
	 */
	public function getAll(bool $single = false, int $resultType = MYSQLI_ASSOC): array{
		$ret = [];
		while($this->next_record($resultType)){
			$ret[] = ($single ? current($this->Record) : $this->Record);
		}
		return $ret;
	}

	/**
	 * @param bool $useArray
	 * @param int $resultType
	 * @return array<int|string,mixed|mixed[]>
	 * @deprecated since version 8.0.5, this function will be private soon
	 */
	public function getAllFirst(bool $useArray = true, int $resultType = MYSQLI_NUM, $keyInt = false): array{
		$ret = [];
		while($this->next_record($resultType)){
			$key = array_shift($this->Record);
			$ret[$keyInt ? (int) $key : (string) $key] = ($useArray ? $this->Record : current($this->Record));
		}
		return $ret;
	}

	/**
	 * @param string $query
	 * @param bool $single
	 * @param int $resultType
	 * @return array<int,string|int|float|mixed[]>
	 */
	public function getAllq(string $query, bool $single = false, int $resultType = MYSQLI_ASSOC): array{
		$hash = 'All' . $resultType . $query;
		try{
			return we_cache_db::load($hash);
		} catch (we_exception_cache $ex){
			$this->query($query);
			$data = $this->getAll($single, $resultType); //@phan-suppress-current-line PhanDeprecatedFunction
			we_cache_db::save($hash, $data);
			return $data;
		}
	}

	/**
	 * @return array<int|string,int|float|string|mixed[]>
	 */
	public function getAllFirstq(string $query, bool $useArray = true, int $resultType = MYSQLI_NUM): array{
		$hash = 'First' . $resultType . $query;
		try{
			return we_cache_db::load($hash);
		} catch (we_exception_cache $ex){
			$this->query($query);
			$data = $this->getAllFirst($useArray, $resultType); //@phan-suppress-current-line PhanDeprecatedFunction
			we_cache_db::save($hash, $data);
			return $data;
		}
	}

	/**
	 * is a handy setter, for executing `a`="\"b\"" set from an assoc array
	 * @param $arr
	 * @param $forValues set to true if used in VALUES (...)
	 */
	static function arraySetter(array $arr, string $imp = ',', bool $forValues = false): string{
		$ret = [];
		foreach($arr as $key => $val){
			if($key === ''){
				continue;
			}
			$escape = !(is_bool($val) || is_int($val) || is_float($val));
			if(is_object($val) && ($val instanceof we_database_function)){
				$val = $val->get();
				$escape = false;
			} elseif($val === null){
				$escape = false;
				$val = 'NULL';
			} elseif(is_object($val) && $val instanceof DateTime){
				$val = $val->format(self::DATE_TIME_FORMAT);
			} elseif(is_object($val) || is_array($val)){
				$val = '';
				t_e('warning', 'data error: db-field cannot contain objects / arrays', 'Key: ' . $key, $arr);
			}

			$val = (is_bool($val) ? intval($val) : $val);
			//we must escape int-values since the value might be an enum element
			$ret[] = ($forValues ? '' : '`' . $key . '`=' ) . ($escape ? '"' . self::escape_sql((string) $val) . '"' : $val);
		}
		return ($forValues ? '(' : '') . implode($imp, $ret) . ($forValues ? ')' : '');
	}

	/** return table metadata
	 * @param string $table
	 * @return string[]
	 */
	public function metaFieldNames(string $table): array{
		//FIXME: this function has to be replaced by SHOW COLUMNS or by INFORMATION_SCHEMA COLUMNS Table
		//select * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_NAME="$table" AND TABLE_SCHEMA="DB_NAME"
		$res = [];
		$hash = 'metaData' . $table . self::META_NAME;
		if(!$table){
			throw new we_exception_database('No query specified.');
		}
		try{
			return we_cache_db::load($hash);
		} catch (Exception $ex){

		}
		if(!$this->query('SELECT * FROM `' . $table . '` LIMIT 1')){
			t_e('Metadata query failed.');
		}

		$count = $this->num_fields();
		if(!$count){
			trigger_error('MYSQL-ERROR' . "\n" . 'Error: Metadata-Query on table ' . $table . ' failed' . "\n", E_USER_WARNING);
		}

		for($i = 0; $i < $count; $i++){
			$res[$i] = $this->field_name($i);
		}

		we_cache_db::save($hash, $res, 100, true);
		$this->free();
		return $res;
	}

	/** return table metadata
	 * Due to compatibility problems with Table we changed the behavior
	 * of metadata();
	 * depending on $full, metadata returns the following values:
	 *
	 * - full is false (default):
	 * $result[]:
	 *   []['table']  table name
	 *   []['name']   field name
	 *   []['type']   field type
	 *   []['len']    field length
	 *   []['flags']  field flags
	 *
	 * - full is true
	 * $result[]:
	 *   ['num_fields'] number of metadata records
	 *   []['table']  table name
	 *   []['name']   field name
	 *   []['type']   field type
	 *   []['len']    field length
	 *   []['flags']  field flags
	 *   ['meta'][field name]  index of field named "field name"
	 *   The last one is used, if you have a field name, but no index.
	 *   Test:  if (isset($result['meta']['myfield'])) {
	 * @param string $table
	 * @return array<string,array{table:string,name:string,type:string,dbType:string,len:int,flags:int,default?:?mixed,values:array,nullable:bool,pos:int,type_short:string,flags_name:string}>
	 */
	public function metadata(string $table, string $field = self::META_SHORT): array{
		//FIXME: this function has to be replaced by SHOW COLUMNS or by INFORMATION_SCHEMA COLUMNS Table
		//select * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_NAME="$table" AND TABLE_SCHEMA="DB_NAME"
		$hash = 'metaData' . $table . $field;

		if(!$table){
			throw new we_exception_database('No query specified.');
		}
		try{
			return we_cache_db::load($hash);
		} catch (Exception $ex){

		}
		$defaults = ($field === self::META_FULL ?
			//adapt default values
			$this->getAllFirstq('SHOW COLUMNS FROM ' . self::escape_sql($table), true, MYSQLI_ASSOC) :
			[]);

		'@phan-var array<int,array{Field:string,Type:string,Null:string,Key:string,Default:string|null,Extra:string}> $defaults';
		if(!$this->query('SELECT * FROM `' . $table . '` LIMIT 1')){
			t_e('Metadata query failed.');
		}
		$count = $this->num_fields();
		if(!$count){
			trigger_error('MYSQL-ERROR' . "\n" . 'Error: Metadata-Query on table ' . $table . ' failed' . "\n", E_USER_WARNING);
		}
		$match = [];
		$res = [];
		'@phan-var array<string,array{table:string,name:string,type:string,dbType:string,len:int,flags:int,default?:?mixed,values:array,nullable:bool,pos:int,type_short:string,flags_name:string}> $res';
		for($i = 0; $i < $count; $i++){
			$name = $this->field_name($i);
			$flags = $this->field_flags($i);
			$default = ($field === self::META_FULL ? $defaults[$name]['Default'] ?? null : $this->field_default($i));
			if($field === self::META_FULL && ($flags & (MYSQLI_ENUM_FLAG | MYSQLI_SET_FLAG))){
				preg_match_all("/'(.*?)'/", $defaults[$name]['Type'], $match);
				$values = $match[1];
			} else {
				$values = [];
			}
			$res[$name] = [
				'table' => $this->field_table($i),
				'name' => $name,
				'type' => $this->field_type($i),
				'dbType' => ($field === self::META_FULL ? $defaults[$name]['Type'] : ''),
				'len' => $this->field_len($i),
				'flags' => $flags,
				'default' => $default,
				'values' => $values,
				'nullable' => !($flags & MYSQLI_NOT_NULL_FLAG),
				'pos' => $i,
				'type_short' => ($field === self::META_FULL ? $this->field_type_short($i) : ''),
				'flags_name' => ($field === self::META_FULL ? $this->flags2txt($flags) : ''),
			];
			if($field === self::META_FULL && $default === null && !($flags & MYSQLI_NOT_NULL_FLAG)){
				unset($res[$name]['default']);
			}
		}

		we_cache_db::save($hash, $res, 100, true);
		$this->free();
		return $res;
	}

	/*	 * checks if this DB connection with this user is allowed to lock a table */

	public function hasLock(): bool{
		static $lock = -1;
		if(is_bool($lock)){
			return $lock;
		}
//lock table
		$this->lock(VALIDATION_SERVICES_TABLE, 'read');
//if lock unavailable this will generate an error 1044 - access denied

		$lock = ($this->Errno == 0);
		$this->unlock();
		return $lock;
	}

	/**
	 * @param string|array $table specify the tables to lock; use numeric array to lock all tables with mode; use named array with [table]=mode to lock specific modes
	 * @param string $mode name the locking mode
	 * @return bool true, on success
	 */
	public function lock(/* mixed */ $table, string $mode = 'write'): bool{
		if(!$this->isConnected() && !$this->_connect()){
			return false;
		}
		if(is_array($table)){
			$query = [];
			foreach($table as $key => $value){
				$query[] = (is_numeric($key) ?
					$value . ' ' . $mode :
					$key . ' ' . $value);
			}
			$query = implode(',', $query);
		} else {
			$query = $table . ' ' . $mode;
		}
//always lock Errlog-Table
		if(!str_contains($query, ERROR_LOG_TABLE)){
			$query .= ',' . ERROR_LOG_TABLE . ' write';
		}

		$this->_query('lock tables ' . $query);
		return true;
	}

	/** Unlock all locked tables
	 * @return bool true, on success
	 */
	public function unlock(): bool{
		if(!$this->isConnected() && !$this->_connect()){
			return false;
		}
		return $this->_query('unlock tables') ? true : false;
	}

	/**
	 * get full query result
	 * @return array
	 */
	public function getRecord(): array{
		return $this->Record;
	}

	public function getInsertId(): int{
		return $this->Insert_ID;
	}

	public function affected_rows(): int{
		return $this->Affected_Rows;
	}

	/**
	 * Check if a colum exists in the specified table
	 * @param string $tab table to check
	 * @param string $col colum to check
	 * @return boolean true if exists
	 */
	public function isColExist(string $tab, string $col): bool{
		if(!$tab || !$col || !$this->isTabExist($tab)){
			return false;
		}
		$col = trim($col, '`');
		return (bool) count($this->getHash('SHOW COLUMNS FROM ' . self::escape_sql($tab) . ' LIKE "' . $this->escapeLikeString($col) . '"'));
	}

	/**
	 * check if a particular table exists
	 * @param string $tab table name
	 * @return boolean true if exists
	 */
	public function isTabExist(string $tab): bool{
		if(!$tab){
			return false;
		}
		$this->query('SHOW TABLES LIKE "' . $this->escapeLikeString($tab) . '"');
		return ($this->next_record());
	}

	/**
	 * @param string $tab
	 * @param array<string,string> $cols
	 * @param string[] $keys
	 * @return bool
	 */
	public function addTable(string $tab, array $cols, array $keys = [], string $engine = 'DEFAULT', bool $temporary = false): bool{
		if(empty($cols)){
			t_e('create table needs an array');
			return false;
		}
		$engine = ($engine === 'DEFAULT' ? $this->getValidDBEngine() : $engine);

		$cols_sql = [];
		foreach($cols as $name => $type){
			$cols_sql[] = "`" . $name . "` " . $type;
		}
		if($keys){
			foreach($keys as $key){
				$cols_sql[] = $key;
			}
		}

		return $this->query('CREATE ' . ($temporary ? 'TEMPORARY' : '') . ' TABLE ' . self::escape_sql($tab) . ' (' . implode(',', $cols_sql) . ') ENGINE=' . $engine . ' ' . self::getCharsetCollation());
	}

	/**
	 * delete an table, checks if it exists.
	 * @param $tab string
	 * @param $temporary boolean
	 */
	public function delTable(string $tab, bool $temporary = false): bool{
		return $this->query('DROP ' . ($temporary ? 'TEMPORARY' : '') . ' TABLE IF EXISTS ' . self::escape_sql($tab));
	}

	/**
	 * Add column to table
	 *
	 * @param string $tab which table to add our column to
	 * @param string $col name of the column to add
	 * @param string $typ type of the column to add
	 * @param string $pos 'FIRST' to add column at first Position, leave empty for last, name of column to add after it
	 */
	public function addCol(string $tab, string $col, string $typ, string $pos = ''): bool{
		$col = trim($col, '`');
		if($this->isColExist($tab, $col)){
			return false;
		}
		return $this->query('ALTER TABLE ' . self::escape_sql($tab) . ' ADD `' . self::escape_sql($col) . '` ' . $typ . ($pos !== '' ? ' ' . ($pos === 'FIRST' ? '' : 'AFTER ') . $pos : ''));
	}

	public function changeColType(string $tab, string $col, string $newtyp): bool{
		$col = trim($col, '`');
		if(!$this->isColExist($tab, $col)){
			return false;
		}

		return $this->query('ALTER TABLE ' . self::escape_sql($tab) . ' MODIFY `' . $col . '` ' . $newtyp);
	}

	public function getColTyp(string $tab, string $col): string{
		return (string) f('SHOW COLUMNS FROM ' . self::escape_sql($tab) . ' LIKE "' . $this->escapeLikeString($col) . '"', 'Type'); //@phan-suppress-current-line PhanDeprecatedFunction
	}

	public function getColDefault(string $tab, string $col): string{
		return (string) f('SHOW COLUMNS FROM ' . self::escape_sql($tab) . ' LIKE "' . $this->escapeLikeString($col) . '"', 'Default'); //@phan-suppress-current-line PhanDeprecatedFunction
	}

	/**
	 * delete a column from a table
	 * @param string $tab
	 * @param string $col
	 * @return boolean
	 */
	public function delCol(string $tab, string $col): bool{
		if(!$this->isColExist($tab, $col)){
			return false;
		}
		return $this->query('ALTER TABLE ' . self::escape_sql($tab) . ' DROP `' . trim($col, '`') . '`');
	}

	public static function removeNonSQL(string $str): string{
		return preg_replace('/(\bCOLLATE=?\b [^ ;,]+|\bENGINE=[^ ;,]+|\bGENERATED ALWAYS\b|\bCHARACTER SET\b [^ ;,]+|DEFAULT CHARSET=[^ ;,]+|\bPAGE_CHECKSUM=[^ ;,]+|ROW_FORMAT=[^ ;,]+)/', '', $str);
	}

	/**
	 *
	 * @param string $tab
	 * @return string[]
	 */
	public function getTableCreateArray(string $tab, bool $removeCollation = false): array{
		$hash = $this->getHash('SHOW CREATE TABLE ' . self::escape_sql($tab));

		return (($data = $hash['Create Table'] ?? [])) ?
			explode("\n", ($removeCollation ? self::removeNonSQL($data) : $data)) :
			[];
	}

	/**
	 *
	 * @param string $tab
	 * @return string[]
	 * @throws InvalidArgumentException
	 */
	public function getTableKeyArray(string $tab): array{
		$zw = $this->getTableCreateArray($tab);
		if(!$zw){
			throw new InvalidArgumentException($tab . ' not found');
		}
		$myarray = [];
		foreach($zw as $v){
			$vv = trim((string) $v);
			$posP = strpos($vv, 'PRIMARY KEY');
			$posU = strpos($vv, 'UNIQUE KEY');
			$posK = strpos($vv, 'KEY');
			if(($posP !== false && $posP == 0) || ($posU !== false && $posU == 0) || ($posK !== false && $posK == 0)){
				$myarray[] = trim(rtrim((string) $v, ','));
			}
		}
		return $myarray;
	}

	public function getPrimaryKeys(string $tab, array $create = []): array{
		$zw = $create ?: $this->getTableCreateArray($tab);
		if(!$zw){
			throw new InvalidArgumentException($tab . ' not found');
		}
		$matches = $mm = [];
		foreach($zw as $v){
			if(preg_match('|PRIMARY KEY\s*\((.*)\)|', (string) $v, $matches)){//be greedy
				$matches = explode(',', $matches[1]);
				foreach($matches as &$m){
					preg_match('|`?([^\(`]*)(\(\d+\))?|', $m, $mm);
					$m = trim($mm[1]);
				}
				return $matches;
			}
		}
		return [];
	}

	/**
	 *
	 * @param string $tab
	 * @param string[] $create
	 * @return array<string,string>
	 * @throws InvalidArgumentException
	 */
	public function getGeneratedColumns(string $tab, array $create = []): array{
		$zw = $create ?: $this->getTableCreateArray($tab, true);
		if(!$zw){
			throw new we_exception_database($tab . ' not found');
		}
		$matches = $ret = [];
		foreach($zw as $v){
			if(preg_match('/`?([^` ]+)`? .*(VIRTUAL|PERSISTENT|STORED)/', $v, $matches)){
				$ret[$matches[1]] = $matches[0];
			}
		}
		return $ret;
	}

	/**
	 * checks if a key with a full key definition exists
	 * @param string $tab table to check
	 * @param string $key name
	 * @return boolean true if exists
	 */
	public function isKeyExistAtAll(string $tab, string $key): bool{
		$zw = $this->getTableKeyArray($tab);
		if($zw){
			foreach($zw as $v){
				if(preg_match('|.*KEY *`?' . $key . '`? \(|', (string) $v)){
					return true;
				}
			}
		}
		return false;
	}

	/**
	 * check if a key matches the EXACT definition given by $key
	 * @param string $tab tablename
	 * @param string[] $definition full key definition what is used in a create statement
	 * @return bool true, if the exact definition is met, false otherwise
	 */
	public function isKeyExist(string $tab, string $name, array $definition, string $type = ''): bool{
		$zw = $this->getTableKeyArray($tab);
		if($zw){
			$definition = array_map('preg_quote', $definition);
			foreach($zw as $v){
				if(preg_match('|' . ($type ?: '.*KEY') . ' *`?' . preg_quote($name) . '`? \(`?' . implode('`?, ?`?', $definition) . '`?\)|', (string) $v)){
					return true;
				}
			}
		}
		return false;
	}

	/**
	 * add a key/index to a table
	 * @param string $tab tablename
	 * @param string $fullKey full key definition what is used in a create statement
	 */
	public function addKey(string $tab, string $fullKey, bool $force = false): bool{
		return $this->query('ALTER ' . ($force ? 'IGNORE' : '') . ' TABLE ' . self::escape_sql($tab) . ' ADD ' . $fullKey);
	}

	/**
	 * delete a key/index from a table
	 * @param string $tab tablename
	 * @param string $keyname ONLY the keyname is wanted here
	 */
	public function delKey(string $tab, string $keyname): bool{
		return $this->query('ALTER TABLE ' . self::escape_sql($tab) . ' DROP ' . ($keyname === 'PRIMARY' ? 'PRIMARY KEY' : 'INDEX `' . $keyname . '`'));
	}

	/**
	 * rename a Col to a new Name
	 * @param string $tab tablename
	 * @param string $oldcol old col-name
	 * @param string $newcol new col-name
	 */
	public function renameCol(string $tab, string $oldcol, string $newcol): bool{
		if($oldcol === $newcol){
			return true;
		}
		return $this->query('ALTER TABLE ' . self::escape_sql($tab) . ' CHANGE `' . $oldcol . '` `' . $newcol . '` ' . $this->getColTyp($tab, $oldcol));
	}

	/**
	 * move a column to a new position inside the table
	 * @param string $tab tablename
	 * @param string $colName the name of the col to move
	 * @param string $newPos the new position (possible: FIRST, colname)
	 */
	public function moveCol(string $tab, string $colName, string $newPos): bool{
		//get the old col def, use for alter table.
		$zw = $this->getTableCreateArray($tab);
		if(!$zw){
			return false;
		}
		$colName = '`' . trim($colName, '`') . '`';
		unset($zw[0]);
		$found = false;
		foreach($zw as $def){
			if(str_contains((string) $def, $colName)){
				$found = trim((string) $def, "\t\n\r\0\x0B ,");
				break;
			}
		}
		if($found){
			return $this->query('ALTER TABLE ' . $tab . ' MODIFY ' . $found . ' ' . ($newPos === 'FIRST' ? 'FIRST' : 'AFTER `' . trim($newPos, '`') . '`'));
		}
		return false;
	}

	/**
	 * @deprecated since version barrhorn
	 * @return string
	 */
	public static function getCharset(): string{//FIXME: remove in cardada
		return defined('DB_CHARSET') ? DB_CHARSET : '';
	}

	/**
	 * @deprecated since version barrhorn
	 * @return string
	 */
	public static function getCollation(): string{//FIXME: remove in cardada
		return defined('DB_COLLATION') ? DB_COLLATION : '';
	}

	public static function getCharsetCollation(): string{
		return ' CHARACTER SET ' . self::DB_CHARSET . ' COLLATE ' . self::DB_COLLATION;
	}

	public static function hasDB(): bool{
		return self::$linkCount > 0;
	}

	public static function t_e(int $cnt = 1): void{
		self::$Trigger_cnt = $cnt;
	}

	/**
	 * @deprecated since version 9.0.8
	 * @param int $cnt
	 * @return void
	 */
	public static function t_e_query(int $cnt = 1): void{
		self::$Trigger_cnt = $cnt;
	}

	public function getInfo(bool $html = true): string{
		if(!$this->isConnected() && !$this->_connect()){
			return '';
		}

		$data = $this->_getInfo();
		$ret = '';
		$this->repool();
		if($html){
			foreach($data as $key => $val){
				$ret .= '<tr><td>' . $key . ':</td><td>' . $val . '</td></tr>';
			}
			return '<table>' . $ret . '</table>';
		}
		foreach($data as $key => $val){
			$ret .= $key . ': ' . $val . "\n";
		}

		return $ret;
	}

	public function getCurrentCharset(): string{
		if(!$this->isConnected() && !$this->_connect()){
			return '';
		}
		$ret = $this->_getCurrentCharset();
		$this->repool();
		return $ret;
	}

	/**
	 * @return array{string,string}
	 */
	public static function getMysqlVer(): array{
		static $version = [];
		if(empty($version)){
			[$res] = explode('-', (string) f('SELECT VERSION()') ?: (string) f('SHOW VARIABLES LIKE "version"', 'Value')); //@phan-suppress-current-line PhanDeprecatedFunction
			$db = (string) f('SHOW VARIABLES LIKE "version_comment"', 'Value'); //@phan-suppress-current-line PhanDeprecatedFunction
			$version = [
				$res,
				preg_match('/mariadb/i', $db) ? self::SERVER_MARIA : self::SERVER_MYSQL
			];
		}
		return $version;
	}

	/**
	 * get the maximum size a db query could be
	 * @return int
	 */
	public function getMaxAllowedPacket(): int{
		return (int) f('SHOW VARIABLES LIKE "max_allowed_packet"', 'Value'); //@phan-suppress-current-line PhanDeprecatedFunction
	}

	/**
	 * get a line from db & give this line as an array
	 * @param $query
	 * @param $resultType
	 * @return array
	 */
	public function getHash(string $query = '', int $resultType = MYSQLI_ASSOC): array{
		if(!$query){
			return [];
		}
		$hash = $resultType . $query;
		try{
			return we_cache_db::load($hash);
		} catch (we_exception_cache $ex){
			$this->query($query);
			$data = ($this->next_record($resultType) ? $this->Record : []);
			$this->free();
			we_cache_db::save($hash, $data);
			return $data;
		}
	}

	public function getValidDBEngine(): string{
		if(self::$defaultEngine){
			return self::$defaultEngine;
		}

		$defaultEngine = (string) f('SHOW variables LIKE "default_storage_engine"', 'Value'); //@phan-suppress-current-line PhanDeprecatedFunction
		if(in_array(strtolower($defaultEngine), self::ENGINES, true)){
			return self::$defaultEngine = $defaultEngine;
		}

		foreach(array_keys($this->getAllFirstq('SHOW ENGINES')) as $eng){
			if(strtolower((string) $eng) === self::ENGINES['ARIA']){
				return self::$defaultEngine = self::ENGINES['ARIA'];
			}
		}

		return self::$defaultEngine = self::ENGINES['MYISAM'];
	}

	public function escapeLikeString(string $str): string{
		return strtr(self::escape_sql($str), ['%' => '\%', '_' => '\_', '[' => '\[', ']' => '\]', '-' => '\-']);
	}

	public static function isReservedField(string $field): bool{
		return in_array(trim(strtolower($field)), self::RESERVED_WORDS, true);
	}

	public function serialize(): string{//deprecated in php 8.1
		$this->repool();
		$this->close();
		return '';
	}

	public function unserialize($serialized): void{//deprecated in php 8.1
	}

	public function __serialize(): array{
		$this->repool();
		$this->close();
		return [];
	}

	public function __unserialize(array $data): void{

	}

}
