<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 562fb8bcf3a8 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2021-06-30 12:28 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
//@phan-file-suppress PhanUndeclaredClassMethod,PhanUndeclaredMethod,PhanUndeclaredExtendedClass
use Sabre\DAV;

class we_davfs_phpFile extends DAV\FSExt\File{

	public function put($data){
		//FIXME: hook?!
		$code = is_resource($data) ? (stream_get_contents($data) ?: '') : $data;

		if(!$code){
			return null;
		}
		try{
			$tmpcode = $code . (!str_ends_with(rtrim($code, "\r\n\t "), '?>') ? ' ?>' : '');
			we_contents_template::checkPhpCode($tmpcode);
		} catch (we_exception_parser $ex){
			t_e('unable to write php document', $ex->errMsg);
			throw new DAV\Exception\PreconditionFailed('internal error');
		}
		$hook = new we_hook_handler('preSave', ['obj' => $this, 'resave' => false, 'code' => $code]);
		//check if doc should be saved
		if(!$hook->executeHook()){
			t_e($hook->getErrorString());
			throw new DAV\Exception\PreconditionFailed('internal error');
		}

		$ret = parent::put($code);

		$hook = new we_hook_handler('save', ['obj' => $this, 'resave' => false, 'code' => $code]);
		//check if doc should be saved
		if(!$hook->executeHook()){
			t_e($hook->getErrorString());
			throw new DAV\Exception\PreconditionFailed('internal error');
		}

		return $ret;
	}

	public function delete(){
		$hook = new we_hook_handler('delete', [$this]);
		$hook->executeHook();
		return parent::delete();
	}

	public function getPath(): string{
		return str_replace(WE_CUSTOM_PATH, '', $this->path);
	}

}
