<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: fdb857335a7f $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2021-11-18 01:31 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
// FIXME: check if we need this...
// => are there relevant diffs between we_doclist_search::searchProperties() and we_search_search::searchProperties()?
class we_doclist_search extends we_search_search{

	/** @var we_modules_view */
	public $View;

	/** @var string */
	protected $whichSearch = '';

	/**
	 * @param we_search_modelBase $model
	 * @param string $table
	 * @return we_search_resultBase[]
	 */
	public function searchProperties(we_search_modelBase $model, string $whichSearch = '', string $table = ''): array{ // FIXME: handle model in like in other searches
		$results = $saveArrayIds = [];
		$_SESSION['weS']['search']['foundItems'] = 0;

		$currentSearchFields = $model->getProperty('currentSearchFields');
		$currentSearch = $model->getProperty('currentSearch');
		$table = $table ?: (($t = $model->getProperty('currentSearchTables')) ? $t[0] : FILE_TABLE);
		$currentLocation = $model->getProperty('currentLocation');
		$currentOrder = (string) $model->getProperty('currentOrder');
		//$view = $model->getProperty('currentSetView');
		$currentSearchstart = $model->getPropertyInt('currentSearchstart');
		$currentAnzahl = $model->getPropertyInt('currentAnzahl');
		$currentFolderID = $model->getPropertyInt('currentFolderID');

		$where = [];
		$this->settable($table);

		if($currentFolderID){
			/*if(true || we_base_request::bool('we_cmd', true, 'newSearch'))*/{
				/*
				 * IMPORTANT: SEARCHRESULT_TABLE holds results from just one search at the time
				 * => because there can be more than one folder open or we can do searches while folders are open, we cannot reuse SEARCHRESULT_TABLE!
				 *
				 * We may need an additional temporary table DOCLISTRESULT_TABLE that can hold more than one doclist (defined by docTable plus ParentID)
				 * => we then need to override selectFromTempTable() and countitems() to geht results by docTable plus ParentID
				 */

				$this->createTempTable();

				foreach($currentSearchFields as $i => $searchField){
					if(isset($currentSearch[0])){
						$searchString = ($currentSearch[$i] ?? $currentSearch[0]);
					}
					if(!empty($searchString)){

						switch($searchField){
							default:
							case 'Filename':
								if(isset($searchField) && isset($currentLocation[$i])){
									$where[] = ($this->searchfor([$searchString], [$searchField], [$currentLocation[$i]], 'WETABLE') ?: ' 0 ');
								}
							case 'Content':
							case we_contents_root::TITLE_FIELD:
							case 'Status':
							case 'Speicherart':
							case 'CreatorName':
							case 'WebUserName':
							case 'temp_category':
								break;
						}

						switch($searchField){
							case 'Content':
								$where[] = ($this->searchContent($searchString, $table) ?: '0');
								break;

							case we_contents_root::TITLE_FIELD:
								break;
							/*
							  $w = $this->searchInTitle($searchString, $table);
							  $where[] = ($w ? $w : '0');
							 *
							 */
							case 'Status':
							case 'Speicherart':
								if($searchString && in_array($table, [FILE_TABLE, OBJECT_FILES_TABLE], true)){
									if($searchString === 'deleted'){
										switch($table){
											case OBJECT_FILES_TABLE:
												$_SESSION['weS']['search']['onlyObjects'] = true;
												break;
											case FILE_TABLE:
												$_SESSION['weS']['search']['onlyDocs'] = true;
												break;
										}

										$table = VERSIONS_TABLE;
									}
									$where[] = $this->getStatusFiles($searchString, $table);
								}
								break;
							case 'CreatorName':
							case 'WebUserName':
								if($searchString){
									$where[] = $this->searchSpecial($searchString, $searchField, $currentLocation[$i]);
								}
								break;
							case 'temp_category':
								$where[] = $this->searchCategory($searchString, $table);
								break;
						}
					}
				}

				$where[] = 'WETABLE.ParentID=' . $currentFolderID;
				switch($table){
					case FILE_TABLE:
					case OBJECT_FILES_TABLE :
					case OBJECT_TABLE :
						$where[] = '(WETABLE.RestrictOwners=0 OR WETABLE.CreatorID=' . we_users_user::getID() . ' OR FIND_IN_SET(' . we_users_user::getID() . ',WETABLE.Owners))';
						break;
					case TEMPLATES_TABLE:
						break;
				}
				$whereQuery = implode(' AND ', array_filter($where));
				$this->where = $whereQuery;
				$this->insertInTempTable($whereQuery, $table, id2path($currentFolderID) . '/');
				$_SESSION['weS']['search']['foundItems'] = $this->founditems = $this->countitems($whereQuery, $table);
			}

			$data = $this->selectFromTempTable($currentSearchstart, $currentAnzahl, $currentOrder);

			foreach($data as $cur){
				if(!isset($saveArrayIds[$cur['ContentType']][$cur['docID']])){
					$saveArrayIds[$cur['ContentType']][$cur['docID']] = $cur['docID'];
					$results[] = array_merge(['Table' => $table], $cur);
				}
			}
		}

		if(!$this->founditems){
			return [];
		}
		//$this->createTempTable();

		foreach($results as &$v){
			$v[we_contents_root::DESCRIPTION_FIELD] = '';
			switch($v['Table']){
				case stripTblPrefix(FILE_TABLE):
					$v[we_contents_root::DESCRIPTION_FIELD] = fS('SELECT c.Dat FROM ' . FILE_TABLE . ' f LEFT JOIN ' . CONTENT_TABLE . ' c ON (f.ID=c.DID) WHERE f.ID=' . intval($v['docID']) . ' AND c.nHash=unhex(md5("Description")) AND c.DocumentTable="' . ($v['IsModified'] ? stripTblPrefix(FILE_TABLE) : TEMPORARY_DOCTARGET) . '"');
			}
		}

		// fill results into result objects
		$resultObjects = [];
		foreach($results as $result){
			$resultObject = null;
			switch($whichSearch){
				case we_doclist_view::DOCLIST_DOCUMENTS:
					$resultObject = new we_doclist_resultDocuments($this);
					break;
				case we_doclist_view::DOCLIST_TEMPLATES:
					$resultObject = new we_doclist_resultTemplates($this);
					break;
				case we_doclist_view::DOCLIST_COLLECTION:
					$resultObject = new we_doclist_resultCollections($this);
					break;
				default:
					continue 2;
			}

			$resultObject->setTable((string) $result['Table'])
					//->setFoundInVersion($result['foundInVersions'] ?? [])
					->setUID((int) $result['UID'])
					->setDocID((int) $result['docID'])
					->setDocTable((string) $result['docTable'])
					->setText((string) $result['Text'])
					->setPath((string) $result['Path'])
					->setParentID(((int) $result['ParentID']))
					->setIsFolder((bool) $result['IsFolder'])
					->setIsProtected((bool) $result['IsProtected'])
					->setTempTemplateId((int) $result['temp_template_id'])
					->setTemplateID((int) $result['TemplateID'])
					->setContentType((string) $result['ContentType'])
					->setSiteTitle((string) $result['SiteTitle'])
					->setDescription((string) $result['Description'])
					->setCreatorID((int) $result['CreatorID'])
					->setCreationDate((string) $result['Creation_Date'])
					->setModDate((string) $result['Mod_Date'])
					->setPublishDate((string) $result['Publish_Date'])
					->setExtension((string) $result['Extension'])
					->setTableID((int) $result['TableID'])
					->setVersionID((int) $result['VersionID'])
					->setMediaAlt((string) $result['media_alt'])
					->setMediaTitle((string) $result['media_title'])
					->setMediaFilesize((int) $result['media_filesize'])
					->setIsUsed((bool) $result['IsUsed'])
					->setRemTable((string) $result['remTable'])
					->setRemCT((string) $result['remCT'])
					->setRemClass((int) $result['remClass'])
					->setIsPublished((bool) $result['IsPublished'])
					->setIsModified((bool) $result['IsModified']);
			/*
			  // seems to be unused
			  ->setNr((string) $result['Nr'])
			  ->setIsNr((bool) $result['isNr']);
			 *
			 */
			$resultObject->processHtml();
			$resultObjects[] = $resultObject;
		}

		return $resultObjects;
	}

}
