<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: adcae6a39191 $
 * $Author: Lukas Imhof $
 * $Date: 2021-10-21 11:48 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_doctype_view extends we_modules_view{
	/** @var we_doctype_docTypes */
	var $Model;

	function __construct(){
		parent::__construct(we_modules_info::DOCTYPE);
		$this->Model = new we_doctype_docTypes();
	}

	function getJSTop(): string{
		return
			parent::getJSTop() .
			we_html_element::jsScript(WE_JS_MODULES_DIR . '/doctype/doctype_top.js', "parent.document.title='" . $this->getModTitle() . "'");
	}

	function getJSProperty(array $jsVars = []): string{
		return we_html_element::jsScript(WE_JS_MODULES_DIR . '/doctype/doctype_prop.js');
	}

	function processCommands(we_base_jsCmd $jsCmd): void{
		switch(we_base_request::string("cmd", '')){
			case 'we_docType_save':
				if(!we_base_permission::hasPerm([we_base_perm::WORK_DOCTYPE_EDIT])){
					$jsCmd->addMsg(g_l('weClass', '[no_perms]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}
				$this->Model->Path = ($this->Model->ParentID ? fS('SELECT Path FROM ' . DOC_TYPES_TABLE . ' WHERE ID=' . intval($this->Model->ParentID)) : '/') . $this->Model->Text;
				if(preg_match('|[\'",]|', $this->Model->Text)){
					$jsCmd->addMsg(g_l('alert', '[doctype_hochkomma]'), we_base_util::WE_MESSAGE_ERROR);
				} else if(!$this->Model->Text){
					$jsCmd->addMsg(g_l('alert', '[doctype_empty]'), we_base_util::WE_MESSAGE_ERROR);
				} elseif(fI('SELECT ID FROM ' . DOC_TYPES_TABLE . ' dt WHERE dt.Text="' . DB_WE::$inst->escape($this->Model->Text) . '" AND ID!=' . intval($this->Model->ID) . ' LIMIT 1')){
					$jsCmd->addMsg(sprintf(g_l('weClass', '[doctype_save_nok_exist]'), $this->Model->Text), we_base_util::WE_MESSAGE_ERROR);
				} elseif($this->Model->save()){
					$jsCmd->addMsg(sprintf(g_l('weClass', '[doctype_save_ok]'), $this->Model->Text), we_base_util::WE_MESSAGE_NOTICE);
					[$cmd, $val] = we_gui_headermenu::getMenuReloadCode();
					$jsCmd->addCmd($cmd, $val);
					if($this->Model->wasUpdate){
						$jsCmd->addCmd('updateTreeEntry', ['id' => $this->Model->ID, 'parentid' => $this->Model->ParentID, 'text' => $this->Model->Text]);
					} else {
						$jsCmd->addCmd('makeTreeEntry', ['id' => $this->Model->ID, 'parentid' => $this->Model->ParentID, 'text' => $this->Model->Text, 'open' => false, 'contenttype' => we_base_ContentTypes::DOCTYPE,
							'table' => DOC_TYPES_TABLE, 'published' => 1]);
					}
				} else {
					t_e('ERROR');
				}
				break;
			case 'we_docType_new':
				//$tmp = $this->Model->Name; // FIXME: find better solution for #11586
				$this->Model = new we_doctype_docTypes();
				//$this->Model->Name = $tmp;
				break;
			case 'we_docType_delete':
				if(!we_base_permission::hasPerm([we_base_perm::WORK_DOCTYPE_EDIT])){
					$jsCmd->addMsg(g_l('alert', '[no_perms]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}
				$id = we_base_request::int('we_cmd', 0, 1) ?: $this->Model->ID; // taking id from request may be obsolete
				$name = fS('SELECT Text FROM ' . DOC_TYPES_TABLE . ' dt WHERE dt.ID=' . $id);
				if($name){
					if(fB('SELECT 1 FROM ' . FILE_TABLE . ' WHERE DocType=' . $id . ' LIMIT 1')){
						$jsCmd->addMsg(sprintf(g_l('weClass', '[doctype_delete_nok]'), $name), we_base_util::WE_MESSAGE_ERROR);
					} else {
						DB_WE::$inst->query('DELETE FROM ' . DOC_TYPES_TABLE . ' WHERE ID=' . $id);
						DB_WE::$inst->query('DELETE FROM ' . LANGLINK_TABLE . ' WHERE DocumentTable="' . stripTblPrefix(DOC_TYPES_TABLE) . '" AND ' . $id . ' IN (DID,LDID)');

						$jsCmd->addMsg(sprintf(g_l('weClass', '[doctype_delete_ok]'), $name), we_base_util::WE_MESSAGE_NOTICE);
						$jsCmd->addCmd('deleteTreeEntry', $id);
						$jsCmd->addCmd('loadHome');
						$_SESSION['weS']['module_sessions']['doctype'] = null;
					}
				}
				break;
			case 'add_dt_template':
				$foo = array_merge($this->Model->Templates, we_base_request::intList('we_cmd', [], 1));
				$this->Model->Templates = array_unique($foo);
				break;
			case 'delete_dt_template':
				$foo = $this->Model->Templates;
				$cmd1 = we_base_request::int('we_cmd', 0, 1);
				if($cmd1 && ($pos = array_search($cmd1, $foo, false)) !== false){
					unset($foo[$pos]);
				}
				if($this->Model->TemplateID == $cmd1){
					$this->Model->TemplateID = ($foo ? $foo[0] : 0);
				}
				$this->Model->Templates = $foo;
				break;
			case 'dt_add_cat':
				if(($id = we_base_request::intList('we_cmd', [], 1))){
					$this->Model->addCat($id);
				}
				break;
			case 'dt_delete_cat':
				if(($id = we_base_request::int('we_cmd', 0, 1))){
					$this->Model->delCat($id);
				}
				break;
			default:
				if(!isset($_SESSION['weS']['module_sessions']['doctype'])){
					$_SESSION['weS']['module_sessions']['doctype'] = null;
				}
				$id = (we_base_request::int('we_cmd', 0, 1) ?: 0);
				if(!$id){
					$dtq = we_doctype_docTypes::getDoctypeQuery();
					$id = fI('SELECT dt.ID FROM ' . DOC_TYPES_TABLE . ' dt LEFT JOIN ' . FILE_TABLE . ' dtf ON dt.ParentFileID=dtf.ID ' . $dtq['join'] . ' WHERE ' . $dtq['where'] . ' LIMIT 1');
				}

				if($id){
					$this->Model->load($id);
				}
		}
		$_SESSION['weS']['module_sessions']['doctype'] = $this->Model;
	}

	function processVariables(): void{
		if(!empty($_SESSION['weS']['module_sessions']['doctype'])){
			$this->Model = $_SESSION['weS']['module_sessions']['doctype'];
			$this->Model->setElementsFromHTTP();
		}
	}

	public function getHomeScreen(we_base_jsCmd $jscmd): string{
		$hiddens = ['cmd' => "home", 'pnt' => 'edbody'];
		$content = we_html_button::create_button('new_doctype', "javascript:we_cmd('we_docType_new')");

		return parent::getActualHomeScreen(we_modules_info::DOCTYPE, $jscmd, $content, we_html_element::form(['name' => we_base_constants::WE_FORM], $this->getCommonHiddens($hiddens) . we_html_element::hidden("home", 0)));
	}

	public function formDocTypeDefaults(we_base_jsCmd $jsCmd): string{
		return '
<table class="default">
	<tr><td colspan="3" style="padding-bottom:5px;">' . $this->formDirChooser() . '</td></tr>
	<tr>
		<td style="padding-right:20px;">' . $this->formSubDir() . '</td>
		<td>' . $this->formExtension() . '</td>
	</tr>
	<tr><td colspan="3" style="padding:5px 0;">' . $this->formTemplatePopup() . '</td></tr>
	<tr>
		<td>' . $this->formIsDynamic() . '</td>
		<td>' . $this->formIsSearchable() . '</td>
	</tr>
	<tr><td colspan="3" style="padding:5px 0;">' . $this->formLangLinks() . '</td></tr>' .
			(we_modules_info::isActive(we_modules_info::CATEGORY) ? '<tr><td colspan="3">' . $this->formCategory($jsCmd) . '</td></tr>' : '') .
			'</table>';
	}

	private function formDirChooser(): string{
		$weSuggest = we_gui_suggest::getInstance();

		$textname = 'ParentPath';
		$idname = 'ParentFileID';
		$weSuggest->setAcId('Path')
			->setContentType(we_base_ContentTypes::FOLDER_TYPES)
			->setInput($textname, id2path($this->Model->ParentFileID, FILE_TABLE))
			->setLabel(g_l('weClass', '[dir]'))
			->setResult($idname, (string) $this->Model->ParentFileID)
			->setSelector(we_gui_suggest::DirSelector)
			->setWidth(399)
			->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_directory', {id:document.we_form.elements['" . $idname . "'].value, table:WE().consts.tables.FILE_TABLE, targetID:'" . $idname . "', targetName:'" . $textname . "', jsCmd:'setHot'})"));

		return $weSuggest->getHTML();
	}

	private function formSubDir(): string{
		return we_html_tools::formElementTable(g_l('weClass', '[subdirectory]'), we_html_tools::select('SubDir', we_base_country::g_l_a('weClass', '[subdir]'), 1, $this->Model->SubDir, false, [
					'onchange' => "we_cmd('setHot')"], true, 300));
	}

	private function formExtension(): string{
		$exts = we_base_ContentTypes::getExtension(we_base_ContentTypes::WEDOCUMENT);
		return we_html_tools::formElementTable(g_l('weClass', '[extension]'), we_html_tools::getExtensionPopup('Extension', $this->Model->Extension, $exts, 100, [
					'onchange' => "we_cmd('setHot')"]));
	}

	/** creates the Template PopupMenue */
	private function formTemplatePopup(): string{
		$tlist = [];
		if($this->Model->TemplateID){
			$tlist[] = $this->Model->TemplateID;
		}
		if($this->Model->Templates){
			$tlist = array_merge($tlist, $this->Model->Templates);
		}
		$list = array_unique(array_filter($tlist));
		$sqlTail = 'IsFolder=0 ' . ($list ? 'AND ID IN(' . implode(',', $list) . ')' : ' AND false' );
		return we_html_forms::formSelectFromDB('', (string) $this->Model->TemplateID, 'TemplateID', TEMPLATES_TABLE, 'ID', 'Path', g_l('weClass', '[standard_template]'), $sqlTail, 1, [
				$this->Model->TemplateID], false, ['onchange' => "we_cmd('setHot')"], '', '', [0, g_l('weClass', '[none]')]);
	}

	private function formIsDynamic(): string{
		return we_html_forms::checkboxWithHidden($this->Model->IsDynamic, 'IsDynamic', g_l('weClass', '[IsDynamic]'), false, 'defaultfont', "we_cmd('setHot');WE().layout.propEdit.switchExt(window," . intval($this->Model->ID) . ");");
	}

	private function formIsSearchable(): string{
		return we_html_forms::checkboxWithHidden($this->Model->IsSearchable, 'IsSearchable', g_l('weClass', '[IsSearchable]'), false, 'defaultfont', "we_cmd('setHot');");
	}

	private function formCategory(we_base_jsCmd $jsCmd): string{
		$addbut = we_html_button::create_button(we_html_button::ADD, "javascript:we_cmd('we_selector_document',{id:-1,table:WE().consts.tables.CATEGORY_TABLE, jsCmd:'dt_add_cat'})");

		$cats = new we_chooser_multiDir($this->Model->Category, 'dt_delete_cat', $addbut, [], '"' . we_base_ContentTypes::CATEGORY . '"', CATEGORY_TABLE);
		return we_html_tools::formElementTable(g_l('weClass', '[category]'), $cats->get($jsCmd));
	}

	/** Form functions for generating the html of the input fields */
	public function formName(): string{
		return $this->formInput('Text', '', 520, 32, ['onchange' => "we_cmd('setHot')"]);
	}

	public function formDocTypeTemplates(we_base_jsCmd $jsCmd): string{
		$addbut = we_html_button::create_button(we_html_button::ADD, "javascript:we_cmd('we_selector_document',{table:WE().consts.tables.TEMPLATES_TABLE,jsCmd:'add_dt_template', filter:WE().consts.contentTypes.TEMPLATE, open_doc:true,multiple:true})");
		$this->Model->Templates = array_filter($this->Model->Templates);
		$templ = new we_chooser_multiDir($this->Model->Templates, 'delete_dt_template', $addbut, [], 'ContentType', TEMPLATES_TABLE);
		return $templ->get($jsCmd);
	}

	private function formLangLinks(): string{
		$value = ($this->Model->Language ?: we_base_WE::$Langs['DefaultFrontendLanguage']);
		$inputName = 'Language';
		$languages = we_base_country::getWeFrontendLanguagesForBackend();

		if(LANGLINK_SUPPORT){
			$langkeys = [];
			$htmlzw = '';
			$LDIDs = $this->db->getAllFirstq('SELECT Locale,LDID FROM ' . LANGLINK_TABLE . ' WHERE DocumentTable="' . stripTblPrefix(DOC_TYPES_TABLE) . '" AND DID=' . $this->Model->ID, false);
			foreach($languages as $langkey => $lang){
				$langkeys[] = $langkey;
				$LDID = empty($LDIDs[$langkey]) ? 0 : (int) $LDIDs[$langkey];
				$htmlzw .= $this->formDocTypes3((string) $lang, (string) $langkey, $LDID);
			}
			return we_html_tools::formElementTable(g_l('weClass', '[language]'), we_html_tools::select($inputName, $languages, 1, $value, false, ['onchange' => "we_cmd('setHot');dieWerte='" . implode(',', $langkeys) . "'; disableLangDefault('LangDocType',dieWerte,this.options[this.selectedIndex].value);"], true, 521)) .
				we_html_element::br() . we_html_tools::formElementTable(g_l('weClass', '[languageLinksDefaults]'), $htmlzw);
		}
		return we_html_tools::formElementTable(g_l('weClass', '[language]'), we_html_tools::select($inputName, $languages, 1, $value, false, ['onchange' => "we_cmd('setHot')"], true, 521));
	}

	private function formDocTypes3(string $headline, string $langkey, int $derDT = 0): string{
		$dtq = we_doctype_docTypes::getDoctypeQuery();
		$vals = [0 => g_l('weClass', '[nodoctype]')] + $this->db->getAllFirstq('SELECT dt.ID,dt.Text FROM ' . DOC_TYPES_TABLE . ' dt LEFT JOIN ' . FILE_TABLE . ' dtf ON dt.ParentFileID=dtf.ID ' . $dtq['join'] . ' WHERE dt.Language="' . $langkey . '" AND ' . $dtq['where'], false);

		return we_html_tools::formElementTable($headline, we_html_tools::select('LangDocType[' . $langkey . ']', $vals, 1, $derDT, false, [
					($langkey == $this->Model->Language ? 'disabled' : null) => "disabled",
					'width' => '328', 'onchange' => "we_cmd('setHot')"]));
	}

}
