<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 80c72f54723f $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2021-03-29 16:27 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_export_tree extends we_tree_base{
	/** @var string[] */
	private static $parentpaths = [];

	/** @var string */
	private static $wsQuery = '';

	public function __construct(we_base_jsCmd $jsCmd){
		parent::__construct($jsCmd);
		$this->autoload = false;
	}

	protected function customJSFile(): string{
		return we_html_element::jsScript(WE_JS_MODULES_DIR . 'export/export_tree.js', 'initTree();');
	}

	function getHTMLMultiExplorer(int $width = 500, int $height = 250, bool $useSelector = true, string $selected = FILE_TABLE, string $exportType = we_exim_ExIm::TYPE_WE): string{
		$js = $this->getJSTreeCode() . we_html_element::cssLink(CSS_DIR . 'tree.css');

		$isTypeWE = !in_array($exportType, [we_exim_ExIm::TYPE_CSV, we_exim_ExIm::TYPE_XML], true);
		if($useSelector){
			$selectorAttribs = [
				'name' => 'headerSwitch',
				'onchange' => "we_cmd('setTreeHead', this.value)",
				'style' => 'width:' . $width . 'px;',
			];
			$selector = new we_html_select(array_filter($selectorAttribs));

			if(we_base_permission::hasPerm([we_base_perm::WORK_DOCUMENTS_CAN_SEE])){
				$selector->addOption(FILE_TABLE, g_l('export', '[documents]'));
			}
			if(we_base_permission::hasPerm([we_base_perm::WORK_TEMPLATES_CAN_SEE])){
				$selector->addOption(TEMPLATES_TABLE, g_l('export', '[templates]'), (!$isTypeWE ? ['disabled' => 'disabled'] : []));
			}
			if(we_modules_info::isActive(we_modules_info::OBJECT) && we_base_permission::hasPerm([we_base_perm::OBJECTFILES_CAN_SEE])){
				$selector->addOption(OBJECT_FILES_TABLE, g_l('export', '[objects]'));
			}
			if(we_modules_info::isActive(we_modules_info::OBJECT) && we_base_permission::hasPerm([we_base_perm::OBJECTS_CAN_SEE])){
				$selector->addOption(OBJECT_TABLE, g_l('export', '[classes]'), (!$isTypeWE ? ['disabled' => 'disabled'] : []));
			}
			if(we_modules_info::isActive(we_modules_info::COLLECTION) && we_base_permission::hasPerm([we_base_perm::WORK_COLLECTIONS_CAN_SEE])){
				$selector->addOption(VFILE_TABLE, g_l('export', '[collections]'), (!$isTypeWE ? ['disabled' => 'disabled'] : []));
			}
			$selector->selectOption($selected);

			$header = we_html_element::div(['style' => 'margin:5px 0;'], $selector->getHtml());
		} else {
			$header = '';
		}
		return $js . $header . we_html_element::div(['id' => 'treetable', 'class' => 'blockWrapper', 'style' => 'position: relative;width: ' . $width . 'px; height: ' . $height . 'px; border:1px #dce6f2 solid;'], '');
	}

	private static function getQueryParents(string $path): string{
		$out = [];
		while($path != '/' && $path){
			$out[] = 'Path="' . $path . '"';
			$path = dirname($path);
		}
		return $out ? implode(' OR ', $out) : '';
	}

	/**
	 *
	 * @param int $ParentID
	 * @param int $offset
	 * @param int $segment
	 * @param bool $sort
	 * @return array<int,array{id:int,parentid:int,text:string,typ:string,open:bool,isfolder:bool,disabled:bool,tooltip:string,offset:int,contenttype:string,published:bool,table:string}>
	 */
	public static function getItems(int $ParentID, int $offset = 0, int $segment = 500, bool $sort = false): array{
		//unused
		return [];
	}

	/**
	 *
	 * @staticvar type $openFolders
	 * @param string $table
	 * @param int $ParentID
	 * @param int[] $of
	 * @return array<int,array{id:int,parentid:int,text:string,contenttype:string,table:string,checked:bool,typ:string,open:bool,published:string,disabled:bool,tooltip:int}>
	 */
	public static function getTreeItems(string $table, int $ParentID, array $of): array{
		static $openFolders = null;
		if($openFolders === null){
			$openFolders = $of;
		}
		$treeItems = [];
		$elem = 'ID,ParentID,Path,Text,IsFolder,Mod_Date,ContentType';
		$DB_WE = we_database_base::$inst;
		switch($table){
			case FILE_TABLE :
				$selDocs = isset($_SESSION['weS']['module_sessions']['export']) ? $_SESSION['weS']['module_sessions']['export']->selDocs : [];
				$elem .= ',Publish_Date,IsPublished';
				break;
			case TEMPLATES_TABLE:
				$selDocs = isset($_SESSION['weS']['module_sessions']['export']) ? $_SESSION['weS']['module_sessions']['export']->selTempl : [];
				$elem .= ',Mod_Date AS Publish_Date,1 AS IsPublished';
				break;
			case OBJECT_FILES_TABLE:
				$elem .= ',Publish_Date,IsPublished';
				$selDocs = isset($_SESSION['weS']['module_sessions']['export']) ? $_SESSION['weS']['module_sessions']['export']->selObjs : [];
				break;
			case OBJECT_TABLE:
				$selDocs = isset($_SESSION['weS']['module_sessions']['export']) ? $_SESSION['weS']['module_sessions']['export']->selClasses : [];
				$elem .= ',Mod_Date AS Publish_Date,1 AS IsPublished';
				break;
		}

		$entries = $DB_WE->getAllq('SELECT ' . $elem . ' FROM ' . $DB_WE->escape($table) . ' WHERE ParentID=' . $ParentID . ' AND((1' . we_users_util::makeOwnersSql() . ')' . self::$wsQuery . ') ORDER BY IsFolder DESC,(text REGEXP "^[0-9]") DESC,ABS(text),Text');
		'@phan-var array<int,array{ID:int,ParentID:int,Path:string,Text:string,IsFolder:int,Mod_Date:string,ContentType:string,Publish_Date:string,IsPublished:int}> $entries';
		foreach($entries as $entry){
			$ID = $entry['ID'];
			$OpenCloseStatus = in_array($ID, $openFolders);

			$treeItems[] = [
				'id' => $ID,
				'parentid' => $entry['ParentID'],
				'text' => $entry['Text'],
				'contenttype' => $entry['ContentType'],
				'table' => $table,
				'checked' => (!empty($selDocs) && in_array($ID, $selDocs)),
				'typ' => $entry['IsFolder'] ? self::TYP_FOLDER : self::TYP_ITEM,
				'open' => $OpenCloseStatus,
				'published' => ($entry['IsPublished']) ? $entry['Publish_Date'] : -1,
				'disabled' => in_array($entry['Path'], self::$parentpaths),
				'tooltip' => $ID
			];

			if($entry['IsFolder'] && $OpenCloseStatus){
				$treeItems = array_merge($treeItems, static::getTreeItems($table, $ID, $of));
			}
		}
		return $treeItems;
	}

	public function loadHTML(string $table, int $parentFolder, array $openFolders): void{
		self::$parentpaths = $wsQuery = [];

		if(($ws = we_users_util::get_ws($table))){
			$wsPathArray = id2pathA($ws, $table);
			foreach($wsPathArray as $path){
				'@phan-var string $path';
				$wsQuery[] = 'Path LIKE "' . DB_WE::$inst->escape($path) . '/%" OR ' . self::getQueryParents($path);
				while($path != '/' && $path){
					self::$parentpaths[] = $path;
					$path = dirname($path);
				}
			}
		}

		switch($table){
			case FILE_TABLE:
				if(!we_base_permission::hasPerm([we_base_perm::WORK_DOCUMENTS_CAN_SEE])){
					return;
				}
				break;
			case TEMPLATES_TABLE:
				if(!we_base_permission::hasPerm([we_base_perm::WORK_TEMPLATES_CAN_SEE])){
					return;
				}
				break;
			case OBJECT_FILES_TABLE :
				if(!we_base_permission::hasPerm([we_base_perm::OBJECTFILES_CAN_SEE])){
					return;
				}
				if(!we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR])){
					$classes = we_users_util::getAllowedClasses();
					if($classes){
						$wsQuery[] = 'TableID IN(' . implode('|', $classes) . ')';
					}
				}
				break;
			case OBJECT_TABLE :
				if(!we_base_permission::hasPerm([we_base_perm::OBJECTS_CAN_SEE])){
					return;
				}
				break;
			case VFILE_TABLE:
				if(!we_base_permission::hasPerm([we_base_perm::WORK_COLLECTIONS_CAN_SEE])){
					return;
				}
				break;
			default:
				return;
		}

		self::$wsQuery = ' ' . ($wsQuery ? ' OR (' . implode(' OR ', $wsQuery) . ')' : '');

		$treeItems = self::getTreeItems($table, $parentFolder, $openFolders);

		$dynVars = [
			'parentFolder' => $parentFolder,
			'clear' => !$parentFolder,
			'treeItems' => $treeItems
		];

		echo we_html_tools::getHtmlTop('', we_html_element::jsScript(WE_JS_MODULES_DIR . 'export/export_cmd_loadTree.js', '', ['id' => 'loadVarExport_cmd_loadTree',
				'data-cmdDynVars' => setDynamicVar($dynVars)]) .
			$this->jsCmd->getCmds(), we_html_element::body());
	}

	public static function loadTree(): void{
//added for export module.
		$jsCmd = new we_base_jsCmd();
		$tree = new we_export_tree($jsCmd);

		$table = we_base_request::_(we_base_request::TABLE, 'we_cmd', FILE_TABLE, 1);

		if($table === FILE_TABLE && !we_base_permission::hasPerm([we_base_perm::WORK_DOCUMENTS_CAN_SEE])){
			if(we_base_permission::hasPerm([we_base_perm::WORK_TEMPLATES_CAN_SEE])){
				$table = TEMPLATES_TABLE;
			} else if(we_modules_info::isActive(we_modules_info::OBJECT) && we_base_permission::hasPerm([we_base_perm::OBJECTFILES_CAN_SEE])){
				$table = OBJECT_FILES_TABLE;
			} else if(we_modules_info::isActive(we_modules_info::OBJECT) && we_base_permission::hasPerm([we_base_perm::OBJECTS_CAN_SEE])){
				$table = OBJECT_TABLE;
			}
		}

		$parentFolder = we_base_request::int('we_cmd', 0, 2);
		$openFolders = array_filter(we_base_request::intList('we_cmd', [], 3));

		$tree->loadHTML($table, $parentFolder, $openFolders);
	}

}
