<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: c288f1fd0858 $
 * $Author: Lukas Imhof $
 * $Date: 2021-11-29 13:23 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_fileupload_resp_import extends we_fileupload_resp_base{

	public function __construct(string $name = ''){
		parent::__construct($name);
		$this->importFile = true;
	}

	protected function initByHttp(): void{
		parent::initByHttp();

		// FIXME: beim überschreiben bestehender grafiken mit einem neuen bild bleiben width und height in der db unveraendert, so dass beim bearbeiten verzerrt wird
		// FIXME: eliminate all serverside editing opts!
		$this->docVars = array_filter([
			'transaction' => we_base_request::_(we_base_request::TRANSACTION, 'we_transaction', $this->docVars['transaction']),
			'importMetadata' => we_base_request::bool('fu_doc_importMetadata', $this->docVars['importMetadata']),
			'isSearchable' => we_base_request::bool('fu_doc_isSearchable', $this->docVars['isSearchable']),
			'importDirs' => we_base_request::bool('fu_doc_importDirs', $this->docVars['importDirs']),
			'focusX' => we_base_request::float('fu_doc_focusX', $this->docVars['focusX']),
			'focusY' => we_base_request::float('fu_doc_focusY', $this->docVars['focusY']),
			// unset the following entries when not in request!
			'categories' => we_modules_info::isActive(we_modules_info::CATEGORY) ? 
				($cats = we_base_request::stringList('fu_doc_categories', we_base_request::NOT_VALID_AS)) === we_base_request::NOT_VALID_AS ? [] :
				path_to_idA($cats, CATEGORY_TABLE) : [],
			'title' => we_base_request::string('fu_doc_title', we_base_request::NOT_VALID_S),
			'alt' => we_base_request::string('fu_doc_alt', we_base_request::NOT_VALID_S),
			'thumbs' => we_base_request::intList('fu_doc_thumbs', we_base_request::NOT_VALID_AI),
			], function($var){
			return $var !== we_base_request::NOT_VALID_S && $var !== we_base_request::NOT_VALID_AI;
		});

		$this->setImageEditProps();
		$this->saveImageEditPropsInSession();
	}

	protected function setImageEditProps(): void{
		$this->imageEditProps = array_merge($this->imageEditProps, [
			'parentID' => $this->fileVars['parentID'],
			'sameName' => $this->fileVars['sameName'],
			'importMetadata' => $this->docVars['importMetadata'],
			'isSearchable' => $this->docVars['isSearchable'],
			'thumbnails' => empty($this->docVars['thumbs']) ? [] : $this->docVars['thumbs'],
			'categories' => empty($this->docVars['categories']) ? [] : $this->docVars['categories'],
			'importDirs' => $this->docVars['importDirs'],
		]);
	}

	protected function postprocess(): array{
		try{
			$we_doc = $this->getWebeditionDocument();
			$response = $this->writeWebeditionDocument($we_doc);
		} catch (we_exception_file $e){
			return array_merge($this->response, ['status' => 'failure', 'message' => $e->getMessage(), 'filename' => $this->fileVars['weFileName']]);
		}
		//TODO: make concise mime and extension test here, taking uploader's typecondition from session...

		$message = isset($response['weDoc']['text']) ? (($response['weDoc']['innerPath'] ?? '') . $response['weDoc']['text'] . ' (ID: ' . $response['weDoc']['id'] . ')') : '';

		return array_merge($this->response, ['status' => 'success', 'completed' => 1, 'message' => $message, 'weDoc' => $response['weDoc'], 'filename' => $this->fileVars['weFileName']]);
	}

	protected function isWorkspaceOk(int $parentID = 0): bool{
		return ($ws = we_users_util::get_ws(FILE_TABLE)) ? (we_users_util::in_workspace($parentID, $ws, FILE_TABLE)) : true;
	}

	protected function getWebeditionDocument(): we_document_files{ // TODO: avoid some more redundancy in this fn

		/* import binary on tab upload of we_document_binary */
		if($this->docVars['transaction']){
			try{
				$we_doc = we_contents_root::getSessionDocument($this->docVars['transaction']);
			} catch (Exception $ex){
				throw new we_exception_file('transaction is not correct');
			}

			if(!$this->isWorkspaceOk($we_doc->ParentID)){
				throw new we_exception_file('workspace not ok');
			}
			assert($we_doc instanceof we_document_files);
//FIXME: use isFileExtensionNotValid and we_filenameNotValid - adapt these functions if needed!
			$we_doc->Extension = strtolower((strpos($this->fileVars['weFileName'], '.') > 0) ? preg_replace('/^.+(\..+)$/', '${1}', $this->fileVars['weFileName']) : ''); //strtolower for feature 3764

			if(!$we_doc->Filename || !$we_doc->ID){ // new we_document_binary, not yet saved
				// Bug Fix #6284
				$we_doc->Filename = we_import_functions::correctFilename( $this->fileVars['weFileName']);
				$we_doc->Filename = preg_replace('/^(.+)\..+$/', '${1}', $we_doc->Filename);
				$we_doc->Path = $we_doc->getParentPath() . (($we_doc->getParentPath() != '/') ? '/' : '') . $we_doc->getText();

				if(fI('SELECT ID FROM ' . FILE_TABLE . ' WHERE Path="' . DB_WE::$inst->escape($we_doc->Path) . '"')){
					// make fn rename to avoid redundant code
					$z = 0;
					$footext = $we_doc->Filename . '_' . $z . $we_doc->Extension;
					while(fI('SELECT ID FROM ' . FILE_TABLE . ' WHERE Text="' . DB_WE::$inst->escape($footext) . '" AND ParentID=' . intval($we_doc->ParentID))){
						$z++;
						$footext = $we_doc->Filename . '_' . $z . $we_doc->Extension;
					}

					$we_doc->Filename = $we_doc->Filename . '_' . $z;
					$we_doc->Path = $we_doc->getParentPath() . (($we_doc->getParentPath() != '/') ? '/' : '') . $we_doc->getText();
				}
			}
			return $we_doc;
		}

		/* import binary on tab upload of we_document_binary */

		// check workspace for new document
		if(!$this->isWorkspaceOk($this->fileVars['parentID'])){
			throw new we_exception_file('workspace not ok');
		}

		// make new we_doc
		$we_ContentType = we_base_ContentTypes::getFromFilename($this->fileVars['weFileName']);
		$we_doc = we_document_files::initDoc($we_ContentType, 0, FILE_TABLE);
		assert($we_doc instanceof we_document_files);

		// set filename, ext and path
		$matches = [];
		preg_match('#^(.*)(\..+)$#', $this->fileVars['weFileName'], $matches);
		if(!$matches){
			throw new we_exception_file(g_l('importFiles', '[save_error]'));
		}
		$we_doc->Filename = $matches[1];
		$we_doc->Extension = strtolower($matches[2]);
		if(!$we_doc->Filename){ // .htaccess
			$we_doc->Filename = $matches[2]; //.htaccess
			$we_doc->Extension = '';
		}
		$we_doc->Filename = we_import_functions::correctFilename($we_doc->Filename, 'document');
		$we_doc->Extension = $we_doc->Extension ? we_import_functions::correctExtension($we_doc->Extension, 'document') : '';

		$this->checkSetParentId();
		$we_doc->setParentID($this->fileVars['parentID']);
		$we_doc->applyWeDocumentCustomerFilterFromFolder();

		// if file exists we have to see if we should create a new one or overwrite it!
		if(($file_id = fI('SELECT ID FROM ' . FILE_TABLE . ' WHERE Path="' . DB_WE::$inst->escape($we_doc->Path) . '"'))){
			switch($this->fileVars['sameName']){
				case we_exim_ExIm::COLLISION_OVERWRITE:
					$tmp = get_class($we_doc);
					$we_doc = new $tmp();
					$we_doc->initByID($file_id, FILE_TABLE);
					break;
				case we_exim_ExIm::COLLISION_RENAME:
					// make fn rename to avoid redundant code
					$z = 0;
					$footext = $we_doc->Filename . '_' . $z . $we_doc->Extension;
					while(fI('SELECT ID FROM ' . FILE_TABLE . ' WHERE Text="' . DB_WE::$inst->escape($footext) . '" AND ParentID=' . intval($this->fileVars['parentID']))){
						$z++;
						$footext = $we_doc->Filename . '_' . $z . $we_doc->Extension;
					}
					$we_doc->Filename = $we_doc->Filename . '_' . $z;
					$we_doc->Path = $we_doc->getParentPath() . (($we_doc->getParentPath() != '/') ? '/' : '') . $we_doc->getText();
					break;
				default:
					throw new we_exception_file(g_l('importFiles', '[same_name]'));
			}
		}

		return $we_doc;
	}

	private function hasPerm(we_document_files $we_doc): bool{
		switch($we_doc->ContentType){
			case we_base_ContentTypes::IMAGE:
				if(!we_base_permission::hasPerm([we_base_perm::WORK_IMAGE_NEW])){
					return false;
				}
				break;
			case we_base_ContentTypes::HTML:
				if(!we_base_permission::hasPerm([we_base_perm::WORK_HTML_NEW])){
					return false;
				}
				break;
			case we_base_ContentTypes::VIDEO:
			case we_base_ContentTypes::AUDIO:
				if(!we_base_permission::hasPerm([we_base_perm::WORK_VIDEO_NEW])){
					return false;
				}
				break;
			case we_base_ContentTypes::JS:
				if(!we_base_permission::hasPerm([we_base_perm::WORK_JS_NEW])){
					return false;
				}
				break;
			case we_base_ContentTypes::CSS:
				if(!we_base_permission::hasPerm([we_base_perm::WORK_CSS_NEW])){
					return false;
				}
				break;
			case we_base_ContentTypes::TEXT:
				if(!we_base_permission::hasPerm([we_base_perm::WORK_TEXT_NEW])){
					return false;
				}
				break;
			default:
				if(!we_base_permission::hasPerm([we_base_perm::WORK_OTHER_NEW])){
					return false;
				}
				break;
		}
		return true;
	}

	protected function writeWebeditionDocument(we_document_files $we_doc): array{
		if(!$this->hasPerm($we_doc)){
			throw new we_exception_file(g_l('importFiles', '[no_perms]')); //$this->fileVars['weFileName']
		}

		if(!$we_doc->uploadSetData($this->fileVars)){
			throw new we_exception_file(g_l('importFiles', '[read_file_error]'));
		}

		try{
			$we_doc->uploadImportFile($this->fileVars, $this->docVars);
		} catch (we_exception_file $e){
			throw new we_exception_parser($e->getMessage());
		}

		return [
			'error' => [],
			'success' => true,
			'weDoc' => ['id' => $we_doc->ID,
				'path' => $we_doc->Path,
				'text' => $we_doc->getText(),
				'ct' => $we_doc->ContentType,
				'table' => FILE_TABLE,
				'currentID' => $we_doc->ID,
				'currentPath' => $we_doc->Path,
				'currentText' => $we_doc->getText(),
				'currentType' => $we_doc->ContentType,
				'currentTable' => FILE_TABLE,
				'innerPath' => $this->fileVars['innerPath'],
			]
		];
	}

}
