<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 0b80d828f85e $
 * $Author: Lukas Imhof $
 * $Date: 2021-11-06 23:04 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_import_files extends we_import_wizard{
	/** @var int */
	var $parentID = 0;

	/** @var int */
	var $step = 0;

	/** @var string */
	private $type = 'FileImport';

	/** @var string */
	var $sameName = 'overwrite';

	/** @var bool */
	private $importMetadata = true;

	/** @var bool */
	private $importDirs = true;

	/** @var bool */
	private $imgsSearchable = false;

	/** @var string */
	var $cmd = '';

	/** @var int[] */
	var $thumbs = [];

	/** @var string[] */
	var $categories = [];

	/** @var string */
	private $cmdUploadFinished = '';

	/** @var string */
	private $fileNameTemp = '';

	/** @var int */
	private $partNum = 0;

	/** @var int */
	private $partCount = 0;

	/** @var bool */
	private $parentFixed = false;

	/** @var bool */
	private $isPrev = false;

	public function __construct(string $what, int $step, int $mode){
		parent::__construct($what, $step, $mode);
		$this->isPrev = we_base_request::bool('we_cmd', false, 'isPrev');
		$this->categories = we_base_request::stringList('fu_doc_categories', $this->categories);
		$this->parentID = we_base_request::int('we_cmd', 0, 1) ?: we_base_request::int('fu_file_parentID', $this->parentID);
		$this->parentFixed = we_base_request::bool('we_cmd', we_base_request::bool('fu_file_parentFixed', $this->parentFixed), 2);
		$this->cmdUploadFinished = we_base_request::string('we_cmd', '', 3) ?: (we_base_request::string('cmdUploadFinished', '') ?: '');
		$this->sameName = we_base_request::string('fu_file_sameName', $this->sameName);
		$this->importMetadata = we_base_request::bool('fu_doc_importMetadata', $this->importMetadata);
		$this->imgsSearchable = we_base_request::bool('fu_doc_isSearchable', $this->imgsSearchable);
		$this->importDirs = we_base_request::bool('fu_doc_importDirs', $this->importDirs);
		$this->step = we_base_request::int('step', $this->step);
		$this->cmd = we_base_request::string('cmd', '');
		$this->thumbs = we_base_request::intList('fu_doc_thumbs', $this->thumbs);
		/* FIXME reimplement these to keep edit opts when navigating back next in editor: scale, scaleWhat, quality
		  $this->scale = we_base_request::int("fu_doc_width", $this->width);
		  $this->height = we_base_request::int("fu_doc_height", $this->height);
		  $this->widthSelect = we_base_request::string("fu_doc_widthSelect", $this->widthSelect);
		  $this->heightSelect = we_base_request::string("fu_doc_heightSelect", $this->heightSelect);
		  $this->keepRatio = we_base_request::bool("fu_doc_keepRatio", $this->keepRatio);
		  $this->quality = we_base_request::int("fu_doc_quality", $this->quality);
		  $this->degrees = we_base_request::int("fu_doc_degrees", $this->degrees);
		 */
		$this->partNum = we_base_request::int('wePartNum', 0);
		$this->partCount = we_base_request::int('wePartCount', 0);
		$this->fileNameTemp = we_base_request::_(we_base_request::FILE, 'weFileNameTemp', '');
	}

	public function getHTML(): string{
		switch($this->cmd){
			case 'content':
				return $this->getWizBody();
			case 'buttons':
				return $this->getWizBusy();
			default:
				if($this->isPrev){
					$fileuploadRest = new we_fileupload_resp_import();
					$fileuploadRest->saveImageEditPropsInSession();
				}
				return $this->getWizFrameset();
		}
	}

	protected function getWizBody(): string{
		switch($this->step){
			default:
			case 1:
				return $this->getStep1();
			case 2:
				return $this->getStep2();
		}
	}

	private function getStep1(): string{
		$jsCmd = new we_base_jsCmd();
		$hiddens = [
			'we_cmd[0]' => 'import_files',
			'cmdUploadFinished' => $this->cmdUploadFinished,
			'cmd' => 'content',
			'step' => '1',
			'type' => $this->type
		];

		$fileupload = new we_fileupload_ui_editor('we_File');
		$fileupload->loadImageEditPropsFromSession();
		$fileupload->setFieldParentID(['setField' => true,
			'presetId' => ($this->parentID ?: 0),
			'setFixed' => $this->parentFixed,
		]);

		$parts = (new we_html_multiIconBoxContent())
			->add($fileupload->getFormParentID(we_base_constants::WE_FORM))
			->add($fileupload->getFormSameName())
			->add($fileupload->getFormImportDirs())
			->add($fileupload->getFormIsSearchable())
			->add($fileupload->getFormImportMeta());

		$content = '';
		if(we_base_permission::hasPerm([we_base_perm::WORK_CATEGORY_EDIT]) && we_modules_info::isActive(we_modules_info::CATEGORY)){
			$parts->add($fileupload->getFormCategories($jsCmd));
		} else {
			$hiddens['fu_doc_categories'] = '';
		}

		if(we_base_permission::hasPerm([we_base_perm::WORK_IMAGE_NEW])){
			$parts->add($fileupload->getFormThumbnails($jsCmd));
		} else {
			$hiddens['fu_doc_thumbs'] = '';
		}

		$content .= we_html_element::hiddens($hiddens) .
			(new we_html_multiIconBox($parts, 'weimportfiles'))
			->marginLeft()
			->fold(-1, false, g_l('importFiles', '[image_options_open]'), g_l('importFiles', '[image_options_close]'))
			->headline(g_l('importFiles', '[step1]'))
			->get();

		$startsrceen = we_html_element::div(
				['id' => 'start'
				], we_html_element::form(
					['action' => WEBEDITION_DIR . 'we_cmd.php',
						'name' => we_base_constants::WE_FORM,
						'method' => 'post'
					], $content));

		return we_html_tools::getHtmlTop('', $fileupload->getJSDialog() . $jsCmd->getCmds(), we_html_element::body(['class' => 'weDialogBody'], $startsrceen));
	}

	private function getStep2(): string{
		$uploader = new we_fileupload_ui_importer('we_File');
		$uploader->setCmd('cmdUploadFinished', $this->cmdUploadFinished);
		$body = $uploader->getHTML($this->getHiddens(true));
		return we_html_tools::getHtmlTop('', $uploader->getCss() . $uploader->getJs(), $body);
	}

	protected function getWizBusy(): string{
		// TODO: let we_fileupload set pb
		$pb = new we_gui_progressBar(0, 200);
		$pb->addText(sprintf(g_l('importFiles', '[import_file]'), 1), we_gui_progressBar::TOP, "progress_title");
		$progressbar = '<div id="progressbar" style="margin:0 0 6px 12px;' . (($this->step == 0) ? 'display:none;' : '') . '">' . $pb->getHTML() . '</div>';

		$table = (new we_html_table(['class' => 'default', 'width' => "100%"], 1, 2))
			->setCol(0, 0, $progressbar)
			->setCol(0, 1, we_html_element::div(['id' => 'normButton'], we_html_button::position_yes_no_cancel(
					we_html_button::create_button(we_html_button::BACK, "javascript:WE().util.dialogTop(window).handleEvent('previous');", '', '', $this->parentFixed, false) .
					we_html_button::create_button(we_html_button::NEXT, "javascript:WE().util.dialogTop(window).handleEvent('next');", '', '', $this->step > 0, false) .
					we_html_button::create_button(we_html_button::UPLOAD, "javascript:WE().util.dialogTop(window).handleEvent('upload');", '', '', true, false, '', false, '', 'weHide'), ''
					, we_html_button::create_button(we_html_button::CANCEL, "javascript:WE().util.dialogTop(window).handleEvent('cancel')", '', '', false, false) .
					we_html_button::create_button(we_html_button::CLOSE, "javascript:WE().util.dialogTop(window).handleEvent('close');", '', '', false, false, '', false, '', 'weHide'))), [
			'style' => 'text-align:right']);

		return we_html_tools::getHtmlTop('', we_gui_progressBar::getJSCode(), we_html_element::body(['class' => "editfooter"], $table->getHtml()));
	}

	private function getHiddens(bool $noCmd = false): string{
		// FIXME: we_fileupload_ui_editor should write these hiddens
		return ($noCmd ? '' : we_html_element::hidden('cmd', 'buttons')) . we_html_element::hiddens(array_merge([
				'step' => '2',
				// these are used by we_fileupload to grasp these values AND by editor to have when going back one step
				'type' => $this->type,
				'fu_file_parentID' => (string) $this->parentID,
				'fu_file_parentFixed' => (string) ($this->parentFixed ? 1 : 0),
				'fu_file_sameName' => $this->sameName,
				'fu_doc_thumbs' => implode(',', $this->thumbs),
				'fu_doc_categories' => implode(',', $this->categories),
				'fu_doc_isSearchable' => (string) $this->imgsSearchable,
				'fu_doc_importMetadata' => (string) $this->importMetadata,
				'fu_doc_importDirs' => (string) $this->importDirs,
		]));
	}

	private function getWizFrameset(): string{
		$body = we_html_element::body(['id' => 'weMainBody']
				, we_html_element::iframe('wizbody', WEBEDITION_DIR . "we_cmd.php?we_cmd[0]=import_files&we_cmd[1]=" . $this->parentID . "&we_cmd[2]=" . ((string) $this->parentFixed) . "&cmd=content" . ($this->step > -1 ? '&step=' . $this->step : '') . '&we_cmd[3]=' . $this->cmdUploadFinished, 'position:absolute;top:0;bottom:40px;left:0;right:0;') .
				we_html_element::iframe('wizbusy', WEBEDITION_DIR . "we_cmd.php?we_cmd[0]=import_files&cmd=buttons" . ($this->step > -1 ? '&step=' . $this->step : '') . '&we_cmd[3]=' . $this->cmdUploadFinished . '&fu_file_parentFixed=' . ((string) $this->parentFixed), 'position:absolute;bottom:0;height:40px;left:0;right:0;overflow: hidden;')
		);

		return we_html_tools::getHtmlTop('', we_html_element::jsScript(JS_DIR . 'dialogs/we_dialog_base.js') . we_html_element::jsScript(JS_DIR . 'import_wizardBase.js'), $body);
	}

}
