<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 365cbfa650e7 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2021-04-09 19:18 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_navigation_ruleFrames{
	/** @var we_navigation_ruleControl */
	protected $Controller;

	/** @var we_base_jsCmd */
	public $jsCmd;

	function __construct(){
		$this->Controller = new we_navigation_ruleControl();
		//$weSuggest = we_gui_suggest::getInstance();
		$this->jsCmd = new we_base_jsCmd();
	}

	function getHTML(string $what): void{
		switch($what){
			case 'ruleFrameset':
				echo $this->getHTMLFrameset();
				break;
			case 'ruleContent':
				echo $this->getHTMLContent();
				break;
			default:
				t_e(__FILE__ . ": unknown reference $what");
		}
	}

	function getHTMLFrameset(): string{
		return we_html_tools::getHtmlTop(g_l('navigation', '[menu_highlight_rules]'), '', we_html_element::body(['class ' => 'weDialogBody']
					, we_html_element::iframe('content', WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod&mod=navigation&pnt=ruleContent', 'position:absolute;top:0;bottom:0;left:0;right:0;overflow: hidden') .
					we_html_element::iframe('cmdFrame', "about:blank", 'position:absolute;bottom:0;height:0;left:0;right:0;overflow: hidden;')
		));
	}

	function getHTMLContent(): string{
		// content contains textarea with all so far existing rules
		$weSuggest = we_gui_suggest::getInstance();

		$allRules = we_navigation_ruleControl::getAllNavigationRules();

		$rules = [];

		foreach($allRules as $navigationRule){
			$rules[$navigationRule->ID] = $navigationRule->NavigationName;
		}
		asort($rules);

		$parts = (new we_html_multiIconBoxContent())
			->add((new we_html_multiIconBoxContentLine(g_l('navigation', '[rules][available_rules]'), '<table class="default">
	<tr><td>' . we_html_tools::select('navigationRules', $rules, 8, '', false, ['style' => "width: 275px;", 'onclick' => "we_cmd('navigation_edit_rule', this.value)"]) . '</td>
		<td style="vertical-align:top">' . we_html_button::create_button(we_html_button::NEW_ENTRY, "javascript:we_cmd('new_navigation_rule');") . '<div style="height:10px;"></div>' . we_html_button::create_button(we_html_button::DELETE, "javascript:we_cmd('delete_navigation_rule')") . '
		</td>
	</tr>
</table>'
				))
				->space(we_html_multiIconBox::SPACE_BIG))
			->add((new we_html_multiIconBoxContentLine(g_l('navigation', '[rules][rule_name]'), we_html_tools::textInput('NavigationName', '', 0, ['style' => "width: 275px;", 'required' => 'required', 'placeholder' => 'New Rule']))) // FIXME: G_L
			->space(we_html_multiIconBox::SPACE_BIG)
			->noline());

		$weSuggest->setAcId('NavigationIDPath')
			->setContentType([we_base_ContentTypes::FOLDER_COMMON, we_base_ContentTypes::NAVIGATION])
			->setInput('NavigationIDPath')
			->setMaxResults(10)
			->setTable(NAVIGATION_TABLE)
			->setResult('NavigationID')
			->setSelector(we_gui_suggest::DocSelector)
			->setWidth(275)
			->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_file', {id:document.we_form.elements.NavigationID.value,table:WE().consts.tables.NAVIGATION_TABLE, targetID:'NavigationID', targetName:'NavigationIDPath'})"));


		$parts->add((new we_html_multiIconBoxContentLine(g_l('navigation', '[rules][rule_navigation_link]'), $weSuggest->getHTML()))
				->space(we_html_multiIconBox::SPACE_BIG)
				->noline());

		$selectionTypes = [we_navigation_navigation::DYN_DOCTYPE => g_l('global', '[documents]')];
		if(we_modules_info::isActive(we_modules_info::OBJECT)){
			$selectionTypes[we_navigation_navigation::DYN_CLASS] = g_l('global', '[objects]');
		}

		$parts->add((new we_html_multiIconBoxContentLine(g_l('navigation', '[rules][rule_applies_for]'), we_html_tools::select('SelectionType', $selectionTypes, 1, 0, false, [
						'style' => "width: 275px;", 'onchange' => "switchType(this.value);"])))
				->space(we_html_multiIconBox::SPACE_BIG)
		);

// getDoctypes
		$dtq = we_doctype_docTypes::getDoctypeQuery();
		$docTypes = [0 => g_l('navigation', '[no_entry]')] + DB_WE::$inst->getAllFirstq('SELECT dt.ID,dt.Text FROM ' . DOC_TYPES_TABLE . ' dt LEFT JOIN ' . FILE_TABLE . ' dtf ON dt.ParentFileID=dtf.ID ' . $dtq['join'] . ' WHERE ' . $dtq['where'], false);

		$weSuggest->setAcId('FolderIDPath')
			->setContentType([we_base_ContentTypes::FOLDER_DOCUMENT])
			->setInput('FolderIDPath')
			->setMaxResults(10)
			->setResult('FolderID')
			->setSelector(we_gui_suggest::DirSelector)
			->setWidth(275)
			->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_directory', {id:document.we_form.elements.FolderID.value,table:WE().consts.tables.FILE_TABLE, targetID:'FolderID', targetName:'FolderIDPath'})"))
			->setTrashButton(we_html_button::create_button(we_html_button::TRASH, "javascript:document.we_form.elements.FolderID.value = '';document.we_form.elements.FolderIDPath.value = '';"));

		$parts->add((new we_html_multiIconBoxContentLine(g_l('navigation', '[rules][rule_folder]'), $weSuggest->getHTML(), we_html_multiIconBox::PROP_PATH))
				->space(we_html_multiIconBox::SPACE_ICON)
				->noline()
				->rightHead()
				->class(' switch switch_doctype')
			)
			->add((new we_html_multiIconBoxContentLine(g_l('navigation', '[rules][rule_doctype]'), we_html_tools::select('DoctypeID', $docTypes, 1, 0, false, ['style' => "width: 275px;"])))
				->space(we_html_multiIconBox::SPACE_ICON)
				->noline()
				->rightHead()
				->class(' switch switch_doctype')
		);

		if(we_modules_info::isActive(we_modules_info::OBJECT)){
			$weSuggest->setAcId('ClassIDPath')
				->setContentType([we_base_ContentTypes::OBJECT])
				->setInput('ClassIDPath')
				->setMaxResults(10)
				->setResult('ClassID')
				->setSelector(we_gui_suggest::DocSelector)
				->setTable(OBJECT_TABLE)
				->setWidth(275)
				->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',{id:document.we_form.elements.ClassID.value,table:WE().consts.tables.OBJECT_TABLE,targetID:'ClassID',targetName:'ClassIDPath',jsCmd:'get_workspaces'})"));

			$weAcSelector = $weSuggest->getHTML();
			$parts->add((new we_html_multiIconBoxContentLine(g_l('navigation', '[rules][rule_class]'), $weAcSelector, we_html_multiIconBox::PROP_CLASS))
					->space(we_html_multiIconBox::SPACE_ICON)
					->noline()
					->rightHead()
					->class(' switch switch_classname')
				)
				->add((new we_html_multiIconBoxContentLine(g_l('navigation', '[rules][rule_workspace]'), we_html_tools::select('WorkspaceID', [], 1, '', false, ['style' => "width: 275px;"])))
					->space(we_html_multiIconBox::SPACE_ICON)
					->noline()
					->rightHead()
					->class(' switch switch_classname')
			);
		}

		$parts->add((new we_html_multiIconBoxContentLine(g_l('navigation', '[rules][rule_categories]'), $this->getHTMLCategory(), we_html_multiIconBox::PROP_CATEGORIES))
				->space(we_html_multiIconBox::SPACE_ICON)
				->preline()
				->rightHead()
				->fold()
				->hide(!we_modules_info::isActive(we_modules_info::CATEGORY))
		);

		$saveButton = we_html_button::create_button(we_html_button::SAVE, "javascript:we_cmd('save_navigation_rule');");
		$closeButton = we_html_button::create_button(we_html_button::CLOSE, "javascript:WE().util.dialogTop(window).doClose();");
		return we_html_tools::getHtmlTop('', we_html_element::jsScript(JS_DIR . 'formFunctions.js') .
				we_html_element::jsScript(WE_JS_MODULES_DIR . 'navigation/navigationRule.js', '', ['id' => 'loadVarNavigationRules', 'data-rules' => setDynamicVar([
						'dependencies' => [
							we_navigation_navigation::DYN_CLASS => ["ClassID", "WorkspaceID", "Categories"],
							we_navigation_navigation::DYN_DOCTYPE => ["FolderID", "DoctypeID", "Categories"]
						],
						'trashButton' => we_html_button::create_button(we_html_button::TRASH, "javascript:" . we_base_constants::PLACEHOLDER)
				])]), we_html_element::body(['onload' => "switchType(document.we_form.SelectionType.value)",
					'class' => 'weDialogBody'], we_html_element::form(['name' => we_base_constants::WE_FORM, 'target' => "cmdFrame", 'method' => "post", 'action' => WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod&mod=navigation&pnt=ruleCmd'], we_html_element::hiddens([
							'cmd' => '',
							'ID' => '0'
						]) .
							(new we_html_multiIconBox($parts, 'navigationRules'))->marginLeft()->buttons(we_html_button::position_yes_no_cancel($saveButton, '', $closeButton))
							->headline(g_l('navigation', '[rules][navigation_rules]'))
							->get()
					)
		));
	}

	private function getHTMLCategory(): string{
		$addbut = we_html_button::create_button(we_html_button::ADD, "javascript:we_cmd('we_selector_document',{id:0,table:WE().consts.tables.CATEGORY_TABLE,jsCmd:'addCat'})");

		$table = (new we_html_table(['id' => 'CategoriesBlock', 'style' => 'display: block;', 'class' => 'default withSpace'], 2, 1))
			->setCol(0, 0, we_html_element::div([
					'id' => 'categories',
					'class' => 'blockWrapper',
					'style' => 'width: 380px; height: 80px; border: #AAAAAA solid 1px;'
			]))
			->setCol(1, 0, we_html_button::create_button(we_html_button::DELETE_ALL, "javascript:removeAllCats()") . $addbut, ['colspan' => '2', 'style' => 'text-align:right']);

		return $table->getHtml() .
			we_html_element::hiddens([
				'CategoriesControl' => '0',
		]);
	}

	public function process(we_base_jsCmd $jscmd): void{
		$this->Controller->processVariables();
		$this->Controller->processCommands($jscmd);
	}

}
