<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: adcae6a39191 $
 * $Author: Lukas Imhof $
 * $Date: 2021-10-21 11:48 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_newsletter_frames extends we_modules_frame{

	const def_width = 450;
	const TAB_PROPERTIES = 0; //make sure to keep 1
	const TAB_MAILING = 1;
	const TAB_EDIT = 2;
	const TAB_REPORTING = 3;

	/** @var we_gui_suggest */
	private $weAutoCompleter;

	/** @var we_newsletter_view */
	protected $View;

	/** @var we_tree_newsletter */
	protected $Tree;

	/** @var we_newsletter_newsletter */
	private $Model;

	function __construct(){
		$this->module = we_modules_info::NEWSLETTER;
		parent::__construct();
		$this->View = new we_newsletter_view();
		$this->Model = &$this->View->Model;
		$this->Tree = new we_tree_newsletter($this->jsCmd);
		$this->weAutoCompleter = we_gui_suggest::getInstance();
	}

	public function getHTML(string $what = '', int $mode = 0, int $step = 0): string{
		switch($what){
			case 'domain_check':
				return $this->getHTMLDCheck();
			case 'show_log':
				return $this->getHTMLLog();
			case 'settings':
				return $this->getHTMLSettings();
			case 'print_lists':
				return $this->getHTMLPrintLists();
			case 'eemail':
				return $this->getHTMLEmailEdit();
			case 'preview':
				return $this->getHTMLPreview();
			case 'black_list':
				return $this->getHTMLBlackList();
			case 'upload_black':
				return $this->getHTMLUploadCsv($what);
			case 'upload_csv':
				return $this->getHTMLUploadCsv($what);
			case 'export_csv_mes':
				return $this->getHTMLExportCsvMessage();
			case 'edit_file':
				return $this->getHTMLEditFile();
			case 'clear_log':
				return $this->getHTMLClearLog();
			case 'send':
				return $this->getHTMLSendWait();
			case 'send_frameset':
				return $this->getHTMLSendFrameset();
			case 'send_body':
				return $this->getHTMLSendBody();
			case 'send_cmd':
				return $this->getHTMLSendCmd();
			case 'send_control':
				return $this->getHTMLSendControl();
			default:
				return parent::getHTML($what, $mode, $step);
		}
	}

	/**
	 * Modul Header
	 *
	 * @return String
	 */
	protected function getHTMLEditorHeader(): string{
		if(we_base_request::bool('home')){
			return parent::getHTMLEditorHeader();
		}

		$group = we_base_request::bool('group');
		$page = ($group ? self::TAB_PROPERTIES : we_base_request::int('page', self::TAB_PROPERTIES));

		$textPre = g_l('modules_newsletter', ($group ? '[group]' : '[newsletter][text]'));
		$textPost = we_base_request::string("txt", g_l('modules_newsletter', ($group ? '[new_newsletter_group]' : '[new_newsletter]')));

		$we_tabs = new we_gui_tabs();
		$we_tabs->addTab('', we_gui_tabs::WE_ICON_PROPERTIES, ($page == self::TAB_PROPERTIES), (string) (self::TAB_PROPERTIES), ['title' => g_l('modules_newsletter', '[property]')]);

		if(!$group){
			$we_tabs->addTab('', 'fa-list', ($page == self::TAB_MAILING), (string) (self::TAB_MAILING), ['title' => sprintf(g_l('modules_newsletter', '[mailing_list]'), "")]);
			$we_tabs->addTab('', we_gui_tabs::WE_ICON_EDIT, ($page == self::TAB_EDIT), (string) (self::TAB_EDIT), ['title' => g_l('modules_newsletter', '[edit]')]);
			//if($this->View->newsletter->ID){ // zusaetzlicher tab fuer auswertung
			$we_tabs->addTab('', 'fa-pie-chart', ($page == self::TAB_REPORTING), (string) (self::TAB_REPORTING), ['title' => g_l('modules_newsletter', '[reporting][tab]')]);
			//}
		}

		$body = we_html_element::body(['onresize' => "weTabs.setFrameSize()", 'onload' => "weTabs.setFrameSize()", 'id' => 'eHeaderBody'], '<div id="main"><div id="headrow"><b>' . htmlspecialchars($textPre) . ':&nbsp;</b><span id="h_path" class="header_small"><b id="titlePath">' . htmlspecialchars($textPost) . '</b></span></div>' .
						$we_tabs->getHTML() .
						'</div>'
		);
		return $this->getHTMLDocument($body, we_html_element::cssLink(CSS_DIR . 'we_tab.css') . we_html_element::jsScript(JS_DIR . 'initTabs.js') . we_html_element::jsScript(WE_JS_MODULES_DIR . 'newsletter/newsletter_header.js'));
	}

	/**
	 * Modul Body
	 *
	 * @return String
	 */
	protected function getHTMLEditorBody(): string{
		return $this->getHTMLProperties();
	}

	/**
	 * Modul Footer
	 *
	 * @return string
	 */
	protected function getHTMLEditorFooter(array $btn_cmd = []): string{
		if(we_base_request::bool('home')){
			return parent::getHTMLEditorFooter([]);
		}
		$group = we_base_request::int('group', 0);

		$select = new we_html_select(['name' => 'gview']);

		$main = $optional = '';
		$main .= ((we_base_permission::hasPerm([we_base_perm::NEWSLETTER_NEW, we_base_perm::NEWSLETTER_EDIT])) ? we_html_button::create_button(we_html_button::FOOTER_SAVE, "javascript:we_save()") : '');
		$optional .= ((we_base_permission::hasPerm([we_base_perm::NEWSLETTER_DELETE])) ? we_html_button::create_button(we_html_button::FOOTER_TRASH, "javascript:top.content.we_cmd('delete_newsletter')") : '');

		if(!$group){
			$main .= $select->getHtml() .
					we_html_button::create_button(we_html_button::PREVIEW, "javascript:we_cmd('popPreview')") .
					(we_base_permission::hasPerm([we_base_perm::NEWSLETTER_SEND]) ? we_html_button::create_button(we_html_button::SEND, "javascript:we_cmd('popSend')") : '');
		}

		$body = we_html_element::body(['id' => "footerBody", 'onload' => "afterLoad();"], we_html_element::form([], we_html_element::hidden('hm', 0) .
								we_html_element::div(['class' => 'footertable'], we_html_element::div(['class' => 'main'], $main) . we_html_element::div(['class' => 'optional'], $optional))
						)
		);

		return $this->getHTMLDocument($body, we_html_element::jsScript(WE_JS_MODULES_DIR . 'newsletter/frames_footer.js'));
	}

	private function getHTMLLog(): string{
		$start = we_base_request::_(we_base_request::DATE_TIME, 'newsletterStartTime');
		$end = we_base_request::_(we_base_request::DATE_TIME, 'newsletterEndTime', new DateTime());
		$status = we_base_request::string('newsletterStatus');

		return $this->getHTMLDocument(
						we_html_element::body(['class' => 'weDialogBody', 'onload' => "self.focus();"], we_html_element::form(['name' => we_base_constants::WE_FORM, 'method' => 'post'], we_html_tools::dialogLayout(
												we_html_element::div(['class' => "blockWrapper", 'style' => "width: 588px; height: 500px; border:1px #dce6f2 solid;margin:5px 10px 15px 10px;"], $this->View->getHTMLLog($status, $start, $end))
												, g_l('modules_newsletter', '[show_log]'), we_html_button::create_button(we_html_button::CLOSE, "javascript:WE().util.dialogTop(window).doClose();")
										)
								)
		));
	}

	protected function getHTMLCmd(): string{
		if(($pid = we_base_request::int('pid', -1)) < 0){
			return $this->getHTMLDocument(we_html_element::body(), $this->jsCmd->getCmds());
		}
		$this->jsCmd->addCmd('loadTree', ['clear' => !$pid, 'items' => we_tree_newsletter::getItems($pid), 'sorted' => true]);

		return $this->getHTMLDocument(we_html_element::body([], we_html_element::form(['name' => we_base_constants::WE_FORM], we_html_element::hiddens([
											'mod' => 'newsletter',
											'pnt' => "cmd",
											'ncmd' => '',
											'nopt' => ''
										])
								)
		));
	}

	private function getHTMLPrintLists(): string{
		$out = '<ul>';
		$count = count($this->Model->groups) + 1;

		$c = 0;
		for($k = 1; $k < $count; $k++){
			$out .= '<li>' . we_html_element::b(sprintf(g_l('modules_newsletter', '[mailing_list]'), $k)) . '<ul>';
			$gc = 0;
			if(we_modules_info::isActive(we_modules_info::CUSTOMER)){
				$out .= '<li>' . g_l('modules_newsletter', '[customers]') . '<ul>';
				$emails = $this->View->getEmails($k, we_newsletter_view::MAILS_CUSTOMER);

				foreach($emails as $email){
					$gc++;
					$out .= '<li>' . $email . '</li>';
				}
				$out .= '</ul>';
			}
			$out .= '</ul>
<li>' . g_l('modules_newsletter', '[emails]') . '<ul>';

			$emails = $this->View->getEmails($k, we_newsletter_view::MAILS_EMAILS);
			foreach($emails as $email){
				$gc++;
				$out .= '<li>' . $email . '</li>';
			}

			$out .= '</ul><li>' . g_l('modules_newsletter', '[file_email]') . '<ul>';

			$emails = $this->View->getEmails($k, we_newsletter_view::MAILS_FILE);
			foreach($emails as $email){
				$gc++;
				$out .= '<li>' . $email . '</li>';
			}

			$c += $gc;
			$out .= '</ul></ul><li>' . we_html_element::b(sprintf(g_l('modules_newsletter', '[sum_group]'), $k) . ':' . $gc) . '</li>';
		}

		$out .= '<li>' . we_html_element::b(g_l('modules_newsletter', '[sum_all]') . ":" . $c) . '</ul>';
		return self::getHTMLDocument(we_html_element::body(['class' => 'weDialogBody'], we_html_element::form(['name' => we_base_constants::WE_FORM, 'method' => 'post',
									'onload' => "self.focus()"], we_html_tools::dialogLayout(
												we_html_element::br() .
												we_html_element::div(['class' => "blockWrapper", 'style' => "width: 588px; height: 500px; border:1px #dce6f2 solid;"], $out) .
												we_html_element::br(), g_l('modules_newsletter', '[lists_overview]'), we_html_button::create_button(we_html_button::CLOSE, "javascript:WE().util.dialogTop(window).doClose();")
										)
		)));
	}

	private function getHTMLDCheck(): string{
		$domain = '';
		$count = count($this->Model->groups) + 1;

		$out = '<ul>' .
				'<li>' . we_html_element::b(g_l('modules_newsletter', '[domain_check_begins]')) . '<ul>';

		for($k = 1; $k < $count; $k++){
			$out .= '<li>' . sprintf(g_l('modules_newsletter', '[domain_check_list]'), $k) . '<ul>';
			$emails = $this->View->getEmails($k, we_newsletter_view::MAILS_ALL);
			foreach($emails as $email){
				'@phan-var string $email';
				if(we_check_email($email)){
					if(!we_base_util::we_checkMailDomain($email, $domain)){
						$out .= '<li>' . sprintf(g_l('modules_newsletter', '[domain_nok]'), $domain) . '</li>';
					}
				} else {
					$out .= '<li>' . sprintf(g_l('modules_newsletter', '[email_malformed]'), $email) . '</li>';
				}
			}
			$out .= '</ul>';
		}
		$out .= '</ul>
<li>' . we_html_element::b(g_l('modules_newsletter', '[domain_check_ends]')) . '</li></ul>';
		return self::getHTMLDocument(we_html_element::body(['class' => 'weDialogBody'], we_html_element::form(['name' => we_base_constants::WE_FORM, 'method' => 'post',
									'onload' => "self.focus()"], we_html_tools::dialogLayout(
												we_html_element::br() .
												we_html_element::div(['class' => "blockWrapper", 'style' => "width: 588px; height: 500px; border:1px #dce6f2 solid;"], $out) .
												we_html_element::br(), g_l('modules_newsletter', '[lists_overview]'), we_html_button::create_button(we_html_button::CLOSE, "javascript:WE().util.dialogTop(window).doClose();")
										)
		)));
	}

	private function getHTMLSettings(): string{
		$settings = we_newsletter_view::getSettings();

		if(we_base_request::string('ncmd') === 'save_settings'){
			$this->View->processCommands($this->jsCmd);
			$this->jsCmd->addCmd('settings_close');
		}

		$js = $this->View->getJSProperty();

		$extra_radio_text = ['use_port'];
		$defaults = [
			'reject_save_malformed' => '1',
			'use_https_refer' => '0',
			'send_wait' => '0',
			'use_port' => '0',
			'use_port_check' => '80',
			'isEmbedImages' => '0',
			'use_base_href' => '1'
		];

		$table = new we_html_table(['class' => 'default withSpace', 'style' => 'width:100%;'], 1, 2);
		$c = 0;

		foreach([
	'send_step' => 'number',
	'send_wait' => 'number',
	'test_account' => 'text',
	'default_sender' => 'text',
	'default_reply' => 'text',
	we_newsletter_newsletter::FEMALE_SALUTATION_FIELD => 'text',
	we_newsletter_newsletter::MALE_SALUTATION_FIELD => 'text'
		] as $text => $type){
			if(!isset($settings[$text])){
				$val = ($defaults[$text] ?? '0');
				$this->View->saveSetting($text, $val);
				$settings[$text] = $val;
			}

			$table
					->setCol($c, 0, g_l('modules_newsletter', '[' . $text . ']', false, true) . ':&nbsp;')
					->setCol($c, 1, we_html_tools::textInput($text, (string) $settings[$text], 0, [], $type));

			$c++;
		}

		if(we_modules_info::isActive(we_modules_info::CUSTOMER)){
			$customerFields = $customerTextFields = ['' => '---'];
			$customerMetas = $this->db->metadata(CUSTOMER_TABLE);
			foreach($this->View->customers_fields as $fv){
				$customerFields[$fv] = $fv;
				$customerTextFields[$fv] = $fv !== 'Password' && isset($customerMetas[$fv]) &&
						($customerMetas[$fv]['type'] === 'text' || ($customerMetas[$fv]['type'] === 'varchar' && (int) $customerMetas[$fv]['len'] >= 64)) ? $fv : '';
			}
			$customerTextFields = array_filter($customerTextFields);

			$table->setCol($c, 0, g_l('modules_newsletter', '[customer_email_field]', false, true) . ':&nbsp;')
					->setCol($c, 1, we_html_tools::select("customer_email_field", $customerTextFields, 1, $settings['customer_email_field'], false, [], true))
					->setCol(++$c, 0, g_l('modules_newsletter', '[customer_salutation_field]', false, true) . ':&nbsp;')
					->setCol($c, 1, we_html_tools::select('customer_salutation_field', $customerFields, 1, $settings['customer_salutation_field'], false, [], true))
					->setCol(++$c, 0, g_l('modules_newsletter', '[customer_title_field]', false, true) . ':&nbsp;')
					->setCol($c, 1, we_html_tools::select('customer_title_field', $customerFields, 1, $settings['customer_title_field'], false, [], true))
					->setCol(++$c, 0, g_l('modules_newsletter', '[customer_firstname_field]', false, true) . ':&nbsp;')
					->setCol($c, 1, we_html_tools::select('customer_firstname_field', $customerTextFields, 1, $settings['customer_firstname_field'], false, [], true))
					->setCol(++$c, 0, g_l('modules_newsletter', '[customer_lastname_field]', false, true) . ':&nbsp;')
					->setCol($c, 1, we_html_tools::select('customer_lastname_field', $customerTextFields, 1, $settings['customer_lastname_field'], false, [], true));
		}


		$radios_code = '';
		foreach([
	'reject_malformed' => 'text',
	'reject_not_verified' => 'text',
	'reject_save_malformed' => 'text',
	'log_sending' => 'text',
	'isEmbedImages' => 'text',
	'title_or_salutation' => 'text',
	'use_base_href' => 'text',
	'use_https_refer' => 'text',
	'use_port' => 'number'
		] as $radio => $type){
			if(!isset($settings[$radio])){
				$val = ($defaults[$radio] ?? '1');
				$this->View->saveSetting($radio, $val);
				$settings[$radio] = $val;
			}
			if(in_array($radio, $extra_radio_text)){
				$radios_code .= we_html_forms::checkbox((string) $settings[$radio], (($settings[$radio] > 0)), $radio . '_check', g_l('modules_newsletter', '[' . $radio . '_check]', false, true), false, '', "if(document.we_form." . $radio . "_check.checked){document.we_form." . $radio . ".value=" . ($defaults[$radio . '_check'] ?? 0) . ";}else{document.we_form." . $radio . ".value=0;}");

				$radio_table = (new we_html_table(['class' => 'default', 'style' => 'margin-left:25px;'], 1, 2))
						->setCol(0, 0, htmlspecialchars(g_l('modules_newsletter', '[' . $radio . ']', false, true)) . ':&nbsp;')
						->setCol(0, 1, we_html_tools::textInput($radio, (string) $settings[$radio], 0, ['onchange' => "if(document.we_form." . $radio . ".value!=0){document.we_form." . $radio . "_check.checked=true;}else{document.we_form." . $radio . "_check.checked=false;}"], $type), [
					'style' => 'padding-left:5px;']);
				$radios_code .= $radio_table->getHtml();
			} else {
				$radios_code .= we_html_forms::checkbox((string) $settings[$radio], (($settings[$radio] == 1)), $radio, htmlspecialchars(g_l('modules_newsletter', '[' . $radio . ']', false, true)), false, '', "if(document.we_form." . $radio . ".checked){document.we_form." . $radio . ".value=1;}else{document.we_form." . $radio . ".value=0;}") . '<br/>';
			}
		}

		$deselect = we_html_button::create_button(we_html_button::TRASH, "javascript:document.we_form.global_mailing_list.value=''");

		$gml_table = (new we_html_table(['class' => 'default withSpace', 'style' => 'width:538px;margin:10px;'], 4, 2))
				->setCol(0, 0, g_l('modules_newsletter', '[global_mailing_list]', false, true))
				->setCol(2, 0, $this->formFileChooser('global_mailing_list', $settings['global_mailing_list']))
				->setCol(2, 1, $deselect, ['style' => 'text-align:right']);

		$body = we_html_element::body(['class' => 'weDialogBody fullSizeInput', 'onload' => "self.focus();"], we_html_element::form(['name' => we_base_constants::WE_FORM], $this->View->getHiddens() .
								we_html_tools::dialogLayout(
										$table->getHtml() .
										$radios_code .
										$gml_table->getHtml(), g_l('modules_newsletter', '[settings]', false, true), we_html_button::position_yes_no_cancel(we_html_button::create_button(we_html_button::SAVE, "javascript:we_cmd('save_settings')"), we_html_button::create_button(we_html_button::CLOSE, "javascript:we_cmd('settings_close')"))
								)
						)
		);

		return $this->getHTMLDocument($body, $js);
	}

	function getHTMLBlockType(string $name, int $selected = 1): string{
		$values = [we_newsletter_block::DOCUMENT => g_l('modules_newsletter', '[newsletter_type_0]'),
			we_newsletter_block::DOCUMENT_FIELD => g_l('modules_newsletter', '[newsletter_type_1]'),
		];

		if(we_modules_info::isActive(we_modules_info::OBJECT)){
			$values[we_newsletter_block::OBJECT] = g_l('modules_newsletter', '[newsletter_type_2]');
			$values[we_newsletter_block::OBJECT_FIELD] = g_l('modules_newsletter', '[newsletter_type_3]');
		}

		if(we_base_permission::hasPerm([we_base_perm::NEWSLETTER_FILES])){
			$values[we_newsletter_block::FILE] = g_l('modules_newsletter', '[newsletter_type_4]');
		}
		$values[we_newsletter_block::TEXT] = g_l('modules_newsletter', '[newsletter_type_5]');
		$values[we_newsletter_block::ATTACHMENT] = g_l('modules_newsletter', '[newsletter_type_6]');
		$values[we_newsletter_block::URL] = g_l('modules_newsletter', '[newsletter_type_7]');

		return we_html_tools::select($name, $values, 1, $selected, false, ['style' => "width:440px;", 'onchange' => "we_cmd('switchPage','',2);"], true, 315);
	}

	function getHTMLCopy(): string{
		return we_html_element::hiddens([
					'copyid' => '0',
					'copyid_text' => '']) .
				we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_file',{id:document.we_form.elements.copyid.value,table:WE().consts.tables.NEWSLETTER_TABLE,targetID:'copyid',targetName:'copyid_text',jsCmd:'copy_newsletter',rootDirID:'" . implode(',', we_users_util::get_ws(NEWSLETTER_TABLE)) . "'})");
	}

	private function getHTMLCustomer(int $group): string{
		$out = we_html_forms::checkbox('1', $this->Model->groups[$group]->isSendAll(), 'sendallcheck_' . $group, g_l('modules_newsletter', '[send_all]'), false, '', "we_cmd('switch_sendall',$group);");
		$out .= '(' . g_l('modules_newsletter', '[reporting][mailing_all_emails]') . ' ' . count($this->View->getEmails(($group + 1), we_newsletter_view::MAILS_ALL)) . ')';

		if(!$this->Model->groups[$group]->isSendAll()){

			$delallbut = we_html_button::create_button(we_html_button::DELETE_ALL, "javascript:we_cmd('del_all_customers'," . $group . ")");
			$addbut = we_html_button::create_button(we_html_button::ADD, "javascript:we_cmd('we_customer_selector',{jsCmd:'add_customer," . $group . "',multiple:true})");

			$cats = new we_chooser_multiDir(array_filter($this->Model->groups[$group]->getCustomers()), 'del_customer', $delallbut . $addbut, [], '"' . we_base_ContentTypes::CUSTOMER . '"', CUSTOMER_TABLE, '', '', 'document.we_form.ngroup.value=' . $group);
			$out .= $cats->get($this->jsCmd);
		}

		return $out . $this->getHTMLCustomerFilter($group);
	}

	private function getHTMLExtern(int $group): string{
		$delallbut = we_html_button::create_button(we_html_button::DELETE_ALL, "javascript:we_cmd('del_all_files'," . $group . ")");
		$addbut = we_html_button::create_button(we_html_button::ADD, "javascript:we_cmd('browse_server','fileselect',WE().consts.contentTypes.TEXT,'/','add_file,top.currentID," . $group . "','',1);");

		$buttons = $delallbut . (we_base_permission::hasPerm([we_base_perm::WORK_EXTERNAL_FILES_CAN_SELECT]) ? $addbut : '');

		$csvs = new we_chooser_multiFile($this->Model->groups[$group]->getExtern(), 'del_file', $buttons, 'edit_file', we_base_ContentTypes::FOLDER_DOCUMENT, we_base_ContentTypes::APPLICATION, 'document.we_form.ngroup.value=' . $group);
		$csvs->setEditable(true);

		return we_html_element::hiddens(['fileselect' => '']) .
				$csvs->get($this->jsCmd);
	}

	private function getHTMLCustomerFilter(int $group): string{
		$custfields = [];
		foreach($this->View->customers_fields as $fv){
			switch($fv){
				case 'ParentID':
				case 'IsFolder':
				case 'Path':
				case 'Text' :
					continue 2;
				default:
					$custfields[$fv] = $fv;
			}
		}

		$operators = [
			we_newsletter_newsletter::OP_EQ => '=',
			we_newsletter_newsletter::OP_NEQ => '<>',
			we_newsletter_newsletter::OP_LE => '<',
			we_newsletter_newsletter::OP_LEQ => '<=',
			we_newsletter_newsletter::OP_GE => '>',
			we_newsletter_newsletter::OP_GEQ => '>=',
			we_newsletter_newsletter::OP_CONTAINS => g_l('modules_newsletter', '[operator][contains]'),
			we_newsletter_newsletter::OP_STARTS => g_l('modules_newsletter', '[operator][startWith]'),
			we_newsletter_newsletter::OP_ENDS => g_l('modules_newsletter', '[operator][endsWith]'),
			we_newsletter_newsletter::OP_LIKE => 'LIKE',
		];
		$logic = [
			'AND' => g_l('modules_newsletter', '[logic][and]'),
			'OR' => g_l('modules_newsletter', '[logic][or]')
		];
		$hours = [];
		for($i = 0; $i < 24; $i++){
			$hours[] = ($i <= 9 ? '0' : '') . $i;
		}
		$minutes = [];
		for($i = 0; $i < 60; $i++){
			$minutes[] = ($i <= 9 ? '0' : '') . $i;
		}

		$filter = $this->Model->groups[$group]->getFilter();

		$colspan = 6;
		$table = (new we_html_table(['class' => 'default'], 1, $colspan))
				->setCol(0, 0, we_html_forms::checkbox(($filter ? '1' : '0'), ($filter ? true : false), 'filtercheck_' . $group, g_l('modules_newsletter', '[filter]'), false, '', "if(document.we_form.filtercheck_" . $group . ".checked){ we_cmd('add_filter'," . $group . ",0); }else{ we_cmd('del_all_filters'," . $group . ");}"), (($filter) ? [
			'colspan' => $colspan] : []));

		$c = 1;
		if($filter){
			foreach($filter as $k => $v){
				if($k != 0){
					$table->setCol($c, 0, we_html_tools::select('filter_logic_' . $group . '_' . $k, $logic, 1, $v['logic'], false, [], true, 70), ['colspan' => $colspan]);
					$c++;
				}

				$table
						->setCol($c, 0, we_html_tools::select('filter_fieldname_' . $group . '_' . $k, $custfields, 1, $v['fieldname'], false, ['onchange' => "top.content.hot=true;changeFieldValue(this.val,'filter_fieldvalue_" . $group . "_" . $k . "');"], true, 170))
						->setCol($c, 1, we_html_tools::select('filter_operator_' . $group . '_' . $k, $operators, 1, $v['operator'], false, ['onchange' => "top.content.hot=true;"], true, 80));
				switch($v['fieldname']){
					case 'MemberSince_Date':
					case 'LastLogin_Date':
					case 'LastAccess_Date':
						$table
								->setCol($c, 2, we_html_tools::getNewDateInput('filter_fieldvalue_' . $group . '_' . $k, new DateTime($v['fieldvalue'] ?? 'now'), we_html_tools::FORMAT_DATE), [
									'id' => 'td_value_fields_' . $group . '_' . $k])
								->setCol($c, 3, we_html_tools::select('filter_hours_' . $group . '_' . $k, $hours, 1, $v['hours'] ?? '', false, ['onchange' => "top.content.hot=true;"]) . '&nbsp;h :')
								->setCol($c, 4, we_html_tools::select('filter_minutes_' . $group . '_' . $k, $minutes, 1, $v['minutes'] ?? '', false, [
											'onchange' => "top.content.hot=true;"]) . '&nbsp;m');
						break;
					default:
						$table->setCol($c, 2, we_html_tools::textInput('filter_fieldvalue_' . $group . '_' . $k, $v['fieldvalue'] ?? '', 0, [
									'oninput' => "top.content.hot=true;"], 'text', '200px'), ['colspan' => '3', 'id' => 'td_value_fields_' . $group . '_' . $k]);
				}
				$table->setCol($c, 5, we_html_button::create_button(we_html_button::PLUS, "javascript:we_cmd('add_filter'," . $group . "," . ($k + 1) . ")") .
						we_html_button::create_button(we_html_button::TRASH, "javascript:we_cmd('del_filter'," . $group . "," . $k . ")"));
				$c++;
			}
		}
		return we_html_element::hiddens(['filter_' . $group => (string) (count($filter))]) .
				$table->getHtml();
	}

	/**
	 *
	 * @param string $name
	 * @param string[] $values
	 * @param int $size
	 * @param string $selectedIndex
	 * @param bool $multiple
	 * @param string $attribs
	 * @param string $compare
	 * @param int $width
	 * @param string $cls
	 * @return string
	 */
	private function htmlSelectEmailList(string $name, array $values, int $size = 1, string $selectedIndex = '', bool $multiple = false, string $attribs = '', string $compare = 'value', int $width = 0, string $cls = 'defaultfont'): string{
		reset($values);
		$ret = '<select class="' . $cls . '" name="' . trim($name) . '" size=' . abs($size) . ' ' . ($multiple ? " multiple" : "") . ($attribs ? " $attribs" : "") . ($width ? ' style="width: ' . $width . 'px"' : '') . '>';
		$selIndex = array_filter(explode(',', $selectedIndex));
		foreach($values as $value => $text){
			'@phan-var string $text';
			$ret .= '<option value="' . htmlspecialchars((string) $value) . '"' . (in_array((($compare === 'value') ? $value : $text), $selIndex) ? ' selected="selected"' : '') . (we_check_email($text) ? ' class="markValid"' : ' class="markNotValid"') . '>' . $text . '</option>';
		}
		return $ret . '</select>';
	}

	/**
	 * Mailing list - block Emails
	 *
	 *
	 * @param integer $group
	 * @return string
	 */
	function getHTMLEmails(int $group): string{
		$arr = we_newsletter_base::getEmailsFromList(htmlspecialchars($this->Model->groups[$group]->Emails));
		'@phan-var string[] $arr';
		// Buttons to handle the emails in  the email list
		$buttons_table = (new we_html_table(['class' => 'default withSpace'], 4, 1))
				->setCol(0, 0, we_html_button::create_button(we_html_button::ADD, "javascript:we_cmd('add_email', " . $group . ");"))
				->setCol(1, 0, we_html_button::create_button(we_html_button::EDIT, "javascript:we_cmd('edit_email', " . $group . ");"))
				->setCol(2, 0, we_html_button::create_button(we_html_button::DELETE, "javascript:deleteit(" . $group . ")"))
				->setCol(3, 0, we_html_button::create_button(we_html_button::DELETE_ALL, "javascript:deleteall(" . $group . ")"));

		// 1. ROW: select status
		$selectStatus = we_html_element::b(g_l('modules_newsletter', '[status]')) . ' ' . we_html_tools::select('weEmailStatus', [
					we_newsletter_newsletter::CSV_STATUS_ALL => g_l('modules_newsletter', '[statusAll]'),
					we_newsletter_newsletter::CSV_STATUS_INVALID => g_l('modules_newsletter', '[statusInvalid]'),
					we_newsletter_newsletter::CSV_STATUS_VALID => g_l('modules_newsletter', '[statusValid]')
						], 0, we_base_request::int('weEmailStatus', 0), false, [
					'onchange' => "weShowMailsByStatus(this.value, $group);",
					'id' => 'weViewByStatus'], true, 150);

		// 2. ROW: Mail list with handling buttons
		$table = (new we_html_table(['class' => 'default withSpace'], 3, 3))
				->setCol(0, 0, $selectStatus, ['style' => 'vertical-align:middle;', 'colspan' => '3'])

				// 3. ROW: Buttons for email import and export
				->setCol(1, 0, $this->htmlSelectEmailList('we_recipient' . $group, $arr, 10, '', false, 'style="width:' . (self::def_width - 110) . 'px; height:140px" id="we_recipient' . $group . '"', 'value', 600), [
					'style' => 'vertical-align:top;'])
				->setColAttributes(1, 1, ['style' => 'vertical-align:middle;width:10px;'])
				->setCol(1, 2, $buttons_table->getHtml(), ['style' => 'vertical-align:top;'])
				->setCol(2, 0, we_html_button::create_button(we_html_button::IMPORT, "javascript:we_cmd('set_import'," . $group . ")") . we_html_button::create_button(we_html_button::EXPORT, "javascript:we_cmd('set_export'," . $group . ")"), [
			'colspan' => '3']);
		// Import dialog
		// Import dialog
		if($this->View->getShowImportBox() === $group){
			$ok = we_html_button::create_button(we_html_button::OK, "javascript:we_cmd('import_csv')");
			$cancel = we_html_button::create_button(we_html_button::CANCEL, "javascript:we_cmd('reset_import');");

			$row = 0;
			$import_options = (new we_html_table(['class' => 'default withSpace'], 6, 3))
					->setCol($row, 0, g_l('modules_newsletter', '[csv_delimiter]') . ':')
					->setCol($row++, 1, we_html_tools::textInput('csv_delimiter' . $group, ','))
					->setCol($row, 0, g_l('modules_newsletter', '[csv_col]') . ':')
					->setCol($row++, 1, we_html_tools::textInput('csv_col' . $group, '1'))
					->setCol($row, 0, g_l('modules_newsletter', '[csv_salutationcol]') . ':')
					->setCol($row, 1, we_html_tools::textInput('csv_salutationcol' . $group, '2'))
					->setCol($row++, 2, '&nbsp;' . g_l('modules_newsletter', '[csv_salutation_explain]'), ['class' => 'lowContrast'])
					->setCol($row, 0, g_l('modules_newsletter', '[csv_titlecol]') . ':')
					->setCol($row, 1, we_html_tools::textInput('csv_titlecol' . $group, '3'))
					->setCol($row++, 2, '&nbsp;' . g_l('modules_newsletter', '[csv_title_explain]'), ['class' => 'lowContrast'])
					->setCol($row, 0, g_l('modules_newsletter', '[csv_firstnamecol]') . ':')
					->setCol($row, 1, we_html_tools::textInput('csv_firstnamecol' . $group, '4'))
					->setCol($row++, 2, '&nbsp;' . g_l('modules_newsletter', '[csv_firstname_explain]'), ['class' => 'lowContrast'])
					->setCol($row, 0, g_l('modules_newsletter', '[csv_lastnamecol]') . ':')
					->setCol($row, 1, we_html_tools::textInput('csv_lastnamecol' . $group, '5'))
					->setCol($row, 2, '&nbsp;' . g_l('modules_newsletter', '[csv_lastname_explain]'), ['class' => 'lowContrast']);


			$import_box = (new we_html_table(['class' => 'default withSpace', 'style' => 'margin-top:10px;'], 4, 1))
					->setCol(0, 0, $this->formFileChooser('csv_file' . $group, '/', ''))
					->setCol(1, 0, we_html_button::create_button(we_html_button::UPLOAD, "javascript:we_cmd('upload_csv'," . $group . ")"))
					->setCol(2, 0, $import_options->getHtml())
					->setCol(3, 0, $ok . $cancel);

			$table->setCol(5, 0, we_html_element::hiddens(['csv_import' => (string) ($group)]) . $import_box->getHtml(), ['colspan' => '3']);
		}

		// Export dialog
		if($this->View->getShowExportBox() === $group){
			$ok = we_html_button::create_button(we_html_button::OK, "javascript:we_cmd('export_csv')");
			$cancel = we_html_button::create_button(we_html_button::CANCEL, "javascript:we_cmd('reset_import');");

			$export_box = (new we_html_table(['class' => 'default withSpace', 'style' => 'margin-top:10px;'], 2, 1))
					->setCol(1, 0, $this->formFileChooser('csv_dir' . $group, '/', '', we_base_ContentTypes::FOLDER_DOCUMENT))
					->setCol(2, 0, $ok . $cancel);

			$table->setCol(5, 0, we_html_element::hiddens(['csv_export' => (string) ($group)]) . $export_box->getHtml(), ['colspan' => '3']);
		}

		return $table->getHtml();
	}

	private function formWeChooser(string $table = FILE_TABLE, int $width = 0, int $rootDirID = 0, string $IDName = 'ID', int $IDValue = 0, string $Pathname = 'Path', string $Pathvalue = '/', string $cmd = '', bool $open_doc = false, $contenttype = [
				we_base_ContentTypes::FOLDER_TEMPLATE, we_base_ContentTypes::TEMPLATE]): string{
		$Pathvalue = $Pathvalue ?: fS('SELECT Path FROM ' . $this->db->escape($table) . ' WHERE ID=' . $IDValue);

		$button = we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',{id:document.we_form.elements['" . $IDName . "'].value,table:'" . $table . "',targetID:'" . $IDName . "',targetName:'" . $Pathname . "',jsCmd:'" . $cmd . "',rootDirID:'" . $rootDirID . "',open_doc:'" . ((string) $open_doc) . "'})");

		$weSuggest = $this->weAutoCompleter;
		$weSuggest->setAcId($IDName)
				->setContentType($contenttype)
				->setInput($Pathname, $Pathvalue)
				->setMaxResults(10)
				->setResult($IDName, (string) ($IDValue))
				->setSelector(we_gui_suggest::DocSelector)
				->setTable($table)
				->setWidth($width)
				->setSelectButton($button);
		return $weSuggest->getHTML();
	}

	private function formWeDocChooser(string $table = FILE_TABLE, int $width = 0, int $rootDirID = 0, string $IDName = 'ID', int $IDValue = 0, string $Pathname = 'Path', string $Pathvalue = '/', string $cmd = '', string $filter = we_base_ContentTypes::WEDOCUMENT): string{
		$Pathvalue = $Pathvalue ?: fS('SELECT Path FROM ' . $this->db->escape($table) . ' WHERE ID=' . $IDValue);

		$button = we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',{id:document.we_form.elements['" . $IDName . "'].value,table:'" . $table . "',targetID:'" . $IDName . "',targetName:'" . $Pathname . "',jsCmd:'" . $cmd . "',rootDirID:'" . $rootDirID . "',filter:'" . $filter . "',open_doc:" . (we_base_permission::hasPerm([
							we_base_perm::WORK_WORKSPACE_CAN_SELECT_OTHER_FILES]) ? 0 : 1) . "})");
		$weSuggest = $this->weAutoCompleter;
		$weSuggest->setAcId($IDName)
				->setContentType(array_merge(we_base_ContentTypes::LINK_ABLE, [we_base_ContentTypes::FOLDER_DOCUMENT]))
				->setInput($Pathname, $Pathvalue)
				->setMaxResults(10)
				->setResult($IDName, (string) ($IDValue))
				->setSelector(we_gui_suggest::DocSelector)
				->setTable($table)
				->setWidth($width)
				->setSelectButton($button);
		return $weSuggest->getHTML();
	}

	private function getHTMLNewsletterBlocks(): string{
		$counter = 0;

		$parts = (new we_html_multiIconBoxContent())
				->add((new we_html_multiIconBoxContentLine('', we_html_element::hiddens(['blocks' => (string) (count($this->Model->blocks))])))
				->space(we_html_multiIconBox::SPACE_MED2)
				->noline()
		);

		foreach($this->Model->blocks as $block){
			$values = [];
			for($i = 1; $i < (count($this->Model->groups) + 1); $i++){
				$values[$i] = sprintf(g_l('modules_newsletter', '[mailing_list]'), $i);
			}

			$selected = $block->getGroups() ?: [1];
			$content = we_html_tools::formElementTable(g_l('modules_newsletter', '[name]'), $this->getHTMLBlockType('block' . $counter . '_Type', $block->Type)) .
					we_html_element::hiddens([
						'block' . $counter . '_Groups' => implode(',', $selected),
					]) .
					we_html_tools::formElementTable(g_l('modules_newsletter', '[block_lists]'), we_html_tools::select('block' . $counter . '_GroupsSel', $values, 1, $selected, true, [
								'class' => 'newSelect', 'style' => 'width:440px;',
								'onchange' => "populateMultipleVar(document.we_form.block" . $counter . "_GroupsSel,document.we_form.block" . $counter . "_Groups);top.content.hot=true"]));

			switch($block->Type){
				case we_newsletter_block::DOCUMENT:
					$content .= we_html_tools::formElementTable(g_l('modules_newsletter', '[block_document]'), $this->formWeDocChooser(FILE_TABLE, 320, 0, "block" . $counter . "_LinkID", $block->getLinkID(), 'block' . $counter . '_LinkPath', '', 'setHot', we_base_ContentTypes::WEDOCUMENT)) .
							we_html_tools::formElementTable('', we_html_forms::checkbox((($block->getField()) ? '0' : '1'), (($block->getField()) ? false : true), "block" . $counter . "_use_def_template", g_l('modules_newsletter', '[use_default]'), false, '', "top.content.hot=true;if(document.we_form.block" . $counter . "_use_def_template.checked){ document.we_form.block" . $counter . "_Field.value=0; document.we_form.block" . $counter . "_FieldPath.value='';}")) .
							we_html_tools::formElementTable(g_l('modules_newsletter', '[block_template]'), $this->formWeChooser(TEMPLATES_TABLE, 320, 0, 'block' . $counter . '_Field', (int) (!is_numeric($block->getField()) ? 0 : $block->getField()), 'block' . $counter . '_FieldPath', '', 'blocks_selectTemplateCallback,' . $counter, false));
					break;

				case we_newsletter_block::DOCUMENT_FIELD:
					$content .= we_html_tools::formElementTable(g_l('modules_newsletter', '[block_document]'), $this->formWeDocChooser(FILE_TABLE, 320, 0, 'block' . $counter . '_LinkID', $block->getLinkID(), 'block' . $counter . '_LinkPath', '', 'switchPage,2,setHot'));

					if($block->getLinkID()){
						$values = $this->View->getFields($block->getLinkID(), FILE_TABLE);

						$content .= (!empty($values) ?
								we_html_tools::formElementTable(g_l('modules_newsletter', '[block_document_field]'), we_html_tools::select('block' . $counter . '_Field', $values, 1, $block->getField(), false, [
											'style' => 'width:440px;',
											'onkeyup' => "top.content.hot=true;"])) :
								we_html_tools::formElementTable(g_l('modules_newsletter', '[block_document_field]'), we_html_element::div(['class' => "lowContrast"], g_l('modules_newsletter', '[none]')))
								);
					}
					break;

				case we_newsletter_block::OBJECT:
					$content .= we_html_tools::formElementTable(g_l('modules_newsletter', '[block_object]'), $this->formWeChooser(OBJECT_FILES_TABLE, 320, 0, 'block' . $counter . '_LinkID', $block->getLinkID(), 'block' . $counter . '_LinkPath', '', 'setHot', !we_base_permission::hasPerm([
												we_base_perm::OBJECTS_CAN_SELECT_OTHER_USERS]), [we_base_ContentTypes::FOLDER_OBJECTFILE, we_base_ContentTypes::OBJECT_FILE])) .
							we_html_tools::formElementTable(g_l('modules_newsletter', '[block_template]'), $this->formWeChooser(TEMPLATES_TABLE, 320, 0, 'block' . $counter . '_Field', (int) (!is_numeric($block->getField()) ? 0 : $block->getField()), 'block' . $counter . '_FieldPath', '', 'setHot', false, [
										we_base_ContentTypes::FOLDER_TEMPLATE, we_base_ContentTypes::TEMPLATE]));
					break;

				case we_newsletter_block::OBJECT_FIELD:
					$content .= we_html_tools::formElementTable(g_l('modules_newsletter', '[block_object]'), $this->formWeChooser(OBJECT_FILES_TABLE, 320, 0, 'block' . $counter . '_LinkID', $block->getLinkID(), 'block' . $counter . '_LinkPath', '', 'switchPage,2,setHot', !we_base_permission::hasPerm([
												we_base_perm::OBJECTS_CAN_SELECT_OTHER_USERS]), [we_base_ContentTypes::FOLDER_OBJECTFILE, we_base_ContentTypes::OBJECT_FILE]));

					if($block->getLinkID()){
						$values = $this->View->getFields($block->getLinkID(), OBJECT_FILES_TABLE);

						$content .= (!empty($values) ?
								we_html_tools::formElementTable(g_l('modules_newsletter', '[block_object_field]'), we_html_tools::select('block' . $counter . '_Field', $values, 1, $block->getField(), false, [
											'OnChange' => "top.content.hot=true;"])) :
								we_html_tools::formElementTable(g_l('modules_newsletter', '[block_document_field]'), we_html_element::div(['class' => "lowContrast"], g_l('modules_newsletter', '[none]')))
								);
					}
					break;

				case we_newsletter_block::FILE:
					$content .= we_html_tools::formElementTable(g_l('modules_newsletter', '[block_file]'), $this->formFileChooser('block' . $counter . '_Field', (is_numeric($block->getField()) ? '' : (!str_starts_with($block->getField(), '/') ? '' : $block->getField()))));
					break;

				case we_newsletter_block::TEXT:
					$blockHtml = preg_replace(
							['/(href=")(\\\\*&quot;)*(.+?)(\\\\*&quot;)*(")/',
								'/(src=")(\\\\*&quot;)*(.+?)(\\\\*&quot;)*(")/'
							], '${1}${3}${5}', stripslashes($block->getHTML()));


					$content .= we_html_tools::formElementTable(g_l('modules_newsletter', '[block_plain]'), we_html_element::textArea(['cols' => '40', 'rows' => '10', 'name' => 'block' . $counter . '_Source',
										'onchange' => "top.content.hot=true;",
										'style' => "width:440px;"], htmlspecialchars($block->getSource()))) .
							we_html_tools::formElementTable(g_l('modules_newsletter', '[block_html]'), we_html_forms::weTextarea('block' . $counter . '_Html', $blockHtml, [
										'wysiwyg' => true,
										'width' => '430',
										'height' => '400',
										'rows' => '10',
										'cols' => '40',
										'style' => "width:440px;",
										'inlineedit' => true,
										'bgcolor' => 'white',
											//'commands' => 'full',
											], true, true, true));
					break;

				case we_newsletter_block::ATTACHMENT:
					$content .= we_html_tools::formElementTable(g_l('modules_newsletter', '[block_attachment]'), $this->formWeChooser(FILE_TABLE, 320, 0, 'block' . $counter . '_LinkID', $block->getLinkID(), 'block' . $counter . '_LinkPath', '', '', false));
					break;

				case we_newsletter_block::URL:
					$content .= we_html_tools::formElementTable(g_l('modules_newsletter', '[block_url]'), we_html_tools::textInput('block' . $counter . '_Field', (is_numeric($block->getField()) ? '' : $block->getField()), 0, [
										'style' => 'width:440px;'], 'text', '0', true));
					break;
			}

			$buttons = '<div style="margin-left:440px;">' .
					we_html_button::create_button(we_html_button::PLUS, "javascript:we_cmd('addBlock','" . $counter . "')") .
					(count($this->Model->blocks) > 1 ?
					we_html_button::create_button(we_html_button::TRASH, "javascript:we_cmd('delBlock','" . $counter . "')") :
					''
					) . '</div>';

			$parts
					->add((new we_html_multiIconBoxContentLine(sprintf(g_l('modules_newsletter', '[block]'), ($counter + 1)), $content))->space(we_html_multiIconBox::SPACE_MED))
					->add((new we_html_multiIconBoxContentLine('', $buttons))->space(we_html_multiIconBox::SPACE_MED));

			$counter++;
		}

		return (new we_html_multiIconBox($parts, 'newsletter_header'))->marginLeft()->get();
	}

	private function getHTMLNewsletterGroups(): string{
		$count = count($this->Model->groups);
		$out = '';

		for($i = 0; $i < $count; $i++){
			$parts = (new we_html_multiIconBoxContent())
					->add(we_modules_info::isActive(we_modules_info::CUSTOMER) ?
							new we_html_multiIconBoxContentLine(g_l('modules_newsletter', '[customers]'), $this->getHTMLCustomer($i), we_html_multiIconBox::PROP_CUSTOMER) : null)
					->add(new we_html_multiIconBoxContentLine(g_l('modules_newsletter', '[file_email]'), $this->getHTMLExtern($i), we_html_multiIconBox::PROP_DOC))
					->add(new we_html_multiIconBoxContentLine(g_l('modules_newsletter', '[emails]'), $this->getHTMLEmails($i), we_html_multiIconBox::PROP_MAIL));

			$buttons = ($i == $count - 1 ? we_html_button::create_button(we_html_button::PLUS, "javascript:we_cmd('addGroup')") : null) .
					($count > 1 ? we_html_button::create_button(we_html_button::TRASH, "javascript:we_cmd('delGroup'," . $i . ")") : null);

			$out .= (new we_html_multiIconBox($parts, 'newsletter_group_box_' . $i))
							->marginLeft()
							->fold(0, ($count < 2), '')
							->headline(sprintf(g_l('modules_newsletter', '[mailing_list]'), ($i + 1)))
							->get() .
					we_html_element::br() .
					'<div style="margin-right:30px;">' . $buttons . '</div>';
		}

		return $out;
	}

	private function formNewsletterDirChooser(int $width = 0, int $rootDirID = 0, string $IDName = 'ID', int $IDValue = 0, string $Pathname = 'Path', string $Pathvalue = '/', string $cmd = ''): string{
		if(!$Pathvalue){
			$Pathvalue = fS('SELECT Path FROM ' . NEWSLETTER_TABLE . ' WHERE ID=' . $IDValue);
		}

		$weSuggest = $this->weAutoCompleter;
		$weSuggest->setAcId('PathGroup')
				->setContentType([we_base_ContentTypes::FOLDER_COMMON])
				->setInput($Pathname, str_replace('\\', '/', $Pathvalue))
				->setMaxResults(10)
				->setResult($IDName, (string) ($IDValue))
				->setSelector(we_gui_suggest::DirSelector)
				->setTable(NEWSLETTER_TABLE)
				->setWidth($width)
				->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_newsletter_dirSelector',{id:document.we_form.elements['" . $IDName . "'].value,targetID:'" . $IDName . "',targetName:'" . $Pathname . "',jsCmd:'" . $cmd . "',rootDirID:'" . $rootDirID . "'})"));

		return $weSuggest->getHTML();
	}

	function getHTMLNewsletterHeader(): string{
		$table = (new we_html_table(['class' => 'default withSpace'], 2, 1))
				->setCol(0, 0, we_html_tools::formElementTable(g_l('modules_newsletter', '[name]'), we_html_tools::textInput('Text', stripslashes($this->Model->Text), 0, [
									'oninput' => "we_cmd('setHot')",
									'id' => 'yuiAcInputPathName', 'onblur' => "we_cmd('syncNewsletterTitle');"], 'text', self::def_width . 'px')))
				->setCol(2, 0, we_html_tools::formElementTable(g_l('modules_newsletter', '[dir]'), $this->formNewsletterDirChooser((self::def_width - 120), 0, "ParentID", $this->Model->ParentID, 'Path', dirname($this->Model->Path), 'setHot')));

		//$table->setCol(2,0,[],we_html_tools::htmlFormElementTable($this->formWeDocChooser(NEWSLETTER_TABLE,320,0,"ParentID",$this->View->newsletter->ParentID,"Path",dirname($this->View->newsletter->Path),"opener.top.content.hot=true;",we_base_ContentTypes::FOLDER),g_l('modules_newsletter','[dir]')));
		$parts = (new we_html_multiIconBoxContent())
				->add(new we_html_multiIconBoxContentLine(g_l('modules_newsletter', '[path]'), $table->getHtml(), we_html_multiIconBox::PROP_PATH));

		if(!$this->Model->IsFolder){
			$chk = ($this->Model->Sender == $this->Model->Reply ?
					we_html_element::input(['type' => "checkbox", 'value' => '1', 'checked' => '', 'name' => "reply_same", 'onclick' => "top.content.hot=true;if(document.we_form.reply_same.checked) document.we_form.Reply.value=document.we_form.Sender.value"]) :
					we_html_element::input(['type' => "checkbox", 'value' => '0', 'name' => "reply_same", 'onclick' => "top.content.hot=true;if(document.we_form.reply_same.checked) document.we_form.Reply.value=document.we_form.Sender.value"])
					);

			// FIXME: inline js
			$inputAttribs = [
				'type' => "checkbox",
				'value' => '1',
				'name' => "isEmbedImagesChk",
				'onclick' => "top.content.hot=true;if(document.we_form.isEmbedImagesChk.checked){document.we_form.isEmbedImages.value=1;}else{document.we_form.isEmbedImages.value=0;}"
			];
			$embedImagesChk = we_html_element::input(array_merge($inputAttribs, ($this->Model->isEmbedImages ? ['checked' => 'checked'] : [])));
			$embedImagesHid = we_html_element::hidden('isEmbedImages', $this->Model->isEmbedImages);
			$embedImagesLab = we_html_element::label(['onclick' => "top.content.hot=true;if(document.we_form.isEmbedImagesChk.checked){ document.we_form.isEmbedImagesChk.checked=false; document.we_form.isEmbedImages.value=0; }else{document.we_form.isEmbedImagesChk.checked=true;document.we_form.isEmbedImages.value=1;}"], g_l('modules_newsletter', '[isEmbedImages]'));


			$table = (new we_html_table(['class' => 'default withSpace'], 4, 1))
					->setCol(0, 0, we_html_tools::formElementTable(g_l('modules_newsletter', '[subject]'), we_html_tools::textInput("Subject", stripslashes($this->Model->Subject), 0, [
										'oninput' => "top.content.hot=true;"], 'text', self::def_width . 'px')))
					->setCol(1, 0, we_html_tools::formElementTable(g_l('modules_newsletter', '[sender]'), we_html_tools::textInput("Sender", $this->Model->Sender, 0, [
										'oninput' => "top.content.hot=true;"], 'text', self::def_width . 'px')))
					->setCol(2, 0, we_html_tools::formElementTable(g_l('modules_newsletter', '[reply]'), we_html_tools::textInput("Reply", $this->Model->Reply, 0, [
										'oninput' => "top.content.hot=true;"]) . "&nbsp;&nbsp;" . $chk . "&nbsp;" . we_html_element::label([
										'onclick' => "top.content.hot=true;if(document.we_form.reply_same.checked){document.we_form.reply_same.checked=false;}else{document.we_form.Reply.value=document.we_form.Sender.value;document.we_form.reply_same.checked=true;}"], g_l('modules_newsletter', '[reply_same]'))))
					->setCol(3, 0, we_html_tools::formElementTable(g_l('modules_newsletter', '[test_email]'), we_html_tools::textInput("Test", $this->Model->Test, 0, [
										'oninput' => "top.content.hot=true;"])))
					->setCol(4, 0, we_html_tools::formElementTable('', $embedImagesHid . $embedImagesChk . '&nbsp;' . $embedImagesLab));

			$parts->add(new we_html_multiIconBoxContentLine(g_l('modules_newsletter', '[newsletter][text]'), $table->getHtml(), we_html_multiIconBox::PROP_NEWSLETTER))
					->add((new we_html_multiIconBoxContentLine(g_l('modules_newsletter', '[copy_newsletter]'), $this->getHTMLCopy(), we_html_multiIconBox::PROP_COPY))
							->noline()
			);
		}

		return (new we_html_multiIconBox($parts, 'newsletter_header'))->marginLeft()->get() .
				we_html_element::br();
	}

	/**
	 * Generates the body for modul frame
	 *
	 * @return string
	 */
	function getHTMLProperties(): string{
		if(we_base_request::bool('home')){
			return $this->View->getHomeScreen($this->jsCmd);
		}
		$js = $this->View->getJSProperty();
		$this->jsCmd->addCmd('setFocus');

		$out = $this->View->getHiddens() .
				$this->View->newsletterHiddens() .
				$this->View->getHiddensProperty() .
				we_html_element::hiddens([
					'newsletterStatus' => '',
					'newsletterStartTime' => '',
					'newsletterEndTime' => ''
		]);

		switch($this->View->getPage()){
			case we_newsletter_frames::TAB_PROPERTIES:
				$out .= we_html_element::hiddens(['home' => '0', 'fromPage' => (string) (we_newsletter_frames::TAB_PROPERTIES)]);

				if($this->Model->IsFolder == 0){
					$out .= $this->View->getHiddensMailingPage() .
							$this->View->getHiddensContentPage();
				}

				$out .= $this->getHTMLNewsletterHeader();
				break;
			case we_newsletter_frames::TAB_MAILING:
				$out .= $this->View->getHiddensPropertyPage() .
						$this->View->getHiddensContentPage() .
						we_html_element::hiddens(['fromPage' => (string) (we_newsletter_frames::TAB_MAILING), 'ncustomer' => '', 'nfile' => '', 'ngroup' => '', 'noCustFilter' => '']) .
						$this->getHTMLNewsletterGroups();
				break;
			case we_newsletter_frames::TAB_EDIT:
				$out .= $this->View->getHiddensMailingPage() .
						$this->View->getHiddensPropertyPage() .
						we_html_element::hiddens(['fromPage' => (string) (we_newsletter_frames::TAB_EDIT), 'blockid' => '0']) .
						$this->getHTMLNewsletterBlocks();
				$js .= we_wysiwyg_editor::isWysiwygInstances() ? we_wysiwyg_editor::getHTMLHeader(false, true) : '';
				break;
			default:
				$out .= $this->View->getHiddensPropertyPage() .
						$this->View->getHiddensMailingPage() .
						$this->View->getHiddensContentPage() .
						we_html_element::hiddens(['fromPage' => (string) (we_newsletter_frames::TAB_REPORTING), 'blockid' => '0']) .
						(new we_html_multiIconBox($this->View->getHTMLReporting()))->marginLeft()->get();
		}

		$body = we_html_element::body(['onload' => "self.loaded=true;if(self.doScrollTo){self.doScrollTo();}; setHeaderTitle();", 'class' => "weEditorBody", 'onunload' => "doUnload()"], we_html_element::form([
							'name' => we_base_constants::WE_FORM, 'method' => 'post', 'onsubmit' => "return false;"], $out
						)
		);

		return $this->getHTMLDocument($body, $js);
	}

	private function getHTMLEmailEdit(): string{
		$type = we_base_request::int('etyp', 0);
		$id = we_base_request::int('eid', 0);
		$email = we_base_request::_(we_base_request::EMAIL, 'email', '');
		$group = we_base_request::string('grp', '');
		$salutation = rawurldecode(str_replace('[:plus:]', '+', we_base_request::string('salutation', '')));
		$title = rawurldecode(str_replace('[:plus:]', '+', we_base_request::string('title', '')));
		$firstname = rawurldecode(str_replace('[:plus:]', '+', we_base_request::string('firstname', '')));
		$lastname = rawurldecode(str_replace('[:plus:]', '+', we_base_request::string('lastname', '')));

		$salut_select = (new we_html_select(['name' => 'salutation', 'style' => "width: 280px"]))
				->addOption('', '');

		if(!empty($this->View->settings[we_newsletter_newsletter::FEMALE_SALUTATION_FIELD])){
			$salut_select->addOption((string) $this->View->settings[we_newsletter_newsletter::FEMALE_SALUTATION_FIELD], (string) $this->View->settings[we_newsletter_newsletter::FEMALE_SALUTATION_FIELD]);
		}
		if(!empty($this->View->settings[we_newsletter_newsletter::MALE_SALUTATION_FIELD])){
			$salut_select->addOption((string) $this->View->settings[we_newsletter_newsletter::MALE_SALUTATION_FIELD], (string) $this->View->settings[we_newsletter_newsletter::MALE_SALUTATION_FIELD]);
		}
		$salut_select->selectOption($salutation);

		$row = 0;
		$table = (new we_html_table([], 12, 3))
				->setCol($row, 0, g_l('modules_newsletter', '[email]'))
				->setCol($row++, 1, we_html_tools::textInput("emailfield", $email, 0, [], 'text', '280px'))
				->setCol($row, 0, g_l('modules_newsletter', '[salutation]'))
				->setCol($row++, 1, $salut_select->getHtml())
				->setCol($row, 0, g_l('modules_newsletter', '[title]'))
				->setCol($row++, 1, we_html_tools::textInput('title', $title, 0, [], 'text', '280px'))
				->setCol($row, 0, g_l('modules_newsletter', '[firstname]'))
				->setCol($row++, 1, we_html_tools::textInput('firstname', $firstname, 0, [], 'text', '280px'))
				->setCol($row, 0, g_l('modules_newsletter', '[lastname]'))
				->setCol($row++, 1, we_html_tools::textInput('lastname', $lastname, 0, [], 'text', '280px'));

		$body = we_html_element::body(['class' => 'weDialogBody', 'onload' => "we_cmd('editEmails_onload')"], we_html_element::form([
							'name' => we_base_constants::WE_FORM, 'onsubmit' => "we_cmd('editEmails_save', " . $type . ");return false;"], we_html_element::hidden("group", $group) .
								($type ?
								we_html_element::hidden('id', $id) :
								''
								) .
								we_html_tools::dialogLayout(
										$table->getHtml(), g_l('modules_newsletter', $type ? '[edit_email]' : '[add_email]'), we_html_button::position_yes_no_cancel(we_html_button::create_button(we_html_button::SAVE, "javascript:we_cmd('editEmails_save', " . $type . ")"), we_html_button::create_button(we_html_button::CLOSE, "javascript:WE().util.dialogTop(window).doClose();"))
								)
						)
		);

		return $this->getHTMLDocument($body, $this->View->getJSProperty());
	}

	private function getHTMLPreview(): string{
		$gview = we_base_request::int('gview', 0);
		$content = '';
		foreach($this->Model->blocks as $block){
			$content .= $this->View->getContent($block, $gview);
		}

		header('Pragma: no-cache;');
		header('Cache-Control: post-check=0, post-check=0, false');
		we_html_tools::headerCtCharset('text/html');


		return '<html><head></head><body><form>
							<textarea name="foo" style="width:100%;height:95%">' .
				htmlspecialchars(trim($content)) .
				'</textarea></form></body></html>';
	}

	private function getHTMLBlackList(): string{
		$this->View->settings['black_list'] = we_base_request::string("black_list", $this->View->settings['black_list']);

		if(($ncmd = we_base_request::string('ncmd'))){
			if($ncmd === 'save_black'){
				$this->View->processCommands($this->jsCmd);
			}
		}

		$js = $this->View->getJSProperty() .
				we_html_element::jsScript(WE_JS_MODULES_DIR . 'newsletter/newsletter_top.js', 'self.focus();');

		switch(we_base_request::string('ncmd')){
			case 'import_black':
				$filepath = we_base_request::_(we_base_request::FILE, "csv_file");
				$delimiter = we_base_request::_(we_base_request::RAW_CHECKED, "csv_delimiter");
				$col = we_base_request::int("csv_col", 0);

				if($col){
					$col--;
				}

				if(str_contains($filepath, '..')){
					$this->jsCmd->addMsg(g_l('modules_newsletter', '[path_not_valid]'), we_base_util::WE_MESSAGE_ERROR);
				} else {
					$row = [];
					$fh = @fopen(WE_DOCUMENT_ROOT . $filepath, 'rb');
					if($fh){
						while(($dat = fgetcsv($fh, 1000, $delimiter))){
							'@phan-var array $dat';
							$alldat = implode('', $dat);
							if(str_replace(' ', '', $alldat) === ''){
								continue;
							}
							$row[] = $dat[$col];
						}

						fclose($fh);

						if(!empty($row)){
							if($this->View->settings['black_list'] === ''){
								$this->View->settings['black_list'] = implode(',', $row);
							} else {
								$this->View->settings['black_list'] .= "," . implode(',', $row);
							}
						}
					} else {
						$this->jsCmd->addMsg(g_l('modules_newsletter', '[path_not_valid]'), we_base_util::WE_MESSAGE_ERROR);
					}
				}
				break;
			case 'export_black':
				$fname = rtrim(we_base_request::_(we_base_request::FILE, "csv_dir", ''), '/') . '/blacklist_export_' . time() . '.csv';
				we_base_file::save(WE_DOCUMENT_ROOT . $fname, str_replace(",", "\n", $this->View->settings['black_list']));
				$this->jsCmd->addCmd('export_csv_window', $fname);
				break;
		}

		$arr = array_filter(explode(',', $this->View->settings['black_list']));

		$buttons_table = (new we_html_table(['class' => 'default withSpace'], 4, 1))
				->setCol(0, 0, we_html_button::create_button(we_html_button::ADD, "javascript:addBlack();"))
				->setCol(1, 0, we_html_button::create_button(we_html_button::EDIT, "javascript:editBlack();"))
				->setCol(2, 0, we_html_button::create_button(we_html_button::DELETE, "javascript:deleteBlack()"))
				->setCol(3, 0, we_html_button::create_button(we_html_button::DELETE_ALL, "javascript:deleteallBlack()"));

		$table = (new we_html_table(['class' => 'default'], 5, 3))
				->setCol(0, 0, we_html_tools::select("blacklist_sel", $arr, 10, '', false, ['style' => 'width:388px'], true, 600), ['style' => 'vertical-align:middle;'])
				->setCol(0, 1, $buttons_table->getHtml(), ['style' => 'vertical-align:top;padding-left:15px;'])
				->setCol(2, 0, we_html_button::create_button(we_html_button::IMPORT, "javascript:set_import(1)") . we_html_button::create_button(we_html_button::EXPORT, "javascript:set_export(1)"), [
			'colspan' => '3', 'style' => 'padding-top:10px;']);

		$sib = we_base_request::_(we_base_request::RAW, 'sib', '0');
		$seb = we_base_request::_(we_base_request::RAW, 'seb', '0');

		if($sib){
			$import_options = (new we_html_table(['class' => 'default withSpace'], 2, 2))
					->setCol(0, 0, g_l('modules_newsletter', '[csv_delimiter]') . ":&nbsp;")
					->setCol(0, 1, we_html_tools::textInput("csv_delimiter", ","))
					->setCol(1, 0, g_l('modules_newsletter', '[csv_col]') . ":&nbsp;")
					->setCol(1, 1, we_html_tools::textInput("csv_col", '1'));

			$import_box = (new we_html_table(['class' => 'default withSpace', 'style' => 'padding-top:10px;'], 4, 1))
					->setCol(0, 0, $this->formFileChooser('csv_file', '/', ''))
					->setCol(1, 0, we_html_button::create_button(we_html_button::UPLOAD, "javascript:we_cmd('upload_black')"))
					->setCol(2, 0, $import_options->getHtml())
					->setCol(3, 0, we_html_button::create_button(we_html_button::OK, "javascript:document.we_form.sib.value=0;we_cmd('import_black');") . we_html_button::create_button(we_html_button::CANCEL, "javascript:set_import(0);"));

			$table->setCol(3, 0, we_html_element::hiddens(['csv_import' => '1']) . $import_box->getHtml(), ['colspan' => '3']);
		} elseif($seb){
			$export_box = (new we_html_table(['class' => 'default withSpace', 'style' => 'padding-top:10px;'], 2, 1))
					->setCol(0, 0, $this->formFileChooser('csv_dir', '/', '', we_base_ContentTypes::FOLDER_DOCUMENT))
					->setCol(1, 0, we_html_button::create_button(we_html_button::OK, "javascript:document.we_form.seb.value=0;we_cmd('export_black');") . we_html_button::create_button(we_html_button::CANCEL, "javascript:set_export(0);"));

			$table->setCol(3, 0, we_html_element::hiddens(['csv_export' => '1']) . $export_box->getHtml(), ['colspan' => '3']);
		}

		$body = we_html_element::body(['class' => 'weDialogBody'], we_html_element::form(['name' => we_base_constants::WE_FORM, 'onsubmit' => "save();return false;"], $this->View->getHiddens() .
								we_html_element::hiddens([
									'black_list' => $this->View->settings['black_list'],
									'sib' => $sib,
									'seb' => $seb]) .
								we_html_tools::dialogLayout(
										$table->getHtml(), g_l('modules_newsletter', '[black_list]'), we_html_button::position_yes_no_cancel(we_html_button::create_button(we_html_button::SAVE, "javascript:we_cmd('save_black')"), '', we_html_button::create_button(we_html_button::CANCEL, "javascript:WE().util.dialogTop(window).doClose();"))
								)
						)
		);

		return $this->getHTMLDocument($body, $js);
	}

	private function getHTMLUploadCsv(string $what): string{
		$weFileupload = new we_fileupload_ui_base('we_File', [we_base_ContentTypes::TEXT]);

		$pb = new we_gui_progressBar(0, 120, 'fu_extProgress_we_File');

		$footerTable = (new we_html_table(['class' => 'default', 'style' => 'width:100%;'], 1, 2))
				->setCol(0, 0, we_html_element::div(['id' => 'progressbar', 'style' => 'display:none;padding-left:10px'], $pb->getHTML()))
				->setCol(0, 1, we_html_button::create_button(we_html_button::IMPORT, "javascript:we_cmd('do_" . $what . "');", '', '', false, false, '_footer') . we_html_button::create_button(we_html_button::CANCEL, "javascript:WE().util.dialogTop(window).doClose();"), [
			'style' => 'text-align:right']);

		$js = $this->View->getJSProperty() . $weFileupload->getJs() . we_gui_progressBar::getJSCode();

		$table = (new we_html_table(['class' => 'default withBigSpace'], 1, 1))
				//$table->setCol(0, 0, ['style' => "padding-right:30px"], $weFileupload->getHtmlAlertBoxes());
				->setCol(0, 0, $weFileupload->getHTML(), ['style' => 'vertical-align:middle;']);

		$body = we_html_element::body(['class' => 'weDialogBody', 'onload' => "self.focus();"], we_html_element::form(['name' => we_base_constants::WE_FORM, 'method' => 'post',
							'enctype' => 'multipart/form-data'], we_html_element::center(
										$this->View->getHiddens() .
										(($grp = we_base_request::string('grp')) ? we_html_element::hiddens(['group' => $grp]) : '') .
										we_html_element::hiddens(['MAX_FILE_SIZE' => (string) (we_fileupload::getMaxUploadSizeB())]) .
										we_html_tools::dialogLayout($table->getHtml(), g_l('modules_newsletter', '[csv_upload]'), $footerTable->getHTML())
								)
						)
		);

		return $this->getHTMLDocument($body, $weFileupload->getCSS() . $js);
	}

	private function getHTMLExportCsvMessage(bool $allowClear = false): string{
		$link = we_base_request::_(we_base_request::URL, 'lnk');
		if($link === false){
			return '';
		}

		$table = (new we_html_table(['class' => 'default'], 3, 1))
				->setCol(0, 0, sprintf(g_l('modules_newsletter', '[csv_export]'), $link), ['style' => 'padding-top:5px;'])
				->setCol(1, 0, we_backup_wizard::getDownloadLinkText(), ['style' => 'padding-top:5px;'])
				->setCol(2, 0, we_html_element::a(['href' => we_http_util::getServerUrl(true) . $link, 'download' => basename($link)], g_l('modules_newsletter', '[csv_download]')), [
			'style' => 'padding:5px 0;']);

		if($allowClear){
			$table->setCol(3, 0, we_html_element::b(g_l('modules_newsletter', '[clearlog_note]')), ['style' => 'padding:5px 0;']);
		}

		$body = we_html_element::body(['class' => 'weDialogBody', 'onload' => "self.focus();"], we_html_element::form(['name' => we_base_constants::WE_FORM, 'method' => 'post'], we_html_element::hidden('group', '') .
								($allowClear ?
								we_html_element::hiddens(['pnt' => 'clear_log', 'ncmd' => 'do_clear_log']) .
								we_html_tools::dialogLayout($table->getHtml(), g_l('modules_newsletter', '[clear_log]'), we_html_button::position_yes_no_cancel(we_html_button::create_button(we_html_button::OK, "javascript:clearLog();"), '', we_html_button::create_button(we_html_button::CANCEL, "javascript:WE().util.dialogTop(window).doClose();"))) :
								we_html_tools::dialogLayout($table->getHtml(), g_l('modules_newsletter', '[csv_download]'), we_html_button::formatButtons(we_html_button::create_button(we_html_button::CLOSE, "javascript:WE().util.dialogTop(window).doClose();")))
								)
						)
		);

		return ($allowClear ? $body : $this->getHTMLDocument($body));
	}

	/**
	 * Edit csv mail list
	 *
	 * @return String
	 */
	private function getHTMLEditFile(): string{
		$sorter_code = $content = [];
		'@phan-var string[] $sorter_code';

		$order = we_base_request::string('order', '');
		for($i = 0; $i < 12; $i += 2){
			$sorter_code[$i] = we_html_element::br() . ($order == $i ?
					we_html_element::input(['type' => 'radio', 'value' => (string) ($i), 'name' => 'order', 'checked' => 'checked', 'onclick' => "submitForm('edit_file')"]) . '&darr;' :
					we_html_element::input(['type' => 'radio', 'value' => (string) ($i), 'name' => 'order', 'onclick' => "submitForm('edit_file')"]) . '&darr;'
					);
			$sorter_code[$i + 1] = ($order == $i + 1 ?
					we_html_element::input(['type' => 'radio', 'value' => (string) ($i + 1), 'name' => 'order', 'checked' => 'checked', 'onclick' => "submitForm('edit_file')"]) . '&uarr;' :
					we_html_element::input(['type' => 'radio', 'value' => (string) ($i + 1), 'name' => 'order', 'onclick' => "submitForm('edit_file')"]) . "&uarr;"
					);
		}

		$headlines = [
			'ID' . $sorter_code[0] . $sorter_code[1],
			g_l('modules_newsletter', '[email]') . $sorter_code[2] . $sorter_code[3],
			g_l('modules_newsletter', '[salutation]') . $sorter_code[4] . $sorter_code[5],
			g_l('modules_newsletter', '[title]') . $sorter_code[6] . $sorter_code[7],
			g_l('modules_newsletter', '[firstname]') . $sorter_code[8] . $sorter_code[9],
			g_l('modules_newsletter', '[lastname]') . $sorter_code[10] . $sorter_code[11],
			g_l('modules_newsletter', '[edit]'),
			g_l('modules_newsletter', '[status]'),
		];

		$csv_file = we_base_request::_(we_base_request::FILE, 'csv_file', we_base_request::_(we_base_request::FILE, 'art'));
		$emails = [];
		$emailkey = [];
		if(!str_contains($csv_file, '..') && is_readable(WE_DOCUMENT_ROOT . $csv_file)){
			if($csv_file){
				$emails = we_newsletter_newsletter::getEmailsFromExtern2([$csv_file], we_base_request::int('weEmailStatus', 0), $emailkey);
			}
		} else {
			$this->jsCmd->addMsg(g_l('modules_newsletter', '[path_not_valid]'), we_base_util::WE_MESSAGE_ERROR);
		}

		$offset = max(we_base_request::int('offset', 0), 0);
		$numRows = we_base_request::int('numRows', 15);
		$anz = count($emails);
		$endRow = min($offset + $numRows, $anz);
		$position = -1;

		switch($order){
			case 2:
			case 3:
				$position = 0;
				break;
			case 4:
			case 5:
				$position = 1;
				break;
			case 6:
			case 7:
				$position = 2;
				break;
			case 8:
			case 9:
				$position = 3;
				break;
			case 10:
			case 11:
				$position = 4;
				break;
			case 12:
			case 13:
				$position = 5;

				break;
		}
		if($position > -1){
			$reverse = ($order % 2 === 0);
			uasort($emails, function (array $a, array $b) use($position, $reverse): int{
				return $reverse ?
						strnatcasecmp($b[$position], $a[$position]) :
						strnatcasecmp($a[$position], $b[$position]);
			});
		}
		$counter = 0;
		foreach($emails as $k => $cols){
			if($k >= $offset && $k < $endRow){

				$edit = we_html_button::create_button(we_html_button::EDIT, "javascript:editEmailFile(" . $emailkey[$k] . ",'" . $cols[0] . "','1','" . $cols[1] . "','" . $cols[2] . "','" . $cols[3] . "','" . $cols[4] . "')");
				$trash = we_html_button::create_button(we_html_button::TRASH, "javascript:delEmailFile(" . $emailkey[$k] . ",'" . $cols[0] . "')");

				$content[$counter] = [
					we_html_element::div(['class' => 'middlefont'], (string) $k),
					we_html_element::div(['class' => 'middlefont'], ($cols[0] ?: '&nbsp;')),
					we_html_element::div(['class' => 'middlefont'], ($cols[1] ?: '&nbsp;')),
					we_html_element::div(['class' => 'middlefont'], ($cols[2] ?: '&nbsp;')),
					we_html_element::div(['class' => 'middlefont'], ($cols[3] ?: '&nbsp;')),
					we_html_element::div(['class' => 'middlefont'], ($cols[4] ?: '&nbsp;')),
					we_html_element::div(['class' => 'middlefont'], $edit . $trash),
					we_html_element::div(['class' => 'middlefont'], '<i class="fa fa-lg ' . (we_check_email($cols[0]) ? 'fa-ok fa-check' : 'fa-cancel fa-close' ) . '"></i>'),
				];
				$counter++;
			}
		}

		$js = $this->View->getJSProperty();

		$close = we_html_button::create_button(we_html_button::CLOSE, "javascript:WE().util.dialogTop(window).doClose();");
		$edit = we_html_button::create_button(we_html_button::SAVE, "javascript:listFile()");
		$nextprev = (new we_html_table(['class' => 'default'], 1, 4))
				->setCol(0, 0, ($offset ?
						we_html_button::create_button(we_html_button::BACK, "javascript:document.we_form.offset.value=" . ($offset - $numRows) . ";submitForm('edit_file');") :
						we_html_button::create_button(we_html_button::BACK, "#", '', '', true)), ['style' => 'padding-right:10px;'])
				->setCol(0, 1, we_html_element::b(( $anz ? $offset + 1 : 0 ) . ' - ' . (($anz - $offset) < $numRows ? $anz : $offset + $numRows) . ' ' .
								g_l('global', '[from]') . ' ' .
								$anz), ['style' => 'padding-right:10px;'])
				->setCol(0, 2, (($offset + $numRows) < $anz ?
				we_html_button::create_button(we_html_button::NEXT, "javascript:document.we_form.offset.value=" . ($offset + $numRows) . ";submitForm('edit_file');") :
				we_html_button::create_button(we_html_button::NEXT, "#", '', '', true)
		));

		if(!empty($emails)){
			$add = we_html_button::create_button(we_html_button::PLUS, "javascript:editEmailFile(" . count($emails) . ",'','','','','','')");
			$end = $nextprev->getHtml();

			$selectStatus = we_html_element::b(g_l('modules_newsletter', '[status]')) . ' ' . we_html_tools::select('weEmailStatus', [
						we_newsletter_newsletter::CSV_STATUS_ALL => g_l('modules_newsletter', '[statusAll]'),
						we_newsletter_newsletter::CSV_STATUS_INVALID => g_l('modules_newsletter', '[statusInvalid]'),
						we_newsletter_newsletter::CSV_STATUS_VALID => g_l('modules_newsletter', '[statusValid]')
							], 0, we_base_request::int('weEmailStatus', 0), false, ['onchange' => "listFile();"], true, 150);

			$nextprev->addCol(3)
					->setCol(0, 3, we_html_element::b(g_l('modules_newsletter', '[show]')) . ' ' . we_html_tools::textInput('numRows', (string) $numRows),
							['style' => 'padding-left:20px;'])
					->setCol(0, 4, $selectStatus, ['style' => 'padding-left:20px;'])
					->setCol(0, 5, $add, ['style' => 'padding-left:20px;']);

			$out = $nextprev->getHtml() .
					we_html_tools::dialogBorder(850, $content, $headlines) .
					$end;
		} else {
			if(empty($csv_file)){
				$nlMessage = g_l('modules_newsletter', '[no_file_selected]');
				$selectStatus2 = '';
			} elseif(($emailStatus = we_base_request::int('weEmailStatus', 0)) == we_newsletter_newsletter::CSV_STATUS_INVALID){
				$nlMessage = g_l('modules_newsletter', '[file_all_ok]');
				$selectStatus2 = '<br/>' . we_html_element::b(g_l('modules_newsletter', '[status]')) . ' ' . we_html_tools::select('weEmailStatus', [
							we_newsletter_newsletter::CSV_STATUS_ALL => g_l('modules_newsletter', '[statusAll]'),
							we_newsletter_newsletter::CSV_STATUS_INVALID => g_l('modules_newsletter', '[statusInvalid]'),
							we_newsletter_newsletter::CSV_STATUS_VALID => g_l('modules_newsletter', '[statusValid]')
								], 0, $emailStatus, false, ['onchange' => "listFile();"], true, 150);
			} else {
				$nlMessage = g_l('modules_newsletter', '[file_all_ok]');
				$selectStatus2 = '';
			}

			$out = we_html_element::div(['class' => "middlefont lowContrast", 'style' => 'text-align:center;padding-bottom:2em;'], '--&nbsp;' . $nlMessage . '&nbsp;--' . $selectStatus2) .
					we_html_button::create_button(we_html_button::PLUS, "javascript:editEmailFile(" . count($emails) . ",'','','','','','')");
		}


		$body = we_html_element::body(['class' => 'weDialogBody'], we_html_element::form(['name' => we_base_constants::WE_FORM], we_html_element::hiddens([
									'ncmd' => 'edit_file',
									'pnt' => 'edit_file',
									'order' => $order,
									'offset' => $offset,
									'nrid' => '',
									'email' => '',
									'htmlmail' => '',
									'salutation' => '',
									'title' => '',
									'firstname' => '',
									'lastname' => '',
									'etyp' => '',
									'eid' => '']) .
								we_html_tools::dialogLayout(
										we_html_element::div(['style' => 'margin-top:10px;'], $this->formFileChooser('csv_file', ($csv_file ?: '/'), 'listFile', we_base_ContentTypes::TEXT))
										. '<br/>' . $out, g_l('modules_newsletter', '[select_file]'), $close . $edit)
						)
		);

		return $this->getHTMLDocument($body, $js);
	}

	function getHTMLClearLog(): string{
		if(we_base_request::string('ncmd') === 'do_clear_log'){
			$this->db->query('TRUNCATE TABLE ' . NEWSLETTER_LOG_TABLE);
			$this->jsCmd->addMsg(g_l('modules_newsletter', '[log_is_clear]'), we_base_util::WE_MESSAGE_NOTICE);
			//return $this->getHTMLDocument(we_html_element::body());
		}

		$csv = '';
		$all = $this->db->getAllq('SELECT n.Text as NewsletterName,DATE_FORMAT(l.stamp,"' . g_l('weEditorInfo', '[mysql_date_format]') . '") AS logTime,l.Log,l.Param FROM ' . NEWSLETTER_TABLE . ' n JOIN ' . NEWSLETTER_LOG_TABLE . ' l ON n.ID=l.NewsletterID');
		foreach($all as $rec){
			$csv .= $rec['NewsletterName'] . ',' . $rec['logTime'] . ',';
			try{
				$csv .= sprintf(g_l('modules_newsletter', '[' . $rec['Log'] . ']'), $rec['Param']);
			}catch(Exception $ex){
				$csv .= $rec['Log'];
			}
			$csv .= "\n";
		}

		$link = WE_BACKUP_DIR . 'download/log_' . time() . '.csv';
		if(we_base_file::save(WE_DOCUMENT_ROOT . $link, $csv)){
			we_base_file::insertIntoCleanUp($link, 3600);
		} else {
			$link = '';
		}

		$_REQUEST['lnk'] = $link;

		return $this->getHTMLDocument($this->getHTMLExportCsvMessage(true), we_html_element::jsScript(WE_JS_MODULES_DIR . 'newsletter/newsletter_top.js'));
	}

	private function getHTMLSendWait(): string{
		$nid = we_base_request::int('nid', 0);
		$test = we_base_request::bool('test');


		return $this->getHTMLDocument(
						we_html_element::body(['class' => 'weDialogBody', 'onload' => "self.focus();setTimeout(function (doc) {doc.we_form.submit();},200,document);"], we_html_element::form([
									'name' => we_base_constants::WE_FORM, 'action' => WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod', 'method' => 'post'], we_html_element::hiddens([
											'mod' => 'newsletter',
											'pnt' => 'send_frameset',
											'nid' => (string) ($nid),
											'test' => (string) ($test)
										]) .
										we_html_element::center('<i class="fa fa-2x fa-spinner fa-pulse"></i>' .
												we_html_element::br() .
												we_html_element::br() .
												we_html_element::div(['class' => 'header_small'], g_l('modules_newsletter', '[prepare_newsletter]'))
										)
								)
						)
		);
	}

	private function getHTMLSendFrameset(): string{
		$nid = we_base_request::int('nid', 0);
		$test = we_base_request::bool('test');
		$ret = $this->View->cacheNewsletter();
		$offset = ($this->Model->Offset) ? ($this->Model->Offset + 1) : 0;

		if((int) $this->View->settings['send_step'] <= $offset){
			$this->Model->Step++;
			$this->Model->Offset = 0;
		}
		$this->jsCmd->addCmd(
				(($this->Model->Step != 0 || $this->Model->Offset != 0) ?
						'ask_camp' :
						'cancel_camp')
		);

		$head = we_html_element::jsScript(WE_JS_MODULES_DIR . 'newsletter/newsletter_top.js', '', ['id' => 'loadVarNewsletter', 'data-newsletter' => setDynamicVar([
						'start' => $this->Model->Step,
						'group' => $this->Model->Offset,
						'nid' => $nid,
						'test' => $test,
						'blockcache' => $ret['blockcache'],
						'emailcache' => $ret['emailcache'],
						'ecount' => $ret['ecount'],
						'gcount' => $ret['gcount']
			])]) . $this->jsCmd->getCmds();

		$body = we_html_element::iframe('send_body', WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod&mod=newsletter&pnt=send_body', 'position:absolute;top:0;bottom:0;left:0;right:0;') .
				we_html_element::iframe('send_cmd', WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod&mod=newsletter&pnt=send_cmd', 'position:absolute;width:0;height:0;') .
				we_html_element::iframe('send_control', WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod&mod=newsletter&pnt=send_control&nid=' . $nid . '&test=' . ((string) $test) . '&blockcache=' . $ret['blockcache'] . '&emailcache=' . $ret['emailcache'] . '&ecount=' . $ret['ecount'] . '&gcount=' . $ret['gcount'], 'position:absolute;width:0;height:0;');

		return $this->getHTMLDocument(we_html_element::body([], $body), $head);
	}

	private function getHTMLSendBody(): string{
		$pb = new we_gui_progressBar(we_base_request::int("pro", 0), 400);
		$pb->addText(g_l('modules_newsletter', '[sending]'), we_gui_progressBar::TOP, "title");

		$footer = '<table style="width:580px;" class="default"><tr><td style="text-align:left">' .
				$pb->getHTML() . '</td><td style="text-align:right">' .
				we_html_button::create_button(we_html_button::CLOSE, "javascript:WE().util.dialogTop(window).doClose();") .
				'</td></tr></table>';

		return $this->getHTMLDocument(
						we_html_element::body(['class' => 'weDialogBody'], we_html_element::form(['name' => we_base_constants::WE_FORM, 'method' => 'post'], we_html_tools::dialogLayout(we_html_element::textArea([
													'name' => "details",
													'style' => "width:530px;height:270px;"
														], g_l('modules_newsletter', (we_base_request::bool("test") ? '[test_no_mail]' : '[sending]')) . "\n" .
														g_l('modules_newsletter', '[campaign_starts]')
												), g_l('modules_newsletter', '[details]'), $footer)
								)
						), we_gui_progressBar::getJSCode());
	}

	private function getHTMLSendCmd(): string{
		$nid = we_base_request::int('nid');
		if(!$nid){
			return '';
		}
		$this->View->Model = new we_newsletter_newsletter($nid);

		$test = we_base_request::bool('test');
		$start = we_base_request::int('start', 0);

		// to calc progress ------------------
		// total number of emails
		$ecount = we_base_request::int('ecount', 0);
		// counter
		$ecs = we_base_request::int('ecs', 0);
		//-----------------------------------

		$blockcache = we_base_request::_(we_base_request::RAW, 'blockcache', 0);

		// emails cache -----------------------
		$emailcache = we_base_request::_(we_base_request::RAW, 'emailcache', 0);
		//
		$egc = we_base_request::int('egc', 0);
		//
		$gcount = we_base_request::int('gcount', 0);
		//-----------------------------------

		$reload = we_base_request::bool('reload');
		$retry = we_base_request::bool('retry');

		if($retry){
			$egc = $this->Model->Step;
			$start = $this->Model->Offset;
			if($start){
				$start++;
			}
			$this->Model->addLog(we_newsletter_newsletter::LOG_RETRY);
		}

		$body = we_html_element::body(['style' => 'margin:10px;', 'onload' => "initControl()"], we_html_element::form([
							'name' => we_base_constants::WE_FORM,
							'method' => 'post'], we_html_element::hiddens([
									'mod' => 'newsletter',
									'nid' => (string) $nid,
									'pnt' => 'send_cmd',
									'test' => (string) $test,
									'blockcache' => $blockcache,
									'emailcache' => $emailcache,
									'ecount' => (string) $ecount,
									'value' => (string) $ecount,
									'gcount' => (string) $gcount,
									'egc' => (string) ($egc + 1),
									'ecs' => (string) $ecs,
									'reload' => '1'
								])
						)
		);
		$progress_update = [
			'log' => [],
			'text' => '',
			'percent' => 0
		];

		if($gcount <= $egc){
			we_cache_file::clean(we_cache_file::DOMAIN_NEWSLETTER . $blockcache);
			$progress_update = [
				'log' => [g_l('modules_newsletter', '[campaign_ends]')],
				'text' => '<span style="color:#006699;font-weight:bold;">' . g_l('modules_newsletter', '[finished]') . '</span>',
				'percent' => 100
			];

			$this->db->query('UPDATE ' . NEWSLETTER_TABLE . ' SET Step=0,Offset=0 WHERE ID=' . $this->Model->ID);
			if(!$test){
				$this->Model->addLog(we_newsletter_newsletter::LOG_END_SEND);
			}
			$this->jsCmd->addCmd('updateLog', $progress_update);
			return $this->getHTMLDocument($body, we_html_element::jsScript(WE_JS_MODULES_DIR . 'newsletter/newsletter_property.js', 'self.focus();'));
		}
		if($start && !$test && !$reload){
			$this->Model->addLog(we_newsletter_newsletter::LOG_CONTINUE_SEND);
		} else if(!$test && !$reload){
			$this->Model->addLog(we_newsletter_newsletter::LOG_START_SEND);
		}

		$content = '';

		try{
			$emails = array_values(we_cache_file::load(we_cache_file::DOMAIN_NEWSLETTER . $emailcache . '_' . $egc));
		}catch(we_exception_cache $e){
			$emails = [];
		}
		if($emails){
			$end = count($emails);

			for($j = $start; $j < $end; $j++){
				$email = trim($emails[$j][0]);

				$user_groups = explode(',', $emails[$j][6]);
				$user_blocks = $emails[$j][7];

				sort($user_blocks);
				$user_blocks = array_unique($user_blocks);

				$salutation = isset($emails[$j][2]) ? str_replace(["\r", "\n"], '', $emails[$j][2]) : '';
				$title = isset($emails[$j][3]) ? str_replace(["\r", "\n"], '', $emails[$j][3]) : '';
				$firstname = isset($emails[$j][4]) ? str_replace(["\r", "\n"], '', $emails[$j][4]) : '';
				$lastname = isset($emails[$j][5]) ? str_replace(["\r", "\n"], '', $emails[$j][5]) : '';
				$customerid = isset($emails[$j][8]) ? str_replace(["\r", "\n"], '', (string) $emails[$j][8]) : '';
				$iscustomer = (isset($emails[$j][9]) && $emails[$j][9] === 'customer' ? 'C' : '');

				$contentDefault = $content_plainDefault = $contentF = $contentF_plain = $contentM = $contentM_plain = $contentTFL = $contentTFL_plain = $contentTL = $contentTL_plain = $contentFL = $contentFL_plain = $contentLN = $contentLN_plain = $contentFN = $contentFN_plain = '';

				$inlines = $atts = [];

				foreach($user_groups as $user_group){
					$atts = array_merge($atts, $this->View->getAttachments(intval($user_group)));
				}

				foreach($user_blocks as $user_block){
					try{
						$html_block = we_cache_file::load(we_cache_file::DOMAIN_NEWSLETTER . $blockcache . '_h_' . $user_block);
						$plain_block = we_cache_file::load(we_cache_file::DOMAIN_NEWSLETTER . $blockcache . '_p_' . $user_block);

						$contentDefault .= $html_block['default' . $iscustomer];
						$content_plainDefault .= $plain_block['default' . $iscustomer];

						$contentF .= $html_block['female' . $iscustomer];
						$contentF_plain .= $plain_block['female' . $iscustomer];

						$contentM .= $html_block['male' . $iscustomer];
						$contentM_plain .= $plain_block['male' . $iscustomer];

						$contentTFL .= $html_block['title_firstname_lastname' . $iscustomer];
						$contentTFL_plain .= $plain_block['title_firstname_lastname' . $iscustomer];

						$contentTL .= $html_block['title_lastname' . $iscustomer];
						$contentTL_plain .= $plain_block['title_lastname' . $iscustomer];

						$contentFL .= $html_block['firstname_lastname' . $iscustomer];
						$contentFL_plain .= $plain_block['firstname_lastname' . $iscustomer];

						$contentLN .= $html_block['lastname' . $iscustomer];
						$contentLN_plain .= $plain_block['lastname' . $iscustomer];

						$contentFN .= $html_block['firstname' . $iscustomer];
						$contentFN_plain .= $plain_block['firstname' . $iscustomer];

						$inlines = array_merge($inlines, $html_block['inlines'] ?? []);
					}catch(we_exception_cache $e){

					}
				}

				if($salutation && $lastname && ($salutation == $this->View->settings[we_newsletter_newsletter::FEMALE_SALUTATION_FIELD]) && ((!$this->View->settings['title_or_salutation']) || (!$title))){
					$rep = [
						we_newsletter_base::FIRSTNAME_REPLACE_TEXT => $firstname,
						we_newsletter_base::LASTNAME_REPLACE_TEXT => $lastname,
						we_newsletter_base::CUSTOMERID_REPLACE_TEXT => $customerid,
						we_newsletter_base::TITLE_REPLACE_TEXT => $title,];
					$content = strtr(($title ? preg_replace('|([^ ])' . we_newsletter_base::TITLE_REPLACE_TEXT . '|', '${1} ' . $title, $contentF) : $contentF), $rep);
					$content_plain = strtr(($title ? preg_replace('|([^ ])' . we_newsletter_base::TITLE_REPLACE_TEXT . '|', '${1} ' . $title, $contentF_plain) : $contentF_plain), $rep);
				} else if($salutation && $lastname && ($salutation == $this->View->settings[we_newsletter_newsletter::MALE_SALUTATION_FIELD]) && ((!$this->View->settings['title_or_salutation']) || (!$title))){
					$rep = [
						we_newsletter_base::FIRSTNAME_REPLACE_TEXT => $firstname,
						we_newsletter_base::LASTNAME_REPLACE_TEXT => $lastname,
						we_newsletter_base::CUSTOMERID_REPLACE_TEXT => $customerid,
						we_newsletter_base::TITLE_REPLACE_TEXT => $title
					];

					$content = strtr(($title ? preg_replace('|([^ ])' . we_newsletter_base::TITLE_REPLACE_TEXT . '|', '${1} ' . $title, $contentM) : $contentM), $rep);
					$content_plain = strtr(($title ? preg_replace('|([^ ])' . we_newsletter_base::TITLE_REPLACE_TEXT . '|', '${1} ' . $title, $contentM_plain) : $contentM_plain), $rep);
				} else if($title && $firstname && $lastname){
					$rep = [
						we_newsletter_base::FIRSTNAME_REPLACE_TEXT => $firstname,
						we_newsletter_base::LASTNAME_REPLACE_TEXT => $lastname,
						we_newsletter_base::CUSTOMERID_REPLACE_TEXT => $customerid,
						we_newsletter_base::TITLE_REPLACE_TEXT => $title
					];
					$content = strtr(preg_replace('|([^ ])' . we_newsletter_base::TITLE_REPLACE_TEXT . '|', '${1} ' . $title, $contentTFL), $rep);
					$content_plain = strtr(preg_replace('|([^ ])' . we_newsletter_base::TITLE_REPLACE_TEXT . '|', '${1} ' . $title, $contentTFL_plain), $rep);
				} else if($title && $lastname){
					$rep = [
						we_newsletter_base::FIRSTNAME_REPLACE_TEXT => $firstname,
						we_newsletter_base::LASTNAME_REPLACE_TEXT => $lastname,
						we_newsletter_base::CUSTOMERID_REPLACE_TEXT => $customerid,
						we_newsletter_base::TITLE_REPLACE_TEXT => $title
					];
					$content = strtr(preg_replace('|([^ ])' . we_newsletter_base::TITLE_REPLACE_TEXT . '|', '${1} ' . $title, $contentTL), $rep);
					$content_plain = strtr(preg_replace('|([^ ])' . we_newsletter_base::TITLE_REPLACE_TEXT . '|', '${1} ' . $title, $contentTL_plain), $rep);
				} else if($lastname && $firstname){
					$rep = [
						we_newsletter_base::FIRSTNAME_REPLACE_TEXT => $firstname,
						we_newsletter_base::LASTNAME_REPLACE_TEXT => $lastname,
						we_newsletter_base::CUSTOMERID_REPLACE_TEXT => $customerid,
					];

					$content = strtr($contentFL, $rep);
					$content_plain = strtr($contentFL_plain, $rep);
				} else if($firstname){
					$rep = [
						we_newsletter_base::FIRSTNAME_REPLACE_TEXT => $firstname,
						we_newsletter_base::CUSTOMERID_REPLACE_TEXT => $customerid
					];
					$content = strtr($contentFN, $rep);
					$content_plain = strtr($contentFN_plain, $rep);
				} else if($lastname){
					$rep = [
						we_newsletter_base::LASTNAME_REPLACE_TEXT => $lastname,
						we_newsletter_base::CUSTOMERID_REPLACE_TEXT => $customerid
					];

					$content = strtr($contentLN, $rep);
					$content_plain = strtr($contentLN_plain, $rep);
				} else {

					$content = $contentDefault;
					$content_plain = $content_plainDefault;
				}

				$secEmail = sha1($email . md5(SECURITY_ENCRYPTION_KEY . WE_DOCUMENT_ROOT)); //use md5, doc root so we will never expose the key
				$content_plain = strtr($content_plain, [we_newsletter_base::EMAIL_REPLACE_TEXT => $email, we_newsletter_base::EMAILTOKEN_REPLACE_TEXT => $secEmail]);
				$content = strtr($content, [we_newsletter_base::EMAIL_REPLACE_TEXT => $email, we_newsletter_base::EMAILTOKEN_REPLACE_TEXT => $secEmail]);

				// damd: Newsletter Platzhalter ersetzten
				$this->replacePlaceholder($content, $content_plain, $emails[$j]);

				//$clean = $this->View->getCleanMail($this->View->newsletter->Reply);

				$not_black = !$this->View->isBlack($email); //Bug #5791 Prüfung muss vor der aufbereitung der Adresse erfolgen
				$emailName = new we_mail_address($email, trim(implode(' ', [$title, $firstname, $lastname])));
				$phpmail = new we_mail_mail([$emailName], $this->Model->Subject, we_mail_address::parse($this->Model->Sender), we_mail_address::parse($this->Model->Reply), $this->Model->isEmbedImages);

				$phpmail->addHTMLPart($content);
				$phpmail->addTextPart(trim($content_plain));

				if(!$this->View->settings['use_base_href']){
					$phpmail->setIsUseBaseHref((bool) $this->View->settings['use_base_href']);
				}

				foreach($atts as $att){
					$phpmail->doaddAttachment($att);
				}
//FIXME: what about inlines?
				$domain = '';
				$not_malformed = ($this->View->settings['reject_malformed']) ? we_check_email($email) : true;
				$verified = ($this->View->settings['reject_not_verified']) ? we_base_util::we_checkMailDomain($email, $domain) : true;

				if($verified && $not_malformed && $not_black){
					if(!$test){
						$phpmail->buildMessage();
						if($phpmail->send()){
							if($this->View->settings['log_sending']){
								$this->Model->addLog(we_newsletter_newsletter::LOG_MAIL_SENT, $email);
							}
						} else {
							if($this->View->settings['log_sending']){
								$this->Model->addLog(we_newsletter_newsletter::LOG_MAIL_FAILED, $email);
							}
							$progress_update['log'][] = sprintf(g_l('modules_newsletter', '[error]') . ': ' . g_l('modules_newsletter', '[mail_failed]'), $email);
						}
						$this->db->query('UPDATE ' . NEWSLETTER_TABLE . ' SET Step=' . intval($egc) . ',Offset=' . intval($j) . ' WHERE ID=' . $this->Model->ID);
					}
				} elseif(!$not_malformed){
					if(!$test && $this->View->settings['log_sending']){
						$this->Model->addLog(we_newsletter_newsletter::LOG_MAIL_MALFORMED, $email);
					}
					$progress_update['log'][] = sprintf(g_l('modules_newsletter', '[error]') . ': ' . g_l('modules_newsletter', '[email_malformed]'), $email);
					$progress_update['log'][] = sprintf(g_l('modules_newsletter', '[mail_not_sent]'), $email);
				} elseif(!$verified){
					if(!$test && $this->View->settings['log_sending']){
						$this->Model->addLog(we_newsletter_newsletter::LOG_DOMAIN_NOK, $email);
					}
					$progress_update['log'][] = sprintf(g_l('modules_newsletter', '[warning]') . ': ' . g_l('modules_newsletter', '[domain_nok]'), $domain);
					$progress_update['log'][] = sprintf(g_l('modules_newsletter', '[mail_not_sent]'), $email);
				} elseif(!$not_black){
					if(!$test && $this->View->settings['log_sending']){
						$this->Model->addLog(we_newsletter_newsletter::LOG_MAIL_BLACK, $email);
					}
					$progress_update['log'][] = sprintf(g_l('modules_newsletter', '[warning]') . ': ' . g_l('modules_newsletter', '[email_is_black]'), $email);
					$progress_update['log'][] = sprintf(g_l('modules_newsletter', '[mail_not_sent]'), $email);
				}
				$ecs++;
			}
		}
		$progress_update['percent'] = ($ecount ? ($ecs / $ecount) * 100 : 0);

		we_cache_file::clean(we_cache_file::DOMAIN_NEWSLETTER . $emailcache . '_' . $egc, true);

		$this->jsCmd->addCmd('updateLog', $progress_update);
		$this->jsCmd->addCmd('nextMails', (!empty($this->View->settings['send_wait']) && intval($this->View->settings['send_wait']) && $egc && intval($this->View->settings['send_step']) && $egc < ceil($ecount / (int) $this->View->settings['send_step']) ? intval($this->View->settings['send_wait']) : 0), $ecs);
		//$laststep = ceil(we_base_request::_(we_base_request::INT, "ecount", 0) / $this->View->settings['send_step']);

		return $this->getHTMLDocument($body, we_html_element::jsScript(WE_JS_MODULES_DIR . 'newsletter/newsletter_property.js', 'self.focus();'));
	}

	private function getHTMLSendControl(): string{
		$nid = we_base_request::int("nid", 0);
		$gcount = we_base_request::int("gcount", 0);
		$ecount = we_base_request::int("ecount", 0);
		$blockcache = we_base_request::_(we_base_request::RAW, "blockcache", '0');
		$ecs = we_base_request::_(we_base_request::RAW, "ecs", '0');
		$emailcache = we_base_request::_(we_base_request::RAW, "emailcache", '0');

		$to = (is_numeric($this->View->settings['send_wait']) ? $this->View->settings['send_wait'] : 0) + 40000;

		return $this->getHTMLDocument(we_html_element::body(['style' => 'margin:10px', 'onload' => "startTimeout();"], we_html_element::form([
									'name' => we_base_constants::WE_FORM,
									'method' => 'post',
									'target' => 'send_cmd',
									'action' => $this->frameset], we_html_element::hiddens([
											'mod' => 'newsletter',
											'nid' => (string) ($nid),
											'pnt' => 'send_cmd',
											'retry' => '1',
											'test' => '0',
											'blockcache' => $blockcache,
											'emailcache' => $emailcache,
											'ecount' => (string) ($ecount),
											'gcount' => (string) ($gcount),
											'ecs' => $ecs,
											'reload' => '0'
										])
								)
						), we_html_element::jsScript(WE_JS_MODULES_DIR . 'newsletter/sendControl.js', 'self.focus();', ['id' => 'loadVarSendControl', 'data-control' => setDynamicVar([
								'to' => $to
		])]));
	}

	/**
	 * Ersetzt die Newsletter Platzthalter
	 *
	 * @author damd
	 * @param String $content
	 * @param String $content_plain
	 * @param array<string,mixed> $customerInfos
	 */
	private function replacePlaceholder(string &$content, string &$content_plain, array $customerInfos): void{
		$placeholderfieldsmatches = [];
		preg_match_all('/' . we_newsletter_util::PLACEHOLDER_PREFIX . '(.[^#]{1,200})####/', $content, $placeholderfieldsmatches);
		$placeholderfields = $placeholderfieldsmatches[1];
		unset($placeholderfieldsmatches);

		$fromCustomer = (isset($customerInfos[8]) && isset($customerInfos[9]) && $customerInfos[9] === 'customer' ?
				array_merge(getHash('SELECT * FROM ' . CUSTOMER_TABLE . ' WHERE ID=' . intval($customerInfos[8])), we_customer_customer::getEncryptedFields()) :
				[]);

		foreach($placeholderfields as $phf){
			$placeholderReplaceValue = $fromCustomer && isset($fromCustomer[$phf]) ? $fromCustomer[$phf] : '';
			$content = str_replace(we_newsletter_util::PLACEHOLDER_PREFIX . $phf . '####', $placeholderReplaceValue, $content);
			$content_plain = str_replace(we_newsletter_util::PLACEHOLDER_PREFIX . $phf . '####', $placeholderReplaceValue, $content_plain);
		}
	}

	function processVariables(): void{
		$this->View->processVariables();
	}

	function processCommands(we_base_jsCmd $jscmd): void{
		$this->View->processCommands($jscmd);
	}

}
