<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: e82607e2d0c1 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2021-10-29 17:17 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_object_cmd extends we_editor_functions implements we_interface_cmd{

	public static function we_cmd(string $cmd): bool{
		[,, $type] = explode('_', $cmd, 4);
		switch($type){
			case 'class':
				self::we_cmdClass($cmd);
				return true;
			case 'file':
				self::we_cmdFile($cmd);
				return true;
		}
		return false;
	}

	private static function we_cmdClass(string $cmd): void{
		$we_transaction = we_base_request::_(we_base_request::TRANSACTION, 'we_transaction');
		$id = we_base_request::string('we_cmd', we_base_request::NOT_VALID_S, 2);
		$we_doc = we_contents_root::getSessionDocument($we_transaction);
		assert($we_doc instanceof we_contents_object);
//
//	---> Setting the Content-Type
//

		$jsCmd = new we_base_jsCmd();
		$content = '';

		switch($cmd){
			case 'we_object_class_insertEntry':
				if($id != we_base_request::NOT_VALID_S){
					[, $after] = explode('_', $id, 2);
					$afterid = $id;
				} else {
					$after = '';
					$afterid = '';
				}
				$identifier = str_replace('.', '', uniqid('', true));
				$uniqid = 'entry_' . $identifier;
				$we_doc->addEntryToClass($identifier, $after);

				$cnt = we_html_element::div(['id' => $uniqid, 'class' => 'objectFileElement'], we_html_element::div(['id' => 'f' . $uniqid, 'class' => 'default'], '<table>' . $we_doc->getFieldHTML($jsCmd, $we_doc->getElementS('wholename' . $identifier), $uniqid, $we_transaction) . '</table>'));
				$content .= we_gui_OrderContainer::wrapField($jsCmd, 'add', $uniqid, $cnt, $afterid);

				$we_doc->saveInSession($we_transaction);
				break;
			case 'we_object_class_changeEntry':
				[, $identifier] = explode('_', $id, 2);
				$fieldname = $we_doc->getElementS('wholename' . $identifier);

				// new fields have no type in name: they are always type input
				$fieldType = count(($tmp = explode('_', $fieldname, 2))) === 2 ? $tmp[0] : we_contents_object::TYPE_INPUT;
				$defaultField = we_contents_objectBase::DEFAULTS_VALUES[$fieldType]['defField'] ?? we_contents_objectBase::DEFAULT_S;
				$defaultValue = we_contents_objectBase::DEFAULTS_VALUES[$fieldType]['defaults'][$defaultField][0];

				switch($defaultField){
					case we_contents_objectBase::DEFAULT_B:
						$we_doc->setElementB($fieldname . we_contents_objectBase::NAME_DIVIDER . $defaultField, (bool) $defaultValue);
						break;
					case we_contents_objectBase::DEFAULT_I:
						$we_doc->setElementI($fieldname . we_contents_objectBase::NAME_DIVIDER . $defaultField, (int) $defaultValue);
						break;
					case we_contents_objectBase::DEFAULT_F:
						$we_doc->setElementF($fieldname . we_contents_objectBase::NAME_DIVIDER . $defaultField, (float) $defaultValue);
						break;
					case we_contents_objectBase::DEFAULT_HTML:
					case we_contents_objectBase::DEFAULT_S:
						$we_doc->setElementS($fieldname . we_contents_objectBase::NAME_DIVIDER . $defaultField, (string) $defaultValue);
						break;
					case we_contents_objectBase::DEFAULT_AI:
					case we_contents_objectBase::DEFAULT_AS:
					case 'arrayHref':
					case 'arrayLink':
						$we_doc->setElementA($fieldname . we_contents_objectBase::NAME_DIVIDER . $defaultField, (array) $defaultValue);
						break;
				}

				$content .= self::reloadElement($jsCmd, $we_transaction, $we_doc, $id);
				break;

			case 'we_object_class_reloadEntry':
				$we_transaction = we_base_request::_(we_base_request::TRANSACTION, 'we_cmd', '', 1);
				$id = we_base_request::string('we_cmd', '', 2);
				$foldIndex = we_base_request::int('we_cmd', 0, 'foldIndex');
				if(!$id){
					return;
				}
				$content .= self::reloadElement($jsCmd, $we_transaction, $we_doc, $id, $foldIndex);
				break;

			case 'we_object_class_selectUp':
				$we_doc->upDownSelectAtClass(we_base_request::string('we_cmd', '', 3), we_base_request::int('we_cmd', 0, 4), true);
				$content .= self::reloadElement($jsCmd, $we_transaction, $we_doc, $id);
				break;
			case 'we_object_class_selectDown':
				$we_doc->upDownSelectAtClass(we_base_request::string('we_cmd', '', 3), we_base_request::int('we_cmd', 0, 4), false);
				$content .= self::reloadElement($jsCmd, $we_transaction, $we_doc, $id);
				break;
			case 'we_object_class_selectInsert':
				$we_doc->addSelectToClass(we_base_request::string('we_cmd', '', 3), we_base_request::int('we_cmd', 0, 4));
				$content .= self::reloadElement($jsCmd, $we_transaction, $we_doc, $id);
				break;
			case 'we_object_class_selectDelete':
				$we_doc->removeSelectFromClass(we_base_request::string('we_cmd', '', 3), we_base_request::int('we_cmd', 0, 4));
				$content .= self::reloadElement($jsCmd, $we_transaction, $we_doc, $id);
				break;

			case 'we_object_class_multiobjectUp':
				$we_doc->upDownMultiobjectAtClass(we_base_request::string('we_cmd', '', 3), we_base_request::int('we_cmd', 0, 4), true);
				$content .= self::reloadElement($jsCmd, $we_transaction, $we_doc, $id);
				break;
			case 'we_object_class_multiobjectDown':
				$we_doc->upDownMultiobjectAtClass(we_base_request::string('we_cmd', '', 3), we_base_request::int('we_cmd', 0, 4), false);
				$content .= self::reloadElement($jsCmd, $we_transaction, $we_doc, $id);
				break;
			case 'we_object_class_multiobjectInsert':
				$we_doc->addMultiobjectToClass(we_base_request::string('we_cmd', '', 3), we_base_request::int('we_cmd', 0, 4));
				$content .= self::reloadElement($jsCmd, $we_transaction, $we_doc, $id);
				break;
			case 'we_object_class_multiobjectDelete':
				$we_doc->removeMultiobjectFromClass(we_base_request::string('we_cmd', '', 3), we_base_request::int('we_cmd', 0, 4));
				$content .= self::reloadElement($jsCmd, $we_transaction, $we_doc, $id);
				break;

			case 'we_object_class_ownersDeleteAll':
				$we_doc->del_all_owners(we_base_request::string('we_cmd', '', 3));
				$content .= self::reloadElement($jsCmd, $we_transaction, $we_doc, $id);
				break;
			case 'we_object_class_fieldAddUser':
				$we_doc->add_user_to_field(we_base_request::intList('we_cmd', [], 3), we_base_request::string('we_cmd', '', 4));
				$content .= self::reloadElement($jsCmd, $we_transaction, $we_doc, $id);
				break;
			case 'we_object_class_fieldDelUser':
				$we_doc->del_user_from_field(we_base_request::int('we_cmd', 0, 3), we_base_request::string('we_cmd', '', 4));
				$content .= self::reloadElement($jsCmd, $we_transaction, $we_doc, $id);
				break;
			case 'we_object_class_removeImage';
				$we_doc->remove_image(we_base_request::string('we_cmd', '', 3));
				$content .= self::reloadElement($jsCmd, $we_transaction, $we_doc, $id);
				break;
			case 'we_object_class_deleteLink':
				$name = we_base_request::string('we_cmd', '', 3);
				$we_doc->delElement($name);
				$content .= self::reloadElement($jsCmd, $we_transaction, $we_doc, $id);
				break;
			case 'we_object_class_changeLink':
				$we_doc->changeLink(we_base_request::string('we_cmd', '', 3));
				$content .= self::reloadElement($jsCmd, $we_transaction, $we_doc, $id);
				break;
			case 'we_object_class_changeMultiobject':
				$we_doc->removeAllMultiobjectFromClass(we_base_request::string('we_cmd', '', 3));
				$content .= self::reloadElement($jsCmd, $we_transaction, $we_doc, $id);
				break;
			case 'we_object_class_deleteEntry':
				if($id){
					[, $identifier] = explode('_', $id, 2);
					$uniqid = 'entry_' . $identifier;
					$we_doc->removeEntryFromClass($identifier);
					$content .= we_gui_OrderContainer::wrapField($jsCmd, 'delete', $uniqid);
					$we_doc->saveInSession($we_transaction);
				}
				break;
			case 'we_object_class_entryUp':
				if($id){
					[, $identifier] = explode('_', $id, 2);
					$uniqid = 'entry_' . $identifier;
					$we_doc->upEntryAtClass($identifier);
					$content .= we_gui_OrderContainer::wrapField($jsCmd, 'up', $uniqid);
					$we_doc->saveInSession($we_transaction);
				}
				break;
			case 'we_object_class_entryDown':
				if($id){
					[, $identifier] = explode('_', $id, 2);
					$uniqid = 'entry_' . $identifier;
					$we_doc->downEntryAtClass($identifier);
					$content .= we_gui_OrderContainer::wrapField($jsCmd, 'down', $uniqid);
					$we_doc->saveInSession($we_transaction);
				}
				break;
			default:
				break;
		}

//	---> Loading the Stylesheets
		$head = '';
		if($we_doc->CSS){
			$cssArr = id2pathA(we_base_request::filterIntList($we_doc->CSS), FILE_TABLE);
			foreach($cssArr as $path){
				'@phan-var string $path';
				$head .= we_html_element::cssLink($path);
			}
		}
		echo we_html_tools::getHtmlTop('', $head . we_editor_script::get($we_doc) . $jsCmd->getCmds(), we_html_element::body([], $content));
	}

	private static function we_cmdFile(string $cmd): void{
//	---> Initalize the document
		$we_transaction = we_base_request::_(we_base_request::TRANSACTION, 'we_cmd', '', 1);
		$identifier = we_base_request::string('we_cmd', '', 2);
		try{
			$we_doc = we_contents_root::getSessionDocument($we_transaction);
		}catch(we_exception_transaction $e){
			//don't do anything.
			return;
		}
		assert($we_doc instanceof we_contents_objectFile);

//	---> Loading the Stylesheets
		$header = '';
		if($we_doc->CSS){
			$cssArr = id2pathA(we_base_request::filterIntList($we_doc->CSS), FILE_TABLE);
			foreach($cssArr as $path){
				'@phan-var string $path';
				$header .= we_html_element::cssLink($path);
			}
		}

		$jsCmd = new we_base_jsCmd();

		switch($cmd){
			case 'we_object_file_reloadEntry':
			case 'we_object_file_multiobjectUp':
			case 'we_object_file_multiobjectDown':
			case 'we_object_file_multiobjectInsert':
			case 'we_object_file_multiobjectDelete':
			case 'we_object_file_objectlinkChange':
			case 'we_object_file_imageRemove':
			case 'we_object_file_linkDelete':
			case 'we_object_file_linkChange':
				$temp = explode('_', $identifier);
				$type = array_shift($temp) ?: ''; //FIXME: can we get a better default?!
				$name = implode('_', $temp);

				switch($cmd){
					case 'we_object_file_multiobjectInsert':
						$we_doc->addSelectToObject($name, we_base_request::int('we_cmd', 0, 3));
						break;
					case 'we_object_file_multiobjectDelete':
						$we_doc->removeSelectFromObject($name, we_base_request::int('we_cmd', 0, 3));
						break;
					case 'we_object_file_multiobjectDown':
						$we_doc->downSelectAtObject($name, we_base_request::int('we_cmd', 0, 3));
						break;
					case 'we_object_file_multiobjectUp':
						$we_doc->upSelectAtObject($name, we_base_request::int('we_cmd', 0, 3));
						break;
					case 'we_object_file_objectlinkChange':
						$we_doc->getLinkedObjects();
						break;
					case 'we_object_file_imageRemove':
						$we_doc->remove_image($name);
						break;
					case 'we_object_file_linkDelete':
						$we_doc->delElement($name);
						break;
					case 'we_object_file_linkChange':
						$we_doc->changeLink($name);
						break;
				}

				$content = we_gui_OrderContainer::wrapField($jsCmd, 'reload', $identifier, '
<div id="' . $identifier . '" class="objectFileElement">
	<div id="f' . $identifier . '" class="default defaultfont">
	' . $we_doc->getFieldHTML($jsCmd, $name, $type, [], []) . '
	</div>
</div>');

				$we_doc->saveInSession($we_transaction);
				break;

			default:
				$content = '';
				break;
		}

		echo we_html_tools::getHtmlTop('', $header . we_editor_script::get($we_doc) . $jsCmd->getCmds(), $content);
	}

	private static function reloadElement(we_base_jsCmd $jsCmd, string $we_transaction, we_contents_object $we_doc, string $id, int $foldIndex = 0): string{
		$idArr = explode('_', $id, 2);
		$identifier = array_pop($idArr);
		$uniqid = 'entry_' . $identifier;
		$wholename = $we_doc->getElementS('wholename' . $identifier);
		if($foldIndex){
			we_html_multiIconBox::setStartPos($foldIndex);
		}

		$content = we_html_element::div(['id' => $uniqid, 'class' => 'objectFileElement', 'data-wholename' => $wholename], we_html_element::div(['id' => 'f' . $uniqid,
							'class' => 'default'], '<table class="defaultfont">' . $we_doc->getFieldHTML($jsCmd, $wholename, $uniqid, $we_transaction) . '	</table>'
		));

		$we_doc->saveInSession($we_transaction);

		return we_gui_OrderContainer::wrapField($jsCmd, 'reload', $uniqid, $content);
	}

}
