<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: b26067fc7281 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2020-02-04 13:57 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_rpcCmd_documentChangeDocType extends we_rpc_cmd{

	function execute(): we_rpc_response{
		$categories = '<tr><td style="font-size:8px">&nbsp;</td></tr>';
		if(($dt = we_base_request::int('docType', -1)) > -1){
			if($dt >= 0){
				$values = getHash('SELECT dt.*,dtf.Path FROM ' . DOC_TYPES_TABLE . ' dt LEFT JOIN ' . FILE_TABLE . ' dtf ON dt.ParentFileID=dtf.ID WHERE dt.ID=' . $dt);

				$ids_arr = array_filter(explode(',', $values['Templates']));

				$paths_arr = id2pathA($values['Templates'], TEMPLATES_TABLE);
				$TPLselect = new we_html_select([
					'name' => 'docTypeTemplateId',
					'class' => 'weSelect',
					'onclick' => (we_modules_info::isActive(we_modules_info::OBJECT)) ? "self.document.forms['" . we_base_constants::WE_FORM . "'].elements['v[import_type]'][0].checked=true;" : '',
					//"onchange"  => "we_submit_form(self.document.forms['we_form'], 'wizbody', '".$this->path."');",
					'style' => 'width: 300px']
				);
				$optid = 0;
				foreach($paths_arr as $templateID => $path){
					$TPLselect->insertOption($optid, (string) $templateID, (string) $path);
					$optid++;
				}
				$templateElement = we_html_tools::formElementTable(g_l('import', '[template]'), $TPLselect->getHTML());
				$categories = ($values['Category'] ? $categories : $this->getCategories('doc', we_base_request::filterIntList($values['Category']), 'v[docCategories]'));

				$categories = strtr($categories, ["\r" => '', "\n" => '']);
				if($ids_arr){
					$docTypeLayerDisplay = 'block';
					$noDocTypeLayerDisplay = 'none';
				} else {
					$docTypeLayerDisplay = 'none';
					$noDocTypeLayerDisplay = 'block';
				}
				$templateName = '';
				if(!empty($values['TemplateID'])){
					$templateName = fS('SELECT Path FROM ' . TEMPLATES_TABLE . ' WHERE ID=' . intval($values['TemplateID']));
				}

				$this->resp->setData('elements', [
					['name' => 'v[store_to_id]', 'type' => 'formelement', 'props' => [['type' => 'attrib', 'name' => 'value', 'val' => ($values['ParentFileID'] ?: 0)]]],
					['name' => 'v[store_to_path]', 'type' => 'formelement', 'props' => [['type' => 'attrib', 'name' => 'value', 'val' => ($values['Path'] ?: '/')]]],
					['name' => 'v[we_TemplateID]', 'type' => 'formelement', 'props' => [['type' => 'attrib', 'name' => 'value', 'val' => ($values['TemplateID'] ?: 0)]]],
					['name' => 'v[we_TemplateName]', 'type' => 'formelement', 'props' => [['type' => 'attrib', 'name' => 'value', 'val' => ($templateName | '')]]],
					['name' => 'v[we_Extension]', 'type' => 'formelement', 'props' => [['type' => 'attrib', 'name' => 'value', 'val' => ($values['Extension'] ?: '')]]],
					['name' => 'v[is_dynamic]', 'type' => 'formelement', 'props' => [['type' => 'attrib', 'name' => 'value', 'val' => ($values['IsDynamic'] ?: 0)]]],
					['name' => 'chbxIsDynamic', 'type' => 'formelement', 'props' => [['type' => 'attrib', 'name' => 'value', 'val' => ($values['IsDynamic'] ?: 0)]]],
					['name' => 'v[docCategories]', 'type' => 'formelement', 'props' => [['type' => 'attrib', 'name' => 'value', 'val' => ($values['Category'] ?: '')]]],
					['name' => 'noDocTypeTemplateId', 'type' => 'formelement', 'props' => [['type' => 'attrib', 'name' => 'value', 'val' => 0]]],
					['name' => 'docTypeLayer', 'type' => 'node', 'props' => [['type' => 'innerHTML', 'val' => $templateElement], ['type' => 'style', 'name' => 'display', 'val' => $docTypeLayerDisplay]]],
					['name' => 'noDocTypeLayer', 'type' => 'node', 'props' => [['type' => 'style', 'name' => 'display', 'val' => $noDocTypeLayerDisplay]]],
					['name' => 'docCatTable', 'type' => 'node', 'props' => [['type' => 'innerHTML', 'val' => addslashes($categories)]]],
					]
				);
			} else {
				$this->resp->setData('elements', [
					['name' => 'v[store_to_id]', 'type' => 'formelement', 'props' => [['type' => 'attrib', 'name' => 'value', 'val' => 0]]],
					['name' => 'v[store_to_path]', 'type' => 'formelement', 'props' => [['type' => 'attrib', 'name' => 'value', 'val' => '/']]],
					['name' => 'v[we_TemplateID]', 'type' => 'formelement', 'props' => [['type' => 'attrib', 'name' => 'value', 'val' => 0]]],
					['name' => 'v[we_TemplateName]', 'type' => 'formelement', 'props' => [['type' => 'attrib', 'name' => 'value', 'val' => '/']]],
					['name' => 'v[we_Extension]', 'type' => 'formelement', 'props' => [['type' => 'attrib', 'name' => 'value', 'val' => '']]],
					['name' => 'v[is_dynamic]', 'type' => 'formelement', 'props' => [['type' => 'attrib', 'name' => 'value', 'val' => 0]]],
					['name' => 'chbxIsDynamic', 'type' => 'formelement', 'props' => [['type' => 'attrib', 'name' => 'value', 'val' => 0]]],
					['name' => 'v[docCategories]', 'type' => 'formelement', 'props' => [['type' => 'attrib', 'name' => 'value', 'val' => '']]],
					['name' => 'noDocTypeTemplateId', 'type' => 'formelement', 'props' => [['type' => 'attrib', 'name' => 'value', 'val' => 0]]],
					['name' => 'docTypeLayer', 'type' => 'node', 'props' => [['type' => 'innerHTML', 'val' => ''], ['type' => 'style', 'name' => 'display', 'val' => "none"]]],
					['name' => 'noDocTypeLayer', 'type' => 'node', 'props' => [['type' => 'style', 'name' => 'display', 'val' => "block"]]],
					['name' => 'docCatTable', 'type' => 'node', 'props' => [['type' => 'innerHTML', 'val' => $categories]]],
					]
				);
			}
		}
		return $this->resp;
	}

	/**
	 *
	 * @param string $obj
	 * @param int[] $categories
	 * @param string $catField
	 * @return string
	 */
	private function getCategories(string $obj, array $categories, string $catField = ''): string{
		$cats = new we_chooser_multiDirExtended($categories, 'delete_' . $obj . 'Cat', '', [], we_base_ContentTypes::CATEGORY, CATEGORY_TABLE);
		$cats->setRowPrefix($obj);
		$cats->setCatField($catField);
		return $cats->getTableRows();
	}

}
