<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 70304633e836 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2021-08-26 11:34 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
//we_modules_info::isActive(we_modules_info::SCHEDULER);

class we_schedule_view extends we_schedule_scheduler{

	/** @var we_schedule_entry */
	public $entry;

	/** @var int */
	var $nr = 0;

	/** @var int[] */
	private static $extraCont = [];

	/** @var we_document_base */
	private $we_doc;

	function __construct(we_document_base $we_doc, we_schedule_entry $entry, int $nr = 0){
		$this->we_doc = $we_doc;
		$this->entry = $entry;
		$this->nr = $nr;
	}

	private function getMonthsHTML(): string{
		$ret = '<table class="schedMonths"><tr>';
		$months = we_base_country::getTranslationList(we_base_country::MONTH, array_search(we_base_WE::$Language, we_base_country::getWELangs(), true) ?: 'de');

		for($i = 1; $i <= 12; $i++){
			$ret .= '<td>' .
					we_html_forms::checkboxWithHidden(!empty($this->entry->months[$i - 1]), 'we[schedule][' . $this->nr . '][' . self::SUBMIT_MONTHS . '][' . ($i - 1) . ']', $months['abbreviated'][$i], false, '', "_EditorFrame.setEditorIsHot(true)") . '</td>';
		}

		return $ret . '</tr></table>';
	}

	private function getDaysHTML(): string{
		$days = '<table class="schedDays"><tr>';

		for($i = 1; $i <= 31; $i++){
			if($i <= 31){
				$days .= '<td>' . we_html_forms::checkboxWithHidden(!empty($this->entry->days[$i - 1]), 'we[schedule][' . $this->nr . '][' . self::SUBMIT_DAYS . '][' . ($i - 1) . ']', sprintf('%02d', $i), false, '', "_EditorFrame.setEditorIsHot(true)") .
						'</td>';
			}
			switch($i){
				case 14:
				case 28:
					$days .= '</tr><tr>';
					break;
			}
		}

		return $days . '</tr></table>';
	}

	private function getWeekdaysHTML(): string{
		$ret = '<table class="schedWDays"><tr>';
		$days = we_base_country::getTranslationList(we_base_country::DAY, array_search(we_base_WE::$Language, we_base_country::getWELangs(), true) ?: 'de');
		$order = array_flip($days['narrow']);

		for($i = 1; $i <= 7; $i++){
			$ret .= '<td>' . we_html_forms::checkboxWithHidden(!empty($this->entry->weekdays[$i - 1]), 'we[schedule][' . $this->nr . '][' . self::SUBMIT_WDAYS . '][' . ($i - 1) . ']', $days['abbreviated'][$order[$i]], false, '', "_EditorFrame.setEditorIsHot(true)") . '</td>';
		}

		return $ret . '</tr></table>';
	}

	//needed to switch description of button publish to "save to scheduler" and vice versa
	public static function getMainJS(we_contents_root $doc): string{
		return we_html_element::jsScript(WE_JS_MODULES_DIR . 'schedule/scheduler.js', '', [
					'id' => 'loadVarScheduler',
					'data-scheduler' => setDynamicVar([
						'we_hasExtraRow' => self::$extraCont,
						'selection' => [
							'DOCTYPE' => self::DOCTYPE,
							'CATEGORY' => self::CATEGORY,
							'DIR' => self::DIR,
							'SCHEDULE_FROM' => self::SCHEDULE_FROM
						],
						'docID' => $doc->ID,
		])]);
	}

	function getHTML(we_base_jsCmd $jsCmd, bool $isobj = false): string{
		$taskpopup = (new we_html_select(['class' => 'weSelect we_schedule_task', 'name' => 'we[schedule][' . $this->nr . '][' . self::SUBMIT_TASK . ']', 'onchange' => "changeSchedOption(this,' . $this->nr . ');"]))
				->addOptions(array_filter([
					self::SCHEDULE_FROM => g_l('modules_schedule', '[task][publish]'),
					self::SCHEDULE_TO => g_l('modules_schedule', '[task][park]'),
					self::DELETE => ((we_base_permission::hasPerm([we_base_perm::WORK_DOCUMENT_DELETE]) && (!$isobj)) || (we_base_permission::hasPerm([we_base_perm::OBJECTFILE_DELETE]) && $isobj) ? g_l('modules_schedule', '[task][delete]') : ''),
					self::DOCTYPE => ($isobj ? '' : g_l('modules_schedule', '[task][doctype]') ),
					self::CALL => ($isobj ? '' : g_l('modules_schedule', '[task][call]')),
					self::CATEGORY => (we_modules_info::isActive(we_modules_info::CATEGORY) ? g_l('modules_schedule', '[task][category]') : ''),
					self::DIR => ((we_base_permission::hasPerm([we_base_perm::WORK_DOCUMENT_MOVE]) && (!$isobj)) || (we_base_permission::hasPerm([we_base_perm::OBJECTFILE_MOVE]) && $isobj) ? g_l('modules_schedule', '[task][directory]') : ''),
					self::SEARCHABLE_ENABLED => g_l('modules_schedule', '[task][search_enable]'),
					self::SEARCHABLE_DISABLED => g_l('modules_schedule', '[task][search_disable]')
				]))
				->selectOption($this->entry->task);

		$extracont = $extraheadl = '';

		switch($this->entry->task){
			case self::DOCTYPE:
				$dtq = we_doctype_docTypes::getDoctypeQuery();
				$all = DB_WE::$inst->getAllFirstq('SELECT dt.ID,dt.Text FROM ' . DOC_TYPES_TABLE . ' dt LEFT JOIN ' . FILE_TABLE . ' dtf ON dt.ParentFileID=dtf.ID ' . $dtq['join'] . ' WHERE ' . $dtq['where'], false);
				'@phan-var array<int,string> $all';
				$doctypepop = (new we_html_select(['class' => 'weSelect', 'name' => 'we[schedule][' . $this->nr . '][' . self::SUBMIT_DOCTYPE . ']', 'onchange' => "_EditorFrame.setEditorIsHot(true);"]))
						->addOptions($all)
						->selectOption($this->entry->doctypeID);

				$extracont = '<table class="default"><tr><td>' . $doctypepop->getHtml() . '</td><td>&nbsp;&nbsp;</td><td>' .
						we_html_forms::checkboxWithHidden($this->entry->doctypeDefault, 'we[schedule][' . $this->nr . '][' . self::SUBMIT_DOCTYPE_DEFAULT . ']', g_l('modules_schedule', '[doctypeAll]'), false, '', "_EditorFrame.setEditorIsHot(true);") .
						'</td></tr></table>';
				$extraheadl = g_l('modules_schedule', '[doctype]');
				break;
			case self::CATEGORY:
				$delallbut = we_html_button::create_button(we_html_button::DELETE_ALL, "javascript:we_cmd('we_schedule_delete_all_schedcats'," . $this->nr . ")");
				$addbut = we_html_button::create_button(we_html_button::ADD, "javascript:we_cmd('we_selector_document',{id:-1,table:WE().consts.tables.CATEGORY_TABLE,jsCmd:'we_schedule_add_schedcat," . $this->nr . "'});setScrollTo(" . we_base_constants::WE_EDITPAGE_SCHEDULER . ");");
				$cats = new we_chooser_multiDir($this->entry->categoryIDs, 'we_schedule_delete_schedcat', $delallbut . $addbut, [], '"' . we_base_ContentTypes::CATEGORY . '"', CATEGORY_TABLE, '', (string) ($this->nr), 'setScrollTo(" . we_base_constants::WE_EDITPAGE_SCHEDULER . ");');
				if(!we_base_permission::hasPerm([we_base_perm::WORK_CATEGORY_EDIT])){
					$cats->setEditable(false);
				}
				$extracont = $cats->get($jsCmd);
				$extraheadl = g_l('modules_schedule', '[categories]');
				break;
			case self::DIR:
				$textname = 'path_we_schedule_' . self::SUBMIT_PARENT . '_' . $this->nr;
				$idname = 'we[schedule][' . $this->nr . '][' . self::SUBMIT_PARENT . ']';
				$myid = $this->entry->parentID;
				$path = id2path($this->entry->parentID, $this->we_doc->Table);

				if($this->we_doc instanceof we_contents_objectFile){
					$path = ($path === '/' ? $this->we_doc->RootDirPath : $path); //	impossible for documents
					$rootDirID = intval($this->we_doc->rootDirID);
				} else {
					$rootDirID = 0;
				}

				$weSuggest = we_gui_suggest::getInstance();
				$weSuggest->setAcId('WsDir')
						->setContentType([we_base_ContentTypes::FOLDER_DOCUMENT])
						->setInput($textname, $path)
						->setMaxResults(20)
						->setRequired(true)
						->setResult($idname, (string) ($myid))
						->setSelector(we_gui_suggest::DirSelector)
						->setTable(FILE_TABLE)
						->setWidth(320)
						->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_directory',{id:document.we_form.elements['" . $idname . "'].value,table:'" . $this->we_doc->Table . "',targetID:'" . $idname . "',targetName:'" . $textname . "',jsCmd:'setHot',rootDirID:" . $rootDirID . "})"));

				$extracont = $weSuggest->getHTML();
				$extraheadl = g_l('modules_schedule', '[dirctory]');
		}

		$typepopup = (new we_html_select(['class' => 'weSelect', 'name' => 'we[schedule][' . $this->nr . '][' . self::SUBMIT_RERUN . ']', 'onchange' => "_EditorFrame.setEditorIsHot(true);setScrollTo(" . we_base_constants::WE_EDITPAGE_SCHEDULER . ");we_cmd('reload_editpage')"]))
				->addOptions([
					self::RERUN_ONCE => g_l('modules_schedule', '[type][' . self::RERUN_ONCE . ']'),
					self::RERUN_HOUR => g_l('modules_schedule', '[type][' . self::RERUN_HOUR . ']'),
					self::RERUN_DAY => g_l('modules_schedule', '[type][' . self::RERUN_DAY . ']'),
					self::RERUN_WEEK => g_l('modules_schedule', '[type][' . self::RERUN_WEEK . ']'),
					self::RERUN_MONTH => g_l('modules_schedule', '[type][' . self::RERUN_MONTH . ']'),
					self::RERUN_YEAR => g_l('modules_schedule', '[type][' . self::RERUN_YEAR . ']'),
				])
				->selectOption($this->entry->rerun);

		$table = '<table class="default">
	<tr style="vertical-align:top">
		<td class=" lowContrast" style="margin-bottom:10px;">' . g_l('modules_schedule', '[task][headline]') . ':</td>
		<td><table class="default"><tr><td>' . $taskpopup->getHtml() . '</td><td>&nbsp;&nbsp;</td><td>' .
				we_html_forms::checkboxWithHidden($this->entry->active, 'we[schedule][' . $this->nr . '][' . self::SUBMIT_ACTIVE . ']', g_l('modules_schedule', '[active]'), false, 'we_schedule_active', '_EditorFrame.setEditorIsHot(true);checkFooter();') .
				'</td></tr></table></td>
		<td>' . we_html_button::create_button(we_html_button::TRASH, "javascript:_EditorFrame.setEditorIsHot(true);we_cmd('we_schedule_del','" . $this->nr . "')") . '</td>
	</tr>' .
				($extracont ? '
	<tr style="vertical-align:top">
		<td class=" lowContrast" style="margin-bottom:10px;">' . $extraheadl . ':</td>
		<td >' . $extracont . '</td>
		<td></td>
	</tr>' :
				'') . '
	<tr style="vertical-align:top">
		<td class=" lowContrast" style="margin-bottom:10px;">' . g_l('modules_schedule', '[type][headline]') . ':</td>
		<td>' . $typepopup->getHtml() . '</td>
		<td></td>
	</tr>';

		switch($this->entry->rerun){
			case self::RERUN_ONCE:
				$table .= '
		<tr style="vertical-align:top">
		<td class=" lowContrast">' . g_l('modules_schedule', '[datetime]') . ':</td>
		<td>' . we_html_tools::getNewDateInput('we[schedule][' . $this->nr . '][' . self::SUBMIT_TIME . '][%s]', $this->entry->time, we_html_tools::FORMAT_DATE_TIME, true, true) . '</td>
		<td></td>
		</tr>';
				break;
			case self::RERUN_HOUR:
				$table .= '
		<tr style="vertical-align:top">
		<td class=" lowContrast">' . g_l('modules_schedule', '[minutes]') . ':</td>
		<td>' . we_html_element::input(['type' => 'number', 'min' => '0', 'max' => '59', 'name' => 'we[schedule][' . $this->nr . '][' . self::SUBMIT_TIME . '][min]',
							'value' => $this->entry->time->format('i'),
							'onchange' => "we_cmd('setHot');",]) . '</td>
		<td></td>
		</tr>';
				break;
			case self::RERUN_DAY:
				$table .= '
		<tr style="vertical-align:top">
		<td class=" lowContrast">' . g_l('modules_schedule', '[time]') . ':</td>
		<td>' . we_html_tools::getNewDateInput('we[schedule][' . $this->nr . '][' . self::SUBMIT_TIME . '][%s]', $this->entry->time, we_html_tools::FORMAT_TIME, true, true) . '</td>
		<td></td>
		</tr>';
				break;
			case self::RERUN_WEEK:
				$table .= '
		<tr style="vertical-align:top">
		<td class=" lowContrast" style="margin-bottom:10px;">' . g_l('modules_schedule', '[time]') . ':</td>
		<td>' . we_html_tools::getNewDateInput('we[schedule][' . $this->nr . '][' . self::SUBMIT_TIME . '][%s]', $this->entry->time, we_html_tools::FORMAT_TIME, true, true) . '</td>
		<td></td>
		</tr>
		<tr style="vertical-align:top">
		<td class=" lowContrast">' . g_l('modules_schedule', '[weekdays]') . ':</td>
		<td>' . $this->getWeekdaysHTML() . '</td>
		<td></td>
		</tr>';
				break;
			case self::RERUN_MONTH:
				$table .= '
		<tr style="vertical-align:top">
		<td class=" lowContrast" style="margin-bottom:10px;">' . g_l('modules_schedule', '[time]') . ':</td>
		<td>' . we_html_tools::getNewDateInput('we[schedule][' . $this->nr . '][' . self::SUBMIT_TIME . '][%s]', $this->entry->time, we_html_tools::FORMAT_TIME, true, true) . '</td>
		<td></td>
		</tr>
		<tr style="vertical-align:top">
		<td class=" lowContrast">' . g_l('modules_schedule', '[days]') . ':</td>
		<td>' . $this->getDaysHTML() . '</td>
		<td></td>
		</tr>';
				break;
			case self::RERUN_YEAR:
				$table .= '
		<tr style="vertical-align:top">
		<td class=" lowContrast" style="margin-bottom:10px;">' . g_l('modules_schedule', '[time]') . ':</td>
		<td>' . we_html_tools::getNewDateInput('we[schedule][' . $this->nr . '][' . self::SUBMIT_TIME . '][%s]', $this->entry->time, we_html_tools::FORMAT_TIME, true, true) . '</td>
		<td></td>
		</tr>
		<tr style="vertical-align:top">
		<td class=" lowContrast" style="margin-bottom:10px;">' . g_l('modules_schedule', '[months]') . ':</td>
		<td>' . $this->getMonthsHTML() . '</td>
		<td></td>
		</tr>
		<tr style="vertical-align:top">
		<td class=" lowContrast">' . g_l('modules_schedule', '[days]') . ':</td>
		<td>' . $this->getDaysHTML() . '</td>
		<td></td>
		</tr>';
				break;
		}
		$table .= '</table>';
		self::$extraCont[$this->nr] = ($extracont ? 1 : 0);
		return $table;
	}

}
