<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 02100279015e $
 * $Author: Lukas Imhof $
 * $Date: 2021-11-05 13:28 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_selector_document extends we_selector_directory{
	protected $userCanMakeNewFile = true;
	protected $titles = [];
	protected $titleName = '';
	protected $startPath;
	protected $ctp = [//FIXME: add audio button
		we_base_ContentTypes::IMAGE => [we_base_perm::WORK_IMAGE_NEW],
		we_base_ContentTypes::VIDEO => [we_base_perm::WORK_VIDEO_NEW],
		we_base_ContentTypes::COLLECTION => [we_base_perm::WORK_COLLECTION_NEW],
	];
	protected $ctb = ['' => "btn_add_file",
		we_base_ContentTypes::IMAGE => 'fa:btn_add_image,fa-upload,fa-lg fa-file-image-o',
		we_base_ContentTypes::VIDEO => 'fa:btn_add_video,fa-upload,fa-lg fa-file-video-o',
		we_base_ContentTypes::COLLECTION => 'fa:btn_add_collection,fa-plus,fa-lg fa-archive',
	];

	public function __construct(int $id, string $table = '', string $JSIDName = '', string $JSTextName = '', string $JSCommand = '', string $order = '', string $newFolderText = '', string $filter = '', int $rootDirID = 0, bool $open_doc = false, bool $multiple = false, bool $canSelectDir = false, int $startID = 0, string $lang = ''){
		parent::__construct($id, $table, $JSIDName, $JSTextName, $JSCommand, $order, $newFolderText, $rootDirID, $multiple, $filter, $startID);

		switch($this->table){
			case FILE_TABLE:
				$this->fields .= ',RestrictOwners,Owners,OwnersReadOnly,CreatorID,Filename,Extension,DATE_FORMAT(Mod_Date,"' . g_l('weEditorInfo', '[mysql_date_format]') . '") AS Mod_Date,IsPublished,ContentType';
				break;
			case VFILE_TABLE:
				$this->fields .= ',RestrictOwners,Owners,OwnersReadOnly,CreatorID,DATE_FORMAT(Mod_Date,"' . g_l('weEditorInfo', '[mysql_date_format]') . '") AS Mod_Date,1 AS IsPublished,Text AS Filename,ContentType';
				break;
			case OBJECT_TABLE :
				$this->fields .= ',RestrictOwners,Owners,OwnersReadOnly,CreatorID,Text AS Filename,DATE_FORMAT(Mod_Date,"' . g_l('weEditorInfo', '[mysql_date_format]') . '") AS Mod_Date,1 AS IsPublished,ContentType';
				$this->useID = true;
				break;
			case OBJECT_FILES_TABLE :
				$this->fields .= ',RestrictOwners,Owners,OwnersReadOnly,CreatorID,Text AS Filename,DATE_FORMAT(Mod_Date,"' . g_l('weEditorInfo', '[mysql_date_format]') . '") AS Mod_Date,1 AS IsPublished,ContentType';
				break;
			case CATEGORY_TABLE:
				$this->fields .= ',Text AS Filename,Title,DATE_FORMAT(Mod_Date,"' . g_l('weEditorInfo', '[mysql_date_format]') . '") AS Mod_Date,1 AS IsPublished';
				$this->filter = implode(',', [we_base_ContentTypes::CATEGORY, we_base_ContentTypes::CATEGORIES]);
				break;
			case TEMPLATES_TABLE:
				$this->useID = true;
			default:
				$this->fields .= ',RestrictOwners,Owners,OwnersReadOnly,CreatorID,Filename,Extension,DATE_FORMAT(Mod_Date,"' . g_l('weEditorInfo', '[mysql_date_format]') . '") AS Mod_Date,1 AS IsPublished,ContentType';
		}

		$this->canSelectDir = $canSelectDir;
		$this->language = $lang;

		$this->title = g_l('fileselector', '[docSelector][title]');
		$this->userCanMakeNewFile = $this->userCanMakeNewFile();
		$this->open_doc = $open_doc;
	}

	protected function printCmdHTML(): void{
		$isWS = we_users_util::in_workspace($this->dir, we_users_util::get_ws($this->table), $this->table);
		$this->jsCmd->addCmd('setButtons', [['NewFileBut', $isWS && $this->userCanMakeNewFile]]);
		parent::printCmdHTML();
	}

	protected function query(): array{
		$filterQuery = '';
		if($this->filter){
			$contentTypes = explode(',', $this->filter);
			$filterQuery .= ' AND (  ' . ($contentTypes ? 'ContentType IN ("' . implode('","', $contentTypes) . '") OR ' : '') . ' isFolder=1)';
		}
		if($this->language){
			$filterQuery .= ' AND (Language="' . $this->language . '" OR isFolder=1)';
		}
		// deal with workspaces
		$wsQuery = '';
		if(we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR]) || ($this->table === FILE_TABLE && we_base_permission::hasPerm([we_base_perm::WORK_WORKSPACE_CAN_SELECT_OTHER_FILES])) || ($this->table === OBJECT_FILES_TABLE && we_base_permission::hasPerm([
				we_base_perm::WORK_WORKSPACE_CAN_SELECT_OTHER_FILES]))){

		} else {
			if(we_users_util::get_ws($this->table)){
				$wsQuery = self::getWsQuery($this->table);
			} else if($this->table === OBJECT_FILES_TABLE && (!we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR]))){
				$ac = we_users_util::getAllowedClasses();
				$wsQuery = ($ac ? 'TableID IN(' . implode(',', $ac) . ')' : '');
			}
			$wsQuery = $wsQuery ?: ' OR RestrictOwners=0 ';
		}

		$owners = we_users_util::makeOwnersSql();

		switch($this->table){
			case FILE_TABLE:
				$this->titles = $this->db->getAllFirstq('SELECT f.ID, c.Dat FROM ' . FILE_TABLE . ' f JOIN ' . CONTENT_TABLE . ' c ON (f.ID=c.DID) WHERE c.DocumentTable="' . stripTblPrefix(FILE_TABLE) . '" AND f.ParentID=' . intval($this->dir) . ' AND c.nHash=unhex(md5("Title")) ' . $filterQuery, false);
				break;
			case OBJECT_FILES_TABLE :
				$hash = getHash('SELECT o.DefaultTitle,o.ID FROM ' . OBJECT_TABLE . ' o JOIN ' . OBJECT_FILES_TABLE . ' `of` ON o.ID=`of`.TableID WHERE `of`.ID=' . intval($this->dir));

				$this->titleName = ($hash ? $hash['DefaultTitle'] : '');
				if($this->titleName && str_contains($this->titleName, '_') && $this->titleName != '_'){
					$this->titles = $this->db->getAllFirstq('SELECT obx.OF_ID,obx.`' . $this->titleName . '` FROM ' . OBJECT_X_TABLE . $hash['ID'] . ' obx JOIN ' . OBJECT_FILES_TABLE . ' `of` ON `of`.ID=obx.OF_ID WHERE `of`.ParentID=' . intval($this->dir), false);
				}
				break;
			case CATEGORY_TABLE:
				$filterQuery = $owners = $wsQuery = '';
				break;
		}
		return $this->db->getAllq('SELECT ' . $this->fields . ' FROM ' . $this->db->escape($this->table) . ' WHERE ParentID=' . intval($this->dir) . ' AND((1 ' .
				$owners . ')' .
				$wsQuery . ')' .
				$filterQuery . //$publ_q.
				($this->order ? (' ORDER BY IsFolder DESC,' . $this->order) : '')
		);
	}

	protected function getFramsetJSFile(): string{
		return parent::getFramsetJSFile() . we_html_element::jsScript(JS_DIR . 'selectors/document_selector.js');
	}

	protected function setDefaultDirAndID(bool $setLastDir): void{
		$ws = we_users_util::get_ws($this->table);
		$rootDirID = ($ws ? reset($ws) : 0);

		$this->dir = (int) ($this->startID ?: ($setLastDir && isset($_SESSION['weS']['we_fs_lastDir'][$this->table]) ? intval($_SESSION['weS']['we_fs_lastDir'][$this->table]) : $rootDirID));
		if($this->rootDirID){
			if(!in_parentID($this->dir, $this->rootDirID, $this->table)){
				$this->dir = $this->rootDirID;
			}
		}
		$this->path = '';
		$this->values = [
			'ParentID' => 0,
			'Text' => '/',
			'Path' => '/',
			'IsFolder' => 1,
			'Mod_Date' => '',
			'RestrictOwners' => 0,
			'Owners' => '',
			'OwnersReadOnly' => '',
			'CreatorID' => 0,
			'ContentType' => ''];
		$this->id = 0;
	}

	protected function getFsQueryString(string $what): string{
		return WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=' . get_class($this) . '&we_cmd[what]=' . $what . '&we_cmd[rootDirID]=' . $this->rootDirID . '&we_cmd[table]=' . $this->table . '&we_cmd[id]=' . $this->id . '&we_cmd[order]=' . $this->order . '&we_cmd[startID]=' . $this->startID . '&we_cmd[filter]=' . $this->filter . '&we_cmd[open_doc]=' . intval($this->open_doc) . '&we_cmd[lang]=' . $this->language;
	}

	protected function printCmdAddEntriesHTML(): void{
		$entries = [];
		foreach($this->query() as $item){
			$title = strip_tags(strtr((string) (empty($this->titles[$item['ID']]) ? ($item['Title'] ?? '-') : $this->titles[$item['ID']]), [
				'"' => '\"',
				"\n\r" => ' ',
				"\n" => ' ',
				"\\" => "\\\\",
				'°' => '&deg;',
			]));

			$entries[] = [
				intval($item['ID']),
				$item['Filename'],
				$item['Extension'] ?? '',
				(int) $item['IsFolder'],
				$item['Path'],
				$item['Mod_Date'],
				$item['ContentType'],
				$item['IsPublished'],
				$title
			];
		}

		if($entries){
			$this->jsCmd->addCmd('addEntries', $entries);
		}
		$buttons = [];
		switch($this->filter){
			case we_base_ContentTypes::TEMPLATE:
			case we_base_ContentTypes::OBJECT:
			case we_base_ContentTypes::OBJECT_FILE:
			case we_base_ContentTypes::WEDOCUMENT:
				break;
			default:
				$tmp = ((we_users_util::in_workspace($this->dir, we_users_util::get_ws($this->table), $this->table)) && $this->userCanMakeNewFile) ? 'true' : 'false';
				$buttons[] = ['NewFileBut', $tmp];
		}

		$buttons[] = ['NewFolderBut', $this->userCanMakeNewDir()];
		$this->jsCmd->addCmd('setButtons', $buttons);
		$this->jsCmd->addCmd('writeBody');
	}

	protected function printHeaderHeadlines(): string{
		switch($this->table){
			case OBJECT_TABLE :
			case TEMPLATES_TABLE:
				return "
<th class='selector selectoricon'></th>
<th class='selector filename'>" . g_l('fileselector', '[filename]') . "</th>
<th class='selector title'>ID</th>
<th class='selector modddate'>" . g_l('fileselector', '[modified]') . "</th>
<th class='selector remain'></th>";
			default:
				return "
<th class='selector selectoricon'></th>
<th class='selector filename'>" . g_l('fileselector', '[filename]') . "</th>
<th class='selector title'>" . g_l('fileselector', '[title]') . "</th>
<th class='selector moddate'>" . g_l('fileselector', '[modified]') . "</th>
<th class='selector remain'></th>";
		}
	}

	protected function printHeaderTable(string $extra = '', bool $append = true): string{
		switch($this->table){
			case FILE_TABLE:
				$extra = '<td>' .
					($this->filter && isset($this->ctb[$this->filter]) ?
					we_html_button::create_button($this->ctb[$this->filter], "javascript:WE().util.dialogTop(window).newFile();", '', "", !$this->userCanMakeNewFile, false, '', false, '', '', 'btn_add_file') :
					(we_base_permission::hasPerm([we_base_perm::WORK_IMAGE_NEW, we_base_perm::WORK_OTHER_NEW]) ? we_html_button::create_button(we_html_button::ADD_FILE, "javascript:WE().util.dialogTop(window).newFile();", '', "", !$this->userCanMakeNewFile, false, '', false, '', '', 'btn_add_file') :
					'')
					) .
					'</td>' .
					$extra;
				break;

			case VFILE_TABLE:
				$extra = '<td>' . we_html_button::create_button($this->ctb[we_base_ContentTypes::COLLECTION], "javascript:WE().util.dialogTop(window).newCollection();", '', "", !$this->userCanMakeNewFile, false, '', false, '', '', 'btn_add_file') . '</td>';
				break;
		}
		return parent::printHeaderTable($extra, true);
	}

	protected function userCanMakeNewFile(){
		if(we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR])){
			return true;
		}
		if(!$this->userCanSeeDir()){
			return false;
		}
		if($this->filter && isset($this->ctp[$this->filter])){
			if(!we_base_permission::hasPerm($this->ctp[$this->filter])){
				return false;
			}
		} elseif(!
			(
			we_base_permission::hasPerm([we_base_perm::WORK_IMAGE_NEW, we_base_perm::WORK_HTML_NEW, we_base_perm::WORK_JS_NEW, we_base_perm::WORK_CSS_NEW, we_base_perm::WORK_TEXT_NEW,
				we_base_perm::WORK_HTACCESS_NEW,
				we_base_perm::WORK_VIDEO_NEW, we_base_perm::WORK_OTHER_NEW, we_base_perm::EXIM_IMPORT_FILE])
			)
		){
			return false;
		}

		return true;
	}

	protected function printSetDirHTML(): void{
		$isWS = $this->userCanMakeNewFile && we_users_util::in_workspace($this->dir, we_users_util::get_ws($this->table), $this->table);
		$this->jsCmd->addCmd('setButtons', [['NewFileBut', $isWS]]);
		parent::printSetDirHTML();
	}

	protected function printFooterTable(string $more = ''): string{
		$ret = '
<table id="footer" class="defaultfont">';
		if(!$this->filter){
			$ret .= '
	<tr>
		<td class="description">' . g_l('fileselector', '[type]') . '</td>
		<td>
			<select name="filter" class="weSelect" onchange="WE().util.dialogTop(window).setFilter(this.options[this.selectedIndex].value)" class="defaultfont" style="width:100%">
				<option value="">' . g_l('fileselector', '[all_Types]') . '</option>';
			foreach(we_base_ContentTypes::getWETypes() as $ctype){
				$ret .= '<option value="' . htmlspecialchars((string) $ctype) . '">' . g_l('contentTypes', '[' . $ctype . ']') . '</option>';
			}
			$ret .= '
			</select></td>
	</tr>';
		}

		$ret .= '
	<tr>
		<td class="description">' . g_l('fileselector', '[name]') . '</td>
		<td>' . we_html_tools::textInput('fname', (string) ($this->values['Text'] === '/' ? '' : $this->values["Text"]), 0, ['style' => "width:100%", 'readonly' => "readonly"]) . '</td>
	</tr>
</table><div id="footerButtons">' . we_html_button::position_yes_no_cancel(we_html_button::create_button(we_html_button::OK, "javascript:press_ok_button();"), we_html_button::create_button(we_html_button::CANCEL, "javascript:WE().util.dialogTop(window).exit_close();"), $more) . '</div>';
		return $ret;
	}

	protected function getFrameset(bool $withPreview = false): string{
		$is_object = $this->table === OBJECT_TABLE;
		return '<body class="selector" onload="startFrameset();showPreview(' . $this->id . ');">' .
			we_html_element::div(['id' => 'fsheader'], $this->printHeaderHTML()) .
			we_html_element::iframe('fsbody', $this->getFsQueryString(we_selector_file::BODY), '', '', '', 'preview' . ($is_object ? ' object' : '')) .
			we_html_element::div(['id' => 'fspreviewDiv', 'class' => ($is_object ? ' object' : '')]) .
			we_html_element::div(['id' => 'fsfooter'], $this->printFooterTable()) .
			we_html_element::div(['id' => 'fspath', 'class' => 'radient']) .
			we_html_element::iframe('fscmd', 'about:blank') .
			'</body>';
	}

	protected function showPreviewFolderFiles(array $data): bool{
		return (isset($data['ContentType']) && in_array($data['ContentType'], we_base_ContentTypes::FOLDER_TYPES)) || ($this->table === VFILE_TABLE && $data['IsFolder']);
	}

	protected function setFramesetJavaScriptOptions(): void{
		parent::setFramesetJavaScriptOptions();

		$this->jsoptions['options']['useID'] = intval($this->useID);
		switch($this->table){
			case FILE_TABLE:
			case VFILE_TABLE:
				$this->jsoptions['data']['newFileState'] = $this->userCanMakeNewFile;
		}
	}

}
