<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: 1b7b553199f4 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2021-12-04 01:26 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_addPercent extends we_weTag_tag{

	public static function parse(array $attribs, string $content): string{
		return '<?php ' . we_weTag_parser::printTag('addPercent', ['_type' => 'start']) . ';?>' .
			$content .
			'<?= ' . we_weTag_parser::printTag('addPercent', $attribs + ['_type' => 'stop']) . ';?>';
	}

	/**
	 * @param array<string,mixed> $attribs parsed tag-attributes
	 * @param string $content content between openingtag and closing tag @unused-param
	 * @param bool $internal if used by we internally @unused-param
	 * @return string output to print
	 * @throws we_exception_parser if any attribute is missing
	 */
	public static function tag(array $attribs = [], string $content = '', bool $internal = false): string{
		//internal Attribute
		switch(self::getAttribute('_type', $attribs, '', we_base_request::STRING)){
			case 'start':
				we_tag_calculate::$active = true;
				ob_start();
				return '';
			case 'stop':
				$data = ob_get_clean();
				$content = ($data === false ? 0 : we_base_util::std_numberformat($data));
				we_tag_calculate::$active = false;
				self::attributeMissing($attribs, ['percent' => false], __CLASS__);

				$percent = self::getAttribute('percent', $attribs, 0, we_base_request::FLOAT);
				$num_format = self::getAttribute('num_format', $attribs, '', we_base_request::STRING);
				$result = ($content / 100) * (100 + $percent);
				return ($num_format ? //bug 6437 gibt immer deutsch zurück (das ist der default von formatnaumber), was das verhalten ändert
					we_base_util::formatNumber($result, $num_format) :
					(string) ($result));
			default:
				self::attributeMissing($attribs, ['_type' => false], __CLASS__);
				return '';
		}
	}

}
