<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: a9d285bed95b $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2021-06-15 16:43 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_block extends we_weTag_tag{
	/** @var int */
	static $blkCnt = -1;

	/** @var ?array{name:string,oname:string,list:string[],listSize:int,ctlShow:bool,ctlShowSelect:bool,pos:int,lastPostName:string} */
	private static $current = null;

	/** @var array */
	private static $stack = [];

	/** @var array<string,array{oname:string,position:int,size:int}> */
	public static $position = [];

	public static function parse(array $attribs, string $content): string{
		self::attributeMissing($attribs, ['name' => false], __CLASS__);

		//cleanup content
		while(str_contains($content, '\\\\$')){
			$content = str_replace('\\\\$', '$', $content);
		}
		//replace all \$ which are not inside we-tags
		$content = preg_replace('|([^\'"])\\\\\$|', '${1}\$', $content);

		$count = 0;
		if(($content = str_replace(we_tag_blockControls::REPLACEMENT, '', $content, $count)) && $count){
			//nothing to do, we have a userdefined blockcontrol
		} else {
			$content = (preg_match('/< ?(tr|td)/i', $content) ?
				//table found
				strtr(preg_replace('|(< ?td[^>]*>)|si', '${1}' . '<?= ' . we_weTag_parser::printTag('blockControls', []) . '?>', strtr($content, [
					'=>' => '#####PHPC####',
					'?>' => '#####PHPE####']), 1), [
					'#####PHPC####' => '=>',
					'#####PHPE####' => '?>']) :
				//no tables found
				'<?= ' . we_weTag_parser::printTag('blockControls', []) . '?>' . $content
				);
		}
//	}
		//here postTagName is explicitly needed, because the control-element is not "inside" the block-tag (no block defined/first element) but controls its elements
		return '<?php
' . we_weTag_parser::printTag('block', $attribs) . ';
while(we_tag_block::condition()){
	?>' . $content . '<?php
}?>';
	}

	/**
	 * @param array<string,mixed> $attribs parsed tag-attributes
	 * @param string $content content between openingtag and closing tag @unused-param
	 * @param bool $internal if used by we internally @unused-param
	 * @return string output to print
	 * @throws we_exception_parser if any attribute is missing
	 */
	public static function tag(array $attribs = [], string $content = '', bool $internal = false): void{
		$origName = self::getAttribute('_name_orig', $attribs, '', we_base_request::STRING);
		$name = self::getAttribute('name', $attribs, '', we_base_request::STRING);
		$start = self::getAttribute('start', $attribs, 0, we_base_request::INT);
		$limit = self::getAttribute('limit', $attribs, 0, we_base_request::INT);
		self::$doc = we_base_WE::getWeDoc();

		if(we_tag_listview::$current){
			if(!($list = we_tag_listview::$current->f($name))){
				$list = we_tag_listview::$current->f($origName);
				$name = $origName;
			}
		} else {
			$list = self::$doc->getElementS($name);
		}

		if(($list = we_unserialize($list, [], true))){
			if(is_array($list) && count($list) && ((count($list) - 1) != max(array_keys($list)))){
				//reorder list!
				$list = array_values($list);
				self::$doc->setElementS($name, we_serialize($list, SERIALIZE_JSON, true, 0, true));
			}
		} elseif($start){
			$list = [];
			if($limit > 0){
				$start = min($start, $limit);
			}
			for($i = 1; $i <= $start; $i++){
				$list[] = '_' . $i;
			}
		}

		$listlen = count($list);
		$show = 10;
		if($limit > 0){
			if(we_base_WE::$editmode){
				$diff = $limit - $listlen;
				$show = ($diff > 0 ? min($show, $diff) : 0);
			} else {
				$listlen = min($listlen, $limit);
			}
		}
		if(self::$current){
			self::$stack[] = self::$current;
		}
		self::$current = [
			'name' => $name,
			'oname' => $origName,
			'list' => $list,
			'listSize' => $listlen,
			'ctlShow' => $show,
			'ctlShowSelect' => self::getAttribute('showselect', $attribs, true, we_base_request::BOOL),
			'pos' => -1,
			'lastPostName' => self::$postTagName,
		];
	}

	public static function condition(): bool{
		if(!self::$current){
			return false;
		}
		//go to next element
		++self::$current['pos']; //@phan-suppress-current-line PhanTypeInvalidUnaryOperandIncOrDec
		if(self::$current['pos'] >= self::$current['listSize']){
			//end of list
			//we need a last add button in editmode
			if(we_base_WE::$editmode){
				echo self::controls();
			}
			//reset data
			unset(self::$position[self::$current['name']]);
			self::$postTagName = (string) self::$current['lastPostName'];
			$last = array_pop(self::$stack);
			self::$current = $last ?: [];
			return false;
		}

		self::$postTagName = 'blk_' . ((string) self::$current['name']) . '_' . ((string) self::$current['list'][(int) self::$current['pos']]); //@phan-suppress-current-line PhanTypeArraySuspiciousNullable

		self::$position[(string) self::$current['name']] = [
			'oname' => (string) self::$current['oname'],
			'position' => self::$current['pos'] + 1,
			'size' => self::$current['listSize']
		];
		return true;
	}

	public static function controls(array $attribs = []): string{
		//if in listview no Buttons are shown!
		if(!we_base_WE::$editmode || we_tag_listview::$current || empty(self::$current) || !self::getAttribute('show', $attribs, true, we_base_request::BOOL)){
			return '';
		}
		static $cnt = 0;
		$showSelect = self::getAttribute('showselect', $attribs, self::$current['ctlShowSelect'], we_base_request::BOOL);

		$ctlName = 'ctl_' . self::$current['name'] . '_' . ($cnt++);

		if(self::$current['pos'] < self::$current['listSize']){
			if($showSelect && self::$current['ctlShow'] > 0){
				$jsSelector = "document.we_form.elements['" . $ctlName . '_' . self::$current['pos'] . "'].options[document.we_form.elements['" . $ctlName . '_' . self::$current['pos'] . "'].selectedIndex].text";

				$selectb = '<select name="' . $ctlName . '_' . self::$current['pos'] . '">';
				for($j = 0; $j < self::$current['ctlShow']; $j++){
					$selectb .= '<option value="' . ($j + 1) . '">' . ($j + 1) . '</option>';
				}
				$selectb .= '</select>';
			} else {
				$jsSelector = 1;
				$selectb = '';
			}
			$tabArray = [
				we_html_button::create_button(we_html_button::LIST_ADD, "javascript:setScrollTo();_EditorFrame.setEditorIsHot(true);we_cmd('insert_entry_at_list','" . self::$current['name'] . "','" . self::$current['pos'] . "'," . $jsSelector . ")", '', '', !(self::$current['ctlShow'] > 0)),
				$selectb,
				((self::$current['pos'] > 0) ?
				//enabled upBtn
				we_html_button::create_button(we_html_button::DIRUP, "javascript:setScrollTo();_EditorFrame.setEditorIsHot(true);we_cmd('up_entry_at_list','" . self::$current['name'] . "','" . self::$current['pos'] . "'," . $jsSelector . ")") :
				//disabled upBtn
				we_html_button::create_button(we_html_button::DIRUP, '', '', '', true)),
				((self::$current['pos'] == self::$current['listSize'] - 1) ?
				//disabled downBtn
				we_html_button::create_button(we_html_button::DIRDOWN, '', '', '', true) :
				//enabled downBtn
				we_html_button::create_button(we_html_button::DIRDOWN, "javascript:setScrollTo();_EditorFrame.setEditorIsHot(true);we_cmd('down_entry_at_list','" . self::$current['name'] . "','" . self::$current['pos'] . "'," . $jsSelector . ")")),
				we_html_button::create_button(we_html_button::TRASH, "javascript:setScrollTo();_EditorFrame.setEditorIsHot(true);we_cmd('delete_list','" . self::$current['name'] . "','" . self::$current['pos'] . "','" . self::$postTagName . "')")
			];

			return we_html_element::div(['class' => 'we we_blockControls'], implode('', $tabArray));
		}
		if($showSelect && self::$current['ctlShow'] > 0){
			$selectb = '<select name="' . $ctlName . '_00">';
			for($j = 1; $j <= self::$current['ctlShow']; $j++){
				$selectb .= '<option value="' . $j . '">' . $j . '</option>';
			}
			$selectb .= '</select>';
			$plusbut = we_html_button::create_button(we_html_button::LIST_ADD, "javascript:setScrollTo();_EditorFrame.setEditorIsHot(true);we_cmd('add_entry_to_list','" . self::$current['name'] . "',document.we_form.elements['" . $ctlName . "_00'].options[document.we_form.elements['" . $ctlName . "_00'].selectedIndex].text);", '', '', !(self::$current['ctlShow'] > 0)) .
				$selectb;
		} else {
			$plusbut = we_html_button::create_button(we_html_button::LIST_ADD, "javascript:setScrollTo();_EditorFrame.setEditorIsHot(true);we_cmd('add_entry_to_list','" . self::$current['name'] . "',1)", '', '', !(self::$current['ctlShow'] > 0));
		}

		$doc = we_base_WE::getWeDoc();
		'@phan-var we_document_webEdition $doc';

		return we_html_element::hidden('we_' . $doc->Name . '_block[' . self::$current['name'] . ']',
																 (empty(self::$current['list']) ? '' : we_serialize(self::$current['list'], SERIALIZE_JSON, true, 0, true))
			) . //FIXME: do we really need this serialized in the document???
			we_html_element::div(['class' => 'we we_blockControls'], $plusbut);
	}

	private static function getMaxListArrayNr(array $la): int{
		$maxnr = 0;
		foreach($la as $val){
			$nr = (int) str_replace('_', '', $val);
			$maxnr = max($maxnr, $nr);
		}
		return $maxnr;
		//new version! - note to remove "+1"
		/* $max=max($la);
		  return min(array_diff(range(1,$max+1),$la));
		 */
	}

	public static function addEntryToList(we_document_base $we_doc, string $name, int $number = 1): void{//FIXME: keep this list unserialized until save.
		$listarray = we_unserialize($we_doc->getElementS($name));
		for($f = 0; $f < $number; $f++){
			$new_nr = self::getMaxListArrayNr($listarray) + 1;
			$listarray[] = '_' . $new_nr;
		}
		$we_doc->setElementS($name, we_serialize(array_values($listarray), SERIALIZE_JSON, true, 0, true), 'block');
	}

	public static function insertEntryAtList(we_document_base $we_doc, string $name, int $nr, int $number = 1): void{
		$listarray = we_unserialize($we_doc->getElementS($name));
		for($f = 0; $f < $number; $f++){
			$new_nr = self::getMaxListArrayNr($listarray) + 1;
			for($i = count($listarray); $i > $nr; $i--){
				$listarray[$i] = $listarray[$i - 1];
			}
			$listarray[$nr] = '_' . $new_nr;
		}
		$we_doc->setElementS($name, we_serialize($listarray, SERIALIZE_JSON, true, 0, true), 'block');
	}

	public static function upEntryAtList(we_document_base $we_doc, string $name, int $nr, int $number = 1): void{
		$listarray = we_unserialize($we_doc->getElementS($name));
		$newPos = max($nr - $number, 0);
		$temp = $listarray[$newPos];
		$listarray[$newPos] = $listarray[$nr];
		$listarray[$nr] = $temp;

		$we_doc->setElementS($name, we_serialize($listarray, SERIALIZE_JSON, true, 0, true), 'block');
	}

	public static function downEntryAtList(we_document_base $we_doc, string $name, int $nr, int $number = 1): void{
		$listarray = we_unserialize($we_doc->getElementS($name));
		$newPos = min($nr + $number, count($listarray) - 1);
		$temp = $listarray[$newPos];
		$listarray[$newPos] = $listarray[$nr];
		$listarray[$nr] = $temp;
		$we_doc->setElementS($name, we_serialize($listarray, SERIALIZE_JSON, true, 0, true), 'block');
	}

	public static function removeEntryFromList(we_document_base $we_doc, string $name, int $nr, string $names): void{
		$listarray = we_unserialize($we_doc->getElementS($name));
		foreach(array_keys($we_doc->elements) as $key){
			if(preg_match('/' . $names . '(__.*)*$/', $key)){// # Bug 6904
				$we_doc->delElement($key);
			}
		}
		unset($listarray[$nr]);
		$we_doc->setElementS($name, we_serialize($listarray, SERIALIZE_JSON, true, 0, true), 'block');
	}

}
