<?php

declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: e9adb31199a3 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2021-05-07 17:54 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_href extends we_weTag_tag{

	/**
	 * @param array<string,mixed> $attribs parsed tag-attributes
	 * @param string $content content between openingtag and closing tag @unused-param
	 * @param bool $internal if used by we internally @unused-param
	 * @return string output to print
	 * @throws we_exception_parser if any attribute is missing
	 */
	public static function tag(array $attribs = [], string $content = '', bool $internal = false): string{
		self::attributeMissing($attribs, ['name' => false], __CLASS__);
		$name = self::getAttribute('name', $attribs, '', we_base_request::STRING);
		$type = self::getAttribute('type', $attribs, we_base_link::TYPE_ALL, we_base_request::STRING);
		$hidedirindex = self::getAttribute('hidedirindex', $attribs, TAGLINKS_DIRECTORYINDEX_HIDE, we_base_request::BOOL);
		$include = self::getAttribute('include', $attribs, false, we_base_request::BOOL);
		self::$doc = we_base_WE::getWeDoc();

		if(self::$doc instanceof we_document_triggerDocument){
			$hrefArr = self::$doc->getElementA($name);
			return we_document_base::getHrefByArray($hrefArr, self::getAttribute('only', $attribs, '', we_base_request::STRING));
		}

		$nint = $name . we_base_link::MAGIC_INT_LINK;
		$nintID = $name . we_base_link::MAGIC_INT_LINK_ID;
		$int = ($type == we_base_link::TYPE_EXT ? false : ($type == we_base_link::TYPE_INT || self::$doc->getElementI($nint)));

		if($int){
			$intID = (int) (self::$doc->getElementI($nintID) ?: self::getAttribute('startid', $attribs, 0, we_base_request::INT));
			$foo = ($intID ? (getHash('SELECT Path,(IsFolder=1||IsPublished=1) AS IsPublished FROM ' . FILE_TABLE . ' WHERE ID=' . $intID)) : []);
			['Path' => $intPath, 'IsPublished' => $isPublished] = $foo ?: ['Path' => '', 'IsPublished' => 0];
			'@phan-var string $intPath';
			$include_path = $intPath ? WEBEDITION_PATH . '..' . $intPath : ''; //(symlink) webEdition always points to the REAL DOC-Root!
			$path_parts = pathinfo($intPath);
			$href = ($hidedirindex && seoIndexHide($path_parts['basename']) ?
					($path_parts['dirname'] != '/' ? $path_parts['dirname'] : '') . '/' :
					$intPath);
			$extPath = '';
		} else {
			$intID = 0;
			$intPath = '';
			$isPublished = false;
// we have to use a html_entity_decode first in case a user has set &amp, &uuml; by himself
			$href = $extPath = htmlspecialchars(html_entity_decode(self::$doc->getElementS($name)));
			$include_path = $href ? WEBEDITION_PATH . '..' . $href : ''; //(symlink) webEdition always points to the REAL DOC-Root!
		}

		return (we_base_WE::$editmode ?
				self::edit($attribs, $name, $extPath, $type, $include, $intPath, $intID, $include_path) :
				self::frontend($attribs, $int, $intID, (bool) $isPublished, $include, $include_path, $href)
				);
	}

	protected static function frontend(array $attribs, bool $int, int $intID, bool $isPublished, bool $include, string $include_path, string $href): string{
		if($int && we_modules_info::isActive(we_modules_info::CUSTOMER) && $intID && self::getAttribute('cfilter', $attribs, true, we_base_request::BOOL)){
			$filter = we_customer_documentFilter::getFilterByIdAndTable($intID, FILE_TABLE);

			if($filter->accessForVisitor($intID, true) != we_customer_documentFilter::ACCESS){
				return '';
			}
		}

		switch(self::getAttribute('only', $attribs, '', we_base_request::STRING)){
			case 'id':
				return (string) ($isPublished ? $intID : 0);
		}
		if($int && !$isPublished){
			return '';
		}
		if($int && $intID && !$include && !isset($attribs['isInternal'])){
			$urlReplace = we_folder_base::getUrlReplacements(DB_WE::$inst, true);
			if($urlReplace){
				return preg_replace($urlReplace, array_keys($urlReplace), $href);
			}
		}
		if($include){
			if($include_path && file_exists($include_path)){
				include($include_path);
			}
			return '';
		}

		return $href;
	}

	protected static function edit(array $attribs, string $name, string $extPath, string $type, bool $include, string $intPath, int $intID, string $include_path): string{
		$reload = self::getAttribute('reload', $attribs, false, we_base_request::BOOL);
		$rootdir = self::getAttribute('rootdir', $attribs, '/', we_base_request::FILE);
//	$seeMode = self::getAttribute((isset($attribs['seem']) ? 'seem' : 'seeMode'), $attribs, true, true);
		$file = self::getAttribute('file', $attribs, true, we_base_request::BOOL);
		$directory = self::getAttribute('directory', $attribs, true, we_base_request::BOOL);
		//$doctypeid = self::getAttribute('doctypeID', $attribs, 0, we_base_request::INT); //FIXME:implement

		$attribs = self::removeAttribs($attribs, ['rootdir', 'file', 'directory', 'type', 'hidedirindex', 'reload', 'doctypeID']);

		$nint = $name . we_base_link::MAGIC_INT_LINK;
		$nintID = $name . we_base_link::MAGIC_INT_LINK_ID;
		$int = ($type == we_base_link::TYPE_EXT ? false : ($type == we_base_link::TYPE_INT || self::$doc->getElementI($nint)));

		if($rootdir[0] != '/'){
			$rootdirid = (int) $rootdir;
			$rootdir = id2path($rootdirid, FILE_TABLE);
		} else {
			$rootdir = ($rootdir === '/' ? $rootdir : rtrim($rootdir, '/'));
			$rootdirid = ($rootdir === '/' ? 0 : path_to_id($rootdir, FILE_TABLE));
		}
// Bug Fix #7045
		if($rootdir === '/'){
			$rootdir = '';
		}

		$int_elem_Name = 'we_' . self::$doc->Name . '_href[' . $nint . '#' . we_contents_base::ETYPE_NUMBER . ']';
		$intPath_elem_Name = 'we_' . self::$doc->Name . '_' . we_contents_base::INTERNAL_VAR . '[' . $name . we_base_link::MAGIC_INT_LINK_PATH . ']';
		$intID_elem_Name = 'we_' . self::$doc->Name . '_href[' . $nintID . '#' . we_contents_base::ETYPE_NUMBER . ']';
		$ext_elem_Name = 'we_' . self::$doc->Name . '_txt[' . $name . ']';

		$span = '<span class="defaultfont">';
		$size = 5 * intval(self::getAttribute('size', $attribs, 20, we_base_request::INT));

		$cmd1 = "document.we_form.elements['" . $intID_elem_Name . "'].value";
		$cmdInt = 'tag_weHref_selectorCallback,' . we_base_link::TYPE_INT . ',' . $type . ',' . $int_elem_Name . ',' . ($include || $reload ? 1 : 0);
		$cmdExt = 'tag_weHref_selectorCallback,' . we_base_link::TYPE_EXT . ',' . $type . ',' . $int_elem_Name . ',0';
		if(($directory && $file) || $file){

			$btnInt = we_html_button::create_button(we_html_button::LINK_EDIT, "javascript:we_cmd('we_selector_document', {id:" . $cmd1 . ",table:WE().consts.tables.FILE_TABLE,targetID:'" . $intID_elem_Name . "',targetName:'" . $intPath_elem_Name . "',jsCmd:'" . $cmdInt . "',rootDirID:'" . $rootdirid . "', open_doc:" . (we_base_permission::hasPerm([
								we_base_perm::WORK_WORKSPACE_CAN_SELECT_OTHER_FILES]) ? 0 : 1) . ", canSelectDir:" . ($directory ? 1 : 0) . "});");
			$btnExt = we_base_permission::hasPerm([we_base_perm::WORK_EXTERNAL_FILES_CAN_SELECT]) ? we_html_button::create_button(we_html_button::LINK_EDIT, "javascript:we_cmd('browse_server', '" . $ext_elem_Name . "', '" . (!$directory ? implode(',', we_base_ContentTypes::LINK_ABLE) : '') . "', document.forms[0].elements['" . $ext_elem_Name . "'].value, '" . $cmdExt . "','" . $rootdir . "')") : '';
		} else {
			$btnInt = we_html_button::create_button(we_html_button::LINK_EDIT, "javascript:we_cmd('we_selector_directory', {id:" . $cmd1 . ",table:WE().consts.tables.FILE_TABLE,targetID:'" . $intID_elem_Name . "',targetName:'" . $intPath_elem_Name . "',jsCmd:'" . $cmdInt . "',rootDirID:'" . $rootdirid . "'});");
			$btnExt = we_base_permission::hasPerm([we_base_perm::WORK_EXTERNAL_FILES_CAN_SELECT]) ? we_html_button::create_button(we_html_button::LINK_EDIT, "javascript:we_cmd('browse_server', '" . $ext_elem_Name . "', WE().consts.contentTypes.FOLDER_DOCUMENT, document.forms[0].elements['" . $ext_elem_Name . "'].value, '" . $cmdExt . "','" . $rootdir . "')") : '';
		}
		$btnEdit = we_html_button::create_button(we_html_button::OPEN_TO_EDIT, "javascript:we_cmd('tag_weHref_openDocument','" . $intID_elem_Name . "');");
		$btnTrashInt = we_html_button::create_button(we_html_button::TRASH, "javascript:we_cmd('tag_weHref_trash','" . we_base_link::TYPE_INT . "','" . $intID_elem_Name . "','" . $intPath_elem_Name . "'," . ($include || $reload ? 1 : 0) . ");");
		$btnTrashExt = we_html_button::create_button(we_html_button::TRASH, "javascript:we_cmd('tag_weHref_trash','" . we_base_link::TYPE_EXT . "','" . $ext_elem_Name . "');");

		$weSuggest = we_gui_suggest::getInstance();
		switch($type){
			case we_base_link::TYPE_ALL:
			case we_base_link::TYPE_INT:
				$weSuggest->setAcId($name . we_base_file::getUniqueId(), $rootdir)
						->setContentType(array_merge($file ? we_base_ContentTypes::LINK_ABLE : [], $directory ? [we_base_ContentTypes::FOLDER_DOCUMENT] : []))
						->setInput($intPath_elem_Name, $intPath)
						->setMaxResults(10)
						->setResult($intID_elem_Name, (string) ($intID))
						->setSelector($directory ? we_gui_suggest::DirSelector : we_gui_suggest::DocSelector)
						->setTable(FILE_TABLE)
						->setWidth($size);
		}

		ob_start();
		if($include && $include_path && file_exists($include_path)){
			include($include_path);
		}

		return
				'<table class="we we_href weEditTable">' .
				($type == we_base_link::TYPE_ALL || $type == we_base_link::TYPE_INT ? '
<tr>
	<td>' . ($type == we_base_link::TYPE_ALL ? we_html_forms::radiobutton('1', $int, $int_elem_Name, g_l('weTag', '[href][int]') . ':') : g_l('weTag', '[href][int]') . ':<input type="hidden" name="' . $int_elem_Name . '" value="1" />' ) . '</td>
	<td>' . $weSuggest->getHTML() . '</td>
	<td>' . $btnInt . '</td>
	<td>' . $btnTrashInt . '</td>
	<td>' . $btnEdit . '</td>
	</tr>' : '') .
				($type == we_base_link::TYPE_ALL || $type == we_base_link::TYPE_EXT ? '
<tr>
	<td>' . ($type == we_base_link::TYPE_ALL ? we_html_forms::radiobutton('0', !$int, $int_elem_Name, $span . g_l('weTag', '[href][ext]') . ':</span>') : $span . g_l('weTag', '[href][ext]') . ':</span><input type="hidden" name="' . $int_elem_Name . '" value="0" />') . '</td>
	<td>' .
				self::htmlTag('input', array_merge(self::removeAttribs($attribs, ['onkeydown', 'onkeydown', 'oninput']), ['style' => 'width:' . ($size) . 'px;',
					'onchange' => ($type == we_base_link::TYPE_ALL ? 'this.form.elements[\'' . $int_elem_Name . '\'][1].checked=true;' : ''),
					'type' => 'text',
					'name' => 'we_' . self::$doc->Name . '_txt[' . $name . ']',
					'placeholder' => 'http://example.org',
					'value' => $extPath
				]))
				. '</td>
	<td>' . $btnExt . '</td>
	<td>' . $btnTrashExt . '</td>
</tr>' : '') . '
</table>' . (ob_get_clean() ?: '');
	}

}
