<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: 220488cfbb47 $
 * $Author: Lukas Imhof $
 * $Date: 2021-08-09 15:55 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_ifVar extends we_weTag_ifTag{

	/**
	 * @param array<string,mixed> $attribs parsed tag-attributes
	 * @param string $content content between openingtag and closing tag @unused-param
	 * @param bool $internal if used by we internally @unused-param
	 * @return string output to print
	 * @throws we_exception_parser if any attribute is missing
	 */
	public static function tag(array $attribs = [], string $content = '', bool $internal = false): bool{
		self::attributeMissing($attribs, ['name' => false, 'match' => true], __CLASS__);

		$match = self::getAttribute('match', $attribs, '', we_base_request::STRING);
		$type = self::getAttribute('type', $attribs, 'document', we_base_request::STRING);
		$operator = self::getAttribute('operator', $attribs, 'equal', we_base_request::STRING);

		if(!str_contains($match, ',')){
			$size = 1;
			$matchArray = [];
		} else {
			$matchArray = array_filter(explode(',', $match));
			$size = count($matchArray);
			$regs = [];
			if($size == 1 && preg_match('|^\\\\?\$([^\[]+)(\[.*\])?|', $match, $regs)){
				$match = (isset($regs[2]) ?
					self::getArrayValue($GLOBALS, $regs[1], $regs[2]) :
					($GLOBALS[$regs[1]] ?? ''));
			}
		}

		$nameOrig = self::getAttribute('_name_orig', $attribs, '', we_base_request::STRING);

		switch($type){
			case 'customer' :
			case 'sessionfield' :
				return ($size == 1 && isset($_SESSION['webuser'][$nameOrig]) ?
					self::op($operator, $_SESSION['webuser'][$nameOrig], $match) :
					(isset($_SESSION['webuser'][$nameOrig]) && in_array($_SESSION['webuser'][$nameOrig], $matchArray)));

			case 'global':
				$var = self::getArrayValue($GLOBALS, null, $nameOrig);
				return ($size == 1 ?
					self::op($operator, $var, $match) :
					(!empty($var) && in_array($var, $matchArray)));

			case 'cookie':
				$var = self::getArrayValue($_COOKIE, null, $nameOrig);
				return ($size == 1 ?
					self::op($operator, $var, $match) :
					(!empty($var) && in_array($var, $matchArray)));

			case 'request':
				$var = self::getArrayValue($_REQUEST, null, $nameOrig);
				return ($size == 1 ?
					self::op($operator, $var, $match) :
					(!empty($var) && in_array($var, $matchArray)));

			case 'post':
				$var = self::getArrayValue($_POST, null, $nameOrig);
				return ($size == 1 ?
					self::op($operator, $var, $match) :
					(!empty($var) && in_array($var, $matchArray)));

			case 'get':
				$var = self::getArrayValue($_GET, null, $nameOrig);
				return ($size == 1 ?
					self::op($operator, $var, $match) :
					(!empty($var) && in_array($var, $matchArray)));

			case 'session':
				$var = self::getArrayValue($_SESSION, null, $nameOrig);
				return ($size == 1 ?
					self::op($operator, $var, $match) :
					(!empty($var) && in_array($var, $matchArray)));

			case 'property':
				$doc = self::getDocForTag(self::getAttribute('doc', $attribs, '', we_base_request::STRING), true);
				if(!isset($doc->$nameOrig)){
					return false;
				}
				$var = $doc->$nameOrig;
				return ($size == 1 ?
					self::op($operator, $var, $match) :
					in_array($var, $matchArray));

			case 'document':
			default:
				$doc = self::getDocForTag(self::getAttribute('doc', $attribs, '', we_base_request::STRING));

				/*
				// is there a better solution to detect elem type href by elemname?
				$elemType = $doc->getElementB($attribs['name'] . we_base_link::MAGIC_INT_LINK) ? 'href' : $doc->getElementType($attribs['name']);
				$val = $doc->getField($attribs, $elemType, true);
				 *
				 */
				$val = $doc->getField($attribs, $doc->getElementType($attribs['name']), true);

				return ($size == 1 ?
					self::op($operator, $val, $match) :
					in_array($val, $matchArray));
		}
	}

	protected static function op(string $operator, /* mixed */ $first, /* mixed */ $match): bool{
		switch($operator){
			default:
			case 'equal':
				return $first == $match;
			case 'less':
				return $first < $match;
			case 'less|equal':
				return $first <= $match;
			case 'greater':
				return $first > $match;
			case 'greater|equal':
				return $first >= $match;
			case 'contains':
				return (str_contains((string) $first, (string) $match));
			case 'isin':
				return (str_contains((string) $match, (string) $first));
		}
	}

}
