<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: 1b7b553199f4 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2021-12-04 01:26 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license		http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_picture extends we_weTag_tag{
	public static $id = 0;

	private const removeAttribs = [
		'id',
		'_type',
		'thumbnail',
		'thumbnailid',
	];

	private static $attribs = [];

	public static function parse($attribs, string $content): string{
		$tagname = self::getTagName();
		return '<?= ' . we_weTag_parser::printTag($tagname, $attribs + ['_type' => 'start']) . ';?>' .
			trim($content) .
			'<?= ' . we_weTag_parser::printTag($tagname, ['_type' => 'stop']) . ';?>';
	}

	/**
	 * @param array<string,mixed> $attribs parsed tag-attributes
	 * @param string $content content between openingtag and closing tag
	 * @param bool $internal if used by we internally @unused-param
	 * @return string output to print
	 * @throws we_exception_parser if any attribute is missing
	 */
	public static function tag(array $attribs = [], string $content = '', bool $internal = false): string{
		switch(self::getAttribute('_type', $attribs, '', we_base_request::STRING)){
			case 'start':
				self::$attribs = $attribs;
				$name = self::getAttribute('name', $attribs, '', we_base_request::STRING);
				self::$id = self::getAttribute('id', $attribs, 0, we_base_request::INT);

				if($name){
					self::$id = (int) ((self::$doc->getElementI($name) ?:
						self::$doc->getElementS($name)) ?: self::$id );
				}

				$tagAttribs = self::removeAttribs($attribs, self::removeAttribs);
				return self::htmlTag('picture', $tagAttribs, '', false, true);
			case 'stop':
				if(we_base_WE::$editmode){
					$name = self::getAttribute('name', self::$attribs, '', we_base_request::STRING);
					if($name){
						return '</picture>' .
							we_tag('img', ['name' => $name]);
					}
				}
				return we_tag('img', [
						'id' => self::$id,
						'thumbnail' => self::getAttribute('thumbnail', self::$attribs, '', we_base_request::STRING),
						'thumbnailid' => self::getAttribute('thumbnailid', self::$attribs, 0, we_base_request::INT),
						'width' => '',
						'height' => '',
					]) .
					'</picture>';
		}
	}

}
