<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: 1b7b553199f4 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2021-12-04 01:26 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license		http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_source extends we_weTag_tag{
	public static $id = 0;
	private static $urlReplace = [];
	private static $val = [];
	private static $attribs = [];

	private const removeAttribs = [
		'id',
		'_type',
		'type',
	];

	public static function parse($attribs, string $content): string{
		$tagname = self::getTagName();
		return '<?php ' . we_weTag_parser::printTag($tagname, $attribs + ['_type' => 'start']) . ';?>' .
			trim($content) .
			'<?= ' . we_weTag_parser::printTag($tagname, ['_type' => 'stop']) . ';?>';
	}

	/**
	 * @param array<string,mixed> $attribs parsed tag-attributes
	 * @param string $content content between openingtag and closing tag
	 * @param bool $internal if used by we internally @unused-param
	 * @return string output to print
	 * @throws we_exception_parser if any attribute is missing
	 */
	public static function tag(array $attribs = [], string $content = '', bool $internal = false): string{
		switch(self::getAttribute('_type', $attribs, '', we_base_request::STRING)){
			case 'start':
				self::$attribs = $attribs;
				self::$val = [];
				self::$urlReplace = self::$urlReplace ?: we_folder_base::getUrlReplacements(DB_WE::$inst, true);
				self::$id = self::getAttribute('id', $attribs, we_tag_picture::$id, we_base_request::INT);
				return '';
			case 'stop':
				$tagAttribs = self::removeAttribs(self::$attribs, self::removeAttribs);
				return self::htmlTag('source', $tagAttribs + [
						'srcset' => implode(',', self::$val),
				]);
		}
		return '';
	}

	public static function append(string $width, string $img): void{
		if(!$img){
			return;
		}
		self::$val[] = trim(
			(self::$urlReplace ? preg_replace(self::$urlReplace, array_keys(self::$urlReplace), $img) : $img) .
			' ' . $width);
	}

}
