<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: adcae6a39191 $
 * $Author: Lukas Imhof $
 * $Date: 2021-10-21 11:48 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_thumbnail_frames extends we_modules_frame{
	const TAB_PROPERTIES = 1;

	/** @var we_thumbnail_view */
	protected $View;

	/** @var we_thumbnail_tree */
	protected $Tree;

	public function __construct(){
		$this->module = we_modules_info::THUMB;
		parent::__construct();
		$this->Tree = new we_thumbnail_tree($this->jsCmd);
		$this->View = new we_thumbnail_view();
	}

	public function getHTML(string $what = '', int $mode = 0, int $step = 0): string{
		switch($what){
			case self::FOOTER:
				return $this->getHTMLEditorFooter([
						'main' => [
							we_html_button::FOOTER_SAVE => [[we_base_perm::ADMINISTRATOR], ['we_thumbnail_save']],
						],
						'optional' => [
							we_html_button::FOOTER_TRASH => [[we_base_perm::ADMINISTRATOR], ['confirm_we_thumbnail_delete', $this->View->Model->Text]]
						]
				]);
			case 'frameset':
				return $this->getHTMLFrameset($this->Tree->getJSTreeCode(), ($sid = we_base_request::int('sid', -1)) !== -1 ? '&sid=' . $sid : '');
			case 'editor':
				return $this->getHTMLEditor(($sid = we_base_request::int('sid', -1)) !== -1 ? '&we_cmd[1]=' . $sid : '');
			default:
				return parent::getHTML($what, $mode, $step);
		}
	}

	protected function getHTMLEditorHeader(): string{
		if(we_base_request::bool('home')){
			return parent::getHTMLEditorHeader();
		}

		$we_tabs = new we_gui_tabs();

		$we_tabs->addTab('', we_gui_tabs::WE_ICON_PROPERTIES, false, (string) (self::TAB_PROPERTIES), ['id' => "tab_" . self::TAB_PROPERTIES, 'title' => g_l('modules_voting', '[property]')]);

		$this->jsCmd->addCmd('setTab', self::TAB_PROPERTIES);
		$tabsHead = we_html_element::cssLink(CSS_DIR . 'we_tab.css') .
			we_html_element::jsScript(JS_DIR . 'initTabs.js') .
			we_html_element::jsScript(WE_JS_MODULES_DIR . '/thumbnail/thumbnail_top.js');


		$body = we_html_element::body([
				'onresize' => "weTabs.setFrameSize()",
				'onload' => "weTabs.setFrameSize();document.getElementById('tab_" . self::TAB_PROPERTIES . "').className='tabActive';",
				'id' => "eHeaderBody"
				], '<div id="main"><div id="headrow"><b>' . g_l('thumbnails', '[thumbnails]') . ':&nbsp;</b><span id="h_path" class="header_small"><b id="titlePath">' . str_replace(' ', '&nbsp;', $this->View->Model->Text) . '</b> (ID:' . $this->View->Model->ID . ')</span></div>' .
				$we_tabs->getHTML() .
				'</div>'
		);

		return $this->getHTMLDocument($body, $tabsHead);
	}

	protected function getHTMLEditorBody(): string{
		if(we_base_request::bool('home')){
			return $this->View->getHomeScreen($this->jsCmd);
		}

		return $this->getHTMLDocument(we_html_element::body(['class' => 'weEditorBody', 'onload' => "loaded=1;", 'onunload' => "doUnload()"], we_html_element::form([
						'name' => we_base_constants::WE_FORM, 'onsubmit' => "return false"], $this->View->getCommonHiddens(['cmd' => 'we_thumbnail_edit', 'pnt' => 'edbody']) . $this->getHTMLProperties())), $this->View->getJSProperty() . $this->jsCmd->getCmds());
	}

	function getHTMLTab1(): we_html_multiIconBoxContent{

		/*		 * ***************************************************************
		 * PROPERTIES
		 * *************************************************************** */

		// Create specify thumbnail dimension input
		$thumbnail_quality = $this->View->Model->Quality;
		$thumbnail_specify_table = (new we_html_table(['class' => 'default inputs'], 3, 2))
			->setCol(0, 0, g_l('thumbnails', '[width]') . ':', ['style' => 'padding-right:10px;'])
			->setCol(0, 1, we_html_tools::textInput('thumbnail_width', (string) $this->View->Model->Width, 4, [], 'number'))
			->setCol(1, 0, g_l('thumbnails', '[height]') . ':')
			->setCol(1, 1, we_html_tools::textInput('thumbnail_height', (string) $this->View->Model->Height, 4, [], 'number'))
			->setCol(2, 0, g_l('thumbnails', '[quality]') . ':', ['class' => 'defaultfont', 'id' => 'thumbnail_quality_text_cell'])
			->setCol(2, 1, we_base_imageEdit::qualitySelect('thumbnail_quality', $thumbnail_quality), ['class' => 'defaultfont', 'id' => 'thumbnail_quality_value_cell']);

		// Create checkboxes for options for thumbnails
		$opt = $this->View->Model->Options;

		$options = [
			'opts' => [2, 1, [
					we_contents_thumbnail::OPTION_MAXSIZE => [($this->View->Model->ID ? intval(in_array(we_contents_thumbnail::OPTION_MAXSIZE, $opt, true)) : -1), g_l('thumbnails', '[maximize]'),
						g_l('thumbnails', '[maximize_desc]')],
					we_contents_thumbnail::OPTION_INTERLACE => [($this->View->Model->ID ? intval(in_array(we_contents_thumbnail::OPTION_INTERLACE, $opt, true)) : -1), g_l('thumbnails', '[interlace]'),
						g_l('thumbnails', '[interlace_desc]')],
				]
			],
			'cutting' => [4, 1, [
					we_contents_thumbnail::OPTION_DEFAULT => [($this->View->Model->ID > 0), g_l('thumbnails', '[cutting_none]'), g_l('thumbnails', '[cutting_none_desc]')],
					we_contents_thumbnail::OPTION_RATIO => [($this->View->Model->ID ? in_array(we_contents_thumbnail::OPTION_RATIO, $opt, true) : false), g_l('thumbnails', '[ratio]'),
						g_l('thumbnails', '[ratio_desc]')],
					we_contents_thumbnail::OPTION_FITINSIDE => [($this->View->Model->ID ? in_array(we_contents_thumbnail::OPTION_FITINSIDE, $opt, true) : false), g_l('thumbnails', '[fitinside]'),
						g_l('thumbnails', '[fitinside_desc]')],
					we_contents_thumbnail::OPTION_CROP => [($this->View->Model->ID ? in_array(we_contents_thumbnail::OPTION_CROP, $opt, true) : false), g_l('thumbnails', '[crop]'),
						g_l('thumbnails', '[crop_desc]')],
				]
			],
			'filter' => [2, 3, [
					we_contents_thumbnail::OPTION_UNSHARP => [($this->View->Model->ID ? in_array(we_contents_thumbnail::OPTION_UNSHARP, $opt, true) : false), g_l('thumbnails', '[unsharp]'),
						g_l('thumbnails', '[unsharp_desc]')],
					we_contents_thumbnail::OPTION_GAUSSBLUR => [($this->View->Model->ID ? in_array(we_contents_thumbnail::OPTION_GAUSSBLUR, $opt, true) : false), g_l('thumbnails', '[gauss]'),
						g_l('thumbnails', '[gauss_desc]')],
					we_contents_thumbnail::OPTION_GRAY => [($this->View->Model->ID ? in_array(we_contents_thumbnail::OPTION_GRAY, $opt, true) : false), g_l('thumbnails', '[gray]')],
					we_contents_thumbnail::OPTION_NEGATE => [($this->View->Model->ID ? in_array(we_contents_thumbnail::OPTION_NEGATE, $opt, true) : false), g_l('thumbnails', '[negate]'),
						g_l('thumbnails', '[negate_desc]')],
					we_contents_thumbnail::OPTION_SEPIA => [($this->View->Model->ID ? in_array(we_contents_thumbnail::OPTION_SEPIA, $opt, true) : false), g_l('thumbnails', '[sepia]')],
				]
			]
		];
		$thumbnail_option_table = [];
		foreach($options as $k => $v){
			if(isset($v[2][we_contents_thumbnail::OPTION_DEFAULT])){
				$v[2][we_contents_thumbnail::OPTION_DEFAULT][0] = ($this->View->Model->ID == 0) || ((count(array_intersect(array_keys($v[2]), $opt))) === 0);
			}

			$thumbnail_option_table[$k] = new we_html_table(['class' => 'editorThumbnailsOptions'], $v[0], $v[1]);
			$i = 0;
			foreach($v[2] as $key => $val){
				switch($k){
					case 'opts':
					case 'filter':
						$thumbnail_option_table[$k]->setCol(($i % $v[0]), intval($i++ / $v[0]), we_html_forms::checkbox($key, (intval($val[0]) > 0), 'Options[' . $key . ']', $val[1], false, 'defaultfont', '', false, '', we_html_tools::TYPE_NONE, 0, '', '', ($val[2] ?? '')));
						break;
					default:
						$thumbnail_option_table[$k]->setCol(($i % $v[0]), intval($i++ / $v[0]), we_html_forms::radiobutton($key, (bool) $val[0], 'Options[' . $k . ']', $val[1], true, "defaultfont", '', false, '', we_html_tools::TYPE_NONE, 0, '', '', ($val[2] ?? '')));
				}
			}
		}

		$window_html = (new we_html_table(['class' => 'editorThumbnailsOptions'], 1, 4))
			->setCol(0, 0, $thumbnail_specify_table->getHtml())
			->setCol(0, 1, we_html_element::div([], g_l('thumbnails', '[output_options]') . ':') . we_html_element::div([], $thumbnail_option_table['opts']->getHtml()))
			->setCol(0, 2, we_html_element::div([], g_l('thumbnails', '[cutting]') . ':') . we_html_element::div([], $thumbnail_option_table['cutting']->getHtml()));

		// OUTPUT FORMAT

		$thumbnail_format = $this->View->Model->Format;

		// Define available formats
		$thumbnails_formats = ['none' => g_l('thumbnails', '[format_original]'), we_contents_thumbnail::FORMAT_GIF => g_l('thumbnails', '[format_gif]'), we_contents_thumbnail::FORMAT_JPG => g_l('thumbnails', '[format_jpg]'),
			we_contents_thumbnail::FORMAT_PNG => g_l('thumbnails', '[format_png]'),
			we_contents_thumbnail::FORMAT_WEBP => g_l('thumbnails', '[format_webp]'),
			];
		$thumbnail_format_select_attribs = ['name' => 'Format', 'id' => 'Format', 'class' => 'weSelect', 'style' => 'width: 225px;'/* , 'onchange' => 'changeFormat()' */];

		$thumbnail_format_select = new we_html_select($thumbnail_format_select_attribs);

		foreach($thumbnails_formats as $k => $v){
			if(in_array($k, we_base_imageEdit::supported_image_types(), true) || $k === 'none'){
				$thumbnail_format_select->addOption($k, $v);

				// Check if added option is selected
				if($thumbnail_format == $k || (!$thumbnail_format && ($k === 'none'))){
					$thumbnail_format_select->selectOption($k);
				}
			}
		}

		// Build dialog
		return (new we_html_multiIconBoxContent())
				->add(new we_html_multiIconBoxContentLine(g_l('navigation', '[general]'), we_html_tools::formElementTable(g_l('thumbnails', '[name]'), we_html_tools::textInput('Text', strtr($this->View->Model->Text, array_flip(get_html_translation_table(HTML_SPECIALCHARS))), 0, [
								'style' => 'width: 520px;', 'onchange' => "top.content.setHot();"])) .
						we_html_tools::formElementTable(g_l('thumbnails', '[description]'), we_html_tools::textInput('description', strtr($this->View->Model->description, array_flip(get_html_translation_table(HTML_SPECIALCHARS))), 0, [
								'style' => 'width: 520px;', 'onchange' => "top.content.setHot();"])), we_html_multiIconBox::PROP_PATH))
				->add(new we_html_multiIconBoxContentLine(g_l('thumbnails', '[properties]'), $window_html->getHtml(), we_html_multiIconBox::PROP_ATTRIB))
				->add(new we_html_multiIconBoxContentLine('Filter', we_html_element::div(['class' => 'editorThumbnailsFilter'], $thumbnail_option_table['filter']->getHtml()), we_html_multiIconBox::PROP_FILTER))
				->add((new we_html_multiIconBoxContentLine(g_l('thumbnails', '[format]'), $thumbnail_format_select->getHtml(), we_html_multiIconBox::PROP_IMAGE))
					->noline()
		);
	}

	function getHTMLProperties(): string{// TODO: move to View
		//$t = we_base_request::int('tabnr', 1);
		return we_html_element::jsScript(JS_DIR . 'we/util/multi_editMulti.js') .
			we_html_element::div(['id' => 'tab1'], (new we_html_multiIconBox($this->getHTMLTab1()))->marginLeft()->get());
	}

	protected function getHTMLCmd(): string{
		if(($pid = we_base_request::int('pid', -1)) < 0){
			return $this->getHTMLDocument(we_html_element::body(), $this->jsCmd->getCmds());
		}

		$offset = we_base_request::int("offset", 0);
		$this->jsCmd->addCmd('loadTree', ['clear' => !$pid, 'items' => we_thumbnail_tree::getItems($pid, $offset, $this->Tree->default_segment), 'sorted' => true]);

		return $this->getHTMLDocument(
				we_html_element::body([], we_html_element::form(
						['name' => we_base_constants::WE_FORM], we_html_element::hiddens([
							'pnt' => "cmd",
							'cmd' => "no_cmd"]
						)
					)
		));
	}

}
