<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: adcae6a39191 $
 * $Author: Lukas Imhof $
 * $Date: 2021-10-21 11:48 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_thumbnail_view extends we_modules_view{
	/** @var we_thumbnail_thumbnail */
	var $Model;

	function __construct(){
		parent::__construct(we_modules_info::THUMB);
		$this->Model = new we_thumbnail_thumbnail();
	}

	function getJSTop(): string{
		return
			parent::getJSTop() .
			we_html_element::jsScript(WE_JS_MODULES_DIR . '/thumbnail/thumbnail_top.js', "parent.document.title='" . $this->getModTitle() . "'");
	}

	function getJSProperty(array $jsVars = []): string{
		return we_html_element::jsScript(WE_JS_MODULES_DIR . '/thumbnail/thumbnail_prop.js');
	}

	function processCommands(we_base_jsCmd $jsCmd): void{
		switch(we_base_request::string('cmd', '')){
			case 'we_thumbnail_save':
				if(!we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR])){
					$jsCmd->addMsg(g_l('weClass', '[no_perms]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}
				//$this->Model->Path = ($this->Model->ParentID ? fS('SELECT Path FROM ' . THUMBNAILS_TABLE . ' WHERE ID=' . intval($this->Model->ParentID)) : '/') . $this->Model->Text;

				if(preg_match('|[\'",]|', $this->Model->Text)){
					$jsCmd->addMsg(g_l('alert', '[thumbnail_hochkomma]'), we_base_util::WE_MESSAGE_ERROR);
				} else if(!$this->Model->Text){
					$jsCmd->addMsg(g_l('alert', '[thumbnail_empty]'), we_base_util::WE_MESSAGE_ERROR);
				} elseif(fI('SELECT ID FROM ' . THUMBNAILS_TABLE . ' WHERE Text="' . $this->db->escape($this->Model->Text) . '" AND ID!=' . intval($this->Model->ID) . ' LIMIT 1')){
					$jsCmd->addMsg(sprintf(g_l('alert', '[thumbnail_exists]'), $this->Model->Text), we_base_util::WE_MESSAGE_ERROR);
				} elseif($this->Model->we_save()){
					$jsCmd->addMsg(sprintf(g_l('thumbnails', '[saved_successfully]'), $this->Model->Text), we_base_util::WE_MESSAGE_NOTICE);
					if($this->Model->wasUpdate){
						$jsCmd->addCmd('updateTreeEntry', ['id' => $this->Model->ID, 'parentid' => $this->Model->ParentID, 'text' => $this->Model->Text]);
					} else {
						$jsCmd->addCmd('makeTreeEntry', ['id' => $this->Model->ID, 'parentid' => $this->Model->ParentID, 'text' => $this->Model->Text, 'open' => false, 'contenttype' => we_base_ContentTypes::THUMBNAIL,
							'table' => THUMBNAILS_TABLE, 'published' => 1]);
					}
				} else {
					echo "ERROR";
				}
				break;
			case 'we_thumbnail_delete':
// Check if we need to delete a thumbnail
				if(!($delId = $this->Model->ID) || !we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR])){
					$jsCmd->addMsg(g_l('weClass', '[no_perms]'), we_base_util::WE_MESSAGE_ERROR);
					return;
				}
				// Delete thumbnails in filesystem
				we_contents_thumbnail::deleteByThumbID($delId);

				$this->db->query('DELETE FROM ' . THUMBNAILS_TABLE . ' WHERE ID=' . $delId);
				$jsCmd->addMsg(sprintf(g_l('thumbnails', '[deleted]'), $this->Model->Text), we_base_util::WE_MESSAGE_INFO);
				$jsCmd->addCmd('deleteTreeEntry', $delId);
				$jsCmd->addCmd('unsetHot');
				$jsCmd->addCmd('loadHome');
				$_SESSION['weS']['module_sessions']['thumbnail'] = null;
				$this->Model = new we_thumbnail_thumbnail();

				break;
			case 'we_thumbnail_new'://new
				$this->Model = new we_thumbnail_thumbnail();
				break;
			default:
				$id = (we_base_request::int('we_cmd', 0, 1) ?: 0);
				if($id){
					$this->Model->initByID($id);
				}
		}
		$_SESSION['weS']['module_sessions']['thumbnail'] = $this->Model;
	}

	function processVariables(): void{
		if(!empty($_SESSION['weS']['module_sessions']['thumbnail'])){
			$this->Model = $_SESSION['weS']['module_sessions']['thumbnail'];

			$this->Model->Text = we_base_request::string('Text', '');
			$this->Model->description = we_base_request::string('description', '');
			$this->Model->Quality = we_base_request::int('thumbnail_quality', 8);
			$this->Model->Height = we_base_request::int('thumbnail_height', 0);
			$this->Model->Width = we_base_request::int('thumbnail_width', 0);
			$this->Model->Format = ($format = we_base_request::string('Format', '')) === 'none' ? '' : $format;
			$this->Model->Options = array_values(array_filter(we_base_request::stringList('Options'), function($v){
					return $v !== we_contents_thumbnail::OPTION_DEFAULT;
				}));
		} else {
			$_SESSION['weS']['module_sessions']['thumbnail'] = null;
		}
	}

	public function getHomeScreen(we_base_jsCmd $jscmd): string{
		$hiddens = ['cmd' => 'home', 'pnt' => 'edbody'];

		$content = we_html_button::create_button('fat:add,fa-lg fa-plus', "javascript:we_cmd('we_thumbnail_new')");

		return parent::getActualHomeScreen(we_modules_info::THUMB, $jscmd, $content, we_html_element::form(['name' => we_base_constants::WE_FORM], $this->getCommonHiddens($hiddens) . we_html_element::hidden('home', 0)));
	}

}
