<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: adcae6a39191 $
 * $Author: Lukas Imhof $
 * $Date: 2021-10-21 11:48 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_users_view extends we_modules_view{

	function __construct(){
		parent::__construct(we_modules_info::USERS);
	}

	function getJSTop(): string{
		if(isset($_SESSION['user_session_data'])){
			unset($_SESSION['user_session_data']);
		}

		return parent::getJSTop() .
			we_html_element::jsScript(WE_JS_MODULES_DIR . 'users/users_view.js', '', ['id' => 'loadVarUsersView', 'data-users' => setDynamicVar([
					'modTitle' => $this->getModTitle(),
					'cgroup' => fI('SELECT ParentID FROM ' . USER_TABLE . ' WHERE ID=' . we_users_user::getID()),
					'Type' => we_users_user::TYPE_USER,
					'Text' => '',
		])]);
	}

	function getJSProperty(array $jsVars = []): string{
		return we_html_element::jsScript(WE_JS_MODULES_DIR . 'users/users_property.js');
	}

	private static function new_group(we_base_jsCmd $jscmd): void{
		if(!we_base_permission::hasPerm([we_base_perm::USER_GROUP_NEW])){
			$jscmd->addMsg(g_l('alert', '[access_denied]'), we_base_util::WE_MESSAGE_ERROR);
			return;
		}

		$user_object = new we_users_user();
		$user_object->ParentID = we_base_request::int("cgroup");
		$user_object->initType(we_users_user::TYPE_USER_GROUP);

		$_SESSION['user_session_data'] = $user_object;

		$jscmd->addCmd('loadUsersContent');
	}

	private static function new_alias(we_base_jsCmd $jscmd): void{
		if(!we_base_permission::hasPerm([we_base_perm::USER_NEW])){
			$jscmd->addMsg(g_l('alert', '[access_denied]'), we_base_util::WE_MESSAGE_ERROR);
			return;
		}

		$user_object = new we_users_user();
		$user_object->ParentID = we_base_request::int("cgroup");
		$user_object->initType(we_users_user::TYPE_ALIAS);

		$_SESSION['user_session_data'] = $user_object;
		$jscmd->addCmd('loadUsersContent');
	}

	private static function new_user(we_base_jsCmd $jscmd): void{
		if(!we_base_permission::hasPerm([we_base_perm::USER_NEW])){
			$jscmd->addMsg(g_l('alert', '[access_denied]'), we_base_util::WE_MESSAGE_ERROR);
			return;
		}
		$user_object = new we_users_user();
		$user_object->ParentID = we_base_request::int("cgroup");
		$user_object->initType(we_users_user::TYPE_USER);

		$_SESSION['user_session_data'] = $user_object;
		$jscmd->addCmd('loadUsersContent', ['oldtab' => 0]);
	}

	private static function display_user(we_base_jsCmd $jscmd): void{
		if(!($uid = we_base_request::int('uid'))){
			return;
		}
		$user_object = new we_users_user();
		$user_object->initFromDB($uid);
		if(!we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR]) && $user_object->checkPermission('ADMINISTRATOR')){
			$jscmd->addMsg(g_l('alert', '[access_denied]'), we_base_util::WE_MESSAGE_ERROR);
			return;
		}

		$_SESSION['user_session_data'] = $user_object;

		$jscmd->addCmd('unsetHot');
		$jscmd->addCmd('setUserData', $user_object->Type == we_users_user::TYPE_USER_GROUP ? $user_object->ID : $user_object->ParentID, $user_object->Type ?: we_users_user::TYPE_USER, $user_object->Path);
		$jscmd->addCmd('loadUsersContent', ['oldtab' => 0]);
	}

	private static function save_user(we_base_jsCmd $jscmd): void{
		$ob = we_base_request::string('obj_name');
		$uname = we_base_request::string($ob . '_username');
		if($uname && !we_users_user::filenameNotValid($uname)){
			$jscmd->addMsg(g_l('global', '[username_wrong_chars]'), we_base_util::WE_MESSAGE_ERROR);
			return;
		}
		if(!isset($_SESSION['user_session_data'])){
			$jscmd->addMsg(g_l('alert', '[no_perms]'), we_base_util::WE_MESSAGE_ERROR);
			return;
		}

		if(($parent = we_base_request::int($ob . '_ParentID'))){
			$folder = fB('SELECT IsFolder FROM ' . USER_TABLE . ' WHERE ID=' . $parent);

			if(empty($folder)){
				$jscmd->addMsg(g_l('alert', '[no_perms]'), we_base_util::WE_MESSAGE_ERROR);
				return;
			}
		}

		$alltables = array_filter([
			FILE_TABLE,
			TEMPLATES_TABLE,
			NAVIGATION_TABLE,
			OBJECT_FILES_TABLE,
			NEWSLETTER_TABLE,
		]);

		foreach($alltables as $table){
			$i = 0;
			while(($wsp = we_base_request::int($ob . '_Workspace_' . $table . '_' . $i))){
				$folder = fB('SELECT IsFolder FROM ' . $table . ' WHERE ID=' . $wsp);
				if(empty($folder)){
					$jscmd->addMsg(g_l('modules_users', '[workspaceFieldError]'), we_base_util::WE_MESSAGE_ERROR);
					return;
				}
				$i++;
			}
		}

		$user_object = $_SESSION['user_session_data'];
		if(($oldtab = we_base_request::int('oldtab', -1)) > -1){//FIXME: is latter ever used?
			$user_object->preserveState($oldtab);
			$_SESSION['user_session_data'] = $user_object;
		}

		if(!we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR]) && $user_object->checkPermission(we_base_perm::ADMINISTRATOR)){
			$jscmd->addMsg(g_l('alert', '[access_denied]'), we_base_util::WE_MESSAGE_ERROR);
			return;
		}
		$oldperm = $user_object->checkPermission(we_base_perm::ADMINISTRATOR);
		if(!$user_object){
			return;
		}

		if(!we_base_permission::hasPerm([we_base_perm::USER_SAVE]) && ($user_object->Type == we_users_user::TYPE_USER || $user_object->Type == we_users_user::TYPE_ALIAS) && $user_object->ID != 0){
			$jscmd->addMsg(g_l('alert', '[access_denied]'), we_base_util::WE_MESSAGE_ERROR);
			return;
		}
		if(!we_base_permission::hasPerm([we_base_perm::USER_NEW]) && ($user_object->Type == we_users_user::TYPE_USER || $user_object->Type == we_users_user::TYPE_ALIAS) && $user_object->ID == 0){
			$jscmd->addMsg(g_l('alert', '[access_denied]'), we_base_util::WE_MESSAGE_ERROR);
			return;
		}
		if(!we_base_permission::hasPerm([we_base_perm::USER_GROUP_SAVE]) && $user_object->Type == we_users_user::TYPE_USER_GROUP && $user_object->ID != 0){
			$jscmd->addMsg(g_l('alert', '[access_denied]'), we_base_util::WE_MESSAGE_ERROR);
			return;
		}
		if(!we_base_permission::hasPerm([we_base_perm::USER_GROUP_NEW]) && $user_object->Type == we_users_user::TYPE_USER_GROUP && $user_object->ID == 0){
			$jscmd->addMsg(g_l('alert', '[access_denied]'), we_base_util::WE_MESSAGE_ERROR);
			return;
		}
		if(($ot = we_base_request::int('oldtab'))){
			$user_object->preserveState($ot);
		}

		$id = $user_object->ID;
		if(!$user_object->username && $user_object->Type != we_users_user::TYPE_ALIAS){
			$jscmd->addMsg(g_l('modules_users', '[username_empty]'), we_base_util::WE_MESSAGE_ERROR);
			return;
		}

		if($user_object->Alias == 0 && $user_object->Type == we_users_user::TYPE_ALIAS){
			$jscmd->addMsg(g_l('modules_users', '[username_empty]'), we_base_util::WE_MESSAGE_ERROR);
			return;
		}
		$exist = (fB('SELECT 1 FROM ' . USER_TABLE . ' WHERE ID!=' . intval($user_object->ID) . ' AND username="' . $user_object->username . '" LIMIT 1'));
		if($exist && $user_object->Type != we_users_user::TYPE_ALIAS){
			$jscmd->addMsg(sprintf(g_l('modules_users', '[username_exists]'), $user_object->username), we_base_util::WE_MESSAGE_ERROR);
			return;
		}
		if(($oldperm) && (!$user_object->checkPermission("ADMINISTRATOR")) && ($user_object->isLastAdmin())){
			$jscmd->addMsg(g_l('modules_users', '[modify_last_admin]'), we_base_util::WE_MESSAGE_ERROR);
			return;
		}

		try{
			if(!$user_object->save()){
					$jscmd->addMsg(g_l('importFiles','[save_error]'), we_base_util::WE_MESSAGE_ERROR);
					$user_object->saveToSession($jscmd);
					return;
			}
			$user_object->saveToSession($jscmd);
		} catch (we_exception_base $e){
			$jscmd->addMsg($e->getMessage(), we_base_util::WE_MESSAGE_ERROR);
			return;
		} finally {
			$_SESSION['user_session_data'] = $user_object;
		}
		//	Save seem_startfile to DB when needed.
		if(($sid = we_base_request::int('seem_start_file', -1)) > -1){
			$uid = we_base_request::int('uid');
			//if($sid || (isset($_SESSION['save_user_seem_start_file'][$uid]))){

			//if($sid !== false){
			//	save seem_start_file from REQUEST
			$seem_start_file = $sid;
			if($user_object->ID == we_users_user::getID()){ // change preferences if user edits his own startfile
				$_SESSION['prefs']['seem_start_file'] = $seem_start_file;
			}
			/* } else {
			  //	Speichere seem_start_file aus SESSION
			  $seem_start_file = $_SESSION['save_user_seem_start_file'][$uid];
			  } */

			DB_WE::$inst->query('REPLACE INTO ' . PREFS_TABLE . ' SET userID=' . $uid . ',`key`="seem_start_file",`value`="' . DB_WE::$inst->escape($seem_start_file) . '"');
			unset($seem_start_file);
			if(isset($_SESSION['save_user_seem_start_file'][$uid])){
				unset($_SESSION['save_user_seem_start_file'][$uid]);
			}
			//}
		}

		$jscmd->addCmd('unsetHot');
		if($id){
			$jscmd->addCmd('updateTreeEntry', ['id' => $user_object->ID, 'parentid' => $user_object->ParentID, 'text' => $user_object->username, 'class' => ($user_object->checkPermission(we_base_perm::ADMINISTRATOR) ? 'bold ' : '') . ($user_object->LoginDenied ? 'red' : '')]);
		} else {
			$jscmd->addCmd('makeTreeEntry', ['id' => $user_object->ID, 'parentid' => $user_object->ParentID, 'text' => $user_object->username, 'open' => false, 'contenttype' => (($user_object->Type == we_users_user::TYPE_USER_GROUP) ? we_base_ContentTypes::USERGROUP : (($user_object->Type == we_users_user::TYPE_ALIAS) ? we_base_ContentTypes::USERALIAS : we_base_ContentTypes::USER)),
				'table' => USER_TABLE, 'published' => ($user_object->LoginDenied ? 0 : 1), 'class' => ($user_object->checkPermission(we_base_perm::ADMINISTRATOR) ? 'bold ' : '')]);
		}

		switch($user_object->Type){
			case we_users_user::TYPE_ALIAS:
				$jscmd->addMsg(sprintf(g_l('modules_users', '[alias_saved_ok]'), $user_object->username), we_base_util::WE_MESSAGE_NOTICE);
				break;
			case we_users_user::TYPE_USER_GROUP:
				$jscmd->addMsg(sprintf(g_l('modules_users', '[group_saved_ok]'), $user_object->username), we_base_util::WE_MESSAGE_NOTICE);
				break;
			case we_users_user::TYPE_USER:
			default:
				$jscmd->addMsg(sprintf(g_l('modules_users', '[user_saved_ok]'), $user_object->username), we_base_util::WE_MESSAGE_NOTICE);
				break;
		}

		$jscmd->addCmd('setUserData', $user_object->Type == we_users_user::TYPE_USER_GROUP ? $user_object->ID : $user_object->ParentID, $user_object->Type ?: we_users_user::TYPE_USER, $user_object->Path);
		$jscmd->addCmd('updateTitle', $user_object->Path);
	}

	private static function delete_user(we_base_jsCmd $jscmd): void{
		if(empty($_SESSION['user_session_data'])){
			$jscmd->addMsg(g_l('alert', '[access_denied]'), we_base_util::WE_MESSAGE_ERROR);
			return;
		}

		$user_object = $_SESSION['user_session_data'];

		if($user_object->ID == we_users_user::getID()){
			$jscmd->addMsg(g_l('modules_users', '[delete_user_same]'), we_base_util::WE_MESSAGE_ERROR);
			return;
		}

		if(we_users_util::isUserInGroups(we_users_user::getID(), [$user_object->ID])){
			$jscmd->addMsg(g_l('modules_users', '[delete_group_user_same]'), we_base_util::WE_MESSAGE_ERROR);
			return;
		}

		if(!we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR]) && $user_object->checkPermission('ADMINISTRATOR')){
			$jscmd->addMsg(g_l('alert', '[access_denied]'), we_base_util::WE_MESSAGE_ERROR);
			return;
		}
		switch($user_object->Type){
			case we_users_user::TYPE_USER:
				if(!we_base_permission::hasPerm([we_base_perm::USER_DELETE])){
					$jscmd->addMsg(g_l('alert', '[access_denied]'), we_base_util::WE_MESSAGE_ERROR);
					return;
				}
				break;
			case we_users_user::TYPE_USER_GROUP:
				if(!we_base_permission::hasPerm([we_base_perm::USER_GROUP_DELETE])){
					$jscmd->addMsg(g_l('alert', '[access_denied]'), we_base_util::WE_MESSAGE_ERROR);
					return;
				}
				break;
		}

		/* if(isset($GLOBALS['user']) && $user_object->username == $GLOBALS['user']['Username']){
		  $jscmd->addMsg(g_l('alert', '[user_same]'), we_base_util::WE_MESSAGE_ERROR);
		  return;
		  } */

		if($user_object->checkPermission('ADMINISTRATOR') && $user_object->isLastAdmin()){
			$jscmd->addMsg(g_l('modules_users', '[modify_last_admin]'), we_base_util::WE_MESSAGE_ERROR);
			return;
		}

		if(!$user_object->deleteMe()){
			$jscmd->addMsg(g_l('alert', '[access_denied]'), we_base_util::WE_MESSAGE_ERROR);
			return;
		}
		$jscmd->addCmd('deleteTreeEntry', $user_object->ID);
		$jscmd->addCmd('loadUsersContent', ['home' => 1]);

		unset($_SESSION['user_session_data']);
	}

	private static function check_user_display(we_base_jsCmd $jscmd): void{
		if(($uid = we_base_request::int('uid'))){
			$mpid = fI('SELECT ParentID FROM ' . USER_TABLE . ' WHERE ID=' . we_users_user::getID());
			$pid = fI('SELECT ParentID FROM ' . USER_TABLE . ' WHERE ID=' . $uid);

			$search = $first = true;
			$found = false;

			if(!we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR])){
				while($search){
					if($mpid == $pid){
						$search = false;
						if(!$first){
							$found = true;
						}
					}
					$first = false;
					if($pid == 0){
						$search = false;
					}
					$pid = fI('SELECT ParentID FROM ' . USER_TABLE . ' WHERE ID=' . $pid);
				}
			}

			if($found || we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR])){
				$jscmd->addCmd('display_user', $uid);
			} else {
				$jscmd->addMsg(g_l('alert', '[access_denied]'), we_base_util::WE_MESSAGE_ERROR);
			}
		}
	}

	function processCommands(we_base_jsCmd $jscmd): void{
		switch(we_base_request::string('cmd')){
			case 'new_group':
				self::new_group($jscmd);
				break;
			case 'new_alias':
				self::new_alias($jscmd);
				break;
			case 'new_user':
				self::new_user($jscmd);
				break;
			case 'display_user':
				self::display_user($jscmd);
				break;
			case 'save_user':
				self::save_user($jscmd);
				break;
			case 'delete_user_do':
				self::delete_user($jscmd);
				break;
			case 'check_user_display':
				self::check_user_display($jscmd);
				break;
		}
	}

	function processVariables(): void{
		if(($page = we_base_request::int('page', -1)) > -1){
			$this->page = $page;
		}
	}

	public function getHomeScreen(we_base_jsCmd $jscmd): string{
		$content = we_html_button::create_button('fat:create_user,fa-lg fa-user-plus', "javascript:we_cmd('new_user');", '', "", !we_base_permission::hasPerm([we_base_perm::USER_NEW])) .
			we_html_button::create_button('fat:create_group,fa-lg fa-users,fa-plus', "javascript:we_cmd('new_group');", '', "", !we_base_permission::hasPerm([we_base_perm::USER_GROUP_NEW])) .
			we_html_button::create_button('fat:create_alias,alias fa-lg fa-user-plus', "javascript:we_cmd('new_alias');", '', "", !we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR]));

		return parent::getActualHomeScreen(we_modules_info::USERS, $jscmd, $content);
	}

}
