<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 36ad0e3aa620 $
 * $Author: Lukas Imhof $
 * $Date: 2021-09-06 19:26 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_weTag_multiSelectorAttribute extends we_weTag_attribute{
	/**
	 * @var string
	 */
	protected $Table;

	/**
	 * @var string
	 */
	protected $Selectable;

	/**
	 * @var string
	 */
	protected $TextName = 'text';

	/**
	 * @param string $name
	 * @param string $table
	 * @param string $selectable
	 * @param string $textName
	 * @param boolean $required
	 */
	function __construct(we_weTag_twbase $tag, string $name, string $default, string $table, string $selectable, string $textName = 'path', bool $required = false, string $module = '', string $description = '', bool $deprecated = false){
		parent::__construct($tag, $name, $default, $required, $module, $description, $deprecated);
		$this->Table = $table;
		$this->Selectable = trim($selectable, ' ,');
		$this->TextName = $textName;
	}

	function getCodeForTagWizard(we_html_tr $row): void{
		switch($this->Table){
			case USER_TABLE :
				$we_cmd = 'we_users_selector';
				break;
			case CATEGORY_TABLE :
				$we_cmd = 'we_selector_document';
				break;
			case CUSTOMER_TABLE :
				$we_cmd = 'we_customer_selector';
				break;
			default:
				$we_cmd = !in_array($this->Selectable, we_base_ContentTypes::FOLDER_TYPES) ? ($this->Selectable === '' ? 'we_selector_file' : 'we_selector_document') : (in_array($this->Selectable, we_base_ContentTypes::FOLDER_TYPES, true) ? 'we_selector_directory' : 'we_selector_file');
		}

		$button = we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('" . $we_cmd . "', {id:0, table:'" . $this->Table . "', jsCmd:'we_multiSelector_writeback,all" . $this->TextName . "s," . $this->getIdName() . "', filter:'" . $this->Selectable . "'})");

		$row->td($this->getLabelCodeForTagWizard(), ['class' => 'attributeName']);
		$row->td(we_html_element::textArea(['name' => $this->Name, 'id' => $this->getIdName(), 'class' => 'wetextinput wetextarea']), ['class' => 'attributeField']);
		$row->td($button, ['class' => 'attributeButton']);
		$row->td($this->defaultValue, ['class' => 'defaultValue']);
	}

}
