<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 4fcb1912b4bf $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-11-16 03:34 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_weTag_selectorAttribute extends we_weTag_attribute{
	const FOLDER = 'folder';

	/** @var string	 */
	protected $Table;

	/** @var string	 */
	protected $Selectable;

	/** @var bool */
	private $useName;

	/**
	 * @param string $name
	 * @param string $table
	 * @param string $selectable
	 * @param boolean $required
	 */
	function __construct(we_weTag_twbase $tag, string $name, string $default, string $table, string $selectable, bool $required = false, string $module = '', bool $useName = false, string $description = '', bool $deprecated = false){
		parent::__construct($tag, $name, $default, $required, $module, $description, $deprecated);
		$this->Table = $table;
		$this->Selectable = $selectable;
		$this->useName = $useName;
	}

	function getCodeForTagWizard(we_html_tr $row): void{
		switch($this->Table){
			case CATEGORY_TABLE:
				$weCmd = 'we_selector_document';
				$this->Selectable = '';
				break;
			case NAVIGATION_TABLE:
				$weCmd = 'we_selector_file';
				break;
			default:
				$weCmd = ($this->Selectable == self::FOLDER ? 'we_selector_directory' : 'we_selector_document');
		}

		$button = we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('" . $weCmd . "', {id:document.getElementById('" . $this->getIdName() . "').value, table:'" . $this->Table . "', targetID:'" . ($this->useName ? '' : $this->getIdName()) . "', targetName:'" . ($this->useName ? $this->getIdName() : '') . "', filter:'" . $this->Selectable . "'})");

		$row->td($this->getLabelCodeForTagWizard(), ['class' => 'attributeName']);
		$row->td(we_html_element::input([
				'name' => $this->Name,
				'value' => $this->Value,
				'id' => $this->getIdName(),
				'class' => 'wetextinput'
			]), ['class' => 'attributeField']);
		$row->td($button, ['class' => 'attributeButton']);
		$row->td($this->defaultValue, ['class' => 'defaultValue']);
	}

}
