<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: be7e56b8b52f $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2021-11-18 23:25 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_widget_mfd extends we_widget_base{//zuletzt bearbeitet

	/** @var string */

	private $lastModified = '';

	public function __construct(string $curID = '', array $aProps = []){
		//preview requested
		$aCols = (!$curID ? we_base_request::stringList('we_cmd') : []);

		$mode = $_SESSION['weS']['we_mode'];
		$uid = we_users_user::getID();
		session_write_close();


		if(empty($aCols) || count($aCols) < 5){
			$aCols = explode(';', $aProps ? $aProps[3] : self::getDefaultConfig()['csv']);
		}
		if(empty($aCols) || count($aCols) < 5){
			$aCols = explode(';', self::getDefaultConfig()['csv']);
		}
		$sTypeBinary = $aCols[0];
		$pos = 0;
		$bTypeDoc = we_base_permission::hasPerm([we_base_perm::WORK_DOCUMENTS_CAN_SEE]) && isset($sTypeBinary[$pos]) && ($sTypeBinary[$pos]);
		$pos++;
		$bTypeTpl = $mode != we_base_constants::MODE_SEE && we_base_permission::hasPerm([we_base_perm::WORK_TEMPLATES_CAN_SEE]) && isset($sTypeBinary[$pos]) && ($sTypeBinary[$pos]);
		$pos++;
		$bTypeObj = we_modules_info::isActive(we_modules_info::OBJECT) && we_base_permission::hasPerm([we_base_perm::OBJECTFILES_CAN_SEE]) && isset($sTypeBinary[$pos]) && ($sTypeBinary[$pos]);
		$pos++;
		$bTypeCls = $mode != we_base_constants::MODE_SEE && we_modules_info::isActive(we_modules_info::OBJECT) && we_base_permission::hasPerm([we_base_perm::OBJECTS_CAN_SEE]) && isset($sTypeBinary[$pos]) && ($sTypeBinary[$pos]);

		$iDate = intval($aCols[1]);

		$doctable = $where = [];

		switch($iDate){
			case 1 :
				$where[] = 'h.Mod_Date=CURDATE()';
				break;
			case 2 :
				$where[] = 'h.Mod_Date>=(CURDATE()-INTERVAL 1 WEEK)';
				break;
			case 3 :
				$where[] = 'h.Mod_Date>=(CURDATE()-INTERVAL 1 MONTH)';
				break;
			default:
			case 4 :
				$where[] = 'h.Mod_Date>=(CURDATE()-INTERVAL 1 YEAR)';
				break;
		}
		$iNumItems = $aCols[2];
		switch($iNumItems){
			default:
				$iMaxItems = 200;
				break;
			case 11:
				$iMaxItems = 15;
				break;
			case 12:
				$iMaxItems = 20;
				break;
			case 13:
				$iMaxItems = 25;
				break;
			case 14:
				$iMaxItems = 50;
				break;
		}
		$sDisplayOpt = $aCols[3];
		$bMfdBy = $sDisplayOpt[0];
		$bDateLastMfd = $sDisplayOpt[1];

		$db = DB_WE::$inst;

		$aUsers = array_filter(array_map('intval', (we_base_permission::hasPerm([we_base_perm::WIDGET_EDIT_MFD]) ?
						array_filter(explode(',', $aCols[4])) :
						[$uid])));

		if($aUsers){
			$aUsers = implode(',', $aUsers);
			$folders = array_map('escape_sql_query', $db->getAllq('SELECT Path FROM ' . USER_TABLE . ' WHERE ID IN (' . $aUsers . ') AND IsFolder=1', true));
			if($folders){
				$aUsers = implode(',', $db->getAllq('SELECT ID FROM ' . USER_TABLE . ' WHERE IsFolder=0 AND (Path REGEXP "^(' . implode('|', $folders) . ')/" OR ID IN (' . $aUsers . '))', true));
			}
			if($aUsers){
				$where[] = 'h.UID IN (' . $aUsers . ')';
			}
		}

		$join = [
			HISTORY_TABLE . ' h',
			LOCK_TABLE . ' l ON l.ID=DID AND l.tbl=h.DocumentTable AND l.UserID!=' . $uid
		];
		$tables = [];
		$admin = we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR]); //reenabled, since this is worse than that disabled by now: rights can be set to groups, not only to users

		if($bTypeDoc){
			$doctable[] = '"' . stripTblPrefix(FILE_TABLE) . '"';
			$ws = we_users_util::get_ws(FILE_TABLE);

			$join[] = FILE_TABLE . ' f ON (h.DocumentTable="' . stripTblPrefix(FILE_TABLE) . '" AND f.ID=h.DID ' .
					($ws ? ' AND f.ParentID IN(' . 'SELECT ID
FROM ' . FILE_TABLE . '
WHERE IsFolder=1 AND (ID IN (' . implode(',', $ws) . ') OR Path LIKE "' . implode('%" OR Path LIKE "', id2pathA($ws, FILE_TABLE)) . '%")' . ')' : '') .
					($admin ? '' : ' AND (f.RestrictOwners=0 OR f.CreatorID=' . we_users_user::getID() . ' OR FIND_IN_SET(' . we_users_user::getID() . ',f.Owners))') .
					')';
			$tables[] = 'f';
		}
		if($bTypeObj){
			$doctable[] = '"' . stripTblPrefix(OBJECT_FILES_TABLE) . '"';
			$ws = we_users_util::get_ws(OBJECT_FILES_TABLE);
			$join[] = OBJECT_FILES_TABLE . ' `of` ON (h.DocumentTable="' . stripTblPrefix(OBJECT_FILES_TABLE) . '" AND `of`.ID=h.DID ' .
					($ws ? ' AND of.ParentID IN(' . 'SELECT ID
FROM ' . OBJECT_FILES_TABLE . '
WHERE IsFolder=1 AND (ID IN (' . implode(',', $ws) . ') OR Path LIKE "' . implode('%" OR Path LIKE "', id2pathA($ws, OBJECT_FILES_TABLE)) . '%")' . ')' : '') .
					($admin ? '' : ' AND (`of`.RestrictOwners=0 OR `of`.CreatorID=' . we_users_user::getID() . ' OR FIND_IN_SET(' . we_users_user::getID() . ',`of`.Owners))') .
					')';
			$tables[] = '`of`';
		}
		if($bTypeTpl){
			$doctable[] = '"' . stripTblPrefix(TEMPLATES_TABLE) . '"';
			$join[] = TEMPLATES_TABLE . ' t ON (h.DocumentTable="' . stripTblPrefix(TEMPLATES_TABLE) . '" AND t.ID=h.DID' .
					($admin ? '' : ' AND (t.RestrictOwners=0 OR t.CreatorID=' . we_users_user::getID() . ' OR FIND_IN_SET(' . we_users_user::getID() . ',t.Owners))') .
					')';
			$tables[] = 't';
		}
		if($bTypeCls){
			$doctable[] = '"' . stripTblPrefix(OBJECT_TABLE) . '"';
			$join[] = OBJECT_TABLE . ' o ON (h.DocumentTable="' . stripTblPrefix(OBJECT_TABLE) . '" AND o.ID=h.DID' .
					(true || $admin ? '' : ' AND (o.RestrictOwners=0 OR o.CreatorID=' . we_users_user::getID() . ' OR FIND_IN_SET(' . we_users_user::getID() . ',o.Owners))') .
					')';
			$tables[] = 'o';
		}

		if(!$tables){
			$this->lastModified = '';
			return;
		}

		if($doctable){
			$where[] = 'h.DocumentTable IN(' . implode(',', $doctable) . ')';
		}

		/* if($mode == we_base_constants::MODE_SEE){
		  $where[] = ' f.ContentType!="' . we_base_ContentTypes::FOLDER . '" ';
		  } */
		/* if(we_users_user::getID() === 150){
		  DB_WE::t_e();
		  } */
		$where = ' WHERE ' . implode(' AND ', $where);
		$all = $db->getAllq('SELECT
	h.DID,
	(SELECT UserName FROM ' . HISTORY_TABLE . ' hh WHERE MAX(h.Mod_Date)=hh.Mod_Date AND hh.DID=h.DID AND h.DocumentTable=hh.DocumentTable) AS UserName,
	h.DocumentTable AS ctable,
	!ISNULL(l.ID) AS isOpen,
	COALESCE(' . implode('.ID,', $tables) . '.ID,"?") AS ID,
	COALESCE(' . implode('.Path,', $tables) . '.Path,"?") AS Path,
	COALESCE(' . implode('.Text,', $tables) . '.Text,"?") AS Text,
	COALESCE(' . implode('.ContentType,', $tables) . '.ContentType,"?") AS ContentType,
	DATE_FORMAT(COALESCE(' . implode('.Mod_Date,', $tables) . '.Mod_Date),"' . g_l('date', '[format][mysql]') . '") AS MDate
FROM ' .
				implode(' LEFT JOIN ', $join) . '
' . $where . '
GROUP BY
	h.DID,h.DocumentTable
ORDER BY
	h.Mod_Date DESC
LIMIT
	0,' . ($iMaxItems));

		$this->lastModified = '<table class="middlefont">';

		foreach($all as $file /* && $j < $iMaxItems */){
			$isOpen = $file['isOpen'];
			$this->lastModified .= '<tr ' . ($isOpen ? '' : 'onclick="WE().layout.weEditorFrameController.openDocument(\'' . addTblPrefix($file['ctable']) . '\',' . $file['ID'] . ',\'' . $file['ContentType'] . '\');" title="' . $file['Path'] . '"') . '><td class="mfdIcon" data-contenttype="' . $file['ContentType'] . '"></td>' .
					'<td class="mfdDoc' . ($isOpen ? ' isOpen' : '') . '" ><span class="cutTextLeft">' .
					$file['Path'] .
					'</span></td>' .
					($bMfdBy ? '<td class="mfdUser">' . $file['UserName'] . /* (($bDateLastMfd) ? ',' : '') . */ '</td>' : '') .
					($bDateLastMfd ? '<td class="mfdDate">' . $file['MDate'] . '</td>' : '') .
					'</tr>';
		}

		$this->lastModified .= '</table>';
	}

	/**
	 * @return array{string,array{string,string}}
	 */
	public function getInsertDiv(int $iCurrId, array $aProps, we_base_jsCmd $jsCmd): array{
		$cfg = self::getDefaultConfig();
		$oTblDiv = we_html_element::div(['id' => 'm_' . $iCurrId . '_inline',
					'class' => 'widget_mfd',
					'style' => 'height:' . ($cfg['height']) . 'px;',
						], we_html_element::div(['id' => 'mfd_data'], $this->lastModified)
		);
		$jsCmd->addCmd('setIconOfDocClass', 'mfdIcon');
		$aLang = [g_l('cockpit', '[last_modified]'), ""];
		return [$oTblDiv, $aLang];
	}

	/**
	 * @return array<string,mixed>
	 */
	public static function getDefaultConfig(): array{
		$shortCuts = (we_base_permission::hasPerm([we_base_perm::WORK_DOCUMENTS_CAN_SEE]) ? '1' : '0') .
				((we_base_permission::hasPerm([we_base_perm::WORK_TEMPLATES_CAN_SEE])) ? '1' : '0') .
				(we_modules_info::isActive(we_modules_info::OBJECT) && we_base_permission::hasPerm([we_base_perm::OBJECTFILES_CAN_SEE]) ? '1' : '0') .
				(we_modules_info::isActive(we_modules_info::OBJECT) && we_base_permission::hasPerm([we_base_perm::OBJECTS_CAN_SEE]) ? '1' : '0');

		return [
			'width' => self::WIDTH_LARGE,
			'expanded' => 1,
			'height' => 210,
			'res' => 1,
			'cls' => 'lightCyan',
			'csv' => $shortCuts . ';0;5;00;',
			'dlgHeight' => 435,
			'isResizable' => false
		];
	}

	public static function showDialog(): void{
		[$jsFile, $oSelCls] = self::getDialogPrefs();
		[$sType, $iDate, $iAmountEntries, $sDisplayOpt, $sUsers] = explode(';', we_base_request::string('we_cmd', '', 1));

		$users = $sUsers ? array_filter(explode(',', trim($sUsers, ','))) : [];

		$content = '<table class="default defaultfont" style="width:300px;margin-bottom:2px;">
<colgroup><col style="width:20px;"/><col style="width:254px;"/><col style="width:26px;"/></colgroup>';

		if(we_base_permission::hasPerm([we_base_perm::WIDGET_EDIT_MFD]) && $users){
			$db = DB_WE::$inst;
			$all = $db->getAllq('SELECT ID,Path,(IF(IsFolder,"' . we_base_ContentTypes::USERGROUP . '",(IF(Alias>0,"' . we_base_ContentTypes::USERALIAS . '","' . we_base_ContentTypes::USER . '")))) AS ContentType FROM ' . USER_TABLE . ' WHERE ID IN (' . implode(',', $users) . ')');
			foreach($all as $rec){
				$content .= '<tr><td class="mfdUIcon" data-contenttype="' . $rec['ContentType'] . '"></td><td>' . $rec['Path'] . '</td><td>' . we_html_button::create_button(we_html_button::TRASH, "javascript:delUser('" . $rec['ID'] . "');") . '</td></tr>';
			}
		} else {
			$content .= '<tr><td class="mfdUIcon" data-contenttype="' . we_base_ContentTypes::USERGROUP . '"></td><td>' . (we_base_permission::hasPerm([we_base_perm::WIDGET_EDIT_MFD]) ? g_l('cockpit', '[all_users]') : $_SESSION['user']['Username']) . '</td><td></td><td></td></tr>';
		}
		$content .= '</table>';

		$sUsrContent = we_html_element::div(['class' => 'multichooserDiv', 'style' => 'width:300px'],
						we_html_element::div(['class' => 'multichooser'], $content) .
						we_html_element::hiddens([
							'UserNameTmp' => '',
							'UserIDTmp' => ''
						]) .
						(we_base_permission::hasPerm([we_base_perm::WIDGET_EDIT_MFD]) ? we_html_element::div(['class' => 'button'],
								we_html_button::create_button(we_html_button::DELETE_ALL, "javascript:delUser(-1)", '', "", ($users ? false : true)) .
								we_html_button::create_button(we_html_button::ADD, "javascript:getUser('we_users_selector',{targetName:'UserNameTmp',targetID:'UserIDTmp',jsCmd:'addUserToField'});")
						) : '')
		);

		$oShowUser = we_html_tools::formElementTable(g_l('cockpit', '[following_users]'), $sUsrContent);

// Typ block
		while(strlen($sType) < 4){
			$sType .= '0';
		}
		if($sType === '0000'){
			$sType = '1111';
		}

		$oChbxDocs = (we_base_permission::hasPerm([we_base_perm::WORK_DOCUMENTS_CAN_SEE]) ?
				we_html_forms::checkbox('1', (bool) ($sType[0]), "chbx_type", g_l('cockpit', '[documents]'), true, "", "", !(we_base_permission::hasPerm([we_base_perm::WORK_DOCUMENTS_CAN_SEE])), '', 0, 0) :
				'<input type="hidden" name="chbx_type" value="0"/>');
		$oChbxTmpl = (we_base_permission::hasPerm([we_base_perm::WORK_TEMPLATES_CAN_SEE]) && we_editor_functions::isNormalMode() ?
				we_html_forms::checkbox('1', (bool) ($sType[1]), "chbx_type", g_l('cockpit', '[templates]'), true, "", "", !(we_base_permission::hasPerm([we_base_perm::WORK_TEMPLATES_CAN_SEE])), "", 0, 0) :
				'<input type="hidden" name="chbx_type" value="0"/>'); //FIXME: this is needed for getBinary!
		$oChbxObjs = (we_base_permission::hasPerm([we_base_perm::OBJECTFILES_CAN_SEE]) ?
				we_html_forms::checkbox('1', (bool) ($sType[2]), "chbx_type", g_l('cockpit', '[objects]'), true, "", "", !(we_modules_info::isActive(we_modules_info::OBJECT) && we_base_permission::hasPerm([
							we_base_perm::OBJECTFILES_CAN_SEE])), "", 0, 0) :
				'<input type="hidden" name="chbx_type" value="0"/>');
		$oChbxCls = (we_base_permission::hasPerm([we_base_perm::OBJECTS_CAN_SEE]) && we_editor_functions::isNormalMode() ?
				we_html_forms::checkbox('1', (bool) ($sType[3]), "chbx_type", g_l('cockpit', '[classes]'), true, "", "", !(we_modules_info::isActive(we_modules_info::OBJECT) && we_base_permission::hasPerm([
							we_base_perm::OBJECTS_CAN_SEE])), "", 0, 0) :
				'<input type="hidden" name="chbx_type" value="0"/>');

		$oDbTableType = $oChbxDocs . $oChbxTmpl . $oChbxObjs . $oChbxCls;

		$oSctDate = new we_html_select(['name' => 'sct_date']);
		$aLangDate = [
			g_l('cockpit', '[all]'),
			g_l('cockpit', '[today]'),
			g_l('cockpit', '[last_week]'),
			g_l('cockpit', '[last_month]'),
			g_l('cockpit', '[last_year]')
		];
		foreach($aLangDate as $k => $v){
			$oSctDate->insertOption($k, (string) ($k), $v);
		}
		$oSctDate->selectOption($iDate);

		$oChbxShowMfdBy = we_html_forms::checkbox('0', (bool) ($sDisplayOpt[0]), 'chbx_display_opt', g_l('cockpit', '[modified_by]'), true, "", "", false, "", we_html_tools::TYPE_NONE, 0);
		$oChbxShowDate = we_html_forms::checkbox('0', (bool) ($sDisplayOpt[1]), 'chbx_display_opt', g_l('cockpit', '[date_last_modification]'), true, "", "", false, "", we_html_tools::TYPE_NONE, 0);
		$oSctNumEntries = (new we_html_select(['name' => "sct_amount_entries", 'class' => 'defaultfont']))
				->insertOption(0, '0', g_l('cockpit', '[all]'));
		for($iCurrEntry = 1; $iCurrEntry <= 50; $iCurrEntry++){
			$oSctNumEntries->insertOption($iCurrEntry, (string) ($iCurrEntry), (string) ($iCurrEntry));
			if($iCurrEntry >= 10){
				$iCurrEntry += ($iCurrEntry == 25) ? 24 : 4;
			}
		}
		$oSctNumEntries->selectOption($iAmountEntries);

		$oSelMaxEntries = new we_html_table(['height' => "100%", 'class' => 'default'], 1, 3);
		$oSelMaxEntries->setCol(0, 0, g_l('cockpit', '[max_amount_entries]'), ['class' => 'defaultfont', 'style' => 'vertical-align:middle;padding-right:5px;']);
		$oSelMaxEntries->setCol(0, 2, $oSctNumEntries->getHTML(), ['style' => 'vertical-align:middle;']);

		$show = $oSelMaxEntries->getHTML() . $oChbxShowMfdBy . $oChbxShowDate . we_html_element::br() . $oShowUser;

		$parts = (new we_html_multiIconBoxContent())
				->add((new we_html_multiIconBoxContentLine(g_l('cockpit', '[type]'), $oDbTableType))
						->space(we_html_multiIconBox::SPACE_MED)
				)
				->add((new we_html_multiIconBoxContentLine(g_l('cockpit', '[date]'), $oSctDate->getHTML()))
						->space(we_html_multiIconBox::SPACE_MED)
				)
				->add((new we_html_multiIconBoxContentLine(g_l('cockpit', '[display]'), $show))
						->space(we_html_multiIconBox::SPACE_MED)
				)
				->add(new we_html_multiIconBoxContentLine('', $oSelCls->getHTML()))
		;

		$save_button = we_html_button::create_button(we_html_button::SAVE, "javascript:save();");
		$preview_button = we_html_button::create_button(we_html_button::PREVIEW, "javascript:preview();");
		$cancel_button = we_html_button::create_button(we_html_button::CLOSE, "javascript:exit_close();");
		$buttons = we_html_button::position_yes_no_cancel($save_button, $preview_button, $cancel_button);

		$sTblWidget = (new we_html_multiIconBox($parts, 'mfdProps'))
				->marginLeft()
				->buttons($buttons)
				->headline(g_l('cockpit', '[last_modified]'))
				->get();

		echo we_html_tools::getHtmlTop(g_l('cockpit', '[last_modified]'), $jsFile .
				we_html_element::jsScript(JS_DIR . 'widgets/mfd.js', '', ['id' => 'loadVarWidget', 'data-widget' => setDynamicVar([
						'sUsers' => $sUsers
			])]), we_html_element::body(
						['class' => 'weDialogBody', 'onload' => "init();"], we_html_element::form([], $sTblWidget)));
	}

	public function showPreview(): void{
		$jsCmd = new we_base_jsCmd();
		$jsCmd->addCmd('initPreview', [
			'id' => we_base_request::string('we_cmd', '', 5),
			'type' => 'mfd',
			'tb' => g_l('cockpit', '[last_modified]'),
		]);
		$jsCmd->addCmd('setIconOfDocClass', 'mfdIcon');

		echo we_html_tools::getHtmlTop(g_l('cockpit', '[last_modified]'), $jsCmd->getCmds(), we_html_element::body(
						['style' => 'margin:10px 15px;',
							'onload' => "init();"
						], we_html_element::div(['id' => 'mfd'], we_html_element::div(['id' => 'mfd_data'], $this->lastModified)
		)));
	}

	public function getLastModified(): string{
		return $this->lastModified;
	}

}
