/* global WE,CodeMirror */
/**
 * webEdition CMS
 *
 * $Rev: 6329b796be62 $
 * $Author: Lukas Imhof $
 * $Date: 2020-11-10 16:09 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
'use strict';

(function () {
	CodeMirror.weHint = function (cm, simbol) {

		if (simbol.length > 0) {
			var cursor = cm.getCursor();
			cm.replaceSelection(simbol);
			cursor = {
				line: cursor.line,
				ch: cursor.ch + 1
			};
			cm.setCursor(cursor);
		}

		CodeMirror.showHint(cm, getHint, {closeCharacters: /[()\[\]{};>,]/});
	};

	var getHint = function (cm) {
		var cursor = cm.getCursor(), i;
		if (cursor.ch > 0) {
			var
				text = cm.getRange({
					line: 0,
					ch: 0
				}, cursor),
				typed = '',
				simbol = '';
			for (i = text.length - 1; i >= 0; i--) {
				if (text[i] === ' ' || text[i] === '<') {
					simbol = text[i];
					break;
				} else {
					typed = text[i] + typed;
				}
			}

			text = text.slice(0, text.length - typed.length);

			var
				path = getOpenTag(text) + simbol,
				hints = WE().layout.editors.CodeMirror.weHints[path];

			//console.log(path+" "+hints);

			if (typeof hints === 'undefined') {
				hints = [''];
			} else {
				hints = hints.slice(0);
				for (i = hints.length - 1; i >= 0; i--) {
					if (hints[i].indexOf(typed) !== 0) {
						hints.splice(i, 1);
					}
				}
			}

			return {
				list: hints,
				from: {
					line: cursor.line,
					ch: cursor.ch - typed.length
				},
				to: cursor
			};
		}
	};


	var getOpenTag = function (text) {
		var
			open = text.lastIndexOf('<'),
			close = text.lastIndexOf('>');

		if (close < open) {
			text = text.slice(open);

			if (text !== '<') {
				var space = text.indexOf(' ');
				if (space < 0) {
					space = text.indexOf('\t');
				}
				if (space < 0) {
					space = text.indexOf('\n');
				}
				if (space < 0) {
					space = text.length;
				}
				return text.slice(0, space);
			}
		}

		return '';
	};

})();