/* global WE, top */

/**
 * webEdition CMS
 *
 * $Rev: 32377a47fa92 $
 * $Author: Lukas Imhof $
 * $Date: 2021-08-30 15:36 +0200 $
 *
 * This source is part of the webEdition SDK. The webEdition SDK is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU Lesser General Public License can be found at
 * http://www.gnu.org/licenses/lgpl-3.0.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionSDK/License.txt
 *
 *
 * @license    http://www.gnu.org/licenses/lgpl-3.0.html  LGPL
 */
'use strict';
var image = WE().util.getDynamicVar(document, 'loadVarDialogImage', 'data-image');

function thumbnailChanged() {
	document.we_form.we_what.value = 'cmd';
	document.we_form['we_cmd[task]'].value = 'changeThumbnail';
	document.we_form.submit();
	resetWecmd();
}

function imageChanged() {
	document.we_form.we_what.value = 'cmd';
	document.we_form['we_cmd[task]'].value = 'changeImage';
	document.we_form.submit();
	resetWecmd();
}

function typeChanged() {
	document.we_form.we_what.value = 'cmd';
	document.we_form['we_cmd[task]'].value = 'changeType';
	document.we_form.submit();
	resetWecmd();
}

function resetWecmd() {
	document.we_form.we_what.value = 'cmd';
	document.we_form['we_cmd[task]'].value = 'writeBack';
}

function extImg_doOnchange(input) {
	if (input.value === '' || input.value === WE().consts.linkPrefix.EMPTY_EXT) {
		WE().layout.button.switch_button_state(document, 'btn_edit_ext', 'disabled');
	} else {
		WE().layout.button.switch_button_state(document, 'btn_edit_ext', 'enabled');
	}
	imageChanged();
}

function update_dialog(data) {
	document.we_form['we_cmd[0]'].value = 'open_dialog_image';

	var inputElem;
	for (var arg in data.args) {
		if (data.args.hasOwnProperty(arg) && arg !== 'cssclass' && arg !== 'width' && arg !== 'height') {
			if ((inputElem = document.we_form.elements['we_dialog_args[' + arg + ']'])) {
				inputElem.value = data.args[arg];
			}
		}
	}
	processDimension(data.task);

	try {
		if (data.displayThumbnailSel === 'none') {
			WE().util.dialogTop(window).document.getElementById('selectThumbnail').setAttribute('disabled', 'disabled');
		} else {
			WE().util.dialogTop(window).document.getElementById('selectThumbnail').removeAttribute('disabled');
		}
	} catch (err) {}
}

function processDimension(task){
	if(parseInt(document.we_form.elements['we_dialog_args[thumbnailID]'].value) && task !== 'changeType'){
		processDimensionThumbnail(task);
		return;
	}
	processDimensionStandard(task);
}

function processDimensionStandard(task){
	var checkWidth = document.we_form.elements['dimension_width'],
		checkHeight = document.we_form.elements['dimension_height'],
		elemWidth = document.we_form.elements['we_dialog_args[width]'],
		elemHeight = document.we_form.elements['we_dialog_args[height]'],
		elemUnit = document.we_form.elements['we_dialog_args[unit]'];

	var imgWidth = parseInt(document.we_form.elements['we_dialog_args[imgWidth]'].value),
		imgHeight = parseInt(document.we_form.elements['we_dialog_args[imgHeight]'].value),
		dimensionState = document.we_form.elements['we_dialog_args[dimensionState]'].value,
		newState = '',
		isSvg = parseInt(document.we_form.elements['we_dialog_args[isSvg]'].value) > 0,
		type = document.we_form.elements['we_dialog_args[type]'].value;

	document.getElementById('span_imgWidth').innerHTML = imgWidth;
	document.getElementById('span_imgHeight').innerHTML = imgHeight;
	elemWidth.readOnly = false;
	elemHeight.readOnly = false;

	switch(task){
		case 'openDialog':
			selectDimension('', false, dimensionState);
			break;
		case 'changeType':
			elemWidth.value = '';
			elemHeight.value = '';
			checkWidth.disabled = false,
			checkHeight.disabled = false,
			elemUnit.disabled = false;
			selectDimension('', false, dimensionState);

			if(type === 'int'){
				processDimension('changeImage');
			}
			break;
		case 'changeThumbnail':
			checkWidth.checked = true;
			checkWidth.disabled = false;
			elemWidth.value = '';
			elemWidth.disabled = false;
			checkHeight.checked = false;
			checkHeight.disabled = false;
			elemHeight.value = '';
			elemHeight.disabled = true;
			elemUnit.value = 'px';
			elemUnit.disabled = false;

			if(isSvg){
				elemWidth.value = imgWidth;
				elemHeight.value = imgHeight;
			}
			break;
		case 'changeImage':
			if(isSvg){
				elemWidth.value = elemWidth.value ? elemWidth.value : imgWidth;
				elemHeight.value = /*elemWidth.value ? '' : */(elemHeight.value ? elemHeight.value : imgHeight);
				newState = dimensionState === 'height' ? 'height' : 'width';
				break;
			}

			if(dimensionState === 'none'){
				elemWidth.value = '';
				elemHeight.value = '';
				selectDimension('', false, 'none');
				break;
			}

			if(elemUnit.value === 'px'){
				switch(dimensionState){
					case 'width':
						newState = 'width';
						if(parseInt(elemWidth.value) === imgWidth){
							elemWidth.value = '';
							newState = 'none';
						}
						break;
					case 'height':
						newState = 'height';
						if(parseInt(elemHeight.value) === imgHeight){
							elemHeight.value = '';
							newState = 'none';
						}
						break;
				}
			} else {
				newState = checkHeight.checked ? 'height' : 'width';
			}
			selectDimension('', false, newState);
			break;
	}
}

function processDimensionThumbnail(){
	var checkWidth = document.we_form.elements['dimension_width'],
		checkHeight = document.we_form.elements['dimension_height'],
		elemWidth = document.we_form.elements['we_dialog_args[width]'],
		elemHeight = document.we_form.elements['we_dialog_args[height]'],
		elemUnit = document.we_form.elements['we_dialog_args[unit]'],
		imgWidth = parseInt(document.we_form.elements['we_dialog_args[imgWidth]'].value),
		imgHeight = parseInt(document.we_form.elements['we_dialog_args[imgHeight]'].value),
		isSvg = parseInt(document.we_form.elements['we_dialog_args[isSvg]'].value) > 0;

	document.getElementById('span_imgWidth').innerHTML = imgWidth;
	document.getElementById('span_imgHeight').innerHTML = imgHeight;

	checkWidth.checked = false;
	checkWidth.disabled = true;
	elemWidth.value = '';
	elemWidth.disabled = true;
	elemWidth.readOnly = true;

	checkHeight.checked = false;
	checkHeight.disabled = true;
	elemHeight.value = '';
	elemHeight.disabled = true;
	elemHeight.readOnly = true;

	elemUnit.value = 'px';
	elemUnit.disabled = true;

	if(isSvg){
		if(imgWidth){
			checkWidth.checked = true;
			elemWidth.value = imgWidth ? imgWidth : '';
			elemWidth.disabled = false;
		}
		if(imgHeight){
			checkHeight.checked = true;
			elemHeight.value = imgHeight ? imgHeight : '';
			elemHeight.disabled = false;
		}
	}
}

function selectDimension(field, checked, state){
	var type = document.we_form.elements['we_dialog_args[type]'].value;

	if(parseInt(document.we_form.elements['we_dialog_args[thumbnailID]'].value) && type === 'int'){
		return;
	}

	var checkWidth = document.we_form.elements['dimension_width'],
		checkHeight = document.we_form.elements['dimension_height'],
		elemWidth = document.we_form.elements['we_dialog_args[width]'],
		elemHeight = document.we_form.elements['we_dialog_args[height]'],
		elemState = document.we_form.elements['we_dialog_args[dimensionState]'],
		isSvgNoWidthAndHeight = parseInt(document.we_form.elements['we_dialog_args[isSvg]'].value) > 0 &&
				!parseInt(document.we_form.elements['we_dialog_args[imgWidth]']) &&
				!parseInt(document.we_form.elements['we_dialog_args[imgHeight]']);

	if(!state){
		switch(field){
			case 'width':
				state = checked || isSvgNoWidthAndHeight ? 'width' : 'none';
				break;
			case 'height':
				state = checked || isSvgNoWidthAndHeight ? 'height' : 'none';
				break;
		}
	}
	elemState.value = state;

	switch(state){
		case 'width':
			checkWidth.checked = true;
			checkHeight.checked = false;
			elemWidth.disabled = false;
			elemHeight.disabled = true;
			break;
		case 'height':
			checkWidth.checked = false;
			checkHeight.checked = true;
			elemWidth.disabled = true;
			elemHeight.disabled = false;
			break;
		case 'none':
			checkWidth.checked = false;
			checkHeight.checked = false;
			elemWidth.disabled = true;
			elemHeight.disabled = true;
	}
}

function changeTypeSelect(s) {
	var elem = document.getElementsByClassName('changetype');
	for (var i = 0; i < elem.length; i++) {
		if (elem[i].classList.contains(s + '_row')) {
			elem[i].classList.remove('hidden');
		} else {
			elem[i].classList.add('hidden');
		}
	}
}

function we_cmd() {
	/*jshint validthis:true */
	var caller = (this && this.window === this ? this : window),
		args = WE().util.getWe_cmdArgsArray(Array.prototype.slice.call(arguments)),
		url = WE().util.getWe_cmdArgsUrl(args);

	switch (args[0]) {
		case 'we_selector_document':
		case 'we_selector_image':
		case 'we_selector_directory':
			(new (WE().layout.overlay)(caller, 'top', url, "we_fileselector", WE().consts.size.dialog.big, WE().consts.size.dialog.medium)).setResizeable().open();
			break;
		case "we_fileupload_editor":
			(new (WE().layout.overlay)(caller, 'top', url, "we_fileupload_editor", WE().consts.size.dialog.small, WE().consts.size.dialog.big)).setResizeable().open();
			break;
		case "dialog_setType":
			changeTypeSelect(args[1]);
			typeChanged();
			break;
		case "dialog_emptyLongdesc":
			caller.document.we_form.elements['we_dialog_args[longdescID]'].value = '';
			caller.document.we_form.elements['we_dialog_args[longdescSrc]'].value = '';
			break;
		case "dialog_imageChanged":
			imageChanged();
			break;
		case "selectDimension":
			selectDimension(args[1].field, args[1].checked);
			break;
		case "changeWidthHeight":
			break;
			var imgValue = parseInt(caller.document.we_form.elements['we_dialog_args[img' + (args[1].field === 'width' ? 'Width' : 'Height') + ']'].value);
			if(caller.document.we_form.elements['we_dialog_args[unit]'].value === 'px' && parseInt(args[1].value) === imgValue){
				caller.document.we_form.elements['we_dialog_args[' + args[1].field + ']'].disabled = true;
				caller.document.we_form.elements['we_dialog_args[dimension]'][(args[1].field === 'width' ? 0 : 1)].checked = false;
			}
			break;
		default :
			WE().util.dialogTop(window).opener.we_cmd.apply(caller, Array.prototype.slice.call(arguments));
			break;
	}
}
