/* global WE, top */

/**
 * webEdition CMS
 *
 * $Rev: 0b80d828f85e $
 * $Author: Lukas Imhof $
 * $Date: 2021-11-06 23:04 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
'use strict';
WE().util.loadConsts(document, 'g_l.import');

function weGetCategories(obj, cats, part) {
	//FIXME: why do we send so much data, if the data is gathered from obj???
	WE().util.rpc(WE().consts.dirs.WEBEDITION_DIR + 'rpc.php?cmd=categoryGet', 'obj=' + obj + '&cats=' + cats + '&part=' + part + '&targetId=docCatTable&catfield=v[' + obj + 'Categories]', function (weResponse) {
		var elements = weResponse.DataArray.elements;
		for (var property in elements) {
			if (elements.hasOwnProperty(property)) {
				top.wizbody.document.getElementById(property).innerHTML = elements[property].innerHTML;
			}
		}

		WE().util.setIconOfDocClass(top.wizbody.document, "chooserFileIcon");
	});
}

function wiz_next(frm, url) {
	window[frm].location.href = url;
}

function we_cmd() {
	/*jshint validthis:true */
	var caller = (this && this.window === this ? this : window),
		args = WE().util.getWe_cmdArgsArray(Array.prototype.slice.call(arguments)),
		url = WE().util.getWe_cmdArgsUrl(args),
		cats, found, i;

	switch (args[0]) {
		case 'we_selector_directory':
		case 'we_selector_image':
		case "we_selector_document":
		case 'we_selector_file':
			(new (WE().layout.overlay)(caller, 'top', url, "we_fileselector", WE().consts.size.dialog.big, WE().consts.size.dialog.medium)).setResizeable().open();
			break;
		case "we_navigation_dirSelector":
			(new (WE().layout.overlay)(caller, 'top', url, "we_navigation_dirselector", WE().consts.size.dialog.small, WE().consts.size.dialog.smaller)).setResizeable().setTitle(WE().consts.g_l.dialogTitle.dirSelector).open();
			break;
		case 'browse_server':
			(new (WE().layout.overlay)(caller, 'top', url, "browse_server", WE().consts.size.dialog.big, WE().consts.size.dialog.medium)).setResizeable().open();
			break;
		case 'add_docCat':
			if (WE().consts.modules.active.indexOf("object") > 0) {
				caller.wizbody.document.we_form.elements['v[import_type]'][0].checked = true;
			}
			caller.we_cmd('add_Cat', args[1], 'doc');
			break;
		case 'add_objCat':
			caller.wizbody.document.we_form.elements['v[import_type]'][1].checked = true;
			caller.we_cmd('add_Cat', args[1], 'obj');
			break;
		case 'add_Cat':
			found = false;
			cats = args[1].allIDs;
			for (i = 0; i < cats.length; i++) {
				if (cats[i] && (caller.wizbody.document.we_form.elements['v[' + args[2] + 'Categories]'].value.indexOf(',' + cats[i] + ',') === -1)) {
					found = true;
					if (caller.wizbody.document.we_form.elements['v[' + args[2] + 'Categories]'].value) {
						caller.wizbody.document.we_form.elements['v[' + args[2] + 'Categories]'].value = caller.wizbody.document.we_form.elements['v[' + args[2] + 'Categories]'].value + cats[i] + ',';
					} else {
						caller.wizbody.document.we_form.elements['v[' + args[2] + 'Categories]'].value = ',' + cats[i] + ',';
					}
				}
				if (found) {
					window.setTimeout(weGetCategories, 100, args[2], caller.wizbody.document.we_form.elements['v[' + args[2] + 'Categories]'].value, 'rows');
				}
			}
			break;
		case 'delete_docCat':
			caller.we_cmd('delete_Cat', 'doc', args[1], false);
			break;
		case 'delete_objCat':
			caller.we_cmd('delete_Cat', 'obj', args[1], false);
			break;
		case 'delete_Cat':
			var obj = args[1],
				cat = args[2],
				reload = !args[3];

			if (caller.wizbody.document.we_form.elements['v[' + obj + 'Categories]'].value.indexOf(',' + cat + ',') !== -1) {
				if (caller.wizbody.document.we_form.elements['v[' + obj + 'Categories]'].value) {
					var re = new RegExp(',' + cat + ',');
					caller.wizbody.document.we_form.elements['v[' + obj + 'Categories]'].value = caller.wizbody.document.we_form.elements['v[' + obj + 'Categories]'].value.replace(re, ',');
					if (!reload) {
						caller.wizbody.document.getElementById(obj + 'Cat' + cat).parentNode.removeChild(caller.wizbody.document.getElementById(obj + 'Cat' + cat));
					}
					if (caller.wizbody.document.we_form.elements['v[' + obj + 'Categories]'].value === ',') {
						caller.wizbody.document.we_form.elements['v[' + obj + 'Categories]'].value = '';
						if (!reload) {
							caller.wizbody.document.getElementById(obj + 'CatTable').innerHTML = '<tr><td style="font-size:8px">&nbsp;</td></tr>';
						}
					}
				}

				if (reload) {
					we_submit_form(WE().util.dialogTop(window).wizbody.document.we_form, 'wizbody', WE().consts.dirs.WEBEDITION_DIR + 'we_cmd.php?we_cmd[0]=import');
				}
			}
			break;
		case 'setFormField':
			// args[1] is not used so we can use this command as selector callback
			setFormField(args[2], args[3], args[4], args[5], args[6]);
			break;
		case 'reload_hot_editpage':
		case 'reload_editpage':
			break;
		case 'handle_event':
			WE().util.dialogTop(window).handleEvent(args[1]);
			break;
		case 'confirm_start_recoverBackup':
			//FIXME: command yes/no missing!
			WE().util.showConfirm(window, "", WE().consts.g_l.import.backup_file_found + ' \n\n' + WE().consts.g_l.import.backup_file_found_question, ["recover_backup"]);
			break;
		case "recover_backup":
			WE().util.dialogTop(window).doClose();
			WE().util.dialogTop(window).opener.top.we_cmd("recover_backup");
			break;
		case 'fileupload_callback':
			WE().util.dialogTop(window).handleEvent('next');
			break;
		case 'fileupload_doOnFileSelect':
			WE().util.dialogTop(window).setFormField('v[rdofloc]', true, 'radio', 1);
			break;
		case 'cycle':
			WE().util.dialogTop(window).cycle();
			break;
		case 'we_import':
			WE().util.dialogTop(window).we_import(args[1], args[2], args[3]);
			break;
		case 'set_button_state':
			WE().util.dialogTop(window).set_button_state();
			break;
		case 'finish':
			WE().util.dialogTop(window).finish(args[1]);
			break;
		case 'doOnImportFinished':
			WE().util.dialogTop(window).doOnImportFinished(args[1].progressText);
			break;
		case 'addLog_buffered':
			if (!args[1]) {
				return;
			}

			for (i = 0; i < args[1].length; i++) {
				WE().util.dialogTop(window).addLog(args[1][i]);
			}
			break;
		case 'setProgress_footer':
			WE().util.dialogTop(window).wizbusy.setProgress(Math.floor(args[1]));
			break;
		case 'setProgressText_footer':
			WE().util.dialogTop(window).wizbusy.setProgressText(args[1], args[2]);
			break;
		case 'call_delayed':
			if (args[1]['function'] && args[1].delay) {
				setTimeout(we_cmd, args[1].delay, args[1].function, args[1].param_1, args[1].param_2, args[1].param_3);
			}
			break;
		case 'close':
			WE().util.dialogTop(window).doClose();
			break;
		default:
			WE().util.dialogTop(window).opener.top.we_cmd.apply(caller, Array.prototype.slice.call(arguments));
	}
}

function deleteCategory(obj, cat) {
	WE().util.dialogTop(window).we_cmd('delete_Cat', obj, cat, false);
}

function we_submit_form(f, target, url) {
	f.target = target;
	f.action = url;
	f.method = 'post';
	f.submit();
}

function toggle(doc, name) {
	var con = doc.getElementById(name);
	con.style.display = (con.style.display === "none" ? "" : "none");
}

function addLog(text) {
	WE().util.dialogTop(window).wizbody.log.innerHTML += text + "<br/>";
	WE().util.dialogTop(window).wizbody.log.scrollTop = 50000;
}

function set_button_state() {
	var f = WE().util.dialogTop(window).wizbody.document.we_form;

// FIXME: something is wrong...
//	WE().layout.button.switch_button_state(WE().util.dialogTop(window).wizbusy.document, 'back', f.elements['v[btnState_back]'].value);
//	WE().layout.button.switch_button_state(WE().util.dialogTop(window).wizbusy.document, 'next', f.elements['v[btnState_next]'].value);
}

function weChangeDocType(f) {
	WE().util.rpc(WE().consts.dirs.WEBEDITION_DIR + 'rpc.php?cmd=documentChangeDocType', 'docType=' + f.value, function (weResponse) {
		var node, prop,
			elems = weResponse.DataArray.elements;

		for (var i = 0; i < elems.length; i++) {
			if ((node = elems[i].type === 'formelement' ? top.wizbody.document.we_form.elements[elems[i].name] : top.wizbody.document.getElementById(elems[i].name))) {
				for (var j = 0; j < elems[i].props.length; j++) {
					prop = elems[i].props[j];
					switch (prop.type) {
						case 'attrib':
							node.setAttribute(prop.name, prop.val);
							break;
						case 'style':
							node.style[prop.name] = prop.val;
							break;
						case 'innerHTML':
							node.innerHTML = prop.val;
							break;
					}
				}
			}
		}
		switchExt();
	});
}

function switchExt() {
	var a = WE().util.dialogTop(window).wizbody.document.we_form.elements;
	a['v[we_Extension]'].value = (a['v[is_dynamic]'].value == 1 ? WE().consts.global.DEFAULT_DYNAMIC_EXT : WE().consts.global.DEFAULT_STATIC_EXT);
}

function setFormField(name, value, type, index, frame) {
	frame = frame ? frame : 'wizbody';
	var f = frame === 'top' ? WE().util.dialogTop(window).document.we_form : WE().util.dialogTop(window).frames[frame].document.we_form;
	var field = f.elements[name];

	if (index !== undefined) {
		field = field[index];
	}

	if (type === 'checkbox' || type === 'radio') {
		field.checked = value;
	} else {
		field.value = value;
	}
}

function handleEvent(evt) {
	var step = parseInt(WE().util.dialogTop(window).wizbody.document.we_form.elements.step.value);
	var type = WE().util.dialogTop(window).wizbody.document.we_form.elements.type.value;
	var task = 'get' + type + '_step_' + step;

	switch (task) {
		case 'getFileImport_step_1':
		case 'getWE_step_1':
		case 'getXML_step_1':
		case 'getCSV_step_1':
			handleEvent_step_1(evt, type);
			break;
		case 'getFileImport_step_2':
			handleEvent_FileImport_step_2(evt);
			break;
		case 'getFileImport_step_3':
			break;
		case 'getWE_step_2':
			handleEvent_WE_step_2(evt);
			break;
		case 'getWE_step_3':
			handleEvent_WE_step_3(evt);
			break;
		case 'getXML_step_2':
			handleEvent_XML_step_2(evt);
			break;
		case 'getXML_step_3':
			handleEvent_XML_step_3(evt);
			break;
		case 'getCSV_step_2':
			handleEvent_CSVStep_2(evt);
			break;
		case 'getCSV_step_3':
			handleEvent_CSVStep_3(evt);
			break;
		default:
			if (step === 0) {
				handleEvent_step_0(evt);
			}
			//
	}
}


function handleEvent_step_0(evt) {
	var f = WE().util.dialogTop(window).wizbody.document.we_form;
	switch (evt) {
		case 'previous':
			break;
		case 'next':
			for (var i = 0; i < f.type.length; i++) {
				if (f.type[i].checked) {
					switch (f.type[i].value) {
						case 'FileImport':
							WE().util.dialogTop(window).location.href = WE().consts.dirs.WEBEDITION_DIR + 'we_cmd.php?we_cmd[0]=import_files';
							break;
						case 'siteImport':
							WE().util.dialogTop(window).location.href = WE().consts.dirs.WEBEDITION_DIR + 'we_cmd.php?we_cmd[0]=siteImport';
							break;
						default:
							f.type.value = f.type[i].value;
							f.step.value = 1;
							f.mode.value = 0;
							f.target = 'wizbody';
							f.action = WE().consts.dirs.WEBEDITION_DIR + 'we_cmd.php?we_cmd[0]=import';
							f.method = 'post';
							f.submit();
							break;
					}
				}
			}
			break;
		case 'cancel':
			WE().util.dialogTop(window).doClose();
			break;
	}
}

function handleEvent_step_1(evt, type) {
	var f = WE().util.dialogTop(window).wizbody.document.we_form;

	if (type === 'XML') {
		f.elements['v[we_TemplateID]'].value = (f.elements['v[docType]'].value == -1 ? f.elements.noDocTypeTemplateId.value :
			f.elements.docTypeTemplateId.value);
	}

	switch (evt) {
		case 'previous':
			f.step.value = 0;
			WE().util.dialogTop(window).location.href = WE().consts.dirs.WEBEDITION_DIR + 'we_cmd.php?we_cmd[0]=import&we_cmd[1]=' + type;
			break;
		case 'next':
			switch (type) {
				case 'WE':
				case 'XML':
				case 'CSV':
					doNext_step1(type);
					break;
				case 'FileImport':
					doNext_FileImportStep1();
					break;
			}
			break;
		case 'cancel':
			WE().util.dialogTop(window).doClose();
			break;
	}

	WE().util.dialogTop(window).wizbusy.back_enabled = WE().layout.button.switch_button_state(WE().util.dialogTop(window).wizbusy.document, 'back', 'enabled');
	WE().util.dialogTop(window).wizbusy.next_enabled = WE().layout.button.switch_button_state(WE().util.dialogTop(window).wizbusy.document, 'next', 'enabled');
}

function handleEvent_FileImport_step_2(evt) {
	var frame = WE().util.dialogTop(window).wizbody,
		form = WE().util.dialogTop(window).wizbody.document.we_form;

	switch (evt) {
		case 'previous':
			var urlParams = '',
				params = {
				fu_file_parentID: form.elements.fu_file_parentID.value,
				fu_file_parentFixed: form.elements.fu_file_parentFixed.value,
				fu_file_sameName: form.elements.fu_file_sameName.value,
				fu_doc_thumbs: form.elements.fu_doc_thumbs.value,
				fu_doc_categories: form.elements.fu_doc_categories.value,
				fu_doc_isSearchable: form.elements.fu_doc_isSearchable.value,
				fu_doc_importMetadata: form.elements.fu_doc_importMetadata.value,
				fu_doc_importDirs: form.elements.fu_doc_importDirs.value
			};
			for (var param in params) {
				if(params.hasOwnProperty(param)){
					urlParams += "&" + param + "=" + params[param];
				}
			}
			WE().util.dialogTop(window).location.href = WE().consts.dirs.WEBEDITION_DIR + 'we_cmd.php?we_cmd[0]=import_files&we_cmd[isPrev]=1' + urlParams;
			break;
		case 'next':
			// do nothing
			break;
		case 'upload':
			if (frame.weFileUpload_instance !== undefined) {
				frame.weFileUpload_instance.startUpload();
			} else {
				WE().t_e('fileupload instance missing');
			}
			break;
		case 'cancel':
			if (frame.weFileUpload_instance !== undefined && frame.weFileUpload_instance.getIsUploading()) {
				frame.weFileUpload_instance.cancelUpload();
			} else {
				WE().util.dialogTop(window).doClose();
			}
			break;
		case 'close':
			WE().util.dialogTop(window).doClose();
			break;
	}
}

function handleEvent_WE_step_2(evt) {
	var we_form = WE().util.dialogTop(window).wizbody.document.we_form;
	switch (evt) {
		case "previous":
			we_form.step.value = 1;
			we_submit_form(we_form, "wizbody", WE().consts.dirs.WEBEDITION_DIR + "we_cmd.php?we_cmd[0]=import");
			break;
		case "next":
			we_form.elements.step.value = 3;
			we_form.mode.value = 1;
			we_form.elements["v[mode]"].value = 1;
			we_submit_form(we_form, "wizbusy", WE().consts.dirs.WEBEDITION_DIR + "we_cmd.php?we_cmd[0]=import&pnt=wizcmd");
			break;
		case "cancel":
			WE().util.dialogTop(window).doClose();
			break;
	}
}

function handleEvent_WE_step_3(evt) {
	switch (evt) {
		case "cancel":
			WE().util.dialogTop(window).doClose();
			break;
	}
}

function handleEvent_XML_step_2(evt) {
	var f = WE().util.dialogTop(window).wizbody.document.we_form;
	switch (evt) {
		case 'previous':
			f.step.value = 1;
			we_submit_form(f, 'wizbody', WE().consts.dirs.WEBEDITION_DIR + 'we_cmd.php?we_cmd[0]=import');
			break;
		case 'next':
			f.elements['v[from_elem]'].value = f.elements['v[from_iElem]'].value;
			f.elements['v[to_elem]'].value = f.elements['v[to_iElem]'].value;
			var iStart = isNaN(parseInt(f.elements['v[from_iElem]'].value)) ? 0 : f.elements['v[from_iElem]'].value;
			var iEnd = isNaN(parseInt(f.elements['v[to_iElem]'].value)) ? 0 : f.elements['v[to_iElem]'].value;
			var iElements = parseInt(f.elements.we_select.options[f.elements.we_select.selectedIndex].value);

			if ((iStart < 1) || (iStart > iElements) || (iEnd < 1) || (iEnd > iElements)) {
				WE().util.showMessage((WE().consts.g_l.import.num_elements + iElements), WE().consts.message.WE_MESSAGE_ERROR, window);
			} else {
				f.elements['v[rcd]'].value = f.we_select.options[f.we_select.selectedIndex].text;
				f.step.value = 3;
				we_submit_form(f, 'wizbody', WE().consts.dirs.WEBEDITION_DIR + 'we_cmd.php?we_cmd[0]=import');
			}
			break;
		case 'cancel':
			WE().util.dialogTop(window).doClose();
			break;
	}
}

function handleEvent_XML_step_3(evt) {
	var f = WE().util.dialogTop(window).wizbody.document.we_form;
	switch (evt) {
		case 'previous':
			f.step.value = 2;
			we_submit_form(f, 'wizbody', WE().consts.dirs.WEBEDITION_DIR + 'we_cmd.php?we_cmd[0]=import');
			break;
		case 'next':
			f.step.value = 3;
			f.mode.value = 1;
			f.elements['v[mode]'].value = 1;
			WE().util.dialogTop(window).wizbusy.next_enabled = WE().layout.button.switch_button_state(WE().util.dialogTop(window).wizbusy.document, 'next', 'disabled');
			we_submit_form(f, 'wizbody', WE().consts.dirs.WEBEDITION_DIR + 'we_cmd.php?we_cmd[0]=import&mode=1');
			break;
		case 'cancel':
			WE().util.dialogTop(window).doClose();
			break;
	}
}

function doNext_FileImportStep1() {
	var frame = WE().util.dialogTop(window).wizbody;

	if (frame.selectCategories !== undefined) {
		frame.selectCategories();
	}
	frame.document.we_form.step.value = 2;
	frame.document.we_form.submit();
}

function doNext_step1(type) {
	var form = WE().util.dialogTop(window).wizbody.document.we_form;

	if (form.elements['v[rdofloc]'][0].checked) { // is import from server
		var file = form.elements['v[fserver]'].value;
		if (file === '/') {
			WE().util.showMessage(WE().consts.g_l.import.select_source_file, WE().consts.message.WE_MESSAGE_ERROR, window);
			return;
		}
		if (file.match(/\.\./) === '..') {
			WE().util.showMessage(WE().consts.g_l.import.invalid_path, WE().consts.message.WE_MESSAGE_ERROR, window);
			return;
		}
		form.elements['v[import_from]'].value = file;
	} else {
		if (WE().util.dialogTop(window).wizbody.weFileUpload_instance.getUploaderState() !== 'finished') {
			WE().util.dialogTop(window).wizbody.weFileUpload_instance.checkStartUpload('fileupload_callback');
			return;
		}
		form.elements['v[import_from]'].value = 'placeholder.xml';
	}

	switch (type) {
		case 'WE':
			form.step.value = 2;
			// timing Problem with Safari
			window.setTimeout(WE().util.dialogTop(window).we_submit_form, 50, WE().util.dialogTop(window).wizbody.document.forms.we_form, 'wizbody', WE().consts.dirs.WEBEDITION_DIR + 'we_cmd.php?we_cmd[0]=import');
			return;
		case 'XML':
			form.elements['v[we_TemplateID]'].value = form.elements['v[docType]'].value == -1 ? form.elements.noDocTypeTemplateId.value : form.elements.docTypeTemplateId.value;

			if (!form.elements['v[we_TemplateID]'].value) {
				form.elements['v[we_TemplateID]'].value = form.elements.noDocTypeTemplateId.value;
			}

			if (WE().consts.modules.active.indexOf("object") !== -1) {
				if ((form.elements['v[import_type]'][0].checked && form.elements['v[we_TemplateID]'].value !== "0") || (form.elements['v[import_type]'][1].checked)) {
					form.step.value = 2;
					WE().util.dialogTop(window).we_submit_form(form, 'wizbody', WE().consts.dirs.WEBEDITION_DIR + 'we_cmd.php?we_cmd[0]=import');
				} else if (form.elements['v[import_type]'][0].checked) {
					WE().util.showMessage(WE().consts.g_l.import.select_docType, WE().consts.message.WE_MESSAGE_ERROR, window);
				}
			} else {
				if (form.elements['v[we_TemplateID]'].value !== "0") {
					form.step.value = 2;
					WE().util.dialogTop(window).we_submit_form(form, 'wizbody', WE().consts.dirs.WEBEDITION_DIR + 'we_cmd.php?we_cmd[0]=import');
				} else {
					WE().util.showMessage(WE().consts.g_l.import.select_docType, WE().consts.message.WE_MESSAGE_ERROR, window);
				}
			}
			return;
		case 'CSV':
			if (form.elements['v[csv_seperator]'].value === '') {
				WE().util.showMessage(WE().consts.g_l.import.select_seperator, WE().consts.message.WE_MESSAGE_ERROR, window);
				return;
			}

			form.step.value = 2;
			WE().util.dialogTop(window).we_submit_form(form, 'wizbody', WE().consts.dirs.WEBEDITION_DIR + 'we_cmd.php?we_cmd[0]=import');
			return;
	}
}

function handleEvent_CSVStep_2(evt) {
	var f = WE().util.dialogTop(window).wizbody.document.we_form;
	if (f.elements['v[import_type]'].value === 'documents') {
		f.elements['v[we_TemplateID]'].value = f.elements['v[docType]'].value == -1 ? f.elements.noDocTypeTemplateId.value : f.elements.docTypeTemplateId.value;
	}

	switch (evt) {
		case 'previous':
			f.step.value = 1;
			we_submit_form(f, 'wizbody', WE().consts.dirs.WEBEDITION_DIR + 'we_cmd.php?we_cmd[0]=import');
			break;
		case 'next':
			if (f.elements['v[import_type]'].value === 'documents') {
				if (!f.elements['v[we_TemplateID]'].value) {
					f.elements['v[we_TemplateID]'].value = f.elements.DocTypeTemplateId.value;
				}
			}
			if (WE().consts.modules.active.indexOf("object") !== -1) {
				if (f.elements['v[import_from]'].value !== '/' && ((f.elements['v[import_type]'][0].checked && f.elements['v[we_TemplateID]'].value !== "0") || (f.elements['v[import_type]'][1].checked))) {
					f.step.value = 3;
					we_submit_form(f, 'wizbody', WE().consts.dirs.WEBEDITION_DIR + 'we_cmd.php?we_cmd[0]=import');
				} else {
					if (f.elements['v[import_from]'].value === '/') {
						WE().util.showMessage(WE().consts.g_l.import.select_source_file, WE().consts.message.WE_MESSAGE_ERROR, window);
					} else if (f.elements['v[import_type]'][0].checked) {
						WE().util.showMessage(WE().consts.g_l.import.select_docType, WE().consts.message.WE_MESSAGE_ERROR, window);
					}
				}
			} else {
				if (f.elements['v[import_from]'].value !== '/' && f.elements['v[we_TemplateID]'].value !== "0") {
					f.step.value = 3;
					we_submit_form(f, 'wizbody', WE().consts.dirs.WEBEDITION_DIR + 'we_cmd.php?we_cmd[0]=import');
				} else {
					if (f.elements['v[import_from]'].value === '/') {
						WE().util.showMessage(WE().consts.g_l.import.select_source_file, WE().consts.message.WE_MESSAGE_ERROR, window);
					} else {
						WE().util.showMessage(WE().consts.g_l.import.select_docType, WE().consts.message.WE_MESSAGE_ERROR, window);
					}
				}
			}
			break;
		case 'cancel':
			WE().util.dialogTop(window).doClose();
			break;
	}
}

function handleEvent_CSVStep_3(evt) {
	var f = WE().util.dialogTop(window).wizbody.document.we_form;
	switch (evt) {
		case 'previous':
			f.step.value = 1;
			we_submit_form(f, 'wizbody', WE().consts.dirs.WEBEDITION_DIR + 'we_cmd.php?we_cmd[0]=import');
			break;
		case 'next':
			f.step.value = 3;
			f.mode.value = 1;
			f.elements['v[mode]'].value = 1;
			f.elements['v[startCSV]'].value = 1;
			WE().util.dialogTop(window).wizbusy.next_enabled = WE().layout.button.switch_button_state(WE().util.dialogTop(window).wizbusy.document, 'next', 'disabled');
			we_submit_form(f, 'wizbody', WE().consts.dirs.WEBEDITION_DIR + 'we_cmd.php?we_cmd[0]=import&mode=1');
			break;
		case 'cancel':
			WE().util.dialogTop(window).doClose();
			break;
	}
}

function onChangeSelectXMLNode(node) {
	node.form.elements['v[to_iElem]'].value = node.options[node.selectedIndex].value;
	node.form.elements['v[from_iElem]'].value = 1;
	node.form.elements['v[sct_node]'].value = node.options[node.selectedIndex].text;
	if (node.options[node.selectedIndex].value == 1) {
		node.form.elements['v[from_iElem]'].disabled = true;
		node.form.elements['v[to_iElem]'].disabled = true;
	} else {
		node.form.elements['v[from_iElem]'].disabled = false;
		node.form.elements['v[to_iElem]'].disabled = false;
	}
}

function onChangeSelectObject(node) {
	var elem = WE().util.dialogTop(window).wizbody.document.we_form.elements['v[classID]'];
	WE().util.dialogTop(window).wizbody.document.we_form.elements['v[obj_path]'].value = '/' + elem.options[elem.selectedIndex].text;
	WE().util.dialogTop(window).wizbody.document.we_form.elements['v[obj_path_id]'].value = WE().util.dialogTop(window).wizbody.document.we_form.elements['v[classID]'].value.split('_')[1];
}

function addField(form, fieldType, fieldName, fieldValue) {
	var input = document.createElement('INPUT');
	input.type = fieldType;
	input.name = fieldName;
	input.value = fieldValue;
	form.appendChild(input);
}

function getField(form, fieldName) {
	return form[fieldName];
}
function removeField(form, fieldName) {
	var field = getField(form, fieldName);
	if (field && !field.length) {
		field.parentNode.removeChild(field);
	}
}
function toggleField(form, fieldName, value) {
	var field = getField(form, fieldName);
	if (field) {
		removeField(form, fieldName);
	} else {
		addField(form, 'hidden', fieldName, value);
	}
}

function finish(rebuild) {
	var std = WE().util.dialogTop(window).wizbusy.document.getElementById('standardDiv');
	if (std !== undefined) {
		std.style.display = 'none';
	}
	var cls = WE().util.dialogTop(window).wizbusy.document.getElementById('closeDiv');
	if (cls !== undefined) {
		cls.style.display = 'block';
	}
	if (rebuild) {
		/* OLD jsWindow */ new (WE().layout.jsWindow)(WE().util.dialogTop(window).opener, WE().consts.dirs.WEBEDITION_DIR + "we_cmd.php?we_cmd[0]=rebuild&step=2&btype=rebuild_all&responseText=' . g_l('import', '[finished_success]') . '", "rebuildwin", WE().consts.size.dialog.small, WE().consts.size.dialog.tiny, false);
	}
}

function cycle() {
	var
		cf = WE().util.dialogTop(window).wizcmd.document.we_form,
		bf = WE().util.dialogTop(window).wizbody.document.we_form;
	for (var i = 0; i < bf.elements.length; i++) {
		if ((bf.elements[i].name.indexOf('v') > -1) || (bf.elements[i].name.indexOf('records') > -1) ||
			(bf.elements[i].name.indexOf('we_flds') > -1) || (bf.elements[i].name.indexOf('attributes') > -1)) {
			addField(cf, 'hidden', bf.elements[i].name, bf.elements[i].value);
		}
	}
}

function we_import(mode, cid, reload) {
	if (reload) {
		WE().util.dialogTop(window).wizbody.location = WE().consts.dirs.WEBEDITION_DIR + 'we_cmd.php?we_cmd[0]=import&pnt=wizbody&step=3&type=WE&noload=1';
	}

	var we_form = WE().util.dialogTop(window).wizcmd.document.we_form;
	we_form.elements['v[mode]'].value = mode;
	we_form.elements['v[cid]'].value = cid;
	we_form.target = 'wizcmd';
	we_form.action = WE().consts.dirs.WEBEDITION_DIR + 'we_cmd.php?we_cmd[0]=import&pnt=wizcmd';
	we_form.method = 'post';
	we_form.submit();
}

function doOnImportFinished(progressText) {
	WE().util.dialogTop(window).wizbusy.setProgressText('pb1', progressText);
	WE().util.dialogTop(window).wizbusy.setProgress(100);
	WE().util.dialogTop(window).opener.top.we_cmd('load', WE().util.dialogTop(window).opener.top.treeData.table, 0);
	if (WE().layout.weEditorFrameController.getActiveDocumentReference().quickstart && WE().layout.weEditorFrameController.getActiveDocumentReference().quickstart !== undefined) {
		WE().layout.weEditorFrameController.getActiveDocumentReference().location.reload();
	}
	if (WE().util.dialogTop(window).wizbusy && WE().util.dialogTop(window).wizbusy.document.getElementById('progress')) {
		var progress = WE().util.dialogTop(window).wizbusy.document.getElementById('progress');
		if (progress !== undefined) {
			progress.style.display = 'none';
		}
	}
}