/* global WE, top */

/**
 * webEdition CMS
 *
 * $Rev: 465018c8caa1 $
 * $Author: Lukas Imhof $
 * $Date: 2021-09-20 08:58 +0200 $
 *
 * This source is part of the webEdition SDK. The webEdition SDK is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU Lesser General Public License can be found at
 * http://www.gnu.org/licenses/lgpl-3.0.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionSDK/License.txt
 *
 *
 * @license    http://www.gnu.org/licenses/lgpl-3.0.html  LGPL
 */
'use strict';
WE().util.loadConsts(document, "search");
WE().util.loadConsts(document, "g_l.search");
var searchConf = WE().util.getDynamicVar(document, 'loadVarSearch_view', 'data-searchConf');

var weSearch = {
	conf: searchConf.conf,
	elems: searchConf.elems,
	elem: null,
	result: {},
	rolloverElem: null,
	init: function () {
		weSearch.conf.editorBodyFrame = (weSearch.conf.editorBody === "window" ? window : top.content.editor.edbody);
		if (weSearch.conf.editorBodyFrame.document.readyState === "complete") {
			if (weSearch.conf.editorBodyFrame.document.getElementById('mouseOverDivs_' + weSearch.conf.whichSearch)) {
				weSearch.sizeScrollContent();
				document.addEventListener('mousemove', weSearch.moveElementOverlay, false);
				document.addEventListener('resize', weSearch.sizeScrollContent(), false);
				WE().util.setIconOfDocClass(document, 'resultIcon');
			}
			if (searchConf.result.elements && searchConf.result.elements.length) {
				weSearch.processResult(searchConf.result.elements, searchConf.result.pagination);
				searchConf.result = {};
			}
			if (weSearch.conf.autostart) {
				weSearch.search(false);
			}
		} else {
			window.setTimeout(weSearch.init, 10);
		}
	},
	setNextPrevData_old: function () {
		var dataElem = this.conf.editorBodyFrame.document.getElementsByClassName('nextPrevData')[0],
			spanText = this.conf.editorBodyFrame.document.getElementsByClassName('spanSearchText'),
			btnBack = this.conf.editorBodyFrame.document.getElementsByClassName('btnSearchBack'),
			btnNext = this.conf.editorBodyFrame.document.getElementsByClassName('btnSearchNext'),
			selPages = this.conf.editorBodyFrame.document.getElementsByClassName('selectSearchPages'),
			selPagesVals = dataElem.getAttribute('data-pagevalue').split(','),
			selPagesTexts = dataElem.getAttribute('data-pagetext').split(','),
			selPagesPage = parseInt(dataElem.getAttribute('data-page')),
			opt, i, j;

		spanText[0].innerHTML = spanText[1].innerHTML = dataElem.getAttribute('data-text');
		btnBack[0].disabled = btnBack[1].disabled = dataElem.getAttribute('data-disableback') === 'true';
		btnNext[0].disabled = btnNext[1].disabled = dataElem.getAttribute('data-disablenext') === 'true';
		for (i = 0; i < 2; i++) {
			selPages[i].innerHTML = '';
			for (j = 0; j < selPagesVals.length; j++) {
				opt = document.createElement("option");
				opt.text = selPagesTexts[j];
				opt.value = selPagesVals[j];
				if (parseInt(selPagesVals[j]) === selPagesPage) {
					opt.selected = 'selected';
				}
				selPages[i].add(opt);
			}
		}
	},
	setNextPrevData: function (data) {
		var spanText = this.conf.editorBodyFrame.document.getElementsByClassName('spanSearchText'),
			btnBack = this.conf.editorBodyFrame.document.getElementsByClassName('btnSearchBack'),
			btnNext = this.conf.editorBodyFrame.document.getElementsByClassName('btnSearchNext'),
			selPages = this.conf.editorBodyFrame.document.getElementsByClassName('selectSearchPages'),
			opt, i, j;

		spanText[0].innerHTML = spanText[1].innerHTML = data.text;
		btnBack[0].disabled = btnBack[1].disabled = data.disableBack;
		btnNext[0].disabled = btnNext[1].disabled = data.disableNext;
		for (i = 0; i < 2; i++) {
			selPages[i].innerHTML = '';
			for (j = 0; j < data.pageValue.length; j++) {
				opt = document.createElement("option");
				opt.text = data.pageText[j];
				opt.value = data.pageValue[j];
				if (parseInt(data.pageValue[j]) === data.page) {
					opt.selected = 'selected';
				}
				selPages[i].add(opt);
			}
		}
	},
	ajaxCallbackResultList: function (response) {
		var responseText = response.DataArray.data;
		if (response.Success) {
			switch (weSearch.conf.whichSearch) {
				case WE().consts.search.SEARCH_DOCS:
				case WE().consts.search.SEARCH_TMPL:
				case WE().consts.search.SEARCH_MEDIA:
				case WE().consts.search.SEARCH_ADV:
				case WE().consts.search.DOCLIST_DOCUMENTS:
				case WE().consts.search.DOCLIST_TEMPLATES:
				case WE().consts.search.DOCLIST_OBJECTFILES:
				case WE().consts.search.DOCLIST_COLLECTION:
					weSearch.processResult(response.DataArray.data.elements, response.DataArray.data.pagination);
					break;
				default:
					weSearch.conf.editorBodyFrame.document.getElementById('scrollContent_' + weSearch.conf.whichSearch).getElementsByTagName("tbody")[0].innerHTML = responseText;

					WE().util.setIconOfDocClass(document, 'resultIcon');

					if (weSearch.conf.editorBodyFrame.document.getElementsByClassName('nextPrevData') && weSearch.conf.editorBodyFrame.document.getElementsByClassName('nextPrevData')[0]) {
						weSearch.setNextPrevData_old();
					} else {
						//weSearch.makeAjaxRequestParametersTop();
						//weSearch.makeAjaxRequestParametersBottom();
					}

					// IMPORTANT: we must move mouseoverdivs because of scrolling. FIXME: use display:none to avoid this
					weSearch.conf.editorBodyFrame.document.getElementById('mouseOverDivs_' + weSearch.conf.whichSearch).innerHTML = weSearch.conf.editorBodyFrame.document.getElementById('movethemaway').innerHTML;
					weSearch.conf.editorBodyFrame.document.getElementById('movethemaway').innerHTML = '';
					window.scrollTo(0, document.body.scrollHeight);
			}
		}
	},
	ajaxCallbackParametersTop: {
		success: function (o) {
			if (o.responseText !== undefined && o.responseText !== '') {
				weSearch.conf.editorBodyFrame.document.getElementById('parametersTop_' + weSearch.conf.whichSearch).innerHTML = o.responseText;
			}
		},
		failure: function (o) {
		}
	},
	ajaxCallbackParametersBottom: {
		success: function (o) {
			if (o.responseText !== undefined && o.responseText !== "") {
				weSearch.conf.editorBodyFrame.document.getElementById('parametersBottom_' + weSearch.conf.whichSearch).innerHTML = o.responseText;
			}
		},
		failure: function (o) {
		}
	},
	processResult: function (elements, pagination) {top.console.log('elements', elements);
		var i;
		switch (weSearch.conf.whichSearch) {
			case WE().consts.search.DOCLIST_OBJECTFILES:
				for (let el of elements) {
					if (el.hasOwnProperty('fields')) {
						for (var property in el.fields) {
							if (el.fields.hasOwnProperty(property)) {
								el[property] = el.fields[property];
							}
						}
						delete el.fields;
					}
				}
			/* fall through */
			case WE().consts.search.SEARCH_DOCS:
			case WE().consts.search.SEARCH_TMPL:
			case WE().consts.search.SEARCH_MEDIA:
			case WE().consts.search.SEARCH_ADV:
			case WE().consts.search.DOCLIST_DOCUMENTS:
			case WE().consts.search.DOCLIST_TEMPLATES:
			case WE().consts.search.DOCLIST_COLLECTION:
				weSearch.result[weSearch.conf.whichSearch] = elements;
				weSearch.fillList();
				weSearch.fillFields();
				weSearch.fillGrid();
				weSearch.setNextPrevData(pagination);
				break;
			default:
			//
		}
	},
	fillList: function () {
		let table = this.conf.editorBodyFrame.document.getElementById('list_' + this.conf.whichSearch),
			tbody = document.createElement('TBODY');

		for (let el of this.result[this.conf.whichSearch]) {
			tbody.innerHTML += el.elementList;
		}
		table.replaceChild(tbody, table.tBodies[0]);
		WE().util.setIconOfDocClass(document, 'resultIcon');
	},
	fillFields: function () {
		let table = this.conf.editorBodyFrame.document.getElementById('fields_' + this.conf.whichSearch),
			tbody = document.createElement('TBODY');

		for (let el of this.result[this.conf.whichSearch]) {
			tbody.innerHTML += el.elementFields;
		}
		table.replaceChild(tbody, table.tBodies[0]);
		WE().util.setIconOfDocClass(document, 'resultIcon');
	},
	fillGrid: function () {
		let
			gridDiv = document.createElement('DIV'),
			overlayDiv = document.createElement('DIV');

		for (let el of this.result[this.conf.whichSearch]) {
			gridDiv.innerHTML += el.elementGrid;
			overlayDiv.innerHTML += el.elementOverlay;
		}
		this.conf.editorBodyFrame.document.getElementById('grid_' + this.conf.whichSearch).innerHTML = gridDiv.innerHTML;
		this.conf.editorBodyFrame.document.getElementById('mouseOverDivs_' + this.conf.whichSearch).innerHTML = overlayDiv.innerHTML;
		WE().util.setIconOfDocClass(document, 'resultIcon');
	},
	search: function (newSearch, sameRange) {
		let Checks = [], m = 0;
		newSearch = newSearch === undefined ? true : newSearch;
		sameRange = sameRange === undefined ? false : sameRange;

		sameRange = !!(sameRange || !newSearch);// if not newSearch we preserve range anyway.

		switch (this.conf.whichSearch) {
			case WE().consts.search.DOCLIST_DOCUMENTS:
			case WE().consts.search.DOCLIST_TEMPLATES:
			case WE().consts.search.DOCLIST_OBJECTFILES:
			case WE().consts.search.DOCLIST_COLLECTION:
				Checks[0] = ''; // no search options needed at all!
				break;
			case WE().consts.search.SEARCH_ADV:
				for (let el of this.conf.editorBodyFrame.document.we_form.elements) {
					let table = el.name;
					if (table.substring(0, 23) === 'search_tables_advSearch') {
						if (encodeURI(el.value) === "1") {
							Checks[m] = "1";
							m++;
						}
					}
				}
				if (!Checks.length) {
					WE().util.showMessage(WE().consts.g_l.search.nothingCheckedAdv, WE().consts.message.WE_MESSAGE_ERROR, window);
				}
				break;
			case WE().consts.search.SEARCH_DOCS:
			case WE().consts.search.SEARCH_MEDIA:
				{
					let thirdName = this.conf.whichSearch === WE().consts.search.SEARCH_DOCS ? 'searchForContent' : 'searchForMeta';
					for (let el of this.conf.editorBodyFrame.document.we_form.elements) {
						switch (el.name) {
							case 'searchForText' + this.conf.whichSearch:
							case 'searchForTitle' + this.conf.whichSearch:
							case thirdName + this.conf.whichSearch:
								if (encodeURI(el.value) === "1") {
									Checks[m] = encodeURI(el.value);
									m++;
								}
						}
					}
				}
				if (!Checks.length) {
					//FIXME: dirty fix => allow to search without searchForXX when no searchFieldsMediaSearch[0] is empty
					if (this.conf.editorBodyFrame.document.we_form.elements['searchMediaSearch[0]'] !== undefined && this.conf.editorBodyFrame.document.we_form.elements['searchMediaSearch[0]'].value) {
						WE().util.showMessage(WE().consts.g_l.search.nothingCheckedTmplDoc, WE().consts.message.WE_MESSAGE_ERROR, window);
					} else {
						Checks[0] = '';
					}
				}
				break;
			case WE().consts.search.SEARCH_TMPL:
				for (let el of this.conf.editorBodyFrame.document.we_form.elements) {
					switch (el.name) {
						case 'searchForText' + this.conf.whichSearch:
						case 'searchForContent' + this.conf.whichSearch:
							if (encodeURI(el.value) === "1") {
								Checks[m] = "1";
								m++;
							}
					}
				}
				if (!Checks.length) {
					WE().util.showMessage(WE().consts.g_l.search.nothingCheckedTmplDoc, WE().consts.message.WE_MESSAGE_ERROR, window);
				}
				break;
		}

		if (Checks.length) {
			if (!sameRange) {
				window.document.we_form.elements['searchstart' + this.conf.whichSearch].value = 0;
			}

			var order = window.document.we_form.elements['Order' + this.conf.whichSearch].value;
			if (!order || !(order.trim())) {
				WE().t_e(this.conf.whichSearch + ': no valid order set', order);
				window.document.we_form.elements['Order' + this.conf.whichSearch].value = 'Text';
			}

			window.document.we_form.elements.newSearch.value = newSearch ? 1 : 0;

			this.makeAjaxRequestDoclist();
		}
	},
	makeAjaxRequestDoclist: function () {
		var args = '',
			cmd = '';
		for (var i = 0, elem; i < this.conf.editorBodyFrame.document.we_form.elements.length; i++) {
			elem = this.conf.editorBodyFrame.document.we_form.elements[i];
			if (!elem.name || (elem.type === "radio" && !elem.checked)
				) {
				continue;
			}
			args += '&we_cmd[' + encodeURI(elem.name) + ']=' + encodeURI(elem.value);
		}

		switch (this.conf.whichSearch) {
			case WE().consts.search.DOCLIST_DOCUMENTS:
			case WE().consts.search.DOCLIST_TEMPLATES:
			case WE().consts.search.DOCLIST_OBJECTFILES:
			case WE().consts.search.DOCLIST_COLLECTION:
				cmd = 'doclistGetSearchResult';
				break;
			default:
				cmd = 'searchGetSearchResult';
		}

		//this.conf.editorBodyFrame.document.getElementById('scrollContent_' + this.conf.whichSearch).getElementsByTagName("tbody")[0].innerHTML = '<tr><td  colspan="10" style="text-align:center"><i class="fa fa-2x fa-spinner fa-pulse"></i><div id="scrollActive"></div></td></tr>';
		WE().util.rpc(WE().consts.dirs.WEBEDITION_DIR + "rpc.php?cmd=" + cmd, 'tab=' + this.conf.tab + '&whichSearch=' + this.conf.whichSearch + '&id=' + this.conf.modelID + '&we_transaction=' + this.conf.we_transaction + args, this.ajaxCallbackResultList);
	},
	setView: function (value) {
		switch (weSearch.conf.whichSearch) {
			case WE().consts.search.DOCLIST_OBJECTFILES:
				var from, to;
				if (document.getElementById('resultFields_' + this.conf.whichSearch).classList.contains('hidden')) {
					from = document.getElementsByClassName('checkboxesProperties');
					to = document.getElementsByClassName('checkboxesElements');
				} else {
					from = document.getElementsByClassName('checkboxesElements');
					to = document.getElementsByClassName('checkboxesProperties');
				}
				for (var i = 0; i < from.length; i++) {
					to[i].checked = from[i].checked;
				}
				/* fall through */
			case WE().consts.search.SEARCH_DOCS:
			case WE().consts.search.SEARCH_TMPL:
			case WE().consts.search.SEARCH_MEDIA:
			case WE().consts.search.SEARCH_ADV:
			case WE().consts.search.DOCLIST_DOCUMENTS:
			case WE().consts.search.DOCLIST_TEMPLATES:
			case WE().consts.search.DOCLIST_COLLECTION:
				this.conf.editorBodyFrame.document.we_form.elements['setView' + this.conf.whichSearch].value = value;
				document.getElementById('resultList' + '_' + weSearch.conf.whichSearch).classList.add('hidden');
				document.getElementById('resultFields' + '_' + weSearch.conf.whichSearch).classList.add('hidden');
				document.getElementById('resultGrid' + '_' + weSearch.conf.whichSearch).classList.add('hidden');
				//document.getElementById('result' + value.charAt(0).toUpperCase() + value.slice(1)+ '_' + weSearch.conf.whichSearch).classList.remove('hidden');
				document.getElementById('result' + (value === 'list' ? 'List' : (value === 'icons' ? 'Grid' : 'Fields')) + '_' + weSearch.conf.whichSearch).classList.remove('hidden');
				break;
			default:
				this.conf.editorBodyFrame.document.we_form.elements['setView' + this.conf.whichSearch].value = value;
				this.search(false);
				break;
		}
	},
	showElementOverlay: function (picID) {
		if ((weSearch.rolloverElem = document.getElementById(picID))) {
			weSearch.rolloverElem.style.display = 'block';
		}
	},
	hideElementOverlay: function (picID) {
		if (weSearch.rolloverElem) {
			weSearch.rolloverElem.style.display = 'none';
		}
	},
	moveElementOverlay: function (e) {
		if (weSearch.rolloverElem && weSearch.rolloverElem.style.display !== 'none') {
			var elem = weSearch.rolloverElem,
				elemW = elem.offsetWidth,
				elemH = elem.offsetHeight,
				frameW = window.innerWidth ? window.innerWidth : document.body.offsetWidth,
				frameH = window.innerHeight ? window.innerHeight : document.body.offsetHeight,
				posX = e.pageX,
				posY = e.pageY,
				scrollY = window.scrollY;

			elem.style.left = (((frameW - posX) < elemW + 10) ? (posX - elemW - 10) : (posX + 10)) + 'px';
			elem.style.top = (Math.max((scrollY + 4), (Math.min((scrollY + frameH - elemH - 4), (posY - Math.round(elemH / 5 * 3)))))) + 'px';
		}
	},
	absLeft: function (el) {
		return (el.offsetParent) ? el.offsetLeft + this.absLeft(el.offsetParent) : el.offsetLeft;
	},
	absTop: function (el) {
		return (el.offsetParent) ? el.offsetTop + this.absTop(el.offsetParent) : el.offsetTop;
	},
	reloadSameRange: function () {// FIXME: add param "newSearchTable"
		var scrollActive = document.getElementById('scrollActive');
		if (scrollActive === null) {
			//this.conf.editorBodyFrame.document.we_form.elements['searchstart' + this.conf.whichSearch].value = parseInt(this.conf.editorBodyFrame.document.we_form.elements['searchstart' + this.conf.whichSearch].value) + anzahl;
			this.search(false);
		}
	},
	next: function () {
		var scrollActive = document.getElementById('scrollActive');
		var doc = this.conf.editorBodyFrame.document; //this.conf.editorBodyFrame ? this.conf.editorBodyFrame.document : document;
		if (scrollActive === null) {
			doc.we_form.elements['searchstart' + this.conf.whichSearch].value = parseInt(doc.we_form.elements['searchstart' + this.conf.whichSearch].value) + parseInt(doc.we_form.elements['anzahl' + this.conf.whichSearch].value);
			this.search(false);
		}
	},
	back: function (anzahl) {
		var scrollActive = document.getElementById('scrollActive');
		var doc = this.conf.editorBodyFrame.document; //this.conf.editorBodyFrame ? this.conf.editorBodyFrame.document : document;
		if (scrollActive === null) {
			doc.we_form.elements['searchstart' + this.conf.whichSearch].value = parseInt(doc.we_form.elements['searchstart' + this.conf.whichSearch].value) - parseInt(doc.we_form.elements['anzahl' + this.conf.whichSearch].value);
			this.search(false);
		}
	},
	openModule: function (mod, id) {
		top.we_cmd('we_' + mod + '_edit', id);
	},
	openCategory: function (id) {
		top.we_cmd('editCat', id);
	},
	compare: function (desc, order, a, b) {
		var
			propA = desc ? b[order] : a[order],
			propB = desc ? a[order] : b[order],
			type = (typeof (propA) === 'string' ||
				typeof (propB) === 'string') ? 'string' : 'number';

		if (type === 'string') {
			return propA.localeCompare(propB);
		}
		return propA - propB;
	},
	setOrder: function (order, whichSearch) {
		var columns = /*whichSearch === 'MediaSearch' ?
		 ['Text', 'media_filesize', 'IsUsed', 'media_alt', 'media_title', 'Creation_Date', 'Mod_Date'] :*/
			['Text', 'SiteTitle', 'Creation_Date', 'Mod_Date', 'docID'],
			deleteArrow,
			arrow,
			arrowSpans,
			name;

		if (whichSearch === 'MediaSearch') {
			var arrowSpans = document.getElementById('list_' + whichSearch).getElementsByClassName('listSortArrow');
			for (var i = 0; i < arrowSpans.length; i++) {
				if (order !== (name = arrowSpans[i].id.replace('_' + whichSearch, ''))) {
					deleteArrow = document.getElementById(columns[i] + '_' + whichSearch);
					if ((deleteArrow = document.getElementById(name + '_' + whichSearch))) {
						deleteArrow.innerHTML = '';
					}
				}
			}
		} else {
			for (var i = 0; i < columns.length; i++) {
				if (order !== columns[i]) {
					deleteArrow = document.getElementById(columns[i] + '_' + whichSearch);
					if (deleteArrow) {
						deleteArrow.innerHTML = '';
					}
				}
			}
		}

		var desc = false;
		arrow = document.getElementById(order + '_' + whichSearch);
		if (order + ' DESC' === document.we_form.elements['Order' + whichSearch].value) {
			document.we_form.elements['Order' + whichSearch].value = order;
			arrow.innerHTML = '<i class="fa fa-sort-asc fa-lg"></i>';
		} else {
			document.we_form.elements['Order' + whichSearch].value = order + ' DESC';
			arrow.innerHTML = '<i class="fa fa-sort-desc fa-lg"></i>';
			desc = true;
		}

		if (whichSearch !== 'MediaSearch' || document.getElementById('parametersTop_' + whichSearch).getElementsByClassName('selectSearchPages')[0].length !== 1) {
			this.search(false);
			return;
		}

		//weSearch.result[whichSearch].sort(compare);
		weSearch.result[weSearch.conf.whichSearch].sort(weSearch.compare.bind(null, desc, order));
		weSearch.fillList();
		weSearch.fillGrid();
	},
	setSort: function (order, changeDirection) {
		var view = this.conf.editorBodyFrame.document.we_form.elements['setView' + this.conf.whichSearch].value,
			old = document.we_form.elements['Order' + this.conf.whichSearch].value,
			desc = false,
			btnClasslist = document.getElementById('btnAsc_' + this.conf.whichSearch).firstChild.classList,
			arrowSpans = document.getElementById('list_' + this.conf.whichSearch).getElementsByClassName('listSortArrow');

		switch (view) {
			case 'icons':
				if (old.split(' ').length === 1) {
					desc = order ? false : changeDirection;
					order = order ? order : old;
				} else {
					desc = order ? true : !changeDirection;
					order = order ? order : old.split(' ')[0];
				}
				break;
			default:
				if (order === old.split(' ')[0]) {
					desc = old.split(' ').length === 1;
				} else {
					desc = false;
				}
		}

		document.we_form.elements['Order' + this.conf.whichSearch].value = order + (desc ? ' DESC' : '');
		for (var i = 0; i < arrowSpans.length; i++) {
			arrowSpans[i].innerHTML = '';
			if (arrowSpans[i].id === order + '_' + this.conf.whichSearch) {
				arrowSpans[i].innerHTML = desc ? '<i class="fa fa-sort-desc fa-lg"></i>' : '<i class="fa fa-sort-asc fa-lg"></i>';
			}
		}
		btnClasslist.remove('fa-sort-alpha-asc');
		btnClasslist.remove('fa-sort-alpha-desc');
		btnClasslist.add(desc ? 'fa-sort-alpha-desc' : 'fa-sort-alpha-asc');
		document.getElementById('gridOrder_' + this.conf.whichSearch).value = order;

		if (this.conf.whichSearch !== WE().consts.search.SEARCH_MEDIA || document.getElementById('parametersTop_' + this.conf.whichSearch).getElementsByClassName('selectSearchPages')[0].length !== 1) {
			this.search(false);
			return;
		}
		weSearch.result[this.conf.whichSearch].sort(weSearch.compare.bind(null, desc, order));
		weSearch.fillList();
		weSearch.fillGrid();
	},
	toggleShowVisible: function (c) {
		this.search(false);
	},
	setWs: function (path, id) {
		document.we_form.elements['WorkspacePath' + weSearch.conf.whichSearch].value = path;
		document.we_form.elements['WorkspaceID' + weSearch.conf.whichSearch].value = id;
		this.search(false);
	},
	sizeScrollContent: function () {
		//disabled for now
		return;
		var frameH = (window.innerHeight ? window.innerHeight : document.body.offsetHeight),
			h = 0, hMin = 120, rows, mode,
			contentParameters = document.getElementById('contentParameters'),
			scrollContent = document.getElementById('scrollContent_' + this.conf.whichSearch),
			paramTop = document.getElementById('parametersTop_' + this.conf.whichSearch);


		if (scrollContent) {
			switch (this.conf.whichSearch) {
				case WE().consts.search.SEARCH_DOCS:
				case WE().consts.search.SEARCH_TMPL:
					h = frameH - contentParameters.clientHeight - (2 * paramTop.clientHeight) - 100;
					break;
				case WE().consts.search.SEARCH_MEDIA:
				case WE().consts.search.SEARCH_ADV:
					//h = frameH - 136;
					h = frameH - contentParameters.clientHeight - (2 * paramTop.clientHeight) - 80;
					hMin = 200;
					break;
				case WE().consts.search.DOCLIST_DOCUMENTS:
				case WE().consts.search.DOCLIST_TEMPLATES:
				case WE().consts.search.DOCLIST_OBJECTFILES:
				case WE().consts.search.DOCLIST_COLLECTION:
					rows = (document.getElementById('filterTable' + this.conf.whichSearch).rows.length);
					mode = document.we_form.mode.value;
					h = parseInt(mode) === 1 ? (frameH - (220 + (rows * 28))) : (frameH - 183);
					break;
			}

			scrollContent.style.height = (Math.max(h, hMin)) + 'px';
		}
	},
	newinput: function () {
		var elem = document.getElementById('filterTable' + this.conf.whichSearch),
			//c = elem.rows.length - 1,
			//scrollContent = document.getElementById('scrollContent_' + this.conf.whichSearch),
			newRow, cell;

		this.conf.rows++;

		if (elem) {
			newRow = document.createElement('TR');
			newRow.setAttribute('id', 'filterRow_' + this.conf.rows);

			cell = document.createElement('TD');
			cell.innerHTML = '<input type="hidden" value="" name="hidden_searchFields' + this.conf.whichSearch + '[' + this.conf.rows + ']" value="">' + this.elems.searchFields.replace(/__we_new_id__/g, this.conf.rows);
			newRow.appendChild(cell);

			newRow.appendChild(this.getCell('location' + this.conf.whichSearch, this.conf.rows));
			newRow.appendChild(this.getCell('search' + this.conf.whichSearch, this.conf.rows));
			newRow.appendChild(this.getCell('delButton', this.conf.rows));
			newRow.appendChild(this.getCell('hiddenLocation_empty', this.conf.rows));

			elem.appendChild(newRow);
			this.sizeScrollContent();
		}
	},
	newinpuMediaSearch: function () {
		//top.console.debug('is fn');

	},
	getCell: function (type, rowID, replacement, value) {
		var cell = document.createElement('TD'),
			locationType;

		switch (type) {
			case 'delButton':
				cell.setAttribute('id', 'td_delButton[' + rowID + ']');
				cell.innerHTML = this.elems.btnTrash.replace(/__we_new_id__/g, rowID);
				break;
			case 'location' + this.conf.whichSearch:
			case 'locationDate' + this.conf.whichSearch:
			case 'locationText' + this.conf.whichSearch:
				locationType = (type === ('locationText' + this.conf.whichSearch) ? 'selLocationText' : (type === ('locationDate' + this.conf.whichSearch) ? 'selLocationDate' : 'selLocation'));
				cell.setAttribute('id', 'td_location' + this.conf.whichSearch + '[' + rowID + ']');
				cell.innerHTML = this.elems[locationType].replace(/__we_new_id__/g, rowID);
				break;

			case 'search' + this.conf.whichSearch:
				cell.setAttribute('id', 'td_search' + this.conf.whichSearch + '[' + rowID + ']');
				cell.innerHTML = this.elems.fieldSearch.replace(/__we_new_id__/g, rowID).replace(/__we_read_only__/g, replacement ? 'readonly="readonly" ' : '');
				break;
			case 'hiddenLocation' + this.conf.whichSearch:
				cell.setAttribute('id', 'td_hiddenLocation[' + rowID + ']');
				cell.innerHTML = '<input type="hidden" name="location' + this.conf.whichSearch + '[' + rowID + ']" value="' + (value ? value : 'IS') + '"/>';
				break;
			case 'hiddenLocation_empty':
				cell.setAttribute('id', 'td_hiddenLocation[' + rowID + ']');
		}
		return cell;

	},
	reload: function () {
		switch (this.conf.whichSearch) {
			case WE().consts.search.DOCLIST_DOCUMENTS:
			case WE().consts.search.DOCLIST_TEMPLATES:
			case WE().consts.search.DOCLIST_OBJECTFILES:
			case WE().consts.search.DOCLIST_COLLECTION:
				top.we_cmd("reload_editpage");
		}
	},
	switchSearch: function (mode) {
		document.we_form.mode.value = mode;
		var defSearch = document.getElementById('defSearch'),
			advSearch = document.getElementById('advSearch'),
			filterTable = document.getElementById('filterTable' + weSearch.conf.whichSearch),
			advSearch3 = document.getElementById('advSearch3');

		if (parseInt(mode) === 1) {
			defSearch.style.display = "none";
			advSearch.style.display = "block";
			filterTable.style.display = "block";
			advSearch3.style.display = "block";
		} else {
			defSearch.style.display = "block";
			advSearch.style.display = "none";
			filterTable.style.display = "none";
			advSearch3.style.display = "none";

			switch (this.conf.whichSearch) {
				case WE().consts.search.DOCLIST_DOCUMENTS:
				case WE().consts.search.DOCLIST_TEMPLATES:
				case WE().consts.search.DOCLIST_OBJECTFILES:
				case WE().consts.search.DOCLIST_COLLECTION:
					this.search(false);
			}
		}
		this.sizeScrollContent();
	},
	delRow: function (id) {
		var //scrollContent = document.getElementById('scrollContent_' + this.conf.whichSearch),
			elem = document.getElementById('filterTable' + this.conf.whichSearch);

		if (elem) {
			var trows = elem.rows,
				rowID = 'filterRow_' + id;

			for (var i = 0; i < trows.length; i++) {
				if (rowID == trows[i].id) {
					elem.deleteRow(i);
				}
			}
		}
		this.sizeScrollContent();
	},
	changeit: function (value, rowNr) {
		var setValue = document.getElementsByName('search' + this.conf.whichSearch + '[' + rowNr + ']')[0].value,
			from = document.getElementsByName('hidden_searchFields' + this.conf.whichSearch + '[' + rowNr + ']')[0].value,
			row = document.getElementById('filterRow_' + rowNr),
			locationTD = document.getElementById('td_location' + this.conf.whichSearch + '[' + rowNr + ']'),
			hiddenLocationTD = document.getElementById('td_hiddenLocation[' + rowNr + ']'),
			searchTD = document.getElementById('td_search' + this.conf.whichSearch + '[' + rowNr + ']'),
			delButtonTD = document.getElementById('td_delButton[' + rowNr + ']'),
			location,
			locationType = '',
			innerhtml,
			cell;

		if (hiddenLocationTD) {
			row.removeChild(hiddenLocationTD);
		}
		if (delButtonTD) {
			row.removeChild(delButtonTD);
		}
		if (searchTD) {
			row.removeChild(searchTD);
		}
		if (locationTD) {
			row.removeChild(locationTD);
		}
		var table;

		switch (value) {
			case 'Content':
				row.appendChild(this.getCell('location' + this.conf.whichSearch, rowNr, '', 'CONTAIN', true));
				location = document.getElementById('location' + this.conf.whichSearch + '[' + rowNr + ']');
				location.value = 'CONTAIN';
				location.disabled = true;

				row.appendChild(this.getCell('search' + this.conf.whichSearch, this.conf.rows));
				row.appendChild(this.getCell('delButton', rowNr));
				row.appendChild(this.getCell('hiddenLocation' + this.conf.whichSearch, rowNr, '', 'CONTAIN'));
				document.getElementById("search" + this.conf.whichSearch + "[" + rowNr + "]").value = setValue;
				break;
			case 'temp_category':
				row.appendChild(this.getCell('location' + this.conf.whichSearch, rowNr, '', 'CONTAIN', true));
				location = document.getElementById('location' + this.conf.whichSearch + '[' + rowNr + ']');
				location.value = 'IS';
				location.disabled = true;

				innerhtml = '<table class="default"><tbody><tr>' +
					'<td>' + this.elems.fieldSearch.replace(/__we_new_id__/g, rowNr).replace(/__we_read_only__/, 'readonly="readonly" ') + '</td>' +
					'<td><input value="" name="search' + this.conf.whichSearch + 'ParentID[' + rowNr + ']" type="hidden"></td><td></td>' +
					'<td>' + this.elems.btnSelector.replace(/__we_new_id__/g, rowNr).replace(/__we_sel_table__/, WE().consts.tables.CATEGORY_TABLE).replace(/__we_selector__/, 'we_selector_document').replace(/__we_content_types__/, '') + '</td>' +
					'</tr></tbody></table>';

				cell = document.createElement('TD');
				cell.setAttribute('id', 'td_search' + this.conf.whichSearch + '[' + rowNr + ']');
				cell.innerHTML = innerhtml;
				row.appendChild(cell);

				row.appendChild(this.getCell('delButton', rowNr));
				row.appendChild(this.getCell('hiddenLocation' + this.conf.whichSearch, rowNr, '', 'IS'));
				break;
			case 'temp_template_id':
			case 'MasterTemplateID':
				row.appendChild(this.getCell('location' + this.conf.whichSearch, rowNr, '', 'CONTAIN', true));
				location = document.getElementById('location' + this.conf.whichSearch + '[' + rowNr + ']');
				location.value = 'IS';
				location.disabled = true;

				innerhtml = '<table class="default"><tbody><tr>' +
					'<td>' + this.elems.fieldSearch.replace(/__we_new_id__/g, rowNr).replace(/__we_read_only__/, 'readonly="readonly" ') + '</td>' +
					'<td><input value="" name="search' + this.conf.whichSearch + 'ParentID[' + rowNr + ']" type="hidden"></td><td></td>' +
					'<td>' + this.elems.btnSelector.replace(/__we_new_id__/g, rowNr).replace(/__we_sel_table__/, WE().consts.tables.TEMPLATES_TABLE).replace(/__we_selector__/, 'we_selector_document').replace(/__we_content_types__/, '') + '</td>' +
					'</tr></tbody></table>';

				cell = document.createElement('TD');
				cell.setAttribute('id', 'td_search' + this.conf.whichSearch + '[' + rowNr + ']');
				cell.innerHTML = innerhtml;
				row.appendChild(cell);

				row.appendChild(this.getCell('delButton', rowNr));
				row.appendChild(this.getCell('hiddenLocation' + this.conf.whichSearch, rowNr, '', 'IS'));
				break;
			case 'ParentIDDoc':
			case 'ParentIDObj':
			case 'ParentIDTmpl':
				row.appendChild(this.getCell('location' + this.conf.whichSearch, rowNr, '', 'CONTAIN', true));
				location = document.getElementById('location' + this.conf.whichSearch + '[' + rowNr + ']');
				location.value = 'IS';
				location.disabled = true;

				table = value === 'ParentIDDoc' ? WE().consts.tables.FILE_TABLE : (value === 'ParentIDObj' ? WE().consts.tables.OBJECT_FILES_TABLE : WE().consts.tables.TEMPLATES_TABLE);
				innerhtml = '<table class="default"><tbody><tr>' +
					'<td>' + this.elems.fieldSearch.replace(/__we_new_id__/g, rowNr).replace(/__we_read_only__/, 'readonly="readonly" ') + '</td>' +
					'<td><input value="" name="search' + this.conf.whichSearch + 'ParentID[' + rowNr + ']" type="hidden"></td><td></td>' +
					'<td>' + this.elems.btnSelector.replace(/__we_new_id__/g, rowNr).replace(/__we_sel_table__/, table).replace(/__we_selector__/, 'we_selector_directory').replace(/__we_content_types__/, '') + '</td>' +
					'</tr></tbody></table>';

				cell = document.createElement('TD');
				cell.setAttribute('id', 'td_search' + this.conf.whichSearch + '[' + rowNr + ']');
				cell.innerHTML = innerhtml;
				row.appendChild(cell);

				row.appendChild(this.getCell('delButton', rowNr));
				row.appendChild(this.getCell('hiddenLocation' + this.conf.whichSearch, rowNr, '', 'IS'));
				break;
			case 'HasReferenceToID':
				row.appendChild(this.getCell('location' + this.conf.whichSearch, rowNr, '', 'CONTAIN', true));
				location = document.getElementById('location' + this.conf.whichSearch + '[' + rowNr + ']');
				location.value = 'IS';
				location.disabled = true;

				table = WE().consts.tables.FILE_TABLE;
				innerhtml = '<table class="default"><tbody><tr>' +
					'<td>' + this.elems.fieldSearch.replace(/__we_new_id__/g, rowNr).replace(/__we_read_only__/, 'readonly="readonly" ') + '</td>' +
					'<td><input value="" name="search' + this.conf.whichSearch + 'ParentID[' + rowNr + ']" type="hidden"></td><td></td>' +
					'<td>' + this.elems.btnSelector.replace(/__we_new_id__/g, rowNr).replace(/__we_sel_table__/, table).replace(/__we_selector__/, 'we_selector_document').replace(/__we_content_types__/, WE().consts.search.MEDIA_CONTENTTYPES_CSV) + '</td>' +
					'</tr></tbody></table>';

				cell = document.createElement('TD');
				cell.setAttribute('id', 'td_search' + this.conf.whichSearch + '[' + rowNr + ']');
				cell.innerHTML = innerhtml;
				row.appendChild(cell);

				row.appendChild(this.getCell('delButton', rowNr));
				row.appendChild(this.getCell('hiddenLocation' + this.conf.whichSearch, rowNr, '', 'IS'));
				break;
			case 'Status':
				row.appendChild(this.getCell('location' + this.conf.whichSearch, rowNr, '', 'CONTAIN', true));
				location = document.getElementById('location' + this.conf.whichSearch + '[' + rowNr + ']');
				location.value = 'IS';
				location.disabled = true;

				cell = document.createElement('TD');
				cell.setAttribute('id', 'td_search' + this.conf.whichSearch + '[' + rowNr + ']');
				cell.innerHTML = this.elems.selStatus.replace(/__we_new_id__/g, rowNr);
				row.appendChild(cell);

				row.appendChild(this.getCell('delButton', rowNr));
				row.appendChild(this.getCell('hiddenLocation' + this.conf.whichSearch, rowNr, '', 'IS'));
				break;
			case 'Speicherart':
				row.appendChild(this.getCell('location' + this.conf.whichSearch, rowNr, '', 'CONTAIN', true));
				location = document.getElementById('location' + this.conf.whichSearch + '[' + rowNr + ']');
				location.value = 'IS';
				location.disabled = true;

				cell = document.createElement('TD');
				cell.setAttribute('id', 'td_search' + this.conf.whichSearch + '[' + rowNr + ']');
				cell.innerHTML = this.elems.selSpeicherart.replace(/__we_new_id__/g, rowNr);
				row.appendChild(cell);

				row.appendChild(this.getCell('delButton', rowNr));
				row.appendChild(this.getCell('hiddenLocation' + this.conf.whichSearch, rowNr, '', 'IS'));
				break;
			case 'Publish_Date':
			case 'Creation_Date':
			case 'Mod_Date':
				row.appendChild(this.getCell('locationDate' + this.conf.whichSearch, rowNr));
				cell = document.createElement('TD');
				cell.setAttribute('id', 'td_search' + this.conf.whichSearch + '[' + rowNr + ']');
				cell.innerHTML = '<span id="search' + this.conf.whichSearch + '[' + rowNr + ']_cell" class="default">' +
					this.elems.fieldDate.replace(/__we_new_id__/g, rowNr) +
					'</span>';
				row.appendChild(cell);

				row.appendChild(this.getCell('delButton', rowNr));
				row.appendChild(this.getCell('hiddenLocation_empty', rowNr));
				break;
			case 'allModsIn':// FIXME: does not work yet
				row.appendChild(this.getCell('location' + this.conf.whichSearch, rowNr, '', 'CONTAIN', true));
				location = document.getElementById('location' + this.conf.whichSearch + '[' + rowNr + ']');
				location.value = 'IN';
				location.disabled = true;

				cell = document.createElement('TD');
				cell.setAttribute('id', 'td_search' + this.conf.whichSearch + '[' + rowNr + ']');
				cell.innerHTML = this.elems.selModFields.replace(/__we_new_id__/g, rowNr);
				row.appendChild(cell);

				row.appendChild(this.getCell('delButton', rowNr));
				row.appendChild(this.getCell('hiddenLocation_empty', rowNr));
				break;
			case 'modifierID':
				//case 'Creator':
				row.appendChild(this.getCell('location' + this.conf.whichSearch, rowNr, '', 'CONTAIN', true));
				location = document.getElementById('location' + this.conf.whichSearch + '[' + rowNr + ']');
				location.value = 'IS';
				location.disabled = true;

				cell = document.createElement('TD');
				cell.setAttribute('id', 'td_search' + this.conf.whichSearch + '[' + rowNr + ']');
				cell.innerHTML = this.elems.selUsers.replace(/__we_new_id__/g, rowNr);
				row.appendChild(cell);

				row.appendChild(this.getCell('delButton', rowNr));
				row.appendChild(this.getCell('hiddenLocation' + this.conf.whichSearch, rowNr, '', 'IS'));
				break;
			default:
				switch (value) {
					case 'meta__Title':
					case 'meta__Description':
					case 'meta__Keywords':
					case 'meta__Autor':
					case 'meta__MIME':
					case 'Path':
					case 'CreatorName': // FIXME: reduce CreatorID and CreatorName to Creator analogue to modifierID
					case 'Text':
					case 'ContentType': // FIXME: make select
					case 'WebUserName':
						locationType = 'Text';
						break;
					case 'ID':
					case 'CreatorID':
					case 'WebUserID':
						locationType = 'Date';
						break;
				}

				row.appendChild(this.getCell('location' + locationType + this.conf.whichSearch, rowNr));
				row.appendChild(this.getCell('search' + this.conf.whichSearch, rowNr));
				row.appendChild(this.getCell('delButton', rowNr));
				row.appendChild(this.getCell('hiddenLocation_empty', rowNr));

				document.getElementById('search' + this.conf.whichSearch + '[' + rowNr + ']').value = setValue;
		}

		switch (from) {// FIXME: this is nonsens! move this to the above cases
			//switch (value) {
			case 'allModsIn':
			case 'MasterTemplateID':
			case 'ParentIDTmpl':
			case 'ParentIDObj':
			case 'ParentIDDoc':
			case 'temp_template_id':
			case 'ContentType':
			case 'temp_category':
			case 'Status':
			case 'Speicherart':
			case 'Publish_Date':
			case 'Creation_Date':
			case 'Mod_Date':
				document.getElementById('search' + this.conf.whichSearch + '[' + rowNr + ']').value = '';
				break;
			default:
				document.getElementById('search' + this.conf.whichSearch + '[' + rowNr + ']').value = setValue;
		}

		document.getElementsByName('hidden_searchFields' + this.conf.whichSearch + '[' + rowNr + ']')[0].value = value;

	},
	ajaxCallbackResetVersion: function (response) {
		WE().util.showMessage(WE().consts.g_l.search.versionsResetAllVersionsOK, WE().consts.message.WE_MESSAGE_NOTICE, window);

		// reload current document => reload all open Editors on demand
		var _usedEditors = WE().layout.weEditorFrameController.getEditorsInUse();
		for (var frameId in _usedEditors) {
			if (_usedEditors[frameId].getEditorIsActive()) { // reload active editor
				_usedEditors[frameId].setEditorReloadAllNeeded(true);
				_usedEditors[frameId].setEditorIsActive(true);
			} else {
				_usedEditors[frameId].setEditorReloadAllNeeded(true);
			}
		}

		//reload tree
		if (top.opener.top.treeData) {
			top.opener.we_cmd('load', top.opener.top.treeData.table, 0);
		}
		document.getElementById('resetBusy' + this.conf.whichSearch).innerHTML = '';
	},
	resetVersionAjax: function (id, documentID, version, table) {
		document.getElementById('resetBusy' + this.conf.whichSearch).innerHTML = "<table border='0' width='100%' height='100%'><tr><td align='center'><i class=\"fa fa-2x fa-spinner fa-pulse\"></i><div id='scrollActive'></div></td></tr></table>";
		WE().util.rpc(WE().consts.dirs.WEBEDITION_DIR + "rpc.php?mod=version&cmd=versionReset", "id=" + id + "&documentID=" + documentID + "&version=" + version + "&documentTable=" + table + "&we_transaction=" + this.conf.we_transaction, this.ajaxCallbackResetVersion);
	},
	resetVersions: function () {
		let checkboxes = [];
		for (let el of document.we_form.querySelectorAll("input[name^=resetVersion]")) {
			if (el.checked === true) {
				let pubElem = document.getElementById("publishVersion_" + el.value);
				checkboxes.push(el.value + (pubElem !== null && pubElem.checked ? "___1" : "___0"));
			}
		}
		if (!checkboxes.length) {
			WE().util.showMessage(WE().consts.g_l.search.versionsNotChecked, WE().consts.message.WE_MESSAGE_NOTICE, window);
			return;
		}
		if (window.confirm(WE().consts.g_l.search.resetVersionsSearchtool) !== true) {
			return;
		}
		this.resetVersionAjax(checkboxes.join(","), 0, 0, 0);

	},
	checkAllActionChecks: function () {
		let
			checkboxes = document.getElementsByName((this.conf.whichSearch === WE().consts.search.SEARCH_MEDIA ? 'delete_docs_MediaSearch' : 'publish_docs_' + this.conf.whichSearch));

		for (let box of checkboxes) {
			box.checked = !box.checked;
		}
	},
	publishDocs: function (action, whichSearch) {
		let
			check = false,
			checkboxes = document.getElementsByName("publish_docs_" + whichSearch);

		for (let box of checkboxes) {
			if (box.checked) {
				check = true;
				break;
			}
		}

		if (!check) {//searchtool__notChecked
			WE().util.showMessage(WE().consts.g_l.search.searchtool__notChecked, WE().consts.message.WE_MESSAGE_NOTICE, window);
			return;
		}
		if (window.confirm(WE().consts.g_l.search.publish_docs) === true) {
			this.publishDocsAjax(action, whichSearch);
		}
	},
	handleActionSelect: function (select, whichSearch) {
		if (!select.value) {
			return;
		}
		let check = false,
			checkboxes;

		switch (whichSearch) {
			case WE().consts.search.DOCLIST_DOCUMENTS:
			case WE().consts.search.DOCLIST_TEMPLATES:
			case WE().consts.search.DOCLIST_COLLECTION:
				checkboxes = document.getElementsByName("publish_docs_" + whichSearch);
				break;
			case WE().consts.search.DOCLIST_OBJECTFILES:
				checkboxes = !document.getElementById('resultFields_' + whichSearch).classList.contains('hidden') ? document.getElementsByClassName('checkboxesElements') :
					document.getElementsByClassName('checkboxesProperties');
				break;
			default:
				checkboxes = document.getElementsByName("publish_docs_" + whichSearch);
		}

		for (var i = 0; i < checkboxes.length; i++) {
			if (checkboxes[i].checked) {
				check = true;
				break;
			}
		}
		if (!check) {//searchtool__notChecked
			select.selectedIndex = 0;
			WE().util.showMessage(WE().consts.g_l.search.searchtool__notChecked, WE().consts.message.WE_MESSAGE_NOTICE, window);
			return;
		}

		switch (select.value) {
			case WE().consts.folderAction.ACTION_CATEGORY_ADD:
			case WE().consts.folderAction.ACTION_CATEGORY_DELETE:
				window.we_cmd('we_selector_document', {table: WE().consts.tables.CATEGORY_TABLE, jsCmd: 'handleCategoryAction,' + select.value + ',' + whichSearch, open_doc: true});
				break;
			default:
				WE().util.showConfirm(window, '', WE().consts.g_l.object.folderAction[select.value], ['documentFolderAction', select.value, whichSearch]);
		}
		select.selectedIndex = 0;
	},
	handleCategoryAction: function (action, whichSearch, IDs) {
		if (!action || !IDs) {
			return;
		}
		this.publishDocsAjax(action, whichSearch, IDs);
	},
	toggleAdditionalContent: function (btn, id) {
		var elem = document.getElementById('infoTable_' + id);

		if (elem) {
			elem.style.display = elem.style.display === 'block' ? 'none' : 'block';
			if (elem.style.display === 'block') {
				btn.firstChild.classList.remove("fa-caret-right");
				btn.firstChild.classList.add("fa-caret-down");
			} else {
				btn.firstChild.classList.remove("fa-caret-down");
				btn.firstChild.classList.add("fa-caret-right");
			}
		}
	},
	ajaxCallbackPublishDocs: function (weResponse) {
		// reload current document => reload all open Editors on demand
		var _usedEditors = WE().layout.weEditorFrameController.getEditorsInUse();
		for (var frameId in _usedEditors) {
			if (_usedEditors[frameId].getEditorIsActive()) { // reload active editor
				_usedEditors[frameId].setEditorReloadAllNeeded(true);
				_usedEditors[frameId].setEditorIsActive(true);
			} else {
				_usedEditors[frameId].setEditorReloadAllNeeded(true);
			}
		}

		//reload tree
		switch (weSearch.conf.whichSearch) {
			case WE().consts.search.DOCLIST_DOCUMENTS:
			case WE().consts.search.DOCLIST_TEMPLATES:
			case WE().consts.search.DOCLIST_OBJECTFILES:
			case WE().consts.search.DOCLIST_COLLECTION:
				top.we_cmd("load", top.treeData.table, 0);
				break;
			default:
				if (top.opener.top.treeData) {
					top.opener.we_cmd("load", top.opener.top.treeData.table, 0);
				}
		}

		document.getElementById("resetBusy" + weSearch.conf.whichSearch).innerHTML = "";
		//document.getElementById("resetBusyDocSearch").innerHTML = "";
		if (weResponse.DataArray.jscmd) {
			for (var i = 0; i < weResponse.DataArray.jscmd.length; i++) {
				WE().util.processJSCmd.call(this, weResponse.DataArray.jscmd[i].cmd, weResponse.DataArray.jscmd[i].data);
			}
		}
		WE().util.showMessage(weResponse.DataArray.message ? weResponse.DataArray.message : WE().consts.g_l.search.searchtool__publishOK, WE().consts.message.WE_MESSAGE_NOTICE, window);
	},
	publishDocsAjax: function (action, whichSearch, IDs) {
		var check = [],
			scroll = document.getElementById("resetBusy" + whichSearch),
			checkboxes,
			cmd;

		switch (whichSearch) {
			case WE().consts.search.DOCLIST_DOCUMENTS:
			case WE().consts.search.DOCLIST_TEMPLATES:
			case WE().consts.search.DOCLIST_COLLECTION:
				checkboxes = document.getElementsByName('publish_docs_' + whichSearch);
				break;
			case WE().consts.search.DOCLIST_OBJECTFILES:
				checkboxes = !document.getElementById('resultFields_' + whichSearch).classList.contains('hidden') ? document.getElementsByClassName('checkboxesElements') :
					document.getElementsByClassName('checkboxesProperties');
				break;
			default:
				checkboxes = document.getElementsByName('publish_docs_' + whichSearch);
		}

		for (var i = 0; i < checkboxes.length; i++) {
			if (checkboxes[i].checked) {
				check.push(checkboxes[i].value);
			}
		}

		switch (this.conf.whichSearch) {
			case WE().consts.search.DOCLIST_OBJECTFILES:
				cmd = 'objectfileFolderAction';
				break;
			default:
				cmd = 'documentFolderAction';
		}

		scroll.innerHTML = "<table border='0' width='100%' height='100%'><tr><td align='center'><i class=\"fa fa-2x fa-spinner fa-pulse\"></i></td></tr></table>";
		WE().util.rpc(WE().consts.dirs.WEBEDITION_DIR + "rpc.php?cmd=" + cmd, "&we_cmd[0]=" + action + "&we_cmd[1]=" + encodeURI(check.join(',')) + (IDs ? "&we_cmd[2]=" + IDs.join(",") : "") + "&we_cmd[transaction]=" + this.conf.we_transaction, this.ajaxCallbackPublishDocs);
	},
	previewVersion: function (table, ID, version) {
		top.we_cmd("we_versions_preview", table, ID, version, 0);
	},
	deleteMediaDocs: function (whichSearch) {
		var checkboxes = document.getElementsByName("delete_docs_" + whichSearch);
		var check = false;

		for (var i = 0; i < checkboxes.length; i++) {
			if (checkboxes[i].checked) {
				check = true;
				break;
			}
		}

		if (!check) {
			WE().util.showMessage(WE().consts.g_l.search.searchtool__notChecked, WE().consts.message.WE_MESSAGE_NOTICE, window);
		} else {
			var conf = window.confirm(WE().consts.g_l.search.confirmDeleteFiles); // FIXME: use we confirm
			if (conf) {
				this.deleteMediaDocsAjax(whichSearch);
			}
		}
	},
	deleteMediaDocsAjax: function (whichSearch) {
		var args = '',
			check = '',
			checkboxes = document.getElementsByName("delete_docs_" + whichSearch);

		for (var i = 0; i < checkboxes.length; i++) {
			if (checkboxes[i].checked) {
				check += (check ? ',' : '') + checkboxes[i].value;
			}
		}
		args += '&we_cmd[0]=' + encodeURI(check);

		var scroll = document.getElementById('resetBusy' + whichSearch);
		scroll.innerHTML = '<div><i class=\"fa fa-2x fa-spinner fa-pulse\"></i></div>';
		WE().util.rpc(WE().consts.dirs.WEBEDITION_DIR + "rpc.php?cmd=mediaDocsDelete", args, this.ajaxCallbackDeleteMediaDocs);
	},
	ajaxCallbackDeleteMediaDocs: function (weResponse) {
		var response = weResponse.DataArray.response;
		WE().util.showMessage(response.message, WE().consts.message.WE_MESSAGE_NOTICE, window);

		// close all Editors with deleted documents
		var _usedEditors = WE().layout.weEditorFrameController.getEditorsInUse(),
			_delete_table = WE().consts.tables.FILE_TABLE,
			_delete_Ids = ',' + response.deletedItems.join() + ',',
			frameId;

		for (frameId in _usedEditors) {
			if (_delete_table == _usedEditors[frameId].getEditorEditorTable() && (_delete_Ids.indexOf(',' + _usedEditors[frameId].getEditorDocumentId() + ',') !== -1)) {
				_usedEditors[frameId].setEditorIsHot(false);
				WE().layout.weEditorFrameController.closeDocument(frameId);
			}
		}

		//reload tree
		if (top.opener.top.treeData) {
			top.opener.we_cmd("load", top.opener.top.treeData.table, 0);
		}

		// reset busy
		document.getElementById("resetBusy" + weSearch.conf.whichSearch).innerHTML = '';

		weSearch.search(true, true);
	},

	/* moved from obsolete object/we_search.js */
	sub: function () {
		document.we_form_search.target = "load";
		document.getElementsByName("SearchStart")[0].value = 0;
		document.we_form_search.action = WE().consts.dirs.WEBEDITION_DIR + "we_cmd.php?we_cmd[0]=we_showMod&mod=search&pnt=cmd&cmd=search_submit";
		document.we_form_search.todo.value = "search";
		document.we_form_search.submit();
	}

	/*
	 newinput: function() {
	 document.we_form_search.target = 'load';
	 document.we_form_search.action = WE().consts.dirs.WEBEDITION_DIR + "we_cmd.php?we_cmd[0]=we_showMod&mod=search&pnt=cmd&cmd=search_submit";
	 document.we_form_search.todo.value = "add";
	 document.we_form_search.submit();
	 },
	 *
	 */

	/*
	 del: function(pos) {
	 document.we_form_search.target = 'load';
	 document.we_form_search.action = WE().consts.dirs.WEBEDITION_DIR + "we_cmd.php?we_cmd[0]=we_showMod&mod=search&pnt=cmd&cmd=search_submit";
	 document.we_form_search.todo.value = "delete";
	 document.we_form_search.position.value = pos;
	 document.we_form_search.submit();
	 }
	 *
	 */

	/*
	 function changeitanyway(f) {
	 document.we_form_search.target = 'load';
	 document.we_form_search.action = WE().consts.dirs.WEBEDITION_DIR + "we_cmd.php?we_cmd[0]=we_showMod&mod=search&pnt=cmd&cmd=search_submit";
	 document.we_form_search.todo.value = "changemeta";
	 document.we_form_search.submit();
	 }
	 *
	 */
};
