/* global top, WE,console */

/**
 * webEdition CMS
 *
 * $Rev: 62067699fe89 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2021-09-01 13:04 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
'use strict';
var weRpcFailedCnt = 0;

window.setInterval(function () {
	var
		data = {},
		activeEditors = WE().layout.weEditorFrameController.getEditorsInUse(),
		frameId,
		table,
		docID;
	for (frameId in activeEditors) {
		if (activeEditors[frameId].getEditorType() !== "model" || (docID = activeEditors[frameId].getEditorDocumentId()) === 0) {
			continue;
		}
		table = activeEditors[frameId].getEditorEditorTable();
		if (!data[table]) {
			data[table] = [];
		}
		data[table].push(docID);
	}
	WE().util.rpc(WE().consts.dirs.WEBEDITION_DIR + "rpc.php?cmd=usersPing", {"activeEditors": data}, function (result) {
		if (result && result.Success) {
			weRpcFailedCnt = 0;

			if (result.DataArray.users) {
				var num_users = result.DataArray.num_users ? result.DataArray.num_users : 0;
				var _ref = WE().layout.cockpitFrame;
				try {
					if (_ref) {
						if (_ref.setUsersOnline && _ref.setUsersListOnline) {
							_ref.setUsersOnline(num_users);
							var usersHTML = result.DataArray.users;
							if (usersHTML) {
								_ref.setUsersListOnline(usersHTML);
							}
						}
						var mfdData = result.DataArray.mfd_data;
						if (_ref.setMfdData && mfdData !== undefined) {
							_ref.setMfdData(mfdData);
						}
					}
				} catch (e) {
					//cockpit is closed
				}
			}

			let releases = result.DataArray.release;
			if (releases) {
				//FIXME: add support for release requests

				//requests handling
				if (releases.requests && releases.requests.length) {
					let msg = '';
					for (let req of releases.requests) {
						let id = parseInt(req.ID),
							tbl = req.tbl,
							obj = {};
						obj[tbl] = [id];
						if (req.forceTime) {
							top.window.setTimeout(forceReleaseDocuments, (req.forceTime > 0 ? req.forceTime - 5 : 30) * 1000, obj);
						}
						obj = getDocumentNames(obj);

						msg += WE().util.sprintf(WE().consts.g_l.main.unlockRequest,
							(req.User.trim() ? req.User : req.username),
							getDocumentTypeTranslation(tbl),
							obj[tbl].names[id],
							(req.forceTime ? " " + WE().consts.g_l.main.unlockRequestForceTo + " " + req.forceDate : ''),
							req.text
							) + '\n\n';
					}
					WE().util.showMessage(msg, WE().consts.message.WE_MESSAGE_INFO, window);
				}

				//reply handling
				if (releases.reply && releases.reply.length) {
					for (let rep of releases.reply) {
						console.log(rep);
					}
				}

				//reload documents handling
				if (releases.unlock && releases.unlock instanceof Object && !(releases.unlock instanceof Array)) {
					WE().layout.reloadUsedEditors(releases.unlock);

					msg = tbl = '';
					var docs = getDocumentNames(releases.unlock);
					for (tbl in docs) {
						if (docs.hasOwnProperty(tbl) && docs[tbl] instanceof Array) {
							for (var i = 0; i < docs[tbl]; i++) {
								if (parseInt(docs[tbl][i])) {
									msg += WE().util.sprintf(WE().consts.g_l.main.unlockRequestReleased,
										getDocumentTypeTranslation(tbl, true),
										docs[tbl][i]
										) + '\n\n';
								}
							}
						}
					}
					releases.unlock = null;
					WE().util.showMessage(msg, WE().consts.message.WE_MESSAGE_INFO, window);
				}
			}
		} else {
			var extra = (result && result.DataArray ? result.DataArray.data : "");
			WE().util.showMessage(WE().consts.g_l.main.unable_to_call_ping + "\n" + extra, WE().consts.message.WE_MESSAGE_ERROR, window, 20000);
		}
	}
	).fail(function (/*jqxhr, textStatus, error*/) {
		if (weRpcFailedCnt++ > 5) {
			//in this case, rpc failed 5 times, this is severe, user should be in informed!
			WE().util.showMessage(WE().consts.g_l.main.unable_to_call_ping, WE().consts.message.WE_MESSAGE_ERROR, window, 20000);
		}
	});
}, WE().consts.global.PING_TIME);


function getDocumentNames(docs) {
	var
		usedEditors = WE().layout.weEditorFrameController.getEditorsInUse(),
		curDoc;
	for (var frameId in usedEditors) {
		curDoc = docs[usedEditors[frameId].getEditorEditorTable()];
		if (curDoc && curDoc.indexOf(usedEditors[frameId].getEditorDocumentId()) !== -1) {
			if (!docs[usedEditors[frameId].getEditorEditorTable()].names) {
				docs[usedEditors[frameId].getEditorEditorTable()].names = {};
			}
			docs[usedEditors[frameId].getEditorEditorTable()].names[usedEditors[frameId].getEditorDocumentId()] = usedEditors[frameId].getEditorDocumentPath();
		}
	}
	return docs;
}

function getDocumentTypeTranslation(table, upper) {
	var translation;
	switch (table) {
		case WE().consts.tables.TEMPLATES_TABLE:
			translation = WE().consts.g_l.main.unlockRequestTemplate;
			break;
		case WE().consts.tables.OBJECT_FILES_TABLE:
			translation = WE().consts.g_l.main.unlockRequestObject;
			break;
		case WE().consts.tables.OBJECT_TABLE:
			translation = WE().consts.g_l.main.unlockRequestClass;
			break;
		case WE().consts.tables.VFILE_TABLE:
			translation = WE().consts.g_l.main.unlockRequestCollection;
			break;
		default:
			translation = WE().consts.g_l.main.unlockRequestDocument;
	}

	if (upper) {
		return translation.charAt(0).toUpperCase() + translation.slice(1);
	}

	return translation;
}

function forceReleaseDocuments(docs) {
	var
		usedEditors = WE().layout.weEditorFrameController.getEditorsInUse(),
		trans,
		url = WE().util.getWe_cmdArgsUrl(['save_document']),
		paths = "",
		curDoc;
	for (var frameId in usedEditors) {
		curDoc = docs[usedEditors[frameId].getEditorEditorTable()];
		if (curDoc && curDoc.indexOf(usedEditors[frameId].getEditorDocumentId()) !== -1) {
			trans = usedEditors[frameId].getEditorTransaction();
			top.doSave(url, trans);
			paths += usedEditors[frameId].getEditorDocumentPath() + "\n";
			WE().layout.weEditorFrameController.closeDocument(frameId);
		}
	}
	if (paths) {
		WE().util.showMessage(WE().consts.g_l.main.unlockReleaseDone + "\n" + paths, WE().consts.message.WE_MESSAGE_INFO, window);
	}
}