/* global WE, top, prefs */

/**
 * webEdition CMS
 *
 * $Rev: 62067699fe89 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2021-09-01 13:04 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
'use strict';
var widget = WE().util.getDynamicVar(document, 'loadVarWidget', 'data-widget');

var _oCsv_;
var _sInitCsv_;
var _oSctDate;
var _oSctNumEntries;
var _bPrev = false;
var _sLastPreviewCsv = '';


function exit_close() {
	if (_bPrev && _sInitCsv_ != _sLastPreviewCsv) {
		var aCsv = _sInitCsv_.split(';');
		WE().layout.cockpitFrame.rpc('mfd', aCsv[0], aCsv[1], aCsv[2], aCsv[3], aCsv[4], prefs._sObjId);
	}
	WE().util.dialogTop(window).exitPrefs();
	WE().util.dialogTop(window).doClose();
}

function isNoError() {
	var elem = document.getElementsByName('chbx_type');
	for (var i = 0; i < elem.length; i++) {
		if (elem[i].checked) {
			return true;
		}
	}
	return false;
}

function getUser() {
	let args = WE().util.getWe_cmdArgsArray(Array.prototype.slice.call(arguments));
	let url = WE().util.getWe_cmdArgsUrl(args);
	(new (WE().layout.overlay)(window, 'self', url, "browse_users", WE().consts.size.dialog.small, WE().consts.size.dialog.smaller)).setResizeable().setTitle(WE().consts.g_l.dialogTitle.selectUser).open();
}

function delUser(iUsrId) {
	let sUsers = '';
	if (iUsrId != -1) {
		let aUsers = widget.sUsers.split(',');
		let iUsersLen = aUsers.length;
		for (let i = 0; i < iUsersLen; i++) {
			if (aUsers[i] == iUsrId) {
				aUsers.splice(i, 1);
				iUsersLen--;
				break;
			}
		}
		for (let i = 0; i < iUsersLen; i++) {
			sUsers += aUsers[i];
			if (i != iUsersLen - 1) {
				sUsers += ',';
			}
		}
	}
	let form = document.forms[0];

	form.action = WE().consts.dirs.WEBEDITION_DIR + 'we_cmd.php?we_cmd[0]=widget_cmd&we_cmd[1]=dialog&we_cmd[2]=we_widget_mfd&we_cmd[]=' +
		prefs._sObjId + '&we_cmd[]=' + getBinary('type') + ';' + _oSctDate.selectedIndex + ';' + _oSctNumEntries.selectedIndex +
		';' + getBinary('display_opt') + ';' + sUsers;
	form.method = 'post';
	form.submit();
}

function getCsv() {
	return getBinary('type') + ';' + _oSctDate.selectedIndex + ';' + _oSctNumEntries.value +
		';' + getBinary('display_opt') + ';' + widget.sUsers;
}

function refresh(bRender) {
	if (bRender) {
		_sLastPreviewCsv = getCsv();
	}
	WE().layout.cockpitFrame.rpc('mfd', getBinary('type'), _oSctDate.selectedIndex, _oSctNumEntries.selectedIndex, getBinary('display_opt'), widget.sUsers, prefs._sObjId);
}

function init() {
	let form = document.forms[0];
	_oCsv_ = WE().util.dialogTop(window).opener.document.getElementById(prefs._sObjId + '_csv');
	_sInitCsv_ = _oCsv_.value;
	_oSctDate = form.elements.sct_date;
	_oSctNumEntries = form.elements.sct_amount_entries;
	WE().util.dialogTop(window).initPrefs();
	WE().util.setIconOfDocClass(document, 'mfdUIcon');
}

function getBinary(postfix) {
	let sBinary = '';
	let form = document.forms[0];
	let oChbx = form.elements['chbx_' + postfix];
	for (let box of oChbx) {
		sBinary += (box.checked) ? '1' : '0';
	}
	return sBinary;
}

function addUserToField() {
	var form = document.forms[0];
	var iNewUsrId = form.elements.UserIDTmp.value;
	var aUsers = widget.sUsers.split(',');
	var bUsrExists = false;
	for (let usr of aUsers) {
		if (usr == iNewUsrId) {
			bUsrExists = true;
			break;
		}
	}
	if (!bUsrExists) {//FIXME change this path!
		form.action = WE().consts.dirs.WEBEDITION_DIR + 'we_cmd.php?we_cmd[0]=widget_cmd&we_cmd[1]=dialog&we_cmd[2]=we_widget_mfd&we_cmd[]=' +
			prefs._sObjId + '&we_cmd[]=' + getBinary('type') + ';' + _oSctDate.selectedIndex + ';' +
			_oSctNumEntries.selectedIndex + ';' + getBinary('display_opt') + ';' + widget.sUsers + ',' + iNewUsrId;
		form.method = 'post';
		form.submit();
	}
}

function save() {
	if (isNoError()) {
		let sCsv = getCsv();
		_oCsv_.value = sCsv;
		WE().util.dialogTop(window).savePrefs();
		WE().util.dialogTop(window).opener.saveSettings();
		if ((!_bPrev && sCsv != _sInitCsv_) || (_bPrev && sCsv != _sLastPreviewCsv)) {
			refresh(false);
		}
		WE().util.showMessage(WE().consts.g_l.main.prefs_saved_successfully, WE().consts.message.WE_MESSAGE_NOTICE, window);
		WE().util.dialogTop(window).doClose();
	} else {
		WE().util.showMessage(WE().consts.g_l.cockpit.no_type_selected, WE().consts.message.WE_MESSAGE_ERROR, window);
	}
}

function preview() {
	if (isNoError()) {
		_bPrev = true;
		WE().util.dialogTop(window).previewPrefs();
		refresh(true);
	} else {
		WE().util.showMessage(WE().consts.g_l.cockpit.no_type_selected, WE().consts.message.WE_MESSAGE_ERROR, window);
	}
}
