/* global tinymce */

/**
 * webEdition CMS
 *
 * $Rev: fb5755151478 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2021-07-09 13:12 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
'use strict';

(function () {
	var wetable = (function () {
		var global = tinymce.util.Tools.resolve('tinymce.PluginManager');

		var Commands = {register: function (editor) {
				editor.addCommand('mceWetable', function () {
					var sel = editor.selection;
					if (editor.dom.getParent(sel.getNode(), 'TABLE') !== null) {
						editor.execCommand('mceTableProps');
					} else {
						editor.execCommand('mceInsertTable');
					}
				});
			}};

		var Buttons = {register: function (editor) {
				editor.ui.registry.addToggleButton('wetableinsert', {
					icon: 'table',
					tooltip: 'Table',
					onAction: function () {
						editor.execCommand('mceWetable');
					},
					onSetup: function (buttonApi) {
						editor.weControlManager.register('button', 'wetableinsert', buttonApi);
					}
				});
				editor.ui.registry.addMenuButton('wetablemenu', {
					tooltip: 'btnTable_' + editor.getParam('id'),
					icon: 'table',
					fetch: function (callback) {
						return callback('inserttable deletetable');
					},
					onSetup: function (buttonApi) {
						var split = editor.getParam('weWin').document.querySelector('[title="btnTable_' + editor.getParam('id') + '"]');
						split.title = 'Table';
						split.removeChild(split.firstChild);
					}
				});

				/*
				editor.ui.registry.addMenuButton('wetabletable', {
					tooltip: 'moretable',
					icon: 'more',
					fetch: function (callback) {
						return callback('');
					},
					onSetup: function (buttonApi) {
						var split = editor.getParam('weWin').document.querySelector('[title="moretable"]');
						split.title = 'Weitere Tabellen-Funktionen';
						split.removeChild(split.firstChild);
					}
				});
				editor.ui.registry.addMenuButton('wetablecells', {
					tooltip: 'morecells',
					icon: 'more',
					fetch: function (callback) {
						return callback('');
					},
					onSetup: function (buttonApi) {
						var split = editor.getParam('weWin').document.querySelector('[title="morecells"]');
						split.title = 'Weitere Zellen-Funktionen';
						split.removeChild(split.firstChild);
					}
				});
				 *
				 */
				editor.ui.registry.addMenuButton('wetablerows', {
					tooltip: 'morerows',
					icon: 'more',
					fetch: function (callback) {
						return callback('tabledeleterow tablecopyrow tablecutrow | tablepasterowbefore tablepasterowafter');
					},
					onSetup: function (buttonApi) {
						var split = editor.getParam('weWin').document.querySelector('[title="morerows"]');
						split.title = WE().consts.g_l.tinyMceTranslationObject.moreRowFunctions;
						split.removeChild(split.firstChild);
					}
				});
				editor.ui.registry.addMenuButton('wetablecols', {
					tooltip: 'morecols',
					icon: 'more',
					fetch: function (callback) {
						return callback('tabledeletecolumn tabledeletecol tablecopycolumn tablecopycol tablecutcolumn tablecutcol | tablepastecolumnbefore tablepastecolbefore tablepastecolumnafter tablepastecolafter');
					},
					onSetup: function (buttonApi) {
						var split = editor.getParam('weWin').document.querySelector('[title="morecols"]');
						split.title = WE().consts.g_l.tinyMceTranslationObject.moreColFunctions;
						split.removeChild(split.firstChild);
					}
				});
			}
		};

		global.add('wetable', function (editor) {
			Commands.register(editor);
			Buttons.register(editor);

			return {
				init: function (ed) {
					ed.on('NodeChange', (function (e) {
						ed.dom.removeClass(ed.dom.select('table'), 'we-table-extra-space-before');

						let n = ed.dom.getParent(e.element, 'TABLE');
						if (n && n.nodeName) {
							let ntop = n.getBoundingClientRect().top;
							/*let nbottom = n.getBoundingClientRect().bottom;
							let h = ed.dom.getParent(e.element, 'BODY').offsetHeight;
*/
							if(ntop >= 0 && ntop < 55 /*&& nbottom > (h - 55)*/){
								n.classList.add('we-table-extra-space-before');
							}
						}
					}));
				}
			};
		});

		function Plugin() {}

		return Plugin;

	}());
})();